% Contents.m
% Adaptive Filtering and Change Detection Toolbox
%
%  SIMULATION 
%    abrupt changes:      y = simchange(z,nnn)  
%    adaptive filter:     y = simadfilter(z,nnn)  
%    compute residuals:   y = simresid(z,nnn)  
%    dig.com. channel:    y = simchannel(z,nnn)  
%  FILTERING  
%    linear filters:   [thhat] = adfilter(z,nnn,adg,adm) 
%    Kalman filters:   [xhat]  = adkalman(z,nnn,adg,alg) 
%  DETECTION  
%    whiteness tests:  [thhat,jhat,thseg] = detect1(z,nnn,dm,sr)
%    sliding window:   [thhat,jhat,thseg] = detect2(z,nnn,L,dm,sr)
%                      [jhat,jtype,thseg] = multihyp(z,nnn,L)
%    multiple models:  [thhat,jhat,thseg] = detectM(z,nnn)
%    off-line methods: [jhat,thseg] = cpe(z,nnn)
%    GLR test:         [xhat,jhat] = GLR(z,nnn,L)
%    MLR test:         [jhat]      = MLR(z,nnn)
%    Gibb's Metropolis:[xhat,jhat] = gibbs(z,nnn)
%    fault detection:  [r]     = diagnosis(z,nnn,L)
%  EQUALIZATION
%    Viterbi algorithm     uhat = viterbi(y,nb,th);
%    Blind equalization    uhat = blindeq(y,nb,adg,adm)
%    ML sequence detection uhat = blindeqM(y,nb)
%  DESIGN  
%    computation of ARL fcn:    L0 = cusumarl(h,th-nu) 
%    computation of RL dist:    L0 = cusumMC(h,th-nu) 
%    computation of threshold:  h  = cusumdesign(L0,th-nu) 
%    computation of power:      beta = power(th0,th1,R,lambda,N,alpha) 
%  MODEL CONVERSION:  
%    piecewise constant to segmentation  thseg = par2seg(TH,jumptimes,N)
%    segmentation to piecewise constant  TH = seg2par(thseg)
%  PLOT    
%    illustration of jumps:     segplot(z,jumphat)  
%    comparison of parameters:  thplot(thhat,th0,jumptimes)  
%    Multiple-model hypothesis  hypplot
%    2D path plot               pathplot(x)
%    2D path plot with radar    radarplot(x)
%  DEMONSTRATION:  
%    Command line demos         demodetect (menu), demo*
%    Manual examples:           reference, tutorial 
%    Wiley book examples:       book 
%    Signal processing book:    dspbook
%  OBJECTS
%    Model or model structure   nnn
%  GUI
%    Graphical user interface   guidetect
%  BENCHMARK DATA (do help 'file' for description)
%    airbag       altdata     ash       bach       carspeed   ceram
%    current      defibrator  eeg_human eeg_rat    ekg        equake
%    ess          fricest     fuel      highway    labmotor   nmt450     
%    nmt900       nose        carpath   photons    planepath  salesment
%    sfquake      sheep       speech    stiction   tpi
%  IMPLEMENTATIONS
%  Example:
%    detect1          basic loop-based implementation
%    detect1c.c       C implementation
%    detect1c.mexsol  C implementation compiled for unix
%    detect1c.dll     C implementation compiled for PC
%    rdetect1         recursive implementation, doing one time recursion
%    rdetect1c.c      recursive C implementation
%    rdetect1c.mexsol C implementation compiled for unix
%    rdetect1c.dll    C implementation compiled for PC
%    sdetect1.mdl     Simulink block for compiled C-version
%    srdetect1.mdl    Simulink block for compiled recursive C-version
%    detect1x.m       Fast implementation using simulink but
%                     matlab syntax as detect1
% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $
