function MakeMatrix(V,filename,decimals,printopt)

% Generates a LaTex matrix from a matlab matrix
%
% function MakeMatrix(V,filename,decimals,printopt)
%
% V         matrix 
% filename  name of the .tex file to be produced
% decimals  number of decimals 
% printopt  if >0, then a full latex file is generated and
%           it is forwarded to latex and dvips

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[nrow,ncol]=size(V);
if nargin<3;   printopt=0; end;

eval(['fid=fopen(''',filename,'.tex'',''w'')']);

if printopt>0;
  fprintf(fid,'\\documentstyle[epsf,12pt]{article} \n');
  fprintf(fid,'\\textwidth 24cm \n');
  fprintf(fid,'\\textheight 18cm \n');
  fprintf(fid,'\\begin{document} \n');
end;

tabarg='c';
for i=2:ncol;
  tabarg=[tabarg,'c'];
end;

fprintf(fid,['\\begin{eqnarray*} \n']);
fprintf(fid,'\\left[ \n');
fprintf(fid,['\\begin{array}{',tabarg,'}  \n']);


dum=['%.',num2str(decimals),'f'];
for i=2:ncol;
  dum=[dum,' & %.',num2str(decimals),'f'];
end;
dum=[dum,' \\\\ \n'];

for j=1:nrow;
  fprintf(fid,dum,V(j,:));
end;

fprintf(fid,'\\end{array}   \n');
fprintf(fid,'\\right] \n');
fprintf(fid,'\\end{eqnarray*}   \n');
fprintf(fid,' \n');

if printopt>0;
  fprintf(fid,'\\end{document} \n ');
  fclose(fid)
  eval(['!latex ',filename,'.tex'])  
  eval(['!dvips ',filename,'.dvi'])  
else;
  fclose(fid)
end;





