function MakeTable(V,filename,decimals,xlabel,ylabel,title,printopt)

% Generates a LaTex table from a matlab matrix
%
% function MakeTable(V,filename,decimals,xlabel,ylabel,title,printopt)
%
% V         matrix to be tabulated
% filename  name of the .tex file to be produced
% decimals  number of decimals 
% xlabel    matrix where each row contains an x label
% ylabel    matrix where each row contains an y label
% printopt  if >0, then a full latex file is generated and
%           it is forwarded to latex and dvips

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[nrow,ncol]=size(V);
if nargin<7;   printopt=0; end;
if nargin<6;   title=' '; end;
if nargin<5;   
  ylabel=[' ']; for i=2:nrow; ylabel=[ylabel;' ']; end;
end;
if nargin<4;   
  xlabel=[' ']; for i=2:ncol; xlabel=[xlabel;' ']; end;
end;

eval(['fid=fopen(''',filename,'.tex'',''w'');']);

if printopt>0;
  fprintf(fid,'\\documentclass[12pt]{article} \n');
  fprintf(fid,'\\textwidth 18cm \n');
  fprintf(fid,'\\textheight 24cm \n');
  fprintf(fid,'\\begin{document} \n');
end;

tabarg='|l|';
for i=1:ncol;
  tabarg=[tabarg,'c|'];
end;

fprintf(fid,'\\begin{small} \n');
fprintf(fid,'\\centerline{ \n');
fprintf(fid,['\\begin{tabular}{',tabarg,'}  \n']);
fprintf(fid,'\\hline \n');
if length(title)>1;
  fprintf(fid,['\\multicolumn{',num2str(ncol+1),'}{|c|}{',title,'} \n \\\\  \\hline \n']);
end;

for i=1:ncol;
  fprintf(fid,[' & ',xlabel(i,:)]);
end;
fprintf(fid,' \\\\ \\hline \\hline \n');


for j=1:nrow;
  dum=[ylabel(j,:)];
  for i=1:ncol;
    dum=[dum,' & %.',num2str(decimals),'f'];
  end;
  dum=[dum,' \\\\ \n'];
  fprintf(fid,dum,V(j,:));
end;


fprintf(fid,'\\hline   \n');
fprintf(fid,'\\end{tabular}   \n');
fprintf(fid,'} \n');
fprintf(fid,'\\end{small} \n');

if printopt>0;
  fprintf(fid,'\\end{document} \n ');
  fclose(fid);
  eval(['!latex ',filename,'.tex'])  
  eval(['!xdvi ',filename,'.dvi'])  
else;
  fclose(fid);
end;





