function [F,G,H,D,Q,R,P0,x0]=airplane(t,x);

%[F,G,H,D,Q,R,P0,x0]=airplane(t,x);
%
% Returns a linear state space model for tracking
% State x=[x1,x2,v1,v2[,a1,a2]]'

if nargin<2; x=[]; end;
if nargin<1; t=0; end;

sigmar=100;
sigmath=0.1*pi/180;

qw=50;        % Tune by adg in adfilter
T=5;
n=4;         % Number of states (4 or 6)
method=1;    % Constant Q (1) or taylored Q (2)


if t==0;    % Initialization
  P0=1e8*eye(n);
  x0=[30000,20000,-200,0]';
  F=zeros(n,n); G=[]; H=zeros(2,n); D=[]; 
  Q=zeros(n); R=zeros(2);
else   % Recursion
  %----Dynamics-----
  if n==4;
    A=[0 0 1 0;
       0 0 0 1;
       0 0 0 0;
       0 0 0 0];
  else
    A=[0 0 1 0 0 0;
       0 0 0 1 0 0;
       0 0 0 0 1 0;
       0 0 0 0 0 1;
       0 0 0 0 0 0;
       0 0 0 0 0 0];
  end;

  %----Measurement-----
  H=[eye(2) zeros(2,n-2)];
 
  %----Sample A----
  F=eye(size(A))+T*A+T^2/2*A*A;   % A nilpotent
  % Compute state noise
  if method==1;
    Qd=T*qw*eye(2);
  elseif method==2;
    theta=atan2(xhat(4),xhat(3));
    qv=0.01*qw;   % State noise parallel to v
    Qd=[qv*cos(theta)^2+qw*sin(theta)^2  (qv-qw)*sin(theta)*cos(theta); ...
       (qv-qw)*sin(theta)*cos(theta)      qv*sin(theta)^2+qw*cos(theta)^2];
  end;
  Qd=[T^3/3*Qd T^2/2*Qd; T^2/2*Qd T*Qd];   % exact sampling
  if n==6;                                 % constant acceleration model
    Qd=[zeros(2,6); zeros(4,2) Qd];        % exact sampling
  end;
  Q=Qd;
  %-----Compute radar noise-----
  y=x(1:2);
  r=sqrt(y(2)^2+y(1)^2);
  th=atan2(y(2),y(1));
  b=(sigmar^2+r^2*sigmath^2)/(sigmar^2-r^2*sigmath^2);
  R=(sigmar^2-r^2*sigmath^2)*[b+cos(2*th) sin(2*th);sin(2*th) b-cos(2*th)];
  %----No input to the model-----
  G=[]; %zeros(n,1);
  D=[]; %zeros(2,1);
%  F,G,H,D,Q,R,pause
end
