function [F,G,H,D,Q,R,P0,x0]=airplanenl(t,x);

% Non-linear model of airplane
% Dynamics given by linearized discretization
% Sampled state space model with x=[x1 x2 v1 v2 w]'
% xp=g(x(t))
% G=[d/dx1(g1dot); ... d/dx5(g1dot)]

sigmar=100;
sigmath=0.1*pi/180;

if nargin<1; t=0; end;
n=5;
T=5;

if t==0;    % Initialization
  P0=1e8*eye(n);
  x0=[30000,20000,-200,pi,0]';
  F=zeros(n,n); G=zeros(n,1); H=zeros(2,n); D=zeros(1,1); 
  G=zeros(n,1);
  D=zeros(2,1);
  Q=zeros(n);
  R=zeros(2);
else   % Recursion
  x1=x(1);
  x2=x(2);
  v1=x(3);
  v2=x(4);
  w=x(5);
  xp=x;

  G=zeros(n,1);
  D=zeros(2,1);
  H=[eye(2) zeros(2,3)];
  qw=5;
  Q=diag([0 0 0 0 qw]);
  %-----Compute radar noise-----
  y=x(1:2);
  r=sqrt(y(2)^2+y(1)^2);
  th=atan2(y(2),y(1));
  b=(sigmar^2+r^2*sigmath^2)/(sigmar^2-r^2*sigmath^2);
  R=(sigmar^2-r^2*sigmath^2)*[b+cos(2*th) sin(2*th);sin(2*th) b-cos(2*th)];


  F=eye(5);
  if w==0;
    F=[1 0 T 0 -v2*T^2/2; 
       0 1 0 T  v1*T^2/2;
       0 0 1 0 -v2*T;
       0 0 0 1 v1*T;
       0 0 0 0 1];
  else;
    F(1,3)=sin(w*T)/w;
    F(1,4)=-1/w*(1-cos(w*T));
    F(1,5)=v1*(T*cos(w*T)/w-sin(w*T)/w^2)-v2*(T*sin(w*T)/w-(1-cos(w*T))/w^2);
    F(2,3)=(1-cos(w*T))/w;
    F(2,4)=sin(w*T)/w;
    F(2,5)=v1*(T*sin(w*T)/w-(1-cos(w*T))/w^2)-v2*(T*cos(w*T)/w-sin(w*T)/w^2);
    F(3,3)=cos(w*T);
    F(3,4)=-sin(w*T);
    F(3,5)=-v1*T*sin(w*T)-v2*T*cos(w*T);
    F(4,3)=sin(w*T);
    F(4,4)=cos(w*T);
    F(4,5)=v1*T*cos(w*T)-v2*T*sin(w*T);
  end
  
end










