function bz=b_sim(fm,pr,fc,fs,N)

% fm Maximum doppler frequency, fm
% pr Pole radius (2nd order)
% fc Butterworth cut-off frequency
% fs Sample frequency (default 25 kHz)
% N  Number of samples to simulate
% 
% White complex valued noise is passed through two filters in
% series.
%

if nargin < 1, fm=70; end
if nargin < 2, pr=0.9994; end
if nargin < 3, fc=pi/2*fm; end
if nargin < 4, fs=25000; end
if nargin < 5, N=20000; end

if rand('seed')==0, rand('seed',563613512); end

Nsim=max([N 1500]);

bz=zeros(N,1);
fd=fm/fs;

A2=[1 -2*pr*cos(2*pi*fd) pr^2];  B2=sum(A2);
[B7,A7]=butter(6,fc/fs);  

tkn=sign(randn(1,2));
init=tkn(1)*(.5+rand(1))+tkn(2)*i*(.5+rand(1));
ans=-conv(A2(2:3),[1 1]*init);
zi=ans(2:3);

e=6.25*(randn(Nsim,1)+i*randn(Nsim,1));

z=filter(B2,A2,e,zi);

[ans,zf]=filter(B7,A7,z(1:Nsim));  % 1500
bz=filter(B7,A7,z,zf);
bz=bz(1:N);

% ---------count level crossing rate ,lcr--------------

r=abs(bz); r_dB=20*log10(r);  r_eff=sqrt(4/pi)*mean(r);
dB=[-30:5:-10 -6:3:9];
n=length(dB); lcr=zeros(n,1); adf=zeros(n,1);

for k=1:n
  r_find=find(r_dB<dB(k));
  r_fix=fix(0.95*diff(r_find));
  xn=find(r_fix);
  n_xn=length(xn)+1;

  lcr(k)=n_xn/(N*fd);
  adf(k)=length(r_find)*fd/n_xn;

end





