/*
 *========================================================================
 *
 *rdetect1
 *
 *MEX-file implementing:
 *detect1 - detects changes using a one-filter approach
 *
 *Jonas Elbornsson July 1998
 *========================================================================
 */

#define S_FUNCTION_NAME rdetect2 /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "stop.h"


#define DM                 ssGetSFcnParam(S,0)    /*distance measure*/
#define SR                 ssGetSFcnParam(S,1)    /*stopping rule*/
#define TS                 ssGetSFcnParam(S,2)    /*sample time*/




static void mdlInitializeSizes(SimStruct *S)
{

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  ssSetNumDiscStates(             S,2);           /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,2)) return;   /*Number of inputports*/
  ssSetInputPortWidth(           S,0,2);         /*Width of inputport 0*/
  ssSetInputPortWidth(           S,1,2);
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/
  ssSetInputPortDirectFeedThrough(S,1,1);

  if (!ssSetNumOutputPorts(       S,2)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(           S,1,2);         /*Width of outputport 1*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,3);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }

    ssSetNumRWork(                  S,0); 
    ssSetNumIWork(                  S,0);           /*Number of integer workvector elements*/
    ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

}

 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}

#define MDL_INITIALIZE_CONDITIONS  
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/


  dstates[0]=0;                   /*initialize the decision function gt*/
  dstates[1]=0;

}
#endif


static void mdlOutputs(SimStruct *S,int_T tid)
{
  InputRealPtrsType eps = ssGetInputPortRealSignalPtrs(S,0);
  InputRealPtrsType lam = ssGetInputPortRealSignalPtrs(S,1);
  int_T dm = mxGetPr(DM)[0];
  real_T *sr = mxGetPr(SR);
  real_T *gt = ssGetRealDiscStates(S);
  int_T alarm;
  real_T st;
  real_T *GT;


  if (*lam[0]>0 && *lam[1]>0)
    {
      if (dm==1)
	st = 0.5*(log((*lam[0])/(*lam[1]))+(*eps[0])*(*eps[0])/(*lam[0])-(*eps[1])*(*eps[1])/(*lam[1]));
      else if (dm==2)
	st = 0.5*((*lam[0])/(*lam[1])-1+ (1+(*lam[0])/(*lam[1]))*(*eps[0])*(*eps[0])/(*lam[0])- 2*(*eps[0])*(*eps[1])/(*lam[1]));
      else if (dm==3)
	st = *eps[0] - *eps[1];
    }
  else
    st = 0;

  stop(gt,st,sr);

  if ((gt[0]>sr[1]) || (gt[1]>sr[1]))
    {
      alarm=1;
      gt[0]=0;
      gt[1]=0;
    }
  else
    alarm=0;
  
  ssGetOutputPortRealSignal(S,0)[0] = alarm;
  GT = ssGetOutputPortRealSignal(S,1);
  GT[0] = gt[0];
  GT[1] = gt[1];
}


static void mdlTerminate(SimStruct *S)
{
  return;
}


#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
