function L=cusumMC(h,mu,Niter,N);

% cusumMC computes the ARL distribution by Monte Carlo simulation
%
% L=cusumMC(h,mu,Niter,N);
%
% Assumptions: increments are white and Gaussian
% L     vector of stopping times
% h     threshold in cusum algorithm
% mu    mean of the signal th0 - drift parameter nu
% Niter number of simulations
% N     number of data in each simulation
%
% Hints: Do L0=mean(L) for mean delay for alarm, and hist(L)

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<3; Niter=10; end;
if nargin<4; N=1000; end;

for i=1:Niter;
  y=randn(N,1); 
  g=0;
  t=1;
  while (g<h) & (t<N)
    g=max([0,g+y(t)+mu]);
    t=t+1;
  end
  L(i)=t-1;
end
