function h=cusumdesign(L0,mu,h0,acc);

% CUSUMDESIGN converts specified mean run times to the design parameters 
%
% h=cusumdesign(L0,mu,h0,acc);
%
% A very crude search scheme using Siegmund's approximation 
% of the arl function
%
% L0    mean time between alarms
% h     threshold in cusum algorithm
% mu    mean E(s(k)) = true mean of signal th0 - drift parameter nu
% acc   relative error in L0. Default is 0.5.
% h0    initial guess
%
% Variance of s(k)=1 assumed
% See also cusumarl

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $


if nargin<4; acc=0.1; end;
if nargin<3; h0=10; end;

L=cusumarl(h0,mu,.01,-1);

while L < L0;
  h0=2*h0;
  L=cusumarl(h0,mu,.01,-1);
end;

h=h0;
h0=0;
iter=0; 
while abs(L-L0)/L0 > acc;
    iter=iter+1;
%    home
%    disp(['Iteration ',num2str(iter)])
%    disp(['L0hat =  ',num2str(floor(L)),'    '])
%    disp(['h =  ',num2str(h),'    '])
    if L > L0;
        h=h0+(h-h0)/2;
    else
        hdum=h;
        h=h+(h-h0)/2;
        h0=hdum;
    end;
    L=cusumarl(h,mu,.01,-1);
end;
h=acc*round(h/acc);
