% demo_altdata

echo on
clc
%          Altitude data
%         ===============

%        Variance increases in an altitude sensor
%       ===========================================
%
% The measurement error of one sensor for height of an aircraft
% based on barometric air pressure increases drastically for velocities
% around one Mach. It's might be convenient to detect variance
% increases so the influence of these data can be decreased.

% First we load real data from a file. The data consist of the 
% residuals from the sensor signal when an accurate estimate of
% the height has been removed.

load altdata
y=hdiff(1:4:15608);

h1=figure(1);
subplot(211)
book('plotfix')
p1=plot(y);
set([p1],'LineWidth',1);
title('Barometric height residuals')

% Make an estimate of the time-varying noise variance using no
% parametric model (nnn=0)

nnn=0;
[thhat,lamhat]=adfilter(y,nnn,0.97);
subplot(212)
book('plotfix')
p2=plot(lamhat);
set([p2],'LineWidth',2);
xlabel('Time [samples]')
title('Low-pass filtered squared residuals')

% We will use the function multihyp, again without a model. 
% Use a prior probability of 0.01 for a jump at each time instant

q=0.01;
[jhat,jtype,thseg,lamseg]=multihyp(y,nnn,100,[0 1 0],q);

%[jh,ths,lams]=detectM(y,0);
% jhat,jh


h2=figure(2);
book('plotfix')
p1=semilogy([lamhat,lamseg']);
set([p1(1)],'LineWidth',1);
set([p1(2)],'LineWidth',2);
title('Low-pass filtered and segmented squared residuals')

% There are totally
length(jhat)
% segments.

% We can use the result to compute a table of noise variances as a function
% of velocity, which might be useful in filtering.




