close all
echo on
% Monte Carlo simulation to find out beta (and alpha) as 
% a function of model order
% See ECC'95, Gustafsson and Ninnes.

Nsim=100;
nb=30;
lambda=.1;  %0.1 in paper
N=50;
alpha0=0.99;                    % confidence level 99%
Calpha=erfinv(2*(alpha0)-1);    % Assuming AsN of V(T)


%B0=fir1(nb,.5);
%B1=fir1(nb,.4);
% The "Astrom system", pole in 0.8376*exp(i*0.4592)
B0=filter(.2/1.5*[0 1 .5],[1 -1.5 0.7],[1;zeros(nb-1,1)]);
r=0.8376*exp(i*0.4);                     
% Move the phase angle from 0.46 to 0.4
B1=filter(.2/1.5*[0 1 .5],[1 -real(r+r') real(r*r')],[1;zeros(nb-1,1)]);

figure(1),plot([B0 B1])
title('Impulse response before and after change')
pause

B0=B0';B1=B1';
order=[1:nb];
th0=B0(order);
th1=B1(order);
echo off
for m=1:nb;
  beta0(m)=power2(B0(1:m)',B1(1:m)',eye(m),lambda,N,alpha0);
end;
echo on

delta=.000001+((B1(order)-B0(order)).^2)';
M=(1:nb)';
limit=lambda/2/N*(1+sqrt(8*Calpha^2./M))./(1+lambda*(M+Calpha*sqrt(2*M))/N./cumsum(delta));
figure(2)
plot([delta limit])
xlabel('m')
title('Variance/change trade-off')
pause

alpha=zeros(1,nb);
beta=zeros(1,nb);
V0=zeros(nb,Nsim);
V1=zeros(nb,Nsim);
echo off
for n=1:Nsim;
  u=randn(N,1);
  e=randn(N,1);
  y0=filter(B0,1,u);
  y0n=y0+sqrt(lambda)*e;
  y1=filter(B1,1,u);
  y1n=y1+sqrt(lambda)*e;
  for m=1:nb;
    Phi=zeros(N,m);
    for l=1:m;
      Phi(:,l)=[zeros(order(l)-1,1);u(1:N-order(l)+1)];
    end;
    P=inv(Phi'*Phi);
    f1=Phi'*(y1n-y0);
    f0=Phi'*(y0n-y0);
    V0(m,n)=(f0'*P*f0-m*lambda)/sqrt(2*m*lambda^2);        % Gaussian test
    V1(m,n)=(f1'*P*f1-m*lambda)/sqrt(2*m*lambda^2);        % Gaussian test
%    V0(m,n)=f0'*P*f0/lambda;                               % chi-2 test
%    V1(m,n)=f1'*P*f1/lambda;                               % chi-2 test
    if V0(m,n) < Calpha; alpha(m)=alpha(m)+1; end;
    if V1(m,n) > Calpha; beta(m)=beta(m)+1; end;
  end;
end;
echo on
alpha=alpha/Nsim;
beta=beta/Nsim;

figure(1)
plot([alpha' beta'])
hold on
plot([alpha0*ones(size(beta0')) beta0'],'--')
hold off
title('Confidence level and power of test')
xlabel('Model order')
echo off