% demo_carpath

echo on
clc
%               Segmentation of driven path
%              =============================

% These data were collected from test drives with a Volvo 850 GLT
% using sensor signals from the ABS system. From these data, the position
% of the car can be calculated very roughly.
%
% Load and look at data

load carpath
N=length(x);

h1=cdfigure(1);
plot(v)
title('Velocity')
h2=cdfigure(2,1);
plot(x,y)
axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x(1),y(1)-50,'t=0 s')
text(x(N)+50,y(N)+50,'t=225 s')

pause

% Next we compute the heading angle of the car
phi=phase(i*diff(y)'+diff(x)')';

% The heading angle within each segment is modeled as a first order 
% polynomial in t, h(t) = c1 + c2 * t, and the segm function is used 
% with 10 parallel filters and required accuracy of the prediction errors
% corresponding to a variance of 0.05.
N=length(phi);
M=10;
nnn=-2;
t=(1:N)';
t0=clock;
[jhat,thseg]=detectM([phi ones(size(t)) t],nnn,[1 0.05],.05,M);
consumedtime=etime(clock,t0)
estpath=thseg'.*[ones(size(t)) t]*[1;1];

figure(h2)
plot(x,y,'-',x(jhat),y(jhat),'o')
axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x(1),y(1)-50,'t=0 s')
text(x(N)+50,y(N)+50,'t=225 s')

figure(h1)
plot(t,phi,'-',jhat,phi(jhat),'o',t,estpath,'--')
title('Actual and segmented heading angle')
xlabel('Time [s]')
ylabel('Heading angle [rad]')
pause

% Do you want to see the optimal segmentation (it takes a minute)?
a=input('y/n','s');
if a=='y';
  t0=clock;
  [jhatopt,thsegopt,V,l,Nfilt]=optseg([phi ones(size(t)) t],nnn,[1 0.05],.05,0.0000001);
  consumedtime=etime(clock,t0)
  % Compare the estimated jump times
  [jhat';jhatopt']
  % The estimates are almost identical!
  %
  segplot(Nfilt',jhatopt) 
  title('Number of parallel filters and optimal segmentation') 
  pause
end

close([h1 h2])



