clc
echo on
%           Demonstration of change point estimation
%          ==========================================
%
% There is a special branch in statistics concerned with estimating
% the change point in the mean of a stochastic process.
% Let's look at some examples.

% H0 signal:
y0=[1*ones(50,1);1*ones(50,1)]+.1*randn(100,1);
% H1 signal:
y1=[1*ones(50,1);2*ones(50,1)]+.1*randn(100,1);

% Use an unknown mean model (required in cpe)
nnn=-1;

% Bayesian approach
[jh,ths,GT01,U01]=cpe(y0,nnn,1);
[jh,ths,GT11,U11]=cpe(y1,nnn,1);
h1=cdfigure(1);
plot([GT01 GT11])
pause

% Maximum likelihood approach
[jh,ths,GT02,U12]=cpe(y0,nnn,2);
[jh,ths,GT12,U12]=cpe(y1,nnn,2);
% Compare with the GLR test
[jh,ths,lr0]=glr(y0,nnn);
[jh,ths,lr1]=glr(y1,nnn);
h2=cdfigure(2,1);
plot([GT02 GT12 2*lr0(1:99) 2*lr1(1:99)])
pause

% Non-parametric approach I
[jh,ths,GT03,U13]=cpe(y0,nnn,3);
[jh,ths,GT13,U13]=cpe(y1,nnn,3);
h3=cdfigure(3,2);
plot([GT03 GT13])
pause

% Non-parametric approach II
[jh,ths,GT04,U14]=cpe(y0,nnn,4);
[jh,ths,GT14,U14]=cpe(y1,nnn,4);
h4=cdfigure(4,3);
plot([GT04 GT14])
pause

echo off
