% demo_equalization

if ~exist('demodigcomarg'); demodigcomarg=0; end
if ~exist('saveplots'); saveplots=0; end


if demodigcomarg==0
clc
disp(' ')
disp('viterbi: Demonstration of equalization for a real SISO noisy channel')
disp(' ')
echo on

N=100;
randn('seed',1);
b=[1 1 1];
alphabet=[-1 1];
[y,u]=simchannel(alphabet,3,N,b);

% BER for bad channel estimate
bhat=[1 0 0];
uhat=viterbi(y,bhat,[-1 1],0.01);
ber=u2ber(u,uhat)

% BER for good channel estimate
bhat=[1 0.7 1.3];
uhat=viterbi(y,bhat,[-1 1],0.01);
ber=u2ber(u,uhat)

% BER when using a short training sequence
[uhat,thhat]=viterbi(y,3,[-1 1],0.01,u(1:7));
thhat
ber=u2ber(u,uhat)

% Using a general input alphabet
alphabet=[-1 1 sqrt(-1) -sqrt(-1)];
[yc,uc]=simchannel(alphabet,3,N,b);
uchat=viterbi(yc,b,alphabet);
echo off
ber=u2ber(uc,uchat);
disp(['BER  = ',num2str(ber)])

end



if demodigcomarg==1
clc
disp(' ')
disp('blindeq: Demonstration for real time-invariant SISO noise-free channel')
disp(' ')
echo on

N=1000;
randn('seed',1);
b=[0.3 1 0.3]
alphabet=[-1 1];
[y,u]=simchannel(alphabet,3,N,b);
y=y; %+0.1*randn(N,1);
nc=5;   % equalizer order for blindeq

c0=[0 -0.1 1 -0.1 0]';

[theq1,uhat1,epsi1]=blindeq(y,nc,0.01,1,c0);
[theq2,uhat2,epsi2]=blindeq(y,nc,0.01,2,c0);

dm1=openeye(b,theq1);
m2=openeye(b,theq2);
cdfigure(1);
h1=plot(m1,'-');  hold on
h2=plot(m2,'--'); hold off
title('Open eye condition')
legend([h1 h2],'Sato','Godard',3)
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq1a.eps, end

cdfigure(2,1);
plot([epsi1' epsi2'])
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq1b.eps, end

%cdfigure(3,2);
figure
h1=plot(1:3,b,'b-');                   hold on
h2=plot(1:5,theq1(:,N)','r:'); 
h3=plot(1:7,conv(b,theq1(:,N)'),'g--'); hold off
legend([h1 h2 h3],'Channel','Equalizer','Combined channel+equalizer',2)
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq1c.eps, end

echo off

ber1=u2ber(u,uhat1);
ber2=u2ber(u,uhat2);
disp(['BER Sato       = ',num2str(ber1)])
disp(['BER Godard     = ',num2str(ber2)])


end


if demodigcomarg==2
clc
disp(' ')
disp('blindeqM: Demonstration for real time-invariant SISO noisy channel using')
disp(' ')
echo on

N=100;
randn('seed',1);
rand('seed',1);
b=[0.5 1 0.7];
alphabet=[-1 1];
[y,u]=simchannel(alphabet,3,N,b);
y=y+0.1*randn(N,1); % Noise on y
nb=3;     % channel order for blindeqM

[thhat,uhat]=blindeqM(y,nb,[-1,1],0.001,16,.1,10,0);

[ber,uhatout,delay,m]=u2ber(u,uhat);
ber

cdfigure(1);
plot(m*thhat') % Correct modulus
hold on
plot(ones(N,1)*b,'--')
hold off
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq2.eps, end

echo off

ber=u2ber(u,uhat); 
disp(['BER blindeqM = ',num2str(ber)])

end


if demodigcomarg==3
clc
disp(' ')
disp('blindeqM: Demonstration of real time-varying SISO channel')
disp(' ')
echo on

randn('seed',1);
N=100;
[y,u,b]=simchannel([-1 1],2,N,[]);
nb=2;

[thhat,uhat]=blindeqM(y,nb,[-1,1],.001,64,.1,10,0);

[ber,uhatout,delay,m]=u2ber(u,uhat);
ber

cdfigure(1);
plot(m*thhat') % Correct modulus
hold on
plot(b')
hold off
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq3.eps, end

echo off

disp(['BER = ',num2str(ber)])

end


if demodigcomarg==4
clc
disp(' ')
disp('blindeqM: Demonstration of real time-invariant SIMO channel')
disp(' ')
echo on


rand('seed',0);
b=[3 2 1   6 5 4];
a=1;
nn=[3;3];

N=200;
sigma=1;

[y,u]=simchannel([-1 1],nn,N,b(:));
y=y+sigma*randn(N,2);

[thhat,uhat,ber]=blindeqM(y,nn,[-1,1],sigma^2,64);

[ber,uhatout,delay,m]=u2ber(u,uhat);
ber

cdfigure(2,1);
plot(m*thhat')
hold on
bdum=b';
b=bdum(:);

plot((ones(N,1)*b'))
hold off
axis([0 N 0 7])
title('Estimated (solid) and true (dashed) parameters')
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq4.eps, end

echo off
end


if demodigcomarg==5
clc
disp(' ')
disp('blindeqM: Demonstration of real time-varying SIMO channel')
disp(' ')
echo on


randn('seed',0);
N=100;
nb=2; ny=2; nu=1;
sigma=0.01;

randn('seed',1);
[y,u,th]=simchannel([-1 1],nb*ones(ny,nu),N,[]);
y=y+sigma*randn(N,ny);

[thhat,uhat]=blindeqM(y,[2;2],[-1,1],sigma^2,64,.1,10,0);
[ber,uhatout,delay,m]=u2ber(u,uhat);
ber
cdfigure(3,2);
plot(m*thhat','-')
hold on
plot(th','--')
hold off
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq5.eps, end

echo off

end


if demodigcomarg==6
clc
disp(' ')
disp('blindeqM: Demonstration of real time-invariant MIMO channel')
disp(' ')
echo on

rand('seed',0);
N=64;
sigma=0.0;

th=[1 0.9   0.8 0.7    0.6 0.5   0.4 0.3];
nb=2;
ny=2;
nu=2;

[y,u]=simchannel([-1 1],nb*ones(ny,nu),N,th);
y=y+sigma*randn(N,ny);

[thhat,uhat]=blindeqM(y,nb*ones(ny,nu),[-1,1]);

%th', thhat(:,N)
disp([th; thhat(:,N)'])

[ber,uhatout,delay,m]=u2ber(u,uhat);

cdfigure(4,3);
plot(m*thhat')
hold on
plot(ones(N,1)*th,'--')
hold off
axis([0 N 0.2 1.1])
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq6.eps, end

echo off
disp(['BER = ',num2str(ber)])
end


if demodigcomarg==7
clc
disp(' ')
disp('blindeqM: Demonstration of complex channel and inputs')
disp(' ')
echo on

randn('seed',0);
N=50;
ny=1; nu=1; nb=2;
sigma=0;
th=[];
[y,u,th]=simchannel([-1 1 -sqrt(-1) sqrt(-1)],nb*ones(ny,nu),N,th);
y=y+sigma*randn(N,ny);

[thhat,uhat]=blindeqM(y,nb,[-1,1],.001,64,.1,10,0);

[ber,uhatout,delay,m]=u2ber(u,uhat);
ber

cdfigure(5,4);
plot(m*[real([thhat' ]) imag([thhat'])],'-')
hold on
plot([real([th' ]) imag([th'])],'--')
hold off
if saveplots==1; print -depsc /home/rt/fredrik/books/manual/figures/tutbleq7.eps, end

echo off

cdfigure(6,5)
plot([real(u) imag(u)])
hold on
plot([real(uhat) imag(uhat)])
hold off

disp(['BER = ',num2str(ber)])

end

