% demo_fricest

echo on
clc
%          Friction estimation
%         =========================

% This example uses real data, but simulates a friction change at
% time 200. First, load data and define the true parameters in the friction 
% model.

  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
%  s(201:N)=s(201:N)+(TH0(2,2)-TH0(2,1))*mu(201:N);  % Friction change

% Next, take a look at the time series of slip and traction force
  pause

  figure(1)
  subplot(211)
  plot(s)
  ylabel('Slip')
  subplot(212)
  plot(mu)
  ylabel('Normalized traction force')
  xlabel('Time [samples]')
  
  % The slip is a linear function of traction force, where the
  % slope depends on the surface, as illustrated in the following
  % plot. 
  pause

  figure(2)
  h1=plot(s(1:200),mu(1:200),'x');
  hold on
  h2=plot(s(201:N),mu(201:N),'o');
  a=axis;
  mumax=a(4);
  h3=plot([threc(1,200) threc(1,200)+threc(2,200)*mumax],[0 mumax]);
  h4=plot([threc(1,N) threc(1,N)+threc(2,N)*mumax],[0 mumax],'--');
  hold off
  ylabel('Normalized traction force')
  xlabel('Slip')
  legend([h1 h2 h3 h4],'Measurements before change','Measurements after change','Friction model before change','Friction model after change',2)
  pause

% Try two different adaptive filters, using a linear regression model
  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);
  figure(3)
  h1=plot(threc');
  set(h1,'LineWidth',1);
  hold on
  h2=plot(threc2');
  set(h2,'LineWidth',2);
  h3=plot(th0','--');
  axis([0 N 0 0.05])
  hold off
  xlabel('Time [samples]')
  legend([h3(1) h1(1) h2(1)],'True parameters','Estimated parameters, fast filter','Estimated parameters, slow filter',2)

% Try now the simplest possible change detector
pause

  h=3; nu=0.2;
  [jhat,thseg,gt,ta,threc]=detect1([s Phi],-2,1,[1 h nu]);

  figure(1)
  subplot(211)
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
  set(h1,'LineWidth',1);
  set(h2,'LineWidth',2);
  hold off
  legend([h1(1) h2(1)],'Estimated parameters','True parameters',2)

  subplot(212)
  h2=plot(gt,'b-');
  axis([0 N 0 h*1.5])
  set(h2,'LineWidth',1)
  hold on, p1=plot([1 N],[h h]);, hold off
  set(p1,'LineWidth',4)
  xlabel('Time [samples]')
  ylabel('Test statistic g_t')

