% demo_fuel

echo on
clc
%          Fuel consumption filter
%         =========================

load fuel
% The data contain instantaneous fuel consumption in a test drive with
% a Volvo 850 GLT
%
% Plot the data and try low-pass filtering to attenuate the noise

N=length(fuel);
h1=cdfigure(1);
plotfix
plot(0.2*(1:N),fuel)
axis([0 N*0.2 0 25])
ylabel('Fuel consumption [liters / 100 km]')
xlabel('Time [s]')
title('Measurements')

lambda=0.8;
fuelf1=filter(1-lambda,[1 -lambda],fuel);
lambda=0.95;
fuelf2=filter(1-lambda,[1 -lambda],fuel);

h2=cdfigure(2,1);
plotfix
plot(0.2*(1:N),fuelf1,'y-',0.2*(1:N),fuelf2,'g--')
axis([0 N*0.2 0 25])
ylabel('Fuel consumption [liters / 100 km]')
xlabel('Time [s]')
title('Low-pass filtered measurements, \lambda=0.8 and 0.95')
%
% Low-pass filtering gives either slow response to changes or
% too noisy results in steady state preventing effective optimization
% of driving style
%
% Let's look at Volvo's compromise (my own implementation)
%
pause

lambda=0.90;
fuelf=filter(1-lambda,[1 -lambda],fuel);
fuelf=round(fuelf*10/3)*3/10;
figure(h1)
plot(0.2*(1:N),fuelf,'-',0.2*(1:N),fuel,'g--')
axis([0 N*0.2 0 25])
ylabel('Fuel consumption [liters / 100 km]')
xlabel('Time [s]')
title('Raw data and low-pass filtered quantized measurements (Volvo)')
if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/fuel1.eps, end
pause

% Now, use recursive segmentation to get both good noise attenuation
% and fast tracking ability.
%
% Use an unknown mean model and an approximate noise variance lam.

nnn=-1;
lam=9;
[jhat,thseg,lamhat,threc]=detectM(fuel,nnn,[1 lam],.5);
figure(h1);
plotfix
plot(0.2*(1:N),fuel,'--',0.2*(1:N),thseg,'-')
axis([0 N*0.2 0 25])
ylabel('Fuel consumption [liters / 100 km]')
xlabel('Time [s]')
title('Raw data and segmentation')
figure(h2);
plot(0.2*(1:N),fuel,'--',0.2*(1:N),threc,'-')
axis([0 N*0.2 0 25])
ylabel('Fuel consumption [liters / 100 km]')
xlabel('Time [s]')
title('Raw data and change detection filter')
if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/fuel1.eps, end
pause

    
close([h1 h2])

% Generate table
load fuel
N=length(y);
t=0.2:0.2:0.2*N;
threc=adfilter(y,-1,0.9);
threc1=detect1(y,-1,2,[2 3 2]);
threc2=detect2(y,-1,1,[2 20 3],3);
threcM=detectM(y,-1,[1 3],[],5);

