% demo_methods1

echo on
% Simulate an ARX model
randn('seed',1)
u=randn(100,1);    % Input
e=randn(100,1);    % Noise
jumptimes=[40 70];
TH=[1.5 .8 2 0.5;...
    1.5 .8 2 1;...
    1.5 .6 2 1]';
nnn=[2 2 1];       % Model structure
y=simchange([0.1*e u],nnn,jumptimes,TH);
z=[y u];
h1=cdfigure(1);
segplot(y,jumptimes);
title('Signal and real change times')
% 
% It's not easy to estimate the change times by visual inspection.
%
% First: a filtering approach

pause
close(h1)
clc

%             FILTERING
%            -----------
%
% First, we examine the signal with RLS with forgetting factor 0.9

thRLS=adfilter(z,nnn,0.9);
thplot(thRLS,TH,jumptimes);
%
% You can guess where the abrupt changes are but the estimates 
% are noisy.
% Other forgetting factors than 0.9 give either slower response or 
% worse estimate.

% Next: multi-model approach
pause
close(h1);


clc
%             MULTI-MODEL DETECTION
%            -----------------------
%
% Second, the segmentation function "segm" is studied
% Here a number of different filters are run in parallel
% and a weighting coefficient (the likelihood) determines
% which is the best one.

[jhat,thseg,lamhat,threc,Alfa]=detectM(z,nnn);jhat
segplot(y,jhat);
title('Signal and estimated change times')
%
% The exact change times are mostly estimated correctly 
% in this example.
pause

h2=cdfigure(2,1);
thplot(thseg,TH,jumptimes);
%
% The parameter estimate looks very good!
pause

plot(Alfa)
axis([0 100 0 10])
title('Negative log likelihood for each model')
%
% The behaviour of the weighting coefficiencts is quite stable 
% before a jump, but rather chaotic afterwards.
% Note that it takes about 6 samples before one model becomes the best

% Next: one-model approach
pause
close([h1 h2])
clc
%             ONE-MODEL APPROACH
%            --------------------
%
% The one-model approach tries to estimate the jumps by just
% looking at the residuals from a standard filter.
%
% Two different loss functions are possible, but the mean residual does
% not make sense here.
% The sum of squared residuals (1) is therefore used.
% There are two conceivable stopping rules: the CUSUM and GMA detectors.
% These decide when the squared residual is large enough to indicate a jump
% Let's test the CUSUM test with threshold 10 and drift parameter 5.

[jhat,thseg,gt2,ta,threc]=detect1(z,nnn,2,[1 10 5],0.01);jhat
thplot(thseg,TH,jumptimes);
%
% Once again the result is very good. The one-model approach is
% furthermore extremely cheap to compute and easy to implement.
pause

% Let's look at the loss function

h2=cdfigure(2,1);
plot([gt2(:,1)])
title('Distance measure for the stopping rule')
% Next: two-model approach
pause
close([h1 h2])

clc
%             TWO-MODEL APPROACH
%            --------------------
%
% In the two-model approach one model is estimated on all past data
% and compared to one estimated in a sliding window.
%
% There are two suggested distance measures, Brandt's GLR test and
% the divergence test. The same stopping rules as for the one-model
% approach are possible.

[jhat,thseg1,gt3,ta]=detect2(z,nnn,1,[1 5 0.3],10,-1);jhat
[jhat,thseg2,gt4,ta]=detect2(z,nnn,2,[1 10 3],10,-1);jhat
thplot(thseg1,TH,jumptimes);
h2=cdfigure(2,1);
thplot(thseg2,TH,jumptimes);

% The jump times are estimated correctly but the parameter estimate
% is not as good as in the segm function.
pause

h3=cdfigure(3,2);
plot([gt3(:,1) gt4(:,1)])
title('Distance measure for the stopping rule')

% Note the scaling on the distance measure for the one-model
% approach. This implies that different thresholds that are required 
% in the stopping rules!
% This makes the design a bit difficult
pause
close([h1 h2 h3])
echo off             



