% script demo_rec1

clc
clg
echo on
echo simchange off
echo rdetect1 off
echo radfilter off
echo detect1 off
echo adfilter off
echo detectM off

%         RECURSIVE ONE-MODEL DETECTION
%        ===============================
%
% There are a number of recursive implementations of the functions
% in DetectTB. These have the prefix 'r', e.g. radfilter.
% The syntax of these functions are almost the same as in the original,
% except that a state has to be included in the input and output arguments.
% The state is initialized by calling the function itself with no data.
% In this way, the user can use his own filters, e.g. EKF, and make
% appropriate actions after a detected change.
%
% In this demo, a first order AR model with one jump is simulated.

randn('seed',1)
N=100;
u=randn(N,1);
e=randn(N,1);
th0=[0.5 1.5;0.8 1.8]';
d=2;
lam=0.1;
nnn=[1 1 1];
y=simchange([lam*e u],nnn,N/2,th0);
pause

% Then, we initialize the recursive versions of filt and detect1

[Xfilt]=radfilter([NaN NaN],nnn);
dm=2;
h=5;
nu=lam;
sr=[1 h nu];
[Xone]=rdetect1([],[],dm,sr,-1);
pause

% Let's also initialize some parameters we want to save for plots

TH=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop does what 
%
%     detect1([y u],nnn,dm,sr,-1)
%
% would do.
%
% Note that the task of residual generation and the detector is split into 
% separate functions in contrast to detect1.

pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt]=radfilter([y(t) u(t)],nnn,0.97,'RLS',Xfilt);
  [alarm,Xone]=rdetect1(epsi/sqrt(S),Xone);
  GT1=[GT1;Xone(1)]; 
  TH=[TH thhat];
  LAM=[LAM;lamhat];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
  end;
end;

% for comparison, we generate the smoothed parameter estimates from SEGM,
% the test statistic off-line from ONEMODEL and the estimates from FILT.

[jhat,thseg]=detectM([y u],nnn);
[jdum,thdum,GT2]=detect1([y u],nnn,dm,sr,-1);
[TH2,LAM2]=adfilter([y u],nnn,0.97);
h1=cdfigure(1);
plot([thseg' TH' TH2'])
axis([0 N -1 3])
h2=cdfigure(2,1);
segplot([GT1],alarmtimes)
hold on, plot(GT2), hold off
title(['Test statistic and alarm times, Threshold = ',num2str(h)])

% From figure 1 we conclude that the same estimates are generated
% and from figure 2 that we get several alarms after the change.
pause

% The most appropriate thing to do after the alarm is to increase 
% the tracking ability of the filter. This is done below by increasing
% the covariance matrix of the parameters a factor 10.

[Xfilt]=radfilter([NaN NaN],nnn);
[Xone]=rdetect1([],[],dm,sr,-1);
TH=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt]=radfilter([y(t) u(t)],nnn,0.97,'RLS',Xfilt);
  [alarm,Xone]=rdetect1(epsi/sqrt(S),Xone);
  GT1=[GT1;Xone(1)]; 
  TH=[TH thhat];
  LAM=[LAM;lamhat];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
    Xfilt(:,5:4+d)=10*Xfilt(:,5:4+d);
  end;
end;
h3=cdfigure(3,2);
segplot([GT1],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])

% This time there is only one alarm.
pause
close(h1)
close(h2)
close(h3)
