% script demo_rec2

echo on
%         RECURSIVE GLR DETECTION
%        =========================
%
% First, a second order state space model with one jump is simulated.

randn('seed',0)
N=100;
d=2;
%w=randn(N,2);
u=randn(N,1);
%e=randn(N,1);
A=[1 1;0 1]; B=[0.5;1]; C=[1 0]; D=0; R1=.01*eye(2); R2=0.1; P0=1*eye(2);
nnn=ss2nnn(A,B,C,D,R1,R2,P0);

th0=[1;...
     2];

%y=simchange([sqrt(R2)*e u w*sqrt(R1)],nnn,N/2,th0);
y=simchange([u],nnn,N/2,th0);

h1=cdfigure(1);
plot(y)
title('Signal')
pause

% First, let's examine the off-line versions of GLR and also MLR

M=-10;  % size of sliding window, batch test here
flops(0)
[jhatglr,taglr,lrglr,Xhat,nuhatglr,mu]=glr([y u],nnn,M);
flopsglr=flops;
[jhatmlr,lrmlr,flopsmlr]=mlr([y u],nnn,1);
jhatglr,jhatmlr
nuhatglr

figure(h1)
plot([y, (C*Xhat)'])
hold on
plot(N,C*(Xhat(:,N)+mu*nuhatglr(:,1)),'o')
hold off
title('Signal and Kalman filtered estimate')

h2=cdfigure(2,1);
plot([lrglr lrmlr])
title('Decision function for GLR and MLR')
  
% That is, a threshold of 70 seems appropriate
% The number of flops compares as follows
disp('GLR   MLR')
disp([flopsglr flopsmlr])
pause
close([h1 h2])

% Then, we initialize the recursive versions of filt and glr

[Xfilt]=radfilter([NaN NaN],nnn);
h=72;
M=10;
[Xglr]=rglr([],[],[],nnn,[],M,h);
pause

% Let's also initialize some parameters we want to save for plots

TH=[];
EPSI=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop uses the recursive implementation of GLR
%
% Note that the task of residual generation and the detector is split into 
% separate functions.
%
% Start recursion
pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt,K]=radfilter([y(t) u(t)],nnn,1,'RLS',Xfilt);
  f0=flops;
  [alarm,Xglr,nuhat,jhat,lrmax,P]=rglr(epsi,S,K,nnn,Xglr);
  f1=flops;
  GT1=[GT1;lrmax]; 
  EPSI=[EPSI;epsi];
  TH=[TH nuhat];
  if alarm~=0; 
    nuhat
    P
    alarmtimes=[alarmtimes t]; 
    Xfilt(:,6:5+d)=100*Xfilt(:,6:5+d);   % Increase P in the Kalman filter
  end;
end;
% The number of flops per iteration is
disp([f1-f0])

h3=cdfigure(3,2);
segplot([GT1],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
pause

close(h1)
close(h2)
close(h3)
