% demo_tracking
disp('Target tracking demo')

if ~exist('saveplots'); saveplots=0; end
if ~exist('demotrackingarg'); demotrackingarg=0; end

if demotrackingarg==0 | demotrackingarg>0
  if demotrackingarg==0 
    clc
    disp(' ')
    disp('Data simulation ')
    disp(' ')
    echo on
  end
  close all
  randn('seed',3);
  N=70;   % Simulation length
  lam=1;
  nnn='nnnplane';  % Motion model
  global NNNmethod % with global switch
  NNNmethod=5;     % Five state model
  u=zeros(N,1);    % No input
  th0=[-0.025;...  % Define state changes in turn rate
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];  % Define true jump times
  % Start a simulation
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse'); 

  cdfigure(1);
  plotfix
  p1=radarplot(y,x0); % Nice illustration
  legend(p1,'Measured position','True position')
  echo off
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking1.eps, end
end
    
if demotrackingarg==1
  clc
  disp(' ')
  disp('adfilter and adkalman: Apply different versions of the Kalman filter')
  disp(' ')
  echo on
  NNNmethod=1; % Change to forth order linear model for estimation
  [xhat0]=adfilter(y,nnn,1); % Kalman filtering
  cdfigure(2,1);
  plotfix
  radarplot(y,xhat0);
  title('Result from adfilter with adgain=1')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking2.eps, end

  [xhat1]=adkalman(y,nnn,1,1); % stationary Kalman filter
  [xhat2]=adkalman(y,nnn,1,2); % Kalman filter
  [xhat3]=adkalman(y,nnn,1,3); % Kalman filter square root
  [xhat4]=adkalman(y,nnn,1,4); % Kalman smoother

  rmse0=sqrt(sum( (x0(1:2,:)-xhat0(1:2,:)).^2 ));
  rmse1=sqrt(sum( (x0(1:2,:)-xhat1(1:2,:)).^2 ));
  rmse2=sqrt(sum( (x0(1:2,:)-xhat2(1:2,:)).^2 ));
  rmse3=sqrt(sum( (x0(1:2,:)-xhat3(1:2,:)).^2 ));
  rmse4=sqrt(sum( (x0(1:2,:)-xhat4(1:2,:)).^2 ));
%  cdfigure(3,2);
  figure
  plotfix
  plot([rmse1' rmse2' rmse3' rmse4'])
  legend('adkalman: stationary','adkalman: time-varying',...
         'adkalman: square-root','adkalman: smoothing');
axis([0 N 600 1600])
if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking3.eps, end
figure
plot(x0(1,:),x0(2,:),'-d')
hold on
plot(xhat1(1,:),xhat1(2,:),'g-')
plot(xhat2(1,:),xhat2(2,:),'g--')
plot(xhat3(1,:),xhat3(2,:),'r-.')
plot(xhat4(1,:),xhat4(2,:),'m:')
hold off
axis('equal')
  legend('true','adkalman: time-varying','adkalman: stationary',...
         'adkalman: square-root','adkalman: smoothing');
axis([1 2.5 3 4]*1e4)
if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking4.eps, end
  echo off

end
    
if demotrackingarg==2
  clc
  disp(' ')
  disp('adkalman: comparing different adaptation gains')
  disp(' ')
  echo on
  NNNmethod=1; % Change to forth order linear model for estimation

  adg=[1e-2 1e-1 1e0 1e1 1e2 1e3 1e4];
  leg=[];
  for i=1:length(adg)
    [xhat]=adkalman(y,nnn,adg(i),[2 1]); % Kalman predictor
    rmse(:,i)=sqrt(sum( (x0(1:2,:)-xhat(1:2,:)).^2 ))';
    leg=str2mat(leg,num2str(adg(i)));
  end
  leg(1,:)=[];
  rmse0=sqrt(sum( (y'-x0(1:2,:)).^2 ))';
  figure
  plotfix
  p1=plot([rmse]);
  hold on, p2=plot(rmse0,'r--'); hold off
  leg=str2mat(leg,'Ad-hoc filter');
  legend([p1;p2],leg,2)
  xlabel('Time [samples]')
  ylabel('Predictive RMSE(t)')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking5.eps, end
  figure
  plotfix
  semilogx(adg,sqrt(sum(rmse.^2)))
  hold on, semilogx(adg,sqrt(sum(rmse0.^2))*ones(size(adg)),'r--'); hold off
  xlabel('Adaptation gain')
  ylabel('Predictive RMSE')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking6.eps, end
  echo off
end
    
if demotrackingarg==3
  clc
  disp(' ')
  disp('adkalman: Compare different state vector coordinates')
  disp(' ')
  echo on
  leg=[];
  for i=1:5
    NNNmethod=i; % State coordinates
    [xhat]=adkalman(y,nnn,1,[2 1]); % Kalman predictor
    rmse(:,i)=sqrt(sum( (x0(1:2,:)-xhat(1:2,:)).^2 ))';
    leg=str2mat(leg,num2str(i));
  end
  leg(1,:)=[];
  figure
  plotfix
  plot([rmse])
  legend(leg,2)
  xlabel('Time [samples]')
  ylabel('Predictive RMSE(t)')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking7.eps, end
  echo off
end
 
if demotrackingarg==4
  clc
  disp(' ')
  disp('detect1: Apply whiteness change detection') 
  disp(' ')
  echo on

  NNNmethod=1;
  h=5;
  nu=0.5;
  [xhat,jhat,xseg,gt,ta]=detect1(y,nnn,1,[2 1e10 nu],1);
  [xhat1,jhat1,xseg1,gt,ta]=detect1(y,nnn,1,[2 h nu],1);
  jumps,jhat1=jhat1' 
  figure
  plotfix
    plot(x0(1,:),x0(2,:),'-',...
         xhat(1,:),xhat(2,:),'--',...
         xhat1(1,:),xhat1(2,:),'-.')
    hold on
    plot(xhat1(1,jhat1),xhat1(2,jhat1),'o')
    hold off
    legend('True trajectory','Kalman filter','KF with change detector')
    axis('equal')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking8.eps, end
  figure
  subplot(211)
  plotfix
  segplot([x0(1,:)'-xhat(1,:)' x0(1,:)'-xhat1(1,:)'],jhat1);
  ylabel('Error in x_1')
  xlabel(' '), title(' ')
  subplot(212)
  plotfix
  segplot([x0(2,:)'-xhat(2,:)' x0(2,:)'-xhat1(2,:)'],jhat1);
  ylabel('Error in x_2')
  xlabel('Time [samples]')
  title(' ')
  legend('Kalman filter','KF with change detector',4)

  echo off
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking9.eps, end
end

if demotrackingarg==5
  clc
  disp(' ')
  disp('detectM: Apply multi-model change detection') 
  disp(' ')
  echo on

  NNNmethod=1;
  [xhat,jhat,xseg,gt,ta]=detectM(y,nnn,[1 10],1e-15,1);
  [xhatM,jhatM,xsegM,gt,ta]=detectM(y,nnn,[1 10],0.5,[]);
  jumps,jhatM=jhatM'
  figure
  plotfix
    plot(x0(1,:),x0(2,:),'-',...
         xhat(1,:),xhat(2,:),'--',...
         xhatM(1,:),xhatM(2,:),'-.')
    hold on
    plot(xhatM(1,jhatM),xhatM(2,jhatM),'o')
    hold off
    legend('True trajectory','Kalman filter','Multiple model KF')
    axis('equal')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking10.eps, end

  figure
  subplot(211)
  plotfix
  segplot([x0(1,:)'-xhat(1,:)' x0(1,:)'-xhatM(1,:)'],jhatM);
  ylabel('Error in x_1')
  xlabel(' '), title(' ')
  axis([0 N -2000 2000])
  subplot(212)
  plotfix
  segplot([x0(2,:)'-xhat(2,:)' x0(2,:)'-xhatM(2,:)'],jhatM);
  ylabel('Error in x_2')
  xlabel('Time [samples]')
  title(' ')
  axis([0 N -2000 2000])
  legend('Kalman filter','Multiple model KF',3)
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking11.eps, end
  echo off
end

if demotrackingarg==6
  clc
  disp(' ')
  disp('detectM: Apply multi-model change detection with CT model using IMM') 
  disp(' ')
  echo on

  NNNmethod=5;     % Coordinated turn model
  e=randn(N,2);    % Measurement noise
  yn=y+500*e;      % Add extra noise
  [xhat,jhat,xseg,gt,ta]=detectM(yn,nnn,[1 1e9],1e-15,1);
  [xhatM,jhatM,xsegM,gt,ta]=detectM(yn,nnn,[1 1e9],0.5,[]);
  [xhatimm]=imm(yn,nnn);
  jumps,jhatM=jhatM'
  figure
  plotfix
    plot(x0(1,:),x0(2,:),'-',...
         xhat(1,:),xhat(2,:),'--',...
         xhatimm(1,:),xhatimm(2,:),':',...
         xhatM(1,:),xhatM(2,:),'-.',...
         yn(:,1),yn(:,2),'o')
    hold on
    plot(xhatM(1,jhatM),xhatM(2,jhatM),'o')
    hold off
    legend('True trajectory','Kalman filter','IMM','Multiple model KF','Measurements')
    axis('equal')
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking12.eps, end

  figure
  subplot(211)
  plotfix
  segplot([x0(1,:)'-xhat(1,:)' x0(1,:)'-xhatM(1,:)' x0(1,:)'-xhatimm(1,:)'],jhatM);
  ylabel('Error in x_1')
  xlabel(' '), title(' ')
  axis([0 N -2000 2000])
  subplot(212)
  plotfix
  segplot([x0(2,:)'-xhat(2,:)' x0(2,:)'-xhatM(2,:)' x0(2,:)'-xhatimm(2,:)'],jhatM);
  ylabel('Error in x_2')
  xlabel('Time [samples]')
  title(' ')
  axis([0 N -2000 2000])
  legend('Kalman filter','Multiple model KF','IMM',3)
  if saveplots==1, print -deps /home/rt/fredrik/books/manual/figures/tuttracking13.eps, end
  echo off
end
