function [P,Pmiss]=diagnosability(nnn,L,Rstruc,indarg,ind2D)

% Computes diagnosability probabilities
% diagnosability(nnn,L,Rstruc,indarg,ind2D)
%
% indarg subset of the following    
%    1 Structured residual
%    2 Observer residual
%    3 Optimal residual
% ind2D Residual indices in plot
% nnn  State space model
% L    Length of sliding window

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<1,  nnn='nnnDCm3';end
if nargin<2, L=3; end
if nargin<3, Rstruc=eye(2); end
if nargin<4, indarg=[1 2 3]; end
if nargin<5, ind2D=[1 2]; end

close all
[n,x0,P0]=feval(nnn,0,0,0,0);
[A,Bu,Bv,Bd,Bf,Q]=feval(nnn,0,x0,0,5);
[C,Du,Df,R]=feval(nnn,0,x0,0,6);
nx=n(1);
nu=n(2);
nv=n(3);
nd=n(4);
nf=n(5);
ny=n(6);


Ordo=C;
Ordotmp=C;
for i=1:L-1;
  Ordotmp=Ordotmp*A;
  Ordo=[Ordo;Ordotmp];
end

Hutmp1=Du;
Hutmp2=Bu;
Hu=[zeros((L-1)*ny,nu);Du];

Dd=zeros(ny,nd);
Hdtmp1=Dd;
Hdtmp2=Bd;
Hd=[zeros((L-1)*ny,nd);Dd];

Hftmp1=Df;
Hftmp2=Bf;
Hf=[zeros((L-1)*ny,nf);Df];

Dv=zeros(ny,nv);
Hvtmp1=Dv;
Hvtmp2=Bv;
Hv=[zeros((L-1)*ny,nv);Dv];
for i=1:L-1;
  Hutmp1=[Hutmp1;C*Hutmp2];
  Hutmp2=A*Hutmp2;
  Hu=[ [zeros((L-1-i)*ny,nu);Hutmp1] Hu];

  Hftmp1=[Hftmp1;C*Hftmp2];
  Hftmp2=A*Hftmp2;
  Hf=[ [zeros((L-1-i)*ny,nf);Hftmp1] Hf];
end

if nd>0; % only for disturbance
  for i=1:L-1;
    Hdtmp1=[Hdtmp1;C*Hdtmp2];
    Hdtmp2=A*Hdtmp2;
    Hd=[ [zeros((L-1-i)*ny,nd);Hdtmp1] Hd];
  end
else
  Hd=[];
end

if nv>0; % only for state noise
  for i=1:L-1;
    Hvtmp1=[Hvtmp1;C*Hvtmp2];
    Hvtmp2=A*Hvtmp2;
    Hv=[ [zeros((L-1-i)*ny,nv);Hvtmp1] Hv];
  end
else
  Hv=[];
end


% ----- Null space ---------
[Usvd,Dsvd,Vsvd]=svd([Ordo Hd]);
n=rank(Dsvd);
Null=Usvd(:,n+1:size(Usvd,2))';
%Null=null([Ordo Hd]')';
%size(Null)
nw=size(Null,1);

Fstruc=eye(nf);
T = Rstruc / ( Null*Hf*kron(ones(L,1),Fstruc));
if any(  T*( Null*Hf*kron(ones(L,1),Fstruc)) - Rstruc > 1e-8)
   disp('Warning: structured residuals could not be achieved')
end


  
disp(['Row rank of null space is ',num2str(nw)])
%    Hv, Null,Hf,L
    m=4; % fault size
    S=Hv*kron(eye(L),Q)*Hv'+kron(eye(L),R); % Residual covariance
    for i=indarg
      switch i
       case 1
	disp('Structured residual')
	w=T*Null;
       case 2
	disp('Observer residual')
	w=Null;
       case 3
	disp('Optimal residual')
	wkf=eye(size(S))-Ordo*inv(Ordo'*inv(S)*Ordo)*Ordo'*inv(S);
	%wkf=(S-Ordo*inv(Ordo'*inv(S)*Ordo)*Ordo')*inv(S)
        [Usvd,Dsvd,Vsvd]=svd(wkf);
        n=rank(Dsvd);
        w=Vsvd(:,1:n)';
      end
      %eval(['w',num2str(i),'=w;'])
      S1=w*S*w';
      Ssqrt=sqrtm(S1);
      clear mu mubar
      for j=1:nf;
	mu(:,j)=m*w*Hf*kron(ones(L,1),[zeros(j-1,1);1;zeros(nf-j,1)]);
        mubar(:,j)=inv(Ssqrt)*mu(:,j);
      end
      P=zeros(nf);
      for k=1:nf
	for l=1:nf
	  xort1=mubar(:,k)-mubar(:,k)'*((mubar(:,k)+mubar(:,l)))/((mubar(:,k)+mubar(:,l))'*(mubar(:,k)+mubar(:,l)))*(mubar(:,k)+mubar(:,l));
	  xort2=mubar(:,l)-mubar(:,l)'*((mubar(:,k)+mubar(:,l)))/((mubar(:,k)+mubar(:,l))'*(mubar(:,k)+mubar(:,l)))*(mubar(:,k)+mubar(:,l)); 
          P(k,l)=0.5*erfc(m*sqrt(xort1'*xort1));
	end
      end
      P=eye(2)-diag(sum(P))+P;
      for j=1:nf;
	Pmiss(j)=0.5*erfc(m*norm(mubar(:,j))/2);
      end
      
      Pmiss;
      
      if nargout==0 % plot illustrations
      figure(i)
      book('plotfix')
      ind=ind2D; % 2D residual indeces in plot
      p(1)=arrowplot([0;0],'-k',inv(S1(ind,ind)),0.5);
      p(nf+2)=arrowplot([0;0],'--k',eye(2),0.5);
      c1=['-b';'-r';'-g';'-m'];
      c2=['--b';'--r';'--g';'--m'];
      for j=1:nf;
	p(j+1)=arrowplot(mu(ind,j),c1(j,:),inv(S1(ind,ind)),0.5);
        p(nf+2+j)=arrowplot(mubar(ind,j),c2(j,:),eye(2),0.5);
      end
  %    p=plot([0 mu1(1)],[0 mu1(2)],'-b',[0 mu2(1)],[0 mu2(2)],'-r',...
  %    [0 mu1bar(1)],[0 mu1bar(2)],'--b',[0 mu2bar(1)],[0 mu2bar(2)],'--r');
  %    set(p,'LineWidth',4)
      legend(p,'r|No fault','r|f^1','r|f^2','S^{-1/2}r|No fault', ...
  	   'S^{-1/2}r|f^1','S^{-1/2}r|f^2','Decision line',-1)
      xlabel(['r_',num2str(ind(1))])
      ylabel(['r_',num2str(ind(2))])
      title(['L=',num2str(L)])
      %eval(['print -depsc
      %../muplot',num2str(i),'L',num2str(L),'.eps'])
      end
      
    end
    
function p=arrowplot(e,m,A,R)
% e  Vector or vectors as  columns in a matrix
% m  Marker style (default '-b')
% A  Quadratic form for ellipsoid
% R  Ellipsoid radius
if nargin<2; m='-b'; end
delta=0.1;  % Rel. arrow length
phi=pi/6;   % Arrow angle
% Positive rotation matrix
A1=[cos(phi) -sin(phi);...
    sin(phi) cos(phi)];     
% Negative rotation matrix
A2=[cos(-phi) -sin(-phi);...
    sin(-phi) cos(-phi)]; 
% Quadratic forms
theta=(0:0.01:2*pi);
x=[cos(theta);sin(theta)];
r=sqrt(diag(x'*A*x));
for i=1:size(e,2);
  p(i)=plot([0 e(1,i)],[0 e(2,i)],m);
  hold on
  v1=delta*A1*e(:,i);
  v2=delta*A2*e(:,i);
  plot([e(1,i) e(1,i)-v1(1)],...
       [e(2,i) e(2,i)-v1(2)],m)
  plot([e(1,i) e(1,i)-v2(1)],...
       [e(2,i) e(2,i)-v2(2)],m)
  plot(e(1,i)+x(1,:)./r'*R,e(2,i)+x(2,:)./r'*R,m)
end
% Arrow looks funny without this
axis('equal')  


    
    
