function dspbook(ex,saveplots)

% Examples and demonstrations from the swedish book in 'Signal
% Processing' published by Studentlitteratur, 2000.
% ex  Switch to choose example. It is of the form 'ex1.2'
% for Examples, and 'sec1.2' for illustrations that are not
% of the form of an example.
% Saveplots used by the author to create all figures

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<2; saveplots=0; end
fontsize=14;

if strcmp(ex,'plotfix')
  set(gca,'FontSize',fontsize)
  set(gcf,'DefaultpatchMarkerSize',10);
  set(gcf,'DefaultsurfaceMarkerSize',10);
  set(gcf,'DefaultlineMarkerSize',8);
  set(gcf,'DefaultlineLineWidth',1);
  set(gcf,'DefaultaxesLineWidth',1);
  set(gcf,'DefaultaxesGridLineStyle',':')
  set(gcf,'DefaulttextFontSize',fontsize)
  set(gcf,'DefaultaxesFontSize',fontsize)
end

if strcmp(ex,'all') 
  pause off
  echo off
  more off
  t000=clock;
end

if strcmp(ex,'all') | strcmp(ex,'intro')
  close all
  figure(1)
  set(gcf,'Position',[100 100 560 600])
  set(gcf,'PaperPosition',[0.25 2.5 8 9])
  subplot(321)
  dspbook('plotfix')
  load nmt450
%  h=plot(y/max(y));
  h=plot(y/1e6);
%  set(h,'LineWidth',2);
  h1=title('Frsljning av NMT450');

  subplot(322)
  dspbook('plotfix')
  load nose
  h=plot(yall(41,:));
%  set(h,'LineWidth',2);
  h1=title('Elektronisk nsa');

  subplot(323)
  dspbook('plotfix')
  load ekg
  h=plot(s);
%  set(h,'LineWidth',1);
  h1=title('EKG');

  subplot(324)
  dspbook('plotfix')
  load eeg_human
  h=plot(Y(:,1));
%  set(h,'LineWidth',1);
  h1=title('EEG');

  subplot(325)
  dspbook('plotfix')
  load photons
  h=plot(diff(y1(1:100:end))/1000);
%  set(h,'LineWidth',1);
  h1=title('Fotoner');

  subplot(326)
  dspbook('plotfix')
  load speech
  h=plot(z/1000);
%  set(h,'LineWidth',1);
  h1=title('Talsignal');

  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/intro.eps, end 

end

if strcmp(ex,'all') | strcmp(ex,'ex1.1')
  close all
  load fuel
  threc1=adfilter(y,-1,0.9);
  hold on
  figure(1)
  dspbook('plotfix')
  set(gca,'box','on')
  t=1:length(y);
  p1=plot(t,y,'b-',t,threc1','b-');
  set(p1(2),'LineWidth',2)
  h1=legend([p1],'Mtning','Filter');
  h2=xlabel('Tid [sampel]');
  h3=ylabel('Brnslefrbrukning [l/100 km]');
%  set([h1' h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex1_1.eps, end 
end


if strcmp(ex,'all') | strcmp(ex,'ex1.2')
  close all
  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
%  s(201:N)=s(201:N)+(TH0(2,2)-TH0(2,1))*mu(201:N);  % Friction change

  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);

  N=200;
  figure(1)
  dspbook('plotfix')
  h1=plot(s(1:N),mu(1:N),'x');
  hold on
  a=axis;
  mumax=a(4);
  h3=plot([threc(1,N) threc(1,N)+threc(2,N)*mumax],[0 mumax]);
  hold off
  hh1=ylabel('Normaliserat motormoment');
  hh2=xlabel('Hjulglidning');
  hh3=legend([h1 h3],'Mtningar','Friktionsmodell',2);
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex1_2.eps, end 
end

if strcmp(ex,'all') | strcmp(ex,'ex1.3')
  load tpi
  close all
  figure(1)
  subplot(211)
  dspbook('plotfix')
  Ts=1/500;
  y=w2(1:2000);
  N=length(y);
  t=(1:N)'*Ts;
  p1=plot(t,y*1.8*3.6*Ts);
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighet [km/h]');
  subplot(212)
  dspbook('plotfix')
%  y=detrend(y);
  Y=fft(y-mean(y));
  n=30;
  Yf=filtfilt(1/n*ones(1,n),1,abs(Y));
  N=length(y);
  w=(0:N-1)'/N/Ts;
  semilogy(w(1:round((N-1)/2)),abs(Yf(1:round((N-1)/2))).^2);
  axis([0 120 1e3 1e5])
  h3=xlabel('Frekvens [Hz]');
  h4=ylabel('Effekt');
%  axis([30 55 1e4 2e5])
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex1_3.eps, end 

end

if strcmp(ex,'all') | strcmp(ex,'ex1.4')
  close all
  rand('seed',0);
  N=50;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+1000*e;

  figure(1);
  dspbook('plotfix')
  %h1=radarplot(y,x0);
  NNNmethod=1;
  [xhat]=adfilter(y,nnn,1);
  h2=radarplot(y,xhat);
  hold on
%  set(h2,'LineWidth',2)
  legend([h2],'Mtt position','Filtrerad position')
  title('Filtrerad position')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex1_4.eps, end 
end



if strcmp(ex,'all') | strcmp(ex,'ex1.5') | strcmp(ex,'ex1tfd')
   close all
   load bach   
   %sound(y)
   figure(1)
   dspbook('plotfix')
   specgram(y,[],8192)
   colormap('gray')
   %set(gcf,'colormap',(1:-1/63:0)'*[1 1 1])
   a=axis;
   %hold on
   %s=1100*2.^((0:7)/8);
   %plot(a(1:2),[1;1]*s)
   %hold off
  h1=title('Tid-frekvensupplsning av Bach'); 
  h2=xlabel('Tid [s]');
  h3=ylabel('Frekvens [Hz]');
  set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/bach1.eps; 
  else
    pause
  end
  t=(1:length(y))/8192;
  
  figure(2)
  subplot(311)
  dspbook('plotfix')
  plot(t,y)     
  hold on
  int=12001:14000;
  plot(1/8192*[int(1),int(1),int(end),int(end),int(1)],4e-3*[-1 1 1 -1 -1],'--')
  hold off
  h1=title('Samplad Bach-signal');
  subplot(312)
  dspbook('plotfix')
  plot(t(int),y(int))
  set(gca,'Xlim',[t(int(1)) t(int(end))])
  hold on
  int=(12001:12100)-12000;
  plot(1/8192*(12000+[int(1),int(1),int(end),int(end),int(1)]),4e-3*[-1 1 1 -1 -1],'--')
  hold off

  subplot(313)
  dspbook('plotfix')
  plot(t(int+12000),y(int+12000))
  set(gca,'Xlim',[t(12000+int(1)) t(12000+int(end))])
  h2=xlabel('Tid [s]');
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/bach2.eps; 
  else
    pause
  end
  figure(3)
  dspbook('plotfix')
  int=12001:12500;
  w=(0:499)/500*8192;
  Y=fft(y(int));
  p1=semilogy(w(1:end/2),abs(Y(1:end/2)));
  h1=xlabel('Frekvens [Hertz]');
  h2=ylabel('Energi');
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/bach3.eps; 
  else
    pause
  end
end   
   
if strcmp(ex,'all') | strcmp(ex,'ex1.6') | strcmp(ex,'ex1chirp')
  % Chirp signal
  close all
  N=5000;
  y=sin(2*pi*(1:N)'.^2./N);

  figure(1)
  subplot(211)
  dspbook('plotfix')
  plot(y)
  hold on
  int=1:200;
  p1=plot([int(1),int(1),int(end),int(end),int(1)],1.5*[-1 1 1 -1 -1],'-');
%  set(p1,'LineWidth',2)
  hold off
  h1=title('Chirp-signal');

  subplot(212)
  dspbook('plotfix')
  plot(y(int))
  h2=xlabel('Tid [sampel]');

  if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ex1chirp1.eps
end

figure(2)
dspbook('plotfix')
specgram(y)
colormap('gray')
h1=ylabel('Frekvens [normaliserad]');
h2=xlabel('Tid [s]');
%set([h1 h2],'FontSize',12)
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ex1chirp2.eps
end
end


if strcmp(ex,'all') | strcmp(ex,'ex1hjul1')
%load carspeed
% y=y/3.6;
load tpi
Ts=1/500;
v=w2*1.8*3.6*Ts;
y=v/3.6;  %km/s -> m/s
y=decimate(y,100);
%xxx
t=.2:.2:length(y)*.2;
figure(1)
dspbook('plotfix')
subplot(211)
plot(t,y)
h1=xlabel('Tid [s]');
h2=ylabel('Hastighet [m/s]');
%set([h1 h2],'FontSize',fontsize)
subplot(212)
plot(t,5*diff([y(1);y(:)]))
axis([0 length(y)*0.2 -0.1 0.1])
h1=xlabel('Tid [s]');
h2=ylabel('Hastighetkning [m/s^2]');
%set([h1 h2],'FontSize',fontsize)
if saveplots==1; 
   print -deps /home/rt/fredrik/digsigfig/hjulv.eps
else
   pause
end
end

if strcmp(ex,'all') | strcmp(ex,'ex1hjul2')
%load carspeed
%y=v/3.6;

load tpi
Ts=1/500;
v=w2*1.8*3.6*Ts;
y=v/3.6;  %km/s -> m/s
y=decimate(y,100);

t=.2:.2:length(y)*.2;
figure(2)
dspbook('plotfix')
subplot(211)
m=mean(y);
Y=fft(y-m);
N=length(y)
w=(0:N-1)/N*2*pi/0.2;
semilogy(w(1:round(N/2)),abs(Y(1:round(N/2))))
h1=xlabel('Frekvens [rad/s]');
%set([h1],'FontSize',fontsize)

subplot(211)
h=0.1;
Ymax=max(abs(Y));
ind=find(abs(Y)<Ymax*h);
disp(['Datareduktionen blir ',num2str(length(ind)/N)])
Y(ind)=zeros(size(ind));
yh=real(ifft(Y))+m;

subplot(211)
p1=plot(t,[yh]);
hold on
p2=plot(t,[y],'--');
hold off
h1=xlabel('Tid [s]');
h2=ylabel('Hastighet [m/s]');
h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',3);
%set([h1 h2 h3],'FontSize',fontsize)

subplot(212)
plot(t,5*diff([yh(1);yh(:)]))
hold on
plot(t,5*diff([y(1);y(:)]),'--')
hold off
%plot(t,y-yh)
axis([0 length(y)*0.2 -0.1 0.1])
h1=xlabel('Tid [s]');
h2=ylabel('Hastighetskning [m/s^2]');
%set([h1 h2],'FontSize',fontsize)

if saveplots==1; 
   print -deps /home/rt/fredrik/digsigfig/hjultransf.eps
else
   pause
end
end

if strcmp(ex,'all') | strcmp(ex,'ex1hjul3')
%  load carspeed
%  y=v/3.6;
  load tpi
  Ts=1/500;
  v=w2*1.8*3.6*Ts;
  y=v/3.6;  %km/s -> m/s
  y=decimate(y,100);

  t=.2:.2:length(y)*.2;
  [b,a]=butter(5,0.1);
  yf=filtfilt(b,a,y);

  figure(3)
  subplot(211)
  dspbook('plotfix')
  p1=plot(t,[yf]);
  hold on
  p2=plot(t,[y],'--');
  hold off
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighet [m/s]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',3);
%  set([h1 h2 h3],'FontSize',fontsize)
  subplot(212)
  dspbook('plotfix')
  p1=plot(t,[5*diff([yf(1);yf(:)])]);
  hold on
  p2=plot(t,[5*diff([y(1);y(:)])],'--');
  hold off
  axis([0 length(y)*0.2 -0.1 0.1])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetkning [m/s^2]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',2);
%  set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/hjulfilter.eps
  else
    pause
  end
end

if strcmp(ex,'all') | strcmp(ex,'ex1hjul4')
%  load carspeed
%  y=v/3.6;
  load tpi
  Ts=1/500;
  v=w2(:)*1.8*3.6*Ts;
  y=v/3.6;  %km/s -> m/s
  y=decimate(y,100);

  t=.2:.2:length(y)*.2;
  N=length(y);

  tt=(0:N-1)/N;
  Phi=[ones(size(tt')) tt' tt'.^2 tt'.^3 tt'.^4 tt'.^5 tt'.^6 tt'.^7 tt'.^8 ...
       tt'.^9 tt'.^10 ];
  %Phi=[[0;-y(1:N-1)] [0;0;-y(1:N-2)] ];
  thhat=Phi\y;
  yhat=Phi*thhat;
  
  figure(3)
  dspbook('plotfix')
  subplot(211)
  p1=plot(t,[yhat]);
  hold on
  p2=plot(t,[y],'--');
  hold off
  %axis([0 N*0.2 -1 1])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighet [m/s]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',3);
  %set([h1 h2 h3],'FontSize',fontsize)
  subplot(212)
  p1=plot(t,[5*diff([yhat(1);yhat(:)])]);
  hold on
  p2=plot(t,[5*diff([y(1);y(:)])],'--');
  hold off
  axis([0 length(y)*0.2 -0.1 0.1])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetkning [m/s^2]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',2);
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/hjulmodell.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'soderasen1')
  load current
  close all
  figure(1),   
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.4])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/stromex.eps
  end
  
  Y=fft(y);
  N=length(y);
  Yapprox=zeros(size(Y));
  ind=N/fs*[50];
  Yapprox([1+ind N-ind])=Y([1+ind N-ind]);
  yapprox1=real(ifft(Yapprox));
  ind=N/fs*[100];
  Yapprox=zeros(size(Y));
  Yapprox([1+ind N-ind])=Y([1+ind N-ind]);
  yapprox2=real(ifft(Yapprox));
  ind=N/fs*[150];
  Yapprox=zeros(size(Y));
  Yapprox([1+ind N-ind])=Y([1+ind N-ind]);
  yapprox3=real(ifft(Yapprox));
  ind=N/fs*[200];
  Yapprox=zeros(size(Y));
  Yapprox([1+ind N-ind])=Y([1+ind N-ind]);
  yapprox4=real(ifft(Yapprox));
  ind=N/fs*[50 100 150 200];
  Yapprox=zeros(size(Y));
  Yapprox([1+ind N-ind])=Y([1+ind N-ind]);
  yapprox=real(ifft(Yapprox));
  size(yapprox),size(y)
  
  figure(2),   
  subplot(211)
  dspbook('plotfix')
  plot(1/fs*(1:N),yapprox1,'b-',...
       1/fs*(1:N),yapprox2,'b--',...
       1/fs*(1:N),yapprox3,'b-.',...
       1/fs*(1:N),yapprox4,'b:')
  set(gca,'Xlim',[0.3 0.4])
  legend('50 Hz','100 Hz','150 Hz','200 Hz')
  subplot(212)
  dspbook('plotfix')
  plot(1/fs*(1:N),2*yapprox,'b--',1/fs*(1:N),y,'b-')
  set(gca,'Xlim',[0.3 0.4])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/stromfour.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'ess')
  load ess
  close all
  figure(1),   
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.7])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/s.eps
  end
  figure(2),   
  subplot(211)
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.4])
  subplot(212)
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.5 0.6])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/soche.eps
  end
end
  
if strcmp(ex,'all') | strcmp(ex,'sigcomp')
  load ess
  close all
  figure(1),   
  subplot(211)
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.5 0.6])

  load current
  subplot(212)
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.4])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/sigcomp.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'fig2.7')
  close all
  N=100;
  t=(1:N)/20;
  y=1.2+sin(5+t/2);
  tk=20:20:N;
  yk=y(tk);
  plot(t,y,'--')
  hold on
  stem(yk)
  hold off
  set(gca,'Xtick',[1 2 3 4 5])
  set(gca,'Xticklabel',[' T';'2T';'3T';'4T';'5T'])
%  get(gca)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/SampFig.eps
  end
end
  
  
if strcmp(ex,'all') | strcmp(ex,'2.2')
  close all
  
  P=100;
  n=20;
  y=kron((-1).^(1:n),ones(1,P));
  Y=fft(y);
  ind=find(abs(Y)>0.001);
  Yt=zeros(size(Y));
  Yt([ind(1:4) ind(end-3:end)])=Y([ind(1:4) ind(end-3:end)]);
  yt=real(ifft(Yt));
  
  figure(1)
  subplot(211)
  dspbook('plotfix')
  ind=5*P+1:11*P;
  plot((1:length(ind))/P/2,y(ind))
  hold on
  h=plot((1:length(ind))/P/2,yt(ind));
  hold off
  set(h,'LineWidth',2)
  set(gca,'Ylim',[-1.2 1.2])
  subplot(212)
  dspbook('plotfix')
  ind=5*P+1:7*P;
  plot((1:length(ind))/P/2,y(ind))
  hold on
  h=plot((1:length(ind))/P/2,yt(ind));
  hold off
  set(h,'LineWidth',2)
  set(gca,'Ylim',[-1.2 1.2])
  
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/gibbs.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'3.5')
  randn('seed',0)
  N=1000;
  e1=randn(N,1);
  e2=randn(N,1);
  u=e1;
  y=filter([0 0 1],[1 -1.5 0.7],e2);
  rhat=covf([y u],25);
  R=[1 -1.5 0.7;-1.5 1+0.7 0; 0.7 -1.5 1]\[1;0;0]
  for k=4:25; R(k)=1.5*R(k-1)-0.7*R(k-2); end

  figure(1),   
  subplot(311)
  dspbook('plotfix')
  plot(0:24,rhat(1,:),'-',0:24,R,'b--')
  ylabel('R_y(\tau)')

  subplot(312)
  dspbook('plotfix')
  plot(0:24,rhat(4,:),'-',[0 24],[0 0],'b--')
  ylabel('R_u(\tau)')

  subplot(313)
  dspbook('plotfix')
  plot(0:24,rhat(3,:),'-',[0 24],[0 0],'b--')
  xlabel('\tau')
  ylabel('R_{yu}(\tau)')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/covex.eps
  end
  
end

if strcmp(ex,'all') | strcmp(ex,'3.6')
  close all
  load equake
  
  figure(1),   
  dspbook('plotfix')
  N=length(y);
  plot(1/fs*(1:1000),y(101:1100))
  y1=y(101:600);  
  y2=y(601:1100);  
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/seis.eps
  end
  
  figure(2),   
  subplot(211)
  dspbook('plotfix')
  Y=fft(y1);
  N
  loglog(fs*(0:250)/500,abs(Y(1:251)).^2/N)
  axis([1e-1 10 1e-3 1e3])
%  set(gca,'Ylim',[1e-3 1e3])
  subplot(212)
  dspbook('plotfix')
  Y=fft(y2);
  loglog(fs*(0:250)/500,abs(Y(1:251)).^2/N)
  axis([1e-1 10 1e-3 1e3])
  xlabel('Frekvens [Hz]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/seisper.eps
  end
  
  figure(3),   
  dspbook('plotfix')
  g10=spa(y2',10);
  g30=spa(y2',30);
  g100=spa(y2',100);
  loglog(g10(2:end,1)/pi*10,g10(2:end,2),'-',...
	 g30(2:end,1)/pi*10,g30(2:end,2),'--',...
	 g100(2:end,1)/pi*10,g100(2:end,2),'-.')
  legend('\gamma=10','\gamma=30','\gamma=100')
  axis([1e-1 1e1 1e-2 1e3])
  xlabel('Frekvens [Hz]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/seisspec.eps
  end
  
end


if strcmp(ex,'all') | strcmp(ex,'3.7')
  randn('seed',0);
  y=randn(1024,1);
  Y1=abs(fft(y(1:32)).^2)/32;
  Y2=abs(fft(y(1:64)).^2)/64;
  Y3=abs(fft(y(1:128)).^2)/128;
  Y4=abs(fft(y(1:1024)).^2)/1024;
  
  figure(1),   
  subplot(411)
  dspbook('plotfix')
  loglog((1:length(Y1)/2)/length(Y1),Y1(1:length(Y1)/2))
  ylabel('a')  
  axis([1e-2 1 1e-2  1e3])
  set(gca,'YtickLabel',[])
  set(gca,'XtickLabel',[])
  subplot(412)
  dspbook('plotfix')
  loglog((1:length(Y2)/2)/length(Y2),Y2(1:length(Y2)/2))
  ylabel('b')  
  axis([1e-2 1 1e-2 1e3])
  set(gca,'YtickLabel',[])
  set(gca,'XtickLabel',[])
  subplot(413)
  dspbook('plotfix')
  loglog((1:length(Y3)/2)/length(Y3),Y3(1:length(Y3)/2))
  ylabel('c')  
  axis([1e-2 1 1e-2 1e3])
  set(gca,'YtickLabel',[])
  set(gca,'XtickLabel',[])
  subplot(414)
  dspbook('plotfix')
  loglog((1:length(Y4)/2)/length(Y4),Y4(1:length(Y4)/2))
  ylabel('d')  
  xlabel('Frekvens [Hz]')  
  set(gca,'YtickLabel',[])
  axis([1e-2 1 1e-2 1e3])

  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/etfeex.eps
  end
  
end


if strcmp(ex,'all') | strcmp(ex,'ex2.1')| strcmp(ex,'ex3fonster')
  close all
  figure(1),   
  dspbook('plotfix')
%  set(gcf,'LineWidth',2)
  N=64;
  w1=boxcar(N);
  w2=bartlett(N);
  w3=hanning(N);
  w4=hamming(N);
  w5=blackman(N);
%  w3([1 64])
%  w4([1 64])
%  w5([1 64])
  subplot(211)
  dspbook('plotfix')
  p(1)=plot([w1]);
  hold on
  p(2)=plot([w2],'--');
  hold off
%  set(p,'LineWidth',2)
  l=legend('Rektangulrt fnster','Triangulrt fnster',4);
  pos=get(l,'Position');
  set(l,'Position',[0.4 pos(2:4)])
  axis([0 64 -0.2 1.2])
  subplot(212)
  dspbook('plotfix')
  p(1)=plot([w3]);
  hold on
  p(2)=plot([w4],'--');
  p(3)=plot([w5],'-.');
  hold off
%  set(p,'LineWidth',2)
  l=legend('Hanning','Hamming','Blackman',4);
  pos=get(l,'Position');
  set(l,'Position',[0.4 pos(2:4)])
  axis([0 64 -0.2 1.2])
  xlabel('Tid')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex2fonster1.eps, end 

  figure(2),
  dspbook('plotfix')
  
  W1=abs(fft([w1/sum(w1); zeros(1024-N,1)]))+1e-5;
  W2=abs(fft([w2/sum(w2); zeros(1024-N,1)]))+1e-5;
  W3=abs(fft([w3/sum(w3); zeros(1024-N,1)]))+1e-5;
  W4=abs(fft([w4/sum(w4); zeros(1024-N,1)]))+1e-5;
  W5=abs(fft([w5/sum(w5); zeros(1024-N,1)]))+1e-5;

  subplot(211)
  dspbook('plotfix')
  p(1)=semilogy([W1]);
  hold on
  p(2)=semilogy([W2],'--');
  hold off
%  set(p,'LineWidth',2)
  l=legend('Rektangulrt fnster','Triangulrt fnster',1);
  set(gca,'Xlim',[0 128]);
%  pos=get(l,'Position');
%  set(l,'Position',[0.4 0.4 pos(3:4)])
%  axis([0 128 0 1])

  subplot(212)
  dspbook('plotfix')
  p(1)=semilogy([W3]);
  hold on
  p(2)=semilogy([W4],'--');
  p(3)=semilogy([W5],'-.');
  hold off
%  set(p,'LineWidth',2)
  l=legend('Hanning','Hamming','Blackman',1);
  set(gca,'Xlim',[0 128]);
%  pos=get(l,'Position');
%  set(l,'Position',[0.4 0.4 pos(3:4)])
%  axis([0 128 0 1])
  xlabel('Frekvens')
  
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex2fonster2.eps, end 
end


if strcmp(ex,'all') | strcmp(ex,'ex2.2')| strcmp(ex,'ex3rekonstruktion')
  % Rekonstruktion av kontinuerlig signal
  % Reconstruction of sampled signal
  % Test signal: sinusoid
  close all
  f0=128;       % Signal frequency [Hz]
  Fs1=8192;      % Sampling frequency [Hz]
  Ts1=1/Fs1;     % Sampling interval [s]
  t1=0:Ts1:1-Ts1;  % 1s time vector
  y1=sin(2*pi*f0*t1);
  
  decfactor=8;  % Decimation factor
  Fs2=Fs1/decfactor; % New sampling frequency
  Ts2=1/Fs2;    % New sampling interval
  t2=t1(1:8:end);
  y2=y1(1:8:end);

  y3=kron(y2,ones(1,decfactor));
  Fs3=decfactor*Fs2;

  dspbook('plotfix')
  p=plot(t1,y1,'-',t1,y3,'--');
  hold on
  p1=stem(t2,y2);
  hold off
  axis([0 0.01 -1 1])
%  set([p p1],'Linewidth',2)
  legend([p p1],'y_1','y_3','y_2')

  [b,a]=butter(4,1/16);
  y3rec=filter(b,a,y3);
  %set(gca,'FontSize',16)
  p=plot(t1,y1,'-',t1,y3rec,'--',t1,y3,'-.');
  hold on
  p1=stem(t2,y2);
  hold off
  axis([0 0.01 -1 1])
%  set([p' p1'],'Linewidth',2)
  %set(gca,'FontSize',fontsize-2)
  legend('Kontinuerlig signal','Rekonstruktion','Hllkrets','Samplad signal',3)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex2rekonstruktion.eps, end 
end

if strcmp(ex,'all') | strcmp(ex,'ex3cepstrum')
  load speechecho
  y=taleko;
  figure(1)
  subplot(211)
  dspbook('plotfix')
  Cy=abs(ifft(log(abs(fft(y)).^2)));
  semilogy(0:449,Cy(1:450))
  hold on
  p1=semilogy(100:100:400,Cy(101:100:401),'o');
  hold off
  h1=xlabel('Tid');
  h2=ylabel('C_y[k]');
  axis([0 450 1e-8 1])

  subplot(212)
  dspbook('plotfix')
  Ry=covf(y,450);
  plot(0:449,Ry)
  hold on
  p2=plot(100,Ry(101),'o');
  hold off
  h3=xlabel('Tid');
  h4=ylabel('R_y[k]');
%  set([p1 p2],'LineWidth',2);
  %set([h1 h2 h3 h4],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex3cepstrum.eps, end 
  
end  
  
  
if strcmp(ex,'all') | strcmp(ex,'ex3tfd')
  close all
  figure(1)
  dspbook('plotfix')
  
  for x=[0 1 2 3 4 5 6 7 8];
    plot([x x],[0 1]);
    hold on
  end;
  for f=[0 1 2 3 4 5 6 7 8]/8;
    plot([0 8],[f f]);
  end;
  hold off
  h1=title('Tid-frekvensupplsning i DFT'); 
  h2=xlabel('Tid');
  h3=ylabel('Normaliserad frekvens');
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/tfd1.eps; 
  else
    pause
  end
  for k=0:7;
    for x=0:2^(4-k):8;
      plot([x x],[2^(-7+k) 1]);
      hold on
    end;
  end;
  for f=[0 .25 .5 1 2  4 8]/8;
    plot([0 8],[f f]);
  end;
  hold off
  h1=title('Tid-frekvensupplsning i wavelettransformen'); 
  h2=xlabel('Tid');
  h3=ylabel('Normaliserad frekvens');
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/tfd2.eps; 
  else
    pause
  end
end



if strcmp(ex,'all') | strcmp(ex,'ex3wavelet1')
   N=100;
   t=(1:6*N)';
   w=2*pi/N/6*(1:6*N);
   phi=hanning(N).*sin(0.2*(1:N)');
   phi0=[zeros(N,1);phi;zeros(4*N,1)];

   phi=hanning(2*N).*sin(0.2/2*(1:2*N)');
   phi1=sqrt(0.5)*[zeros(N,1);phi;zeros(3*N,1)];
   phi=hanning(4*N).*sin(0.2/4*(1:4*N))';
   phi2=0.5*[zeros(N,1);phi;zeros(N,1)];
   
   Phi0=abs(fft(phi0));
   Phi1=abs(fft(phi1));
   Phi2=abs(fft(phi2));
   figure(1)
   subplot(311)
   dspbook('plotfix')
   plot(t,phi0)
   h1=title('Wavelet i tidsplanet');
   subplot(312)
   dspbook('plotfix')
   plot(phi1)
   subplot(313)
   dspbook('plotfix')
   plot(phi2)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/krusning1.eps; 
  else
    pause
  end

   figure(2)
   subplot(311)
   dspbook('plotfix')
   plot(w(1:end/2),Phi0(1:end/2))
   axis([0 0.5 0 30])
   h2=title('Wavelet i frekvensplanet');
   subplot(312)
   dspbook('plotfix')
   plot(w(1:end/2),Phi1(1:end/2))
   axis([0 0.5 0 40])
   subplot(313)
   dspbook('plotfix')
   plot(w(1:end/2),Phi2(1:end/2))
   axis([0 0.5 0 60])
   %set([h1 h2],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/krusning2.eps; 
  else
    pause
  end
   
   n=3;
   N=n*100;
   t=(1:N)'/n;
   phi=hanning(N).*sin(t);
   phi=[zeros(N,1);phi;zeros(N,1)];
   figure(3)
   dspbook('plotfix')
   specgram(phi,[128],n*2*pi)
   set(gcf,'colormap',(1:-1/63:0.4)'*[1 1 1])
   h3=title('Wavelet i tid-frekvensplanet');
   h4=ylabel('Frekvens [rad/s]');
   h5=xlabel('Tid [s]');
   %set([h3 h4 h5],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/krusningtfd.eps; 
  else
    pause
  end
end

if strcmp(ex,'all') | strcmp(ex,'ex3wavelethaar')
   N=100;
   t=(1:6*N)';
   w=2*pi/N/6*(1:6*N);
   phi=[ones(N/2,1);-ones(N/2,1)];
   phi0=sqrt(0.5)*[zeros(N,1);phi;zeros(4*N,1)];

   phi=[ones(N,1);-ones(N,1)];
   phi1=0.5*[zeros(N,1);phi;zeros(3*N,1)];
   phi=[ones(N*2,1);-ones(N*2,1)];
   phi2=sqrt(1/8)*[zeros(N,1);phi;zeros(N,1)];
   
   Phi0=abs(fft(phi0));
   Phi1=abs(fft(phi1));
   Phi2=abs(fft(phi2));
   figure(1)
   subplot(311)
   dspbook('plotfix')
   plot(t,phi0)
   axis([0 6*N -2 2])
   h1=title('Haar-transformen  i tidsplanet');
   subplot(312)
   dspbook('plotfix')
   plot(phi1)
   axis([0 6*N -1 1])
   subplot(313)
   dspbook('plotfix')
   plot(phi2)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/haar1.eps; 
  else
    pause
  end


   figure(2)
   subplot(311)
   dspbook('plotfix')
   plot(w(1:end/2),Phi0(1:end/2))
   axis([0 0.5 0 80])
   h2=title('Haar-transformen  i frekvensplanet');
   subplot(312)
   dspbook('plotfix')
   plot(w(1:end/2),Phi1(1:end/2))
   axis([0 0.5 0 80])
   subplot(313)
   dspbook('plotfix')
   plot(w(1:end/2),Phi2(1:end/2))
   axis([0 0.5 0 100])
   %set([h1 h2],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/haar2.eps; 
  else
    pause
  end
end

if strcmp(ex,'all') | strcmp(ex,'ex3wavelet2')
  addpath('/home/rt/fredrik/matlab/wavelets');
  figure(3)
  dspbook('plotfix')
  nmax=10;
  for n=1:nmax;
    psi=daubechies(n);
    plot(n+[zeros(5,1);psi(:);zeros(20-length(psi),1)])
    hold on
    plot(n+[zeros(5,1);psi(:);zeros(20-length(psi),1)],'o')
  end
  hold off
  axis([0 20 0 nmax+1])
  hold off
  h1=title('Daubechies wavelets av ordning 1 till 10');
  %set([h1],'FontSize',fontsize)
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/daubechies.eps; 
  else
    pause
  end
end
  

if strcmp(ex,'all') | strcmp(ex,'ex3wavelet3')
  close all
  
  load carspeed
  y=v/3.6;
  t=.2:.2:length(y)*.2;
  
  t=.2:.2:length(y)*.2;
  N=length(y);
  
 % yf=bodinfilter(y-mean(y),0.05,10)+mean(y);
  [yf,comprate,h]=bodinfilter(y,0.05,10);
  comprate,h
  figure(1)
  subplot(211)
  dspbook('plotfix')
  p1=plot(t,[yf]);
  hold on
  p2=plot(t,[y],'--');
  hold off
  %axis([0 N*0.2 -1 1])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighet [m/s]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',3);

  subplot(212)
  dspbook('plotfix')
  p1=plot(t,[5*diff([0;yf(:)])]);
  hold on
  p2=plot(t,1+[5*diff([0;y(:)])],'--');
  hold off
  axis([0 length(y)*0.2 -3 5])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetkning [m/s^2]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal (offset 1)',2);
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/donoho1.eps; 
  else
    pause
  end

  figure(2)
  subplot(211)
  dspbook('plotfix')
  plot(t,[y-yf])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetsskillnad [m/s]');

  subplot(212)
  dspbook('plotfix')
  sep=0.0025;
  x=-0.05:sep:0.2;
  hist((y-yf),x)       
  h2=xlabel('Frdelning fr hastighetsskillnad [m/s]');
  ylabel('Frekvens')
  %set([h1 h2],'FontSize',fontsize)
  if saveplots==1; 
     print -deps /home/rt/fredrik/digsigfig/donoho2.eps; 
  end
end


if strcmp(ex,'all') | strcmp(ex,'ex3wavelet4')
  
  load carspeed
  y=v/3.6;
  t=.2:.2:length(y)*.2;
  N=length(y)
  
  psi=daubechies(10);
  Y=fwt(psi,y);
  [dum,ind]=sort(abs(Y));
  
  figure(1)
  clf
  dspbook('plotfix')
  subplot(611)
  p0=plot(y,'-');
  text(20,24,['Verklig signal'])
%  set(p0,'LineWidth',2);
  axis([0 N -5 25])
  axis('off')
  hold on
  nn=[10 20 40 70 100];
  for n=1:5;
    Y1=Y;
    Y1(ind(1:end-nn(n)))=zeros(N-nn(n),1);
    y1=ifwt(psi,Y1);
    subplot(6,1,n+1)
    p(n)=plot(y1);
    text(20,26,['Rekonstruktion med ',num2str(nn(n)),' termer'])
    set(p(n),'LineWidth',1.5);
    axis([0 N -5 25])
    axis('off')
  end
  hold off
%  legend()
  

  if saveplots==1; 
     print -deps /home/rt/fredrik/digsigfig/donoho3.eps; 
  end
  
end
  
  
  
if strcmp(ex,'all') | strcmp(ex,'ex3wavelet3x')
  
  %load carspeed
  %y=v/3.6;
  %t=.2:.2:length(y)*.2;
  
  load tpi
  Ts=1/500;
  v=w2(:)*1.8*3.6*Ts;
  y=v/3.6;  %km/s -> m/s
  y=decimate(y,100);

  t=.2:.2:length(y)*.2;
  N=length(y);
  
  yf=bodinfilter(y-mean(y),0.01,10)+mean(y);

  figure(1)
  subplot(211)
  dspbook('plotfix')
  p1=plot(t,[yf]);
  hold on
  p2=plot(t,[y],'--');
  hold off
  %axis([0 N*0.2 -1 1])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighet [m/s]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal',3);

  subplot(212)
  dspbook('plotfix')
  p1=plot(t,[5*diff([0;yf(:)])]);
  hold on
  p2=plot(t,[5*diff([0;y(:)])],'--');
  hold off
  axis([0 length(y)*0.2 -0.3 0.3])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetkning [m/s^2]');
  h3=legend([p1 p2],'Filtrerad signal','Ursprunglig signal (offset 1)',3);
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/carw.eps; 
  else
    pause
  end

  figure(2)
  subplot(211)
  dspbook('plotfix')
  plot(t,[y-yf])
  h1=xlabel('Tid [s]');
  h2=ylabel('Hastighetsskillnad [m/s]');

  subplot(212)
  dspbook('plotfix')
  sep=0.001;
  x=-0.05:sep:0.05;
  hist((y-yf),x)       
  h2=xlabel('Frdelning fr hastighetsskillnad [m/s]');
  %set([h1 h2],'FontSize',fontsize)
  if saveplots==1; 
     print -deps /home/rt/fredrik/digsigfig/carwhist.eps; 
  end
end

if strcmp(ex,'all') | strcmp(ex,'circconv')
  % Circular convolution
  close all
  k=-31:32;
  x=sin(.5*k)./(k+eps);
  x(32)=0.5;
  figure(1)
  dspbook('plotfix')
  subplot(211)
  plot(x), hold on, stem(x), hold off
  h=title('x');

  y=1:64;
  y=y/64;
  subplot(212)
  plot(y), hold on, stem(y), hold off
  h(2)=title('y');
  %set(h,'FontSize',fontsize)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/circconv1.eps
  end

  z=conv(x,y);
  figure(2)
  dspbook('plotfix')
  subplot(211)
  plot(z), hold on, stem(z), hold off
  h=title('z=x*y');

  zc=real(ifft(fft(y).*fft(x)));
  subplot(212)
  plot(zc), hold on, stem(zc), hold off
  h(2)=title('zc=ifft(fft(y).*fft(x))');
  %set(h,'FontSize',fontsize)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/circconv2.eps
  end

  tic
  z=conv(x,y);
  toc

  xn=[x zeros(size(x))];
  yn=[y zeros(size(y))];
  tic
  znc=real(ifft(fft(yn).*fft(xn)));
  toc
end

if strcmp(ex,'all') | strcmp(ex,'specest')
  % Spectrum estimation
  close all
  N=10000;
  t=(1:N)';
  e=randn(N,1);
  y=filter([0 0.2],[1 -.9],e)+cos(.7*t)+cos(.8*t);

  [H,w]=freqz(.1,[1 -.9]);
  ind=find(w>.7);
  H(ind(1))=H(ind(1))+10;
  ind=find(w>.8);
  H(ind(1))=H(ind(1))+10;

  figure
  dspbook('plotfix')
  loglog(w,abs(H))
  title('Teoretiskt spektrum')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest1.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3Phi.eps

  figure
  dspbook('plotfix')
  plot(y(1:200))
  title('y=filter([0 .1],[1 -.9],e)+cos(.7*t)+cos(.8*t);')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest2.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3y.eps

  figure
  dspbook('plotfix')
  bodeplot([etfe(y) etfe(y(1:1000))])
  title('Periodogram fr N=1000 och N=10000')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest2.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3etfe.eps
  figure
  dspbook('plotfix')
  bodeplot(etfe(y,30))
  title('Spektrum med gamma=30')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest3.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3etfe30.eps
  figure
  dspbook('plotfix')
  bodeplot(etfe(y,20))
  title('Spektrum med gamma=20')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest4.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3etfe20.eps
  figure
  dspbook('plotfix')
  bodeplot(etfe(y,50))
  title('Spektrum med gamma=50')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/specest5.eps
  end
  %print -deps /home/rt/fredrik/edu/digsig/figures/le3etfe50.eps
end

if strcmp(ex,'all') | strcmp(ex,'chirp')
% Chirp signal
close all
N=5000;
y=sin(2*pi*(1:N)'.^2./N);

figure(1)
subplot(211)
dspbook('plotfix')
h1=plot(4*pi*(1:N)'./N,'--');
hold on
h2=plot(1:1*N/4,4*pi*(1:N/4)'./N,'-');
h3=plot(1*N/4:2*N/4,2*pi-4*pi*(N/4:2*N/4)'./N,'-');
h4=plot(2*N/4:3*N/4,4*pi*(0:N/4)'./N,'-');
h5=plot(3*N/4:4*N/4,2*pi-4*pi*(N/4:2*N/4)'./N,'-');
hold off
%set([h1 h2 h3 h4 h5],'LineWidth',2)
legend([h1 h2],'Fourier Transform','Discrete Fourier Transform',2)
subplot(212)
dspbook('plotfix')
plot(y)
xlabel('Tid [sampel]')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/chirp0.eps
end

figure(2)
dspbook('plotfix')
specgram(y)
h1=ylabel('Frekvens [Hz]');
h2=xlabel('Tid [sampel]');
%set([h1 h2],'FontSize',12)
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/chirp1.eps
end

figure(3)
dspbook('plotfix')
N=500;
y=sin(2*pi*(1:N)'.^2./N);
[th,lamhat]=adfilter(y,2,20); 
th2tfd(th,lamhat,y,64,[],[1 0]); 
h1=ylabel('Frekvens [rad/s]');
h2=xlabel('Tid [5 sampel]');
%set([h1 h2],'FontSize',12)
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/chirp2.eps
end
end



if strcmp(ex,'all') | strcmp(ex,'skalning')
%[b,a]=cheby1(3,0.5,0.3);
%[b,a]=butter(3,0.3);
%h=dimpulse(b,a,128);
  close all
h=[hamming(16);zeros(112,1)];
H=abs(fft(h));
figure(1)
dspbook('plotfix')
subplot(211)
plot(0:127,H)
subplot(212)
plot([-63:64]/128 ,[H(66:128);H(1:65)])
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/skalning1.eps
end

H=H(1:65);
figure(2)
dspbook('plotfix')
subplot(221)
plot(H)
subplot(222)
semilogy(H)
a=axis
axis([a(1:2) 1e-4 1e1]);
subplot(223)
semilogx(H)
subplot(224)
loglog(H)
a=axis;
axis([a(1:2) 1e-4 1e1])
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/skalning2.eps
end

end

if strcmp(ex,'all') | strcmp(ex,'ickekausal')
%if strcmp(ex,'ickekausal')
close all
a=conv([1 -0.7],[ -0.7 1]);
[r,p,k]=residue([0 1 0],a);
N=100;
e=randn(N,1);   
y1=filtfilt([0 1],[1 -0.7],e);
y2a=filter([0 r(2)],[1 -p(2)],e);
y2br=filter([-r(1)/p(1) 0],[1 -1/p(1)],e(N:-1:1));
y2b=y2br(N:-1:1);
y2=y2a+y2b;
figure
dspbook('plotfix')
plot(y1)
hold on
plot(y2,'--')
hold off
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ickekausal.eps
end

end

if strcmp(ex,'all') | strcmp(ex,'adaptar')
%if  strcmp(ex,'adaptar')
  close all
close all
TH=[-1.5 0.7;1.5 0.7]';
jumps=150;
N=300;
nnn=2;
randn('seed',0);
e=randn(N,1);
[y,th0]=simchange(e,nnn,jumps,TH);
figure(1);
dspbook('plotfix')
plot(y)
title('Data')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/data10.eps
end

figure(2);
dspbook('plotfix')
%thplot(thhat,th0);
p=plot([th0(1,:)']);
set(p,'Linewidth',2)
hold on
ltype=['- ';'--';'-.'];
% RLS
lam=[1 0.95 0.9];
for k=1:length(lam)
  [thhat,lamhat]=adfilter(y,nnn,lam(k),'RLS');
  plot([thhat(1,:)'],ltype(k,:))
end
axis([0 300 -2 2])
legend('Sann parameter','\lambda=1','\lambda=0.95','\lambda=0.9',2)
hold off
title('Skattad a_1-parameter: RLS')
% LMS
figure(3);
dspbook('plotfix')
p=plot([th0(1,:)']);
set(p,'Linewidth',2)
hold on
mu=[0.05 0.1 0.5];
for k=1:length(mu)
  [thhat,lamhat]=adfilter(y,nnn,mu(k),'NLMS');
  plot([thhat(1,:)'],ltype(k,:))
end
axis([0 300 -2 2])
legend('Sann parameter','\mu=0.05','\mu=0.1','\lambda=0.5',2)
hold off
title('Skattad a_1-parameter: LMS')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/rek10a.eps
end
% KF
figure(4);
dspbook('plotfix')
p=plot([th0(1,:)']);
set(p,'Linewidth',2)
hold on
q1=[0.001 0.01 0.1];
for k=1:length(mu)
  [thhat,lamhat]=adfilter(y,nnn,[q1(k) 0;0 0],'KF');
  plot([thhat(1,:)'],ltype(k,:))
end
axis([0 300 -2 2])
legend('Sann parameter','q_1=0.001','q_1=0.01','q_1=0.1',2)
hold off
title('Skattad a_1-parameter: KF')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/rek10c.eps
end


lam=[0.7:0.01:1];
for k=1:length(lam)
  [thhat,lamhat]=adfilter(y,nnn,lam(k),'RLS');
  e=simresid(y,nnn,thhat);
  V(k)=e'*e/N;
end
figure(5);
dspbook('plotfix')
subplot(311)
plot(lam,V)
ylabel('V^{RLS}')
xlabel('\lambda')
%title('Frlustfunktion fr RLS som funktion av \lambda')

clear V
mu=logspace(-3,0,20);
for k=1:length(mu)
  [thhat,lamhat]=adfilter(y,nnn,mu(k),'NLMS');
  e=simresid(y,nnn,thhat);
  V(k)=e'*e/N;
end
subplot(312)
plot(mu,V)
ylabel('V^{NLMS}')
xlabel('\mu')
%title('Frlustfunktion fr NLMS som funktion av \mu')

clear V
r1=logspace(-3,0,20);
for k=1:length(r1)
  [thhat,lamhat]=adfilter(y,nnn,[r1(k) 0;0 0],'KF');
  e=simresid(y,nnn,thhat);
  V(k)=e'*e/N;
end
subplot(313)
plot(mu,V)
ylabel('V^{KF}')
xlabel('q_1')
%title('Frlustfunktion fr KF som funktion av q_1')

if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/rekkomp.eps
end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.1')
  close all
  wL=0.1*pi;
  k=1:50;
  h=sin(wL*k)./k./pi;
  h=[h(end:-1:1) wL/pi h];
  k=[-k(end:-1:1) 0 k];
  figure
  dspbook('plotfix')
  plot(k,h)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_1a.eps
  end
  N=length(k);
  ind=25:75;
  ht=zeros(size(h));
  ht(ind+25)=h(ind);
  figure
  dspbook('plotfix')
  plot(k,ht)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_1b.eps
  end
  Ht=fft([ht zeros(1,1024-N)]);
  w=2*(0:1024-1)/1024;
  figure
  dspbook('plotfix')
  plot(w,abs(Ht))
  set(gca,'Xlim',[0 1])
  xlabel('Frekvens (normaliserad)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_1c.eps
  end
  
end

 
if strcmp(ex,'all') | strcmp(ex,'ex4.2')
close all
load ekg,
figure
dspbook('plotfix')
plot(y)
xlabel('Tid (sampel)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgfilt1.eps
end
g = etfe(y);
figure
dspbook('plotfix')
%ffplot([g]) 
semilogy(g(2:end,1),g(2:end,2))
title(' ')
xlabel('Frekvens (Hz)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgfilt2.eps
end

[b,a] = butter(5,[45/100, 55/100],'stop');
ekgf = filtfilt(b, a, y);
figure
dspbook('plotfix')
plot(ekgf)
xlabel('Tid (sampel)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgfilt3.eps
end

Y=fft(y);
Y([ [73:79],302-[73:79] ])=zeros(1,14);
yf=real(ifft(Y));
figure
dspbook('plotfix')
plot(yf)
xlabel('Tid (sampel)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgftfilt.eps
end


end


if strcmp(ex,'all') | strcmp(ex,'fig4.5')
  close all
  h1=[ones(1,300) zeros(1,700)];
  h2=[zeros(1,700) ones(1,300)];
  h3=[zeros(1,350) ones(1,300) zeros(1,350)];
  h4=[ones(1,350) zeros(1,300) ones(1,350)];
  w=(0:999)/1000;
  figure
  dspbook('plotfix')
  plot(w,h1);
  axis([0 1 -0.2 1.2])
  xlabel('Frekvens (normaliserad)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_5a.eps
  end
  figure
  dspbook('plotfix')
  plot(w,h2);
  axis([0 1 -0.2 1.2])
  xlabel('Frekvens (normaliserad)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_5b.eps
  end
  figure
  dspbook('plotfix')
  plot(w,h3);
  axis([0 1 -0.2 1.2])
  xlabel('Frekvens (normaliserad)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_5c.eps
  end
  figure
  dspbook('plotfix')
  plot(w,h4);
  axis([0 1 -0.2 1.2])
  xlabel('Frekvens (normaliserad)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/fig4_5d.eps
  end
end


if strcmp(ex,'all') | strcmp(ex,'4.10')
  close all
%  y=[(0:10) (9:-1:1)];
  y=[0 0 10 10 10 0 (1:10)];
  t=1:length(y);
  t2=1:length(y)/2;
  yhp=filter([1 -1],1,y);
  yhpdec=yhp(2:2:end);
  ylp=filter([1 1],1,y);
  ylpdec=ylp(2:2:end);
  yhpdecint=kron(yhpdec,[1 0]);
  ylpdecint=kron(ylpdec,[1 0]);
  yhpdecintihp=filter(0.5*[-1 1],1,yhpdecint);
  ylpdecintilp=filter(0.5*[1 1],1,ylpdecint);
  ylpdecintilp+yhpdecintihp
  y
  
  figure
  subplot(5,1,1)
  dspbook('plotfix')
  stem(y)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,3)
  dspbook('plotfix')
  stem(t,yhp)
%  title('HP')
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,4)
  dspbook('plotfix')
  stem(t,ylp)
%  title('LP')
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,5)
  dspbook('plotfix')
  stem(t2,yhpdec)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,6)
  dspbook('plotfix')
  stem(t2,ylpdec)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,7)
  dspbook('plotfix')
  stem(t,yhpdecint)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,8)
  dspbook('plotfix')
  stem(t,ylpdecint)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,9)
  dspbook('plotfix')
  stem(t,yhpdecintihp)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  subplot(5,2,10)
  dspbook('plotfix')
  stem(t,ylpdecintilp)
  set(gca,'XtickLabel',[])
  set(gca,'YtickLabel',[])
  
  
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/filterbank.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'fig5.1')
  load current
  close all
  figure(1),   
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.5])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/fig5_1a.eps
  end
  load equake
  figure(2),   
  dspbook('plotfix')
  N=length(y);
  plot(1/fs*(1:1000),y(101:1100))
  set(gca,'Xlim',[25 50])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/fig5_1b.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'fig5.2')
  load ess
  close all
  figure(1),   
  dspbook('plotfix')
  plot(1/fs*(1:length(y)),y)
  set(gca,'Xlim',[0.3 0.35])
  xlabel('Tid [s]')
  if saveplots==1; 
    print -deps /home/rt/fredrik/digsigfig/fig5_2.eps
  end
end



if strcmp(ex,'all') | strcmp(ex,'5.4')
  close all
load ekg, 
g = etfe(y,100);
g=sett(g,1/200);
%g.Ts=1/200;
m=poly2th([1 0 0.99],[],[],[],[], 0.03); 
gm =trf(m); 
gm=sett(gm,1/200);
figure(1)
subplot(311)
dspbook('plotfix')
%ffplot([g gm]) 
semilogy(g(2:end,1)/pi/2,g(2:end,2),'-',...
	 gm(2:end,1)/pi/2,gm(2:end,2),'--')
legend('ETFE','Modell 1',1)
axis([0 100 1e-4 1e0])
num=conv([1 -0.5],[1 -0.5]);
[b,a]=butter(3,0.4);
b1=b/b(1);
mm=poly2th(num,[],b1,a,[],b(1)*b(1)); 
gmm=trf(mm);
gmm=sett(gmm,1/200);
%figure(2)
subplot(312)
dspbook('plotfix')
%ffplot([g gmm])
semilogy(g(2:end,1)/pi/2,g(2:end,2),'-',...
	 gmm(2:end,1)/pi/2,gmm(2:end,2),'--')
legend('ETFE','Modell 2',1)
%set(gca,'Ylim',[1e-5 1e1])
axis([0 100 1e-4 1e0])
gs=gm; gs(2:129,2)=gs(2:129,2)+gmm(2:129,2);
%figure(3)
subplot(313)
dspbook('plotfix')
%ffplot([g gs]) 
semilogy(g(2:end,1)/pi/2,g(2:end,2),'-',...
	 gs(2:end,1)/pi/2,gs(2:end,2),'--')
legend('ETFE','Modell 1 och 2',1)
axis([0 100 1e-4 1e0])
title(' ')
xlabel('Frekvens (Hz)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgmod.eps
end
end

if strcmp(ex,'all') | strcmp(ex,'6.3')
close all
b=[1 -1.5 0.7];
a=1;
e=randn(500,1);
y=filter(b,a,e);

figure
dspbook('plotfix')
plot(101:200,y(101:200))
title(' ')
xlabel('Tid (sampel)')
hold off
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ma0.eps
end


gar2=trf(ar(y,2));
gar4=trf(ar(y,4));
gar6=trf(ar(y,6));
garma22=trf(armax(y,[2 2]));
gma2=trf(armax(y,[0 2]));

figure
dspbook('plotfix')
%bodeplot([gar2 gar4 gar6])
semilogy(gar2(2:end,1)/pi,gar2(2:end,2),'-',...
       gar4(2:end,1)/pi,gar4(2:end,2),'--',...
       gar6(2:end,1)/pi,gar6(2:end,2),'-.')
legend('AR(2)','AR(4)','AR(6)',2)
hold on
[g,w] = freqz(b,a,100);
h=semilogy(w/pi,abs(g).^2);
set(h,'LineWidth',2)
title(' ')
xlabel('Frekvens (normaliserad)')
hold off
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ma1.eps
end
figure
dspbook('plotfix')
%bodeplot([garma22 gma2])
semilogy(garma22(2:end,1)/pi,garma22(2:end,2),'-',...
       gma2(2:end,1)/pi,gma2(2:end,2),'-.')
legend('ARMA(2,2)','MA(2)',2)
hold on
h=semilogy(w/pi,abs(g).^2);
set(h,'LineWidth',2)
hold off
title(' ')
xlabel('Frekvens (normaliserad)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ma2.eps
end
end



if strcmp(ex,'all') | strcmp(ex,'6.4')
close all
load ekg, 
g = etfe(y,100);
m33=armax(y,[3 3]);
garma33 = trf(m33); 
m44=armax(y,[4 4]);
garma44 = trf(m44); 
figure
dspbook('plotfix')
%ffplot([g garma33])
semilogy(g(2:end,1)/pi*100,g(2:end,2),'-',...
       garma33(2:end,1)/pi*100,garma33(2:end,2),'--')
title(' ')
legend('ETFE','ARMA(3,3)',2)
xlabel('Frekvens (Hz)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgmodest1.eps
end
figure
dspbook('plotfix')
%ffplot([g garma44])
semilogy(g(2:end,1)/pi*100,g(2:end,2),'-',...
       garma44(2:end,1)/pi*100,garma44(2:end,2),'--')
title(' ')
legend('ETFE','ARMA(4,4)',2)
xlabel('Frekvens (Hz)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgmodest2.eps
end
figure
dspbook('plotfix')
compare(y,m44,10,151:200);
title(' ')
xlabel('Tid (sampel)')
if saveplots==1;
  print -deps /home/rt/fredrik/digsigfig/ekgmodest3.eps
end

end


if strcmp(ex,'all') | strcmp(ex,'zeropad')
  close all
  N=32; T=1;
  t=0:N-1;
  w0=2*pi/N/T;
  x1=sin(5*w0*t);
  X1=fft(x1);
  subplot(211)
  stem(abs(X1))
  axis([0 N 0 20])
  h=ylabel('x_1=sin(5 \omega_0 t)');
  xlabel('Frekvens (sampel)')
  hold on
  x2=sin(5.5*w0*t);
  X2=fft(x2);
  subplot(212)
  stem(abs(X2))
  axis([0 N 0 20])
  h(2)=ylabel('x_2=sin(5.5 \omega_0 t)');
  xlabel('Frekvens (sampel)')
  %set(h,'FontSize',fontsize)
  hold on
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/zeropad1.eps
  end
  if saveplots==0; pause; end
  X1=fft([x1 zeros(1,1024-N)]);
  subplot(211)
  plot(1+(1:1024)/N,abs(X1))
  axis([0 N 0 20])
  h=ylabel('x_1=sin(5 \omega_0 t)');
  xlabel('Frekvens (sampel)')
  X2=fft([x2 zeros(1,1024-N)]);
  subplot(212)
  plot(1+(1:1024)/N,abs(X2))
  axis([0 N 0 20])
  h(2)=ylabel('x_2=sin(5.5 \omega_0 t)');
  xlabel('Frekvens (sampel)')
  %set(h,'FontSize',fontsize)
  hold off
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/zeropad2.eps
  end
  
end

if strcmp(ex,'all') | strcmp(ex,'zeropadmultisine')
  close all
  N=32; T=1;
  t=0:N-1;
  w0=2*pi/N/T;
  w1=5.1;
  w2=5.9;
  x1=sin(w1*w0*t);
  x2=sin(w2*w0*t);
  x=x1+x2;
  X=fft(x);
  subplot(311)
  stem(abs(X))
  hold on
  plot(abs(X))
  hold off
  axis([0 N 0 20])
  h=title(['x(t)=sin(',num2str(w1),' t)+sin(',num2str(w2),' t)']);
  %set(h,'FontSize',fontsize)

  X=fft([x zeros(1,2*N-N)]);
  subplot(312)
  stem(abs(X))
  hold on
  plot(abs(X))
  hold off
  axis([0 2*N 0 20])

  X=fft([x zeros(1,4*N-N)]);
  subplot(313)
  stem(abs(X))
  hold on
  plot(abs(X))
  hold off
  axis([0 4*N 0 20])
  xlabel('Frekvens (sampel)')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/zeropadmultisine.eps
  end
  
end

if strcmp(ex,'all') | strcmp(ex,'truncation')
  close all
  N=64; T=1;
  t=0:N-1;
  w0=2*pi/N/T;
  w1=1;
  x=sin(w1*t);
  X1=fft([x(1:N/4) zeros(1,9*N+3*N/4)]);
  X2=fft([x(1:N/2) zeros(1,9*N+N/2)]);
  X3=fft([x(1:N)   zeros(1,9*N)]);
  w=2*pi/N/10/T*(0:10*N-1);
  subplot(311)
  plot(w,abs(X1))
  axis([0 2*pi 0 30])
  h=title(['TDFT av x(t)=sin(',num2str(w1),' t)']);
  %set(h,'FontSize',fontsize)
  subplot(312)
  plot(w,abs(X2))
  axis([0 2*pi 0 30])
  subplot(313)
  plot(w,abs(X3))
  axis([0 2*pi 0 30])
  xlabel('Frekvens \omega [rad/s]')

  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/truncation.eps
  end
  
end


if strcmp(ex,'all') | strcmp(ex,'truncationhamming')
  close all
  N=64; T=1;
  t=0:N-1;
  w0=2*pi/N/T;
  w1=1;
  x=sin(w1*t);
  X1=fft([hamming(N/4)'.*x(1:N/4) zeros(1,9*N+3*N/4)]);
  X2=fft([hamming(N/2)'.*x(1:N/2) zeros(1,9*N+N/2)]);
  X3=fft([hamming(N)'.*x(1:N)   zeros(1,9*N)]);
  w=2*pi/N/10/T*(0:10*N-1);
  subplot(311)
  plot(w,abs(X1))
  axis([0 2*pi 0 30])
  h=title(['TDFT av w(t)sin(',num2str(w1),' t)']);
  %set(h,'FontSize',fontsize)
  subplot(312)
  plot(w,abs(X2))
  axis([0 2*pi 0 30])
  subplot(313)
  plot(w,abs(X3))
  axis([0 2*pi 0 30])
  xlabel('Frekvens \omega [rad/s]')

  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/truncationhamming.eps
  end
  
end

if strcmp(ex,'all') | strcmp(ex,'truncationmultisine')
  close all
  N=64; T=1;
  t=0:N-1;
  w0=2*pi/N/T;
  w1=1; w2=1.1;
  x=sin(w1*t)+sin(w2*t);
  X1=fft([x(1:N/4) zeros(1,9*N+3*N/4)]);
  X2=fft([x(1:N/2) zeros(1,9*N+N/2)]);
  X3=fft([x(1:N)   zeros(1,9*N)]);
  w=2*pi/N/10/T*(0:10*N-1);
  subplot(311)
  plot(w,abs(X1))
  axis([0 2*pi 0 30])
  h=title(['TDFT av x(t)=sin(',num2str(w1),' t)+sin(',num2str(w2),' t)']);
  %set(h,'FontSize',fontsize)
  subplot(312)
  plot(w,abs(X2))
  axis([0 2*pi 0 30])
  subplot(313)
  plot(w,abs(X3))
  axis([0 2*pi 0 30])
  xlabel('Frekvens \omega [rad/s]')

  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/truncationmultisine.eps
  end
  
end


if strcmp(ex,'all') | strcmp(ex,'fig4.6')
   close all
   figure(1)
   dspbook('plotfix')
   i=sqrt(-1);
   [h,w]=freqz(ones(6,1),poly([0.7+0.5*i,0.7-0.5*i]),100);
   h=abs(h/h(1));
   w=w/pi;
   plot(w,h)   
   hold on
   ind05=find(h<0.5); ind05=ind05(1);
   [hmax,ind]=max(h);
   plot([0 w(ind05)],[hmax hmax],'--')
   text(w(ind),hmax+0.03,'R_p \leftrightarrow \rho')
   ind0=find(h<0.01);
   [hmin,ind]=max(h(ind0(1):end));
   plot([w(ind05) 1],[hmin hmin],'--')
   text(w(ind0(1)+ind),hmin+0.03,'R_s')
    
   A1=0.9; A2=0.2;
   ind=find(h<A1);
   plot([0 w(ind(1))],[A1 A1],':')
   plot(w(ind(1))*[1 1],[0 h(ind(1))],':')
   text(w(ind(1))-0.05,A1-0.1,'\omega_1')
   ind=find(h<A2);
   plot([0 w(ind(1))],[A2 A2],':')
   plot(w(ind(1))*[1 1],[0 h(ind(1))],':')
   text(w(ind(1))-0.05,A2-0.1,'\omega_2')
   hold off
   xlabel('Normaliserad frekvens')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/filterdef.eps
  end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.7')
   close all
   w1=0.25;
   w2=0.26;
   t=1:1000;
   y1=sin(w1*t);
   y2=sin(w2*t);
   y=y1+y2;
   figure(1)
   subplot(311)
   dspbook('plotfix')
   plot(y1)
   subplot(312)
   dspbook('plotfix')
   plot(y2)
   subplot(313)
   dspbook('plotfix')
   plot(y)
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_7.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.8')
   close all
   u=[zeros(1,10) ones(1,10) zeros(1,80)];
   b1=[0 0 0 0 1];
   a1=1;
   y1=filter(b1,a1,u);
   [H1,w1]=freqz(b1,a1,256);
   b2=[1 -1.1];
   a2=[1.1 -1];
   y2=filter(b2,a2,u);
   [H2,w2]=freqz(b2,a2,256);
   
   figure(1)
   subplot(221)
   dspbook('plotfix')
   plot(y1)
   title('Filtrerad puls')
   axis([0 100 -1.2 1.2])
   subplot(222)
   dspbook('plotfix')
   plot(w1,unwrap(angle(H1)))
   title('Filtrets fas')
   subplot(223)
   dspbook('plotfix')
   plot(y2)
   axis([0 100 -1.2 1.2])
   subplot(224)
   dspbook('plotfix')
   plot(w2,angle(H2))
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_8.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.9')
   close all
   t=1:200;
   w=0.1;
   u=sin(w*t);
   e=0.316*randn(size(u));
   [b1,a1]=butter(2,0.1);
   y1=filter(b1,a1,u+e);
   [b2,a2]=butter(10,0.1);
   y2=filter(b2,a2,u+e);
   
   figure
   dspbook('plotfix')
   plot(t,y1,'-',t,u,'--')
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_9a.eps
   end
   figure
   dspbook('plotfix')
   plot(t,y2,'-',t,u,'--')
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_9b.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.10')
   close all
   w1=logspace(-2,2,1024);
   w2=logspace(-2,pi,1024);
   K=1;, alpha=0.1; T=1;
   H1=freqs([0 K],[1 alpha],w1);
   H2=freqz(K*T/(2+alpha*T)*[1 1],[1 (alpha*T-2)/(alpha*T+2)],w2);
   
   figure
   dspbook('plotfix')
   loglog(w1,abs(H1),'-',w2,abs(H2),'--')
   set(gca,'Ylim',[1e-2 1e1])
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_10a.eps
   end
   figure
   dspbook('plotfix')
   semilogx(w1,angle(H1),'-',w2,angle(H2),'--')
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_10b.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.11')
   close all
   wg=0.1;
   dw=0.001;
   w=0.04:dw:0.2;
   [b,a]=butter(5,wg);
   H1=freqz(b,a,w*pi);
   h1=filter(b,a,[1 zeros(1,99)]);
   [b,a]=cheby2(5,30,wg);
   H2=freqz(b,a,w*pi);
   h2=filter(b,a,[1 zeros(1,99)]);
   [b,a]=ellip(5,0.1,30,wg);
   H3=freqz(b,a,w*pi);
   h3=filter(b,a,[1 zeros(1,99)]);
   [b,a]=cheby1(5,0.1,wg);
   H4=freqz(b,a,w*pi);
   h4=filter(b,a,[1 zeros(1,99)]);
   
   figure
   subplot(321)
   dspbook('plotfix')
   plot(w,abs(H1),'-',w,abs(H2),'--')
   axis([0.04 0.2 0 1])
   ylabel('Amplitudkurva')
   title('butter och cheby2')
   subplot(322)
   dspbook('plotfix')
   plot(w,abs(H3),'-',w,abs(H4),'--')
   axis([0.04 0.2 0 1])
   title('ellip och cheby1')

   dw=dw*pi;
   subplot(323)
   dspbook('plotfix')
   H1d=diff(phase(H1));
   for k=1:length(H1d);
     if H1d(k)>pi-0.5; H1d(k)=H1d(k)-pi; end
     if H1d(k)<-pi+0.5; H1d(k)=H1d(k)+pi; end
   end
   H2d=diff(phase(H2));
   for k=1:length(H2d);
     if H2d(k)>pi-0.5; H2d(k)=H2d(k)-pi;  end
     if H2d(k)<-pi+0.5; H2d(k)=H2d(k)+pi; end
   end
   plot(w,[0 -H1d]/dw,'-',w,[0 -H2d]/dw,'--')
   axis([0.04 0.2 0 50])
   ylabel('Grupplptider')
%   set(gca,'Xlim',[0.04 0.2])
   subplot(324)
   dspbook('plotfix')
   H3d=diff(phase(H3));
   for k=1:length(H3d);
     while H3d(k)>pi-0.5; H3d(k)=H3d(k)-pi; end
     while H3d(k)<-pi+0.5; H3d(k)=H3d(k)+pi; end
   end
   H4d=diff(phase(H4));
   for k=1:length(H4d);
     if H4d(k)>pi-0.5; H4d(k)=H4d(k)-pi;  end
     if H4d(k)<-pi+0.5; H4d(k)=H4d(k)+pi; end
   end
   plot(w,[0 -H3d]/dw,'-',w,[0 -H4d]/dw,'--')
   axis([0.04 0.2 0 50])

   t=1:length(h1);
   subplot(325)
   dspbook('plotfix')
   plot(t,h1,'-',t,h2,'--')
   axis([0 100 -0.15 0.15])
   ylabel('Pulssvar')
   subplot(326)
   dspbook('plotfix')
   plot(t,h3,'-',t,h4,'--')
   axis([0 100 -0.15 0.15])
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_11.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.12')
   close all
   wg=0.1;
   dw=0.001;
   w=0.04:dw:0.2;
   [b,a]=cheby1(3,0.1,wg);
   H1=freqz(b,a,w*pi);
   h1=filter(b,a,[1 zeros(1,99)]);
   [b,a]=cheby1(5,0.1,wg);
   H2=freqz(b,a,w*pi);
   h2=filter(b,a,[1 zeros(1,99)]);
   [b,a]=cheby1(10,0.1,wg);
   H3=freqz(b,a,w*pi);
   h3=filter(b,a,[1 zeros(1,99)]);
   
   figure
   subplot(311)
   dspbook('plotfix')
   plot(w,abs(H1),'-',w,abs(H2),'--',w,abs(H3),'-.')
   axis([0.04 0.2 0 1])
   ylabel('Amplitudkurva')
   legend('n=3','n=5','n=10',3)

   subplot(312)
   dspbook('plotfix')
   H1d=diff(phase(H1));
   H2d=diff(phase(H2));
   H3d=diff(phase(H3));
%   for k=1:length(H4d);
%     if H4d(k)>pi-0.5; H4d(k)=H4d(k)-pi;  end
%     if H4d(k)<-pi+0.5; H4d(k)=H4d(k)+pi; end
%   end
   dw=dw*pi;
   plot(w,[0 -H1d]/dw,'-',w,[0 -H2d]/dw,'--',w,[0 -H3d]/dw,'-.')
   axis([0.04 0.2 0 120])
   ylabel('Grupplptid')

   t=1:length(h1);
   subplot(313)
   dspbook('plotfix')
   plot(t,h1,'-',t,h2,'--',t,h3,'-.')
   axis([0 100 -0.15 0.15])
   ylabel('Pulssvar')
   
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_12.eps
   end
end

if strcmp(ex,'all') | strcmp(ex,'fig4.14')
   close all
   t=1:100;
   w1=0.09;
   w2=0.1;
   u=sin(2*pi*w1*t)+sin(2*pi*w2*t);
   U=fft(u);
   ind=11:91;
   U(ind)=zeros(size(ind));
   u1=real(ifft(U));
   U=fft(u(1:99));
   ind=11:90;
   U(ind)=zeros(size(ind));
   u2=real(ifft(U));
   
   figure
   subplot(311)
   dspbook('plotfix')
   plot(t,u)
   subplot(312)
   dspbook('plotfix')
   plot(t,u1)
   subplot(313)
   dspbook('plotfix')
   plot(1:99,u2)
   if saveplots==1;
     print -deps /home/rt/fredrik/digsigfig/fig4_14.eps
   end
end


if strcmp(ex,'all') | strcmp(ex,'ex6.1')
  close all
  randn('seed',0)
  % estimating AR models
  a=[1 -1.5 0.7];
  N=350;
  e=randn(N,1);
  y=filter(1,a,e);
  % correct model
  th2=rarx(y,2,'ff',1);
  figure(1)
  dspbook('plotfix')
  p=plot(th2);
%  set(p,'LineWidth',2)
  h=title('AR(2) modell, \theta^0=[-1.5; 0.7]');
  xlabel('Antal data N')
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval1.eps, end

  % validation
  th2=arx(y,2);
  figure(2)
  dspbook('plotfix')
%  bodeplot([spa(y) th2ff(th2)])
  g=spa(y);
  g1=th2ff(th2);
  semilogy(g(2:end,1),g(2:end,2),'--',g1(2:end,1),g1(2:end,2),'-')
  legend('ETFE','AR(2)',3)
  h1=title('Spektrum fr data och AR(2)-modell');
  h2=xlabel('Frekvens [rad/s]')
%  set([h1 h2],'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval2.eps, end
    
  figure(3)
  dspbook('plotfix')
  plot(1:N,y,':',1:N,predict(y,th2,1),'-')
  h=title('Enstegsprediktioner (heldragen) och signal (prickad)');
  xlabel('Tid (sampel)')
  %set(h,'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval3.eps, end

  figure(4)
  dspbook('plotfix')
  [e,r]=resid(y,th2);
%  plot(r(1:25))
  h1=title('Korrelationen mellan \epsilon(t) och y(t-\tau)');
  h2=xlabel('\tau');
  set([h1 h2],'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval4.eps, end

  figure(5)
  dspbook('plotfix')
  plot(e)
  h=title('Residualer frn AR(2) modell');
  xlabel('Tid (sampel)')
  %set(h,'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval5.eps, end

  % under-modeling
  th1=rarx(y,1,'ff',1);
  figure(6)
  dspbook('plotfix')
  p=plot(th1);
%  set(p,'LineWidth',2)
  h=title('AR(1) modell');
  xlabel('Antal data N')
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval6.eps, end
  % validation
  th1=arx(y,1);

  figure(7)
  dspbook('plotfix')
  %bodeplot([spa(y) th2ff(th1)])
  g=spa(y);
  g1=th2ff(th1);
  semilogy(g(2:end,1),g(2:end,2),'--',g1(2:end,1),g1(2:end,2),'-')
  legend('ETFE','AR(1)',3)
  h1=title('Spektrum fr data och AR(1)-modell');
  h2=xlabel('Frekvens [rad/s]')
  set([h1 h2],'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval7.eps, end

  figure(8)
  dspbook('plotfix')
  plot(1:N,y,':',1:N,predict(y,th1,1),'-')
  h=title('Enstegsprediktioner (heldragen) och signal (prickad)');
  xlabel('Tid (sampel)')
  %set(h,'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval8.eps, end

  figure(9)
  dspbook('plotfix')
  [e,r]=resid(y,th1);
  %plot(r(1:25))
  h1=title('Korrelationen mellan \epsilon(t) och y(t-\tau)');
  h2=xlabel('\tau');
  set([h1 h2],'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval9.eps, end

  figure(10)
  dspbook('plotfix')
  plot(e)
  h=title('Residualer frn AR(1) modell');
  xlabel('Tid (sampel)')
  %set(h,'FontSize',fontsize)
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modval10.eps, end
end

if strcmp(ex,'all') | strcmp(ex,'ex6.2')
  close all
  randn('seed',0)
  % estimating AR models
  a=[1 -1.5 0.7];
  N=350;
  e=randn(N,1);
  y=filter(1,a,e);
  lamhat=zeros(1,10);
  for n=1:10;
    th=arx(y,n);
    [par,P,lamhat(n)]=th2par(th);
    disp([num2str(n),'  ',num2str(par)])
    disp([num2str(n),'  ',num2str(sqrt(diag(P)'))])
    disp(' ')
  end
  n=1:10;
  disp(num2str(n))
  disp(num2str(lamhat(n)))

  figure
  dspbook('plotfix')
  p1=plot(n,lamhat(n));
  hold on
  p2=plot(n,lamhat(n),'o');
  p3=plot(n,lamhat(n).*(1+2*n/N),'--');
  p4=plot(n,lamhat(n).*(1+2*n/N),'o');
  hold off
%  set([p1 p2 p3 p4],'LineWidth',2)
  h=legend([p1 p3],'W_N(n)','U_N(n)');
  %set(h,'FontSize',fontsize)
  xlabel('Modellordning n')
  if saveplots==1; print -deps /home/rt/fredrik/digsigfig/modvalaic.eps, end
end


if strcmp(ex,'all') | strcmp(ex,'ex6A.1')
% Gauss-Newton
close all
clear xhat
randn('state',1)
disp('Minimera x1^2+x1*x2+x2^2')
phi=(0:0.01:2*pi);
x=[sin(phi); cos(phi)];
x=[1 -0.5;-0.5 1]*x;  % 
s=1:4;
figure(1)
dspbook('plotfix')
plot(x(1,:)'*s,x(2,:)'*s,'b:')
title('Newton-Raphson minimering av f(x)=x_1^2+x_1x_2+x_2^2')
hold on
figure(2)
dspbook('plotfix')
plot(x(1,:)'*s,x(2,:)'*s,'b:')
title('Stokastisk Newton-Raphson minimering av f(x)=x_1^2+x_1x_2+x_2^2')
hold on
N=100;

figure(1)
dspbook('plotfix')
xhat(:,1)=[4;4];
mu=0.03*inv([2 1; 1 2]);  % mu=-inv(fbis)
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*fprime;
end
plot(xhat(1,:)',xhat(2,:),'-b')
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')

xhat(:,1)=[4;1];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*fprime;
end
plot(xhat(1,:)',xhat(2,:),'-b')
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')

xhat(:,1)=[4;-1];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*fprime;
end
plot(xhat(1,:)',xhat(2,:),'-b')
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')

xhat(:,1)=[4;-4];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*fprime;
end
plot(xhat(1,:)',xhat(2,:),'-b')
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
hold off
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/gauss1.eps, end

figure(2)
dspbook('plotfix')
e=3*randn(2,N);
xhat(:,1)=[4;4];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*(fprime+e(:,k));
end
plot(xhat(1,:)',xhat(2,:),'b')

xhat(:,1)=[4;1];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*(fprime+e(:,k));
end
plot(xhat(1,:)',xhat(2,:),'b')

xhat(:,1)=[4;-1];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*(fprime+e(:,k));
end
plot(xhat(1,:)',xhat(2,:),'b')

xhat(:,1)=[4;-4];
for k=2:N;
  x1=xhat(1,k-1);
  x2=xhat(2,k-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,k)=xhat(:,k-1)-mu*(fprime+e(:,k));
end
plot(xhat(1,:)',xhat(2,:),'b')
hold off
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/gauss2.eps, end
end

if strcmp(ex,'all') | strcmp(ex,'ex6A.2')
% Gauss-Newton with local minimum
close all
randn('state',1)

x=0:0.1:4;
f=x.^5-6*x.^4+6*x.^3+20*x.^2-38*x+20;
plot(x,f)
title('f=x^5-6*x^4+6*x^3+20*x^2-38*x+20')

xstart=[0 4];
XI=[];
FI=[];
for k=1:2;
q=8;xi=zeros(1,q); 
xi(1)=xstart(k);
for l=1:q
  x0=xi(k);
  f0=x0^5-6*x0^4+6*x0^3+20*x0^2-38*x0+20;
  fprim=5*x0^4-24*x0^3+18*x0^2+40*x0-38;
  fbis=20*x0^3-72*x0^2+36*x0+40;
  xi(l+1)=xi(l)-.5*fprim/(fbis);
end
fi=xi.^5-6*xi.^4+6*xi.^3+20*xi.^2-38*xi+20;
XI(k,:)=xi;
FI(k,:)=fi;
end;

figure(1)
dspbook('plotfix')
hold on
plot(XI(1,:),FI(1,:),'o')
plot(XI(2,:),FI(2,:),'*')
hold off
title('f=x^5-6*x^4+6*x^3+20*x^2-38*x+20 fr x0=0 (o) och x0=4 (*)')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/gauss3.eps, end

end


if strcmp(ex,'all') | strcmp(ex,'ex6A.3')
% ARMA estimation
close all
randn('state',0)
a0=-0.5;
b0=0.5;
b=[1 b0];
a=[1 a0];
y=filter(b,a,randn(200,1));
thi=poly2th([1 0],[],[1 0]);
par=[0 0];
for k=1:10
  par(k+1,:)=th2par(armax(y,thi,k));
end
figure(4)
dspbook('plotfix')
plot(0:10,par),hold on
plot(0:10,par,'o'), hold off
xlabel('Iteration'),
title('ARMA(1,1), \theta_0=[-0.5; 0.5]')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/arma1.eps, end


sep=0.04;
B=-1:sep:1;
A=-1:sep:1;
nA=length(A);
nB=length(B);

for k=1:nA
  for l=1:nB
    e=filter([1 A(k)],[1 B(l)],y);
    V(k,l)=sum(e.*e);
  end
end
for k=1:size(par,1);
    e=filter([1 par(k,1)],[1 par(k,2)],y);
    V2(k)=sum(e.*e);
end

figure(1)
dspbook('plotfix')
mesh(B(1:1:nB),A(1:1:nA),log(V(1:1:nA,1:1:nB)))

xlabel('c')
ylabel('a')
title('log(V) fr ARMA(1,1) modell')
hold on
h=plot3(par(:,2),par(:,1),log(V2));
set(h,'Linewidth',3)
%view(150,30)

if saveplots==1, print -deps /home/rt/fredrik/digsigfig/arma2.eps, end

figure(2)
dspbook('plotfix')
inda=find(A>a0);
indb=find(B>b0);
inda=inda(1)-5:inda(1)+5;
indb=indb(1)-5:indb(1)+5;
mesh(B(indb),A(inda),log(V(inda,indb)))
xlabel('c')
ylabel('a')
ax=axis;
hold on
h=plot3(par(:,2),par(:,1),log(V2));
%set(h,'Linewidth',3)
axis(ax)
title('log(V) fr ARMA(1,1) modell')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/arma3.eps, end

figure(3)
dspbook('plotfix')
V=log(V);
Vmin=min(min(V));
Vmax=max(max(V));
contour(B,A,V,[logspace(log10(Vmin),log10(1.2*Vmin),10) ])
hold on
plot(0.5,-0.5,'*')
plot(par(:,2),par(:,1))
plot(par(:,2),par(:,1),'o')
hold off
%contour(B,A,V,Vmin:(Vmax-Vmin)/50:Vmax)
xlabel('c')
ylabel('a')
title('log(V) fr ARMA(1,1) modell')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/arma4.eps, end

end

if strcmp(ex,'all') | strcmp(ex,'3.11')

close all
y=5*sin(0.5*(1:1024)')+randn(1024,1); 
m=3;
r=covf(y,m);
R=toeplitz(r);                       
[U,S,V]=svd(R);                      
angle(roots(U(:,3)))    
abs(roots(U(:,3)))    
  
w=0:0.1:pi;
j=sqrt(-1);
W=exp(-j*(0:m-1)'*w);
Phihat=1./abs(U(:,3)'*W).^2;
figure
dspbook('plotfix')
h1=semilogy(w,Phihat);
xlabel('Frekvens [rad/s]')
ylabel('Amplitud')
%set(h1,'LineWidth',2)
hold on
h=h1;
mark=['--';'-.';'b:'];
for m=4:6;
  W=exp(-j*(0:m-1)'*w);
  r=covf(y,m);
  R=toeplitz(r);
  [U,S,V]=svd(R);
  Phihat=1./sum(abs(U(:,3:m)'*W).^2);
%  figure
%  dspbook('plotfix')
  h2=semilogy(w,Phihat,mark(m-3,:));
%  set(h2,'LineWidth',(7-m)/2)
  h=[h h2];
end
legend(h,'m=3','m=4','m=5','m=6')
hold off
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/music1.eps, end

end

if strcmp(ex,'all') | strcmp(ex,'ex7.all')

% Fredrik Gunnarsson 001122
% function void=nytt_ex(alf,b,v_var,hplp)
% alf   - pole of H(q)
% b     - pole of F_s(q)
% a     - zero of F_s(q), computed to give alf
% v_var - noise variance of v
% hplp  - 'high'/'low' (default)

close all
randn('state',77)

%ex1
alf=0.5;
b=0.8;
v_var=1;

%ex1
%alf=0.4
%b=0.6;
%v_var=1;

if b>0
  hplp='low';
else
  hplp='high';
end
n=2; % Berkna h(k), k=-n:n
N=8192;% Antal simulerade data
ind=[21:N-20]; % Intervall fr MSE-berkning

disp('Exempel 7.1 (spektralberkningar)')
a=sqrt(b*v_var*(alf+1/alf)-v_var*(1+b^2))
disp('Fs(q)=')
Fs=tf([a,0],[1,-b],1,'variable','z^-1')
disp('T(q)=')
T=tf([1,-alf],[1,-b],1,'variable','z^-1')
sig2_e=b/alf,

e=sqrt(b/alf)*randn(N,1);
y1=filter([1,-alf],[1,-b],e);
w=randn(N,1);
v=sqrt(v_var)*randn(N,1);
s=filter([0,a],[1,-b],w);
y2=s+v;
th=linspace(0,1,200);
Pss=a^2./(1+b^2-2*b*cos(pi*th));
Pyy=(v_var*(1+b^2)+a^2-2*b*v_var*cos(pi*th))./(1+b^2-2*b*cos(pi*th));
figure
dspbook('plotfix')
plot(th,Pss,[0,1],v_var*[1,1],'--',th,Pyy,'-.')
legend('Spektrum fr s(t)','Spektrum fr n(t)','Spektrum fr y(t)')
xlabel('Normaliserad frekvens')
ylabel('Energi')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_1a.eps, end
mseall(1)=sum(v.^2)/(N-40);
varall(6)=v_var;
hall(1,:)=[zeros(1,n) 1 zeros(1,n)];

disp('Exempel 7.2 (LP-filter)')
Wn=[0.01:0.01:0.99];
mse=zeros(99,1);
for idx=1:99,
  [B,A]=butter(3,Wn(idx),hplp);
  shat=filtfilt(B,A,y2);
  mse(idx)=sum((s(ind)-shat(ind)).^2)/(N-40);
end
figure
dspbook('plotfix')
plot(Wn,mse)
xlabel('Normaliserad grnsfrekvens \omega_g')
ylabel('MSE')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_1c.eps, end
[mse,idx]=min(mse);
mse,th_min=th(idx)
mseall(2)=mse;
varall(2)=NaN;
[B,A]=butter(3,Wn(idx),hplp);
h=filtfilt(B,A,[zeros(10,1);1;zeros(10,1)]);
hall(2,:)=h(11-n:11+n)';

disp('Exempel 7.3 (FIR-WF)')
disp(['Signal-brus-frhllande: ',num2str(a^2/(1-b^2)/v_var)])
Rsy=(b.^[0:n]')*a^2/(1-b^2);
Ryy=Rsy; Ryy(1)=Ryy(1)+v_var;
idx=toeplitz([1:n+1]',[1:n+1]);
h=Ryy(idx)\Rsy;
shat=filter(h',[1,zeros(1,n)],y2);
mse=sum((s(ind)-shat(ind)).^2)/(N-40)
mseall(3)=mse;
var=Rsy(1)-sum(Rsy.*h)
varall(3)=var;
h=[zeros(1,n),h'];
disp(['Koeff: ',num2str(h)])
hall(3,:)=h(:)';

disp('Exempel 7.5 (Kausal wienerfiltrering)')
sig2=a^2/(b/alf);
shat=filtfilt([0,sqrt(sig2)],[1,-alf],y2);
mse=sum((s(ind)-shat(ind)).^2)/(N-40)
mseall(4)=mse;

disp('Direktterm i baktrekursion')
echo on
[r,p,k]=residue([0,sig2/alf,0],[1,-(alf+1/alf),1])
echo off
disp(['residue of ',num2str([0,sig2/alf,0],[1,-(alf+1/alf),1])])
disp('Direktterm i framtrekursion')
echo on
[r,p,k]=residue([0,0,sig2/alf],[1,-(alf+1/alf),1])
echo off
fk=find(abs(p)<1);
fik=find(abs(p)>1);
var=1-(-r(fk)+r(fik)*alf*b)*(a^2/(1-b^2))*1/(1-alf*b)
varall(4)=var;
h=[r(fik)*alf.^[n:-1:1],-r(fk)*alf.^[0:n]];
disp(['Koeff: ',num2str(h)])
hall(4,:)=h(:)';

disp('Exempel 7.6 (kausal wienerfiltrering)')
echo on
[r,p,k]=residue([0,0,-a^2/alf],conv([1,-b],[1,-1/alf]))
echo off
%disp(['residue of ',num2str([0,0,-a^2/alf],conv([1,-b],[1,-1/alf]))])
f=find(abs(p)<1);
r=r(f);
disp('H_0(q)=')
H=tf([r/(b/alf),0],[1,-alf],1)
shat=filter([r/(b/alf),0],[1,-alf],y2);
mse=sum((s(ind)-shat(ind)).^2)/(N-40)
mseall(5)=mse;
var=1 - (r/(b/alf))*(a^2/(1-b^2))*1/(1-alf*b)
varall(5)=var;
h=[zeros(1,n),(r/(b/alf))*alf.^[0:n]];
disp(['Koeff: ',num2str(h)])
hall(5,:)=h(:)';

disp('Exempel 7.6 (enstegsprediktion)')
sig2_e=b/alf,T=tf([1,-alf],[1,-b],1)
[r,p,k]=residue([0,-a^2/alf,0],conv([1,-b],[1,-1/alf]))
f=find(abs(p)<1);
r=r(f);
disp('H_1(q)=')
H=tf([0,r/(b/alf)],[1,-alf],1)
shat=filter([0,r/(b/alf)],[1,-alf],y2);
mse=sum((s(ind)-shat(ind)).^2)/(N-40)
mseall(6)=mse;
var=1 - (r/(b/alf))*(a^2/(1-b^2))*1/(1-alf*b)*b
varall(6)=var;
h=[zeros(1,n+1),(r/(b/alf))*alf.^[0:n-1]];
disp(['Koeff: ',num2str(h)])
hall(6,:)=h(:)';

disp('Exempel 7.6 (enstegsglttning)')
sig2_e=b/alf,T=tf([1,-alf],[1,-b],1)
[r,p,k]=residue([0,0,0,-a^2/alf],[conv([1,-b],[1,-1/alf]),0])
f=find(p==0);
r0=r(f);
f=find((abs(p)<1)&(p~=0));
r=r(f);
disp('H_{-1}(q)=')
H=tf([(r+r0)/(b/alf),-b*r0/(b/alf),0],[0,1,-alf],1)
shat=filter([(r+r0)/(b/alf),-b*r0/(b/alf)],[1,-alf],y2);
ind1=[21:N-20];
ind2=[22:N-19];
mse=sum((s(ind1)-shat(ind2)).^2)/(N-40)
mseall(7)=mse;
K=-b*r0/(b/alf)+(r+r0)/(b/alf)*alf;
var=1 - (a^2/(1-b^2))*((r+r0)/(b/alf)*b + K*1/(1-alf*b))
varall(7)=var;
h=[zeros(1,n-1), (r+r0)/(b/alf) ,(r/(b/alf))*alf.^[0:n]];
disp(['Koeff: ',num2str(h)])
hall(7,:)=h(:)';

disp(' ')
disp('Exempel 7.8 (jmfrelse)')
disp(' ')
tab=str2mat('Inget filter','Lgpass-filter',...
	    'FIR-wienerfilter','Icke-kausalt filter',...
	    'Kausalt filter','Enstegsprediktion','Enstegsglttare');

disp('Tabell 7.2 ver MSE och empiriskt MSE')
disp([tab,char(zeros(7,3)),num2str(mseall(:)),char(zeros(7,3)),num2str(varall(:))])
disp('Tabell 7.3 ver impulssvar h(-2),...,h(2)')
disp([tab,char(zeros(7,3)),num2str(hall)])
  
end


if strcmp(ex,'all') | strcmp(ex,'ex7.0')
  N=8*1024;
  eta=randn(N,1);
  w=randn(N,1);
  [bs,as]=butter(3,[0.2 0.4]);
  [bv,av]=butter(3,[0.6 0.8]);
  v=filter(bv,av,eta);
  s=filter(bs,as,w);
  y=s+v;
  n=100;
%  Phis=(filtfilt(ones(1,n)/n,1,abs(fft(s)).^2))/N;
%  Phiv=(filtfilt(ones(1,n)/n,1,abs(fft(v)).^2))/N;
%  Phiy=(filtfilt(ones(1,n)/n,1,abs(fft(y)).^2))/N;
  Nf=100;
  f=0:1/Nf:1;
  Phiv=abs(freqz(bv,av,pi*f)).^2;
  Phis=abs(freqz(bs,as,pi*f)).^2;
  Phiy=Phiv+Phis;
%  bodeplot([spa(s) spa(v) spa(y)])
  close all
  figure
  dspbook('plotfix')
  p=plot(f,Phis,'-b',f,Phiv,'--b',f,Phiy,'-.b');
%  p=plot(f,Phis(1:N/2+1),'-b',f,Phiv(1:N/2+1),'--b',f,Phiy(1:N/2+1),'-.b');
  legend(p,'Spektrum fr s(t)','Spektrum fr n(t)','Spektrum fr y(t)')
  xlabel('Normaliserad frekvens')
  ylabel('Energi')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_0a.eps, end

%  figur b
  [bs,as]=butter(3,[0.2 0.4]);
  [bv,av]=butter(1,[0.1 0.8]);
  v=filter(bv,av,eta);
  s=filter(bs,as,w);
  y=s+v;
  n=100;
%  Phis=(filtfilt(ones(1,n)/n,1,abs(fft(s)).^2))/N;
%  Phiv=(filtfilt(ones(1,n)/n,1,abs(fft(v)).^2))/N;
%  Phiy=(filtfilt(ones(1,n)/n,1,abs(fft(y)).^2))/N;
  Phiv=abs(freqz(bv,av,pi*f)).^2;
  Phis=abs(freqz(bs,as,pi*f)).^2;
  Phiy=Phiv+Phis;
%  bodeplot([spa(s) spa(v) spa(y)])
  figure
  dspbook('plotfix')
  p=plot(f,Phis,'-b',f,Phiv,'--b',f,Phiy,'-.b');
  legend(p,'Spektrum fr s(t)','Spektrum fr n(t)','Spektrum fr y(t)')
  xlabel('Normaliserad frekvens')
  ylabel('Energi')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_0b.eps, end
end  

if strcmp(ex,'all') | strcmp(ex,'ex7.1')
  close all
  randn('seed',3)
  N=8*1024;
  e=sqrt(1.6)*randn(N,1);
  bT=[1 -0.5]; aT=[1 -0.8];
  y1=filter(bT,aT,e);
  randn('seed',3)
  w=randn(N,1);
  v=randn(N,1);
  bF=[0 0.6];  aF=[1 -0.8];
  s=filter(bF,aF,w);
  y2=s+v;
  Nf=128;
  f=0:1/Nf:1;

  Phiy=abs(freqz(bT,aT,pi*f)).^2;
  Phis=abs(freqz(bF,aF,pi*f)).^2;
  Phiv=ones(size(f));
  figure
  dspbook('plotfix')
  p=plot(f,Phis,'-b',f,Phiv,'--b',f,Phiy,'-.b');
%  p=plot(f,Phis(1:N/2+1),'-b',f,Phiv(1:N/2+1),'--b',f,Phiy(1:N/2+1),'-.b');
  legend(p,'Spektrum fr s(t)','Spektrum fr n(t)','Spektrum fr y(t)')
  xlabel('Normaliserad frekvens')
  ylabel('Energi')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_1a.eps, end
  
  
  n=100;
  Phihats=(cdfiltfilt(ones(1,n)/n,1,abs(fft(s)).^2))/N;
  Phihatv=(cdfiltfilt(ones(1,n)/n,1,abs(fft(v)).^2))/N;
  Phihaty1=(cdfiltfilt(ones(1,n)/n,1,abs(fft(y1)).^2))/N;
  Phihaty2=(cdfiltfilt(ones(1,n)/n,1,abs(fft(y2)).^2))/N;
  figure
  dspbook('plotfix')
  p(1)=plot(f,Phihats(1:N/Nf/2:N/2+1),'-b');
  hold on
  p(2)=plot(f,Phihatv(1:N/Nf/2:N/2+1),'--b');
  p(3)=plot(f,Phihaty1(1:N/Nf/2:N/2+1),'-.b');
  p(4)=plot(f,Phihaty2(1:N/Nf/2:N/2+1),'-.b');
  hold off
%  bodeplot([spa(s) spa(v) spa(y2) spa(y1)]);
  legend(p,'Spektralskattning fr s(t)','Spektralskattning fr n(t)','Spektralskattning fr y_1(t)','Spektralskattning fr y_2(t)')
  xlabel('Normaliserad frekvens')
  ylabel('Energi')
  
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_1b.eps, end


  mse=[];
  ind=20:N-20;
  fg=0.02:0.02:0.98;
  for fgtmp=fg
    [b,a]=butter(3,fgtmp);
    shat=filtfilt(b,a,y2);
    mse=[mse (sum((s(ind)-shat(ind)).^2)/(N-40))];
  end
  [msemin,indmin]=min(mse); msemin
  v_mse=sum((v(ind)).^2)/(N-40)
  fgopt=fg(indmin)
  figure
  dspbook('plotfix')
  plot(fg,mse)
  xlabel('Normaliserad grnsfrekvens')
  ylabel('MSE')

  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/ex7_1c.eps, end

end  

  
if strcmp(ex,'all') | strcmp(ex,'ex7.2')
  Rss=inv([1 -0.8; -0.8 1])*[0.36;0]
  
  h=inv([1+1 0.8 0.64;0.8 1+1 0.8; 0.64 0.8 1+1])*[1; 0.8; 0.64]
  varshat=1-0.8.^(0:2)*h  
  randn('seed',3);
  N=8*1024;
  w=randn(N,1);
  v=randn(N,1);
  bF=[0 0.6];  aF=[1 -0.8];
  s=filter(bF,aF,w);
  y2=s+v;
  ind=21:N-20;
  shat=filter(h,1,y2);
  mse=(sum((s(ind)-shat(ind)).^2)/(N-40))
  
end


if strcmp(ex,'all') | strcmp(ex,'ex7.4')
   format compact
   [r,p,k]=residue([0  -2*0.36/1.6 0],[1 -2.5 1])

   [r,p,k]=residue([0  0 -2*0.36/1.6],[1 -2.5 1])
   
  randn('seed',3);
  N=8*1024;
  w=randn(N,1);
  v=randn(N,1);
  bF=[0 0.6];  aF=[1 -0.8];
  s=filter(bF,aF,w);
  y2=s+v;
  ind=21:N-20;
  shat=filtfilt([0 sqrt(0.36/1.6)],[1 -0.5],y2);
  mse=(sum((s(ind)-shat(ind)).^2)/(N-40))
   
  h=filtfilt([0 sqrt(0.36/1.6)],[1 -0.5],[zeros(4,1);1;zeros(5,1)])
end

if strcmp(ex,'all') | strcmp(ex,'ex7.5')
   format compact
   [r,p,k]=residue([0 0 -2*0.36],[1 -2.8 1.6])
   bk=[0.6/1.6 0];
   ak=[1 -0.5];
   hk=filter(bk,ak,[1;zeros(9,1)])
   hfir=inv([1+1 0.8 0.64;0.8 1+1 0.8; 0.64 0.8 1+1])*[1; 0.8; 0.64];
   hk(1)/hfir(1)
   stem(hk)
   hold on
   stem(hfir,'r')
   hold off

  randn('seed',3);
  N=8*1024;
  w=randn(N,1);
  v=randn(N,1);
  bF=[0 0.6];  aF=[1 -0.8];
  s=filter(bF,aF,w);
  y2=s+v;
  ind=21:N-20;
  shat=filter(bk,ak,y2);
  mse=(sum((s(ind)-shat(ind)).^2)/(N-40))

end

  
if strcmp(ex,'all') | strcmp(ex,'ex8.1')
  
  P0=0.1*eye(4);
  P0(1,2)=0.07;
  P0(2,1)=0.07;
  x0=[0,0,1,1]';
  
  phi=(3*pi/4:-0.05:pi/4)';
  v=[sin(phi) cos(phi)];
  xo=[2+2*sqrt(2)*cos(phi) -2+2*sqrt(2)*sin(phi)];
  Q=0.2*eye(2);
  R=0.1*eye(2);
  B=[0 0; 0 0; 1 0; 0 1];
  A=[1 0 1 0;0 1 0 1; 0 0 1 0; 0 0 0 1];
  C=[1 0 0 0;0 1 0 0];
  x1=A*x0;
  P1=A*P0*A'+B*Q*B';
  x2=A*x1;
  P2=A*P1*A'+B*Q*B';
  x3=A*x2;
  P3=A*P2*A'+B*Q*B';
  
  figure(1)
  dspbook('plotfix')
  p1=plot(xo(:,1),xo(:,2));
  text(2.5,0.5,'x(t)')
%  set(p1,'LineWidth',2)
  hold on
  p=plot(xo(10:10:30,1),xo(10:10:30,2),'o');

  phi=(0:0.1:6.3)';
  v=[sin(phi) cos(phi)];
  p0=plot(x0(1),x0(2),'o');
  vt=v*(sqrtm(P0(1:2,1:2)));
  plot(x0(1)+vt(:,1),x0(2)+vt(:,2))
  text(x0(1)+0.25,x0(2),'Cx(0|0)=Cx(0)')
  
  p1=plot(x1(1),x1(2),'o');
  vt=v*(sqrtm(P1(1:2,1:2)));
  plot(x1(1)+vt(:,1),x1(2)+vt(:,2))
  text(x1(1)+0.1,x1(2),'Cx(1|0)')

  p2=plot(x2(1),x2(2),'o');
  vt=v*(sqrtm(P2(1:2,1:2)));
  plot(x2(1)+vt(:,1),x2(2)+vt(:,2))
  text(x2(1)+0.1,x2(2),'Cx(2|0)')
  
  p3=plot(x3(1),x3(2),'o');
  vt=v*(sqrtm(P3(1:2,1:2)));
  plot(x3(1)+vt(:,1),x3(2)+vt(:,2))
  text(x3(1)+0.1,x3(2),'Cx(3|0)')

%  set([p p0 p1 p2 p3],'LineWidth',2)

  plot([0 0.75],[0 0.75])
  plot([0.75-0.2 0.75],[0.75-0.1 0.75])
  plot([0.75-0.1 0.75],[0.75-0.2 0.75])
  axis('equal')
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kfintro1.eps, end

  
  figure(2)
  dspbook('plotfix')
  p1=plot(xo(:,1),xo(:,2));
  text(2.5,0.5,'Cx(t)')
%  set(p1,'LineWidth',2)
  hold on
  p=plot(xo(10:10:30,1),xo(10:10:30,2),'o');

  phi=(0:0.1:6.3)';
  v=[sin(phi) cos(phi)];
  p0=plot(x0(1),x0(2),'o');
  vt=v*(sqrtm(P0(1:2,1:2)));
  plot(x0(1)+vt(:,1),x0(2)+vt(:,2))
  text(x0(1)+0.25,x0(2),'Cx(0|0)=Cx(0)')

  vo1=(xo(10,:)-xo(9,:))/0.1
  sqrt(sum(vo1.^2))
  xo1=[xo(10,:) vo1]';
  y=xo(10,:)';
  
  K=P1*C'*inv(R+C*P1*C');
  P1g1=P1-P1*C'*inv(R+C*P1*C')*C*P1;
  x1g1=x1+K*(y-C*x1);
  
  p1=plot(x1(1),x1(2),'o');
  vt=v*(sqrtm(P1(1:2,1:2)));
  plot(x1(1)+vt(:,1),x1(2)+vt(:,2))
  text(x1(1)+0.1,x1(2)+0.1,'Cx(1|0)')

  p2=plot(x1g1(1),x1g1(2),'o');
  vt=v*(sqrtm(P1g1(1:2,1:2)));
  plot(x1g1(1)+vt(:,1),x1g1(2)+vt(:,2))
  text(x1g1(1)+0.1,x1g1(2)+0.1,'Cx(1|1)')

%  set([p p0 p1 p2],'LineWidth',2)
  axis('equal')
%  axis([-0.5 2 -0.5 2])
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kfintro2.eps, end
  
  
  P0=zeros(4);
  Q=1*eye(2);
  R=1*eye(2);
  xhat=x0;
  P=P0;

  % t=1
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(10,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);  %K=0: Note the time delay caused by A,B,Q!
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

  % t=2
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(20,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

  % t=3
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(30,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

end

  
if strcmp(ex,'all') | strcmp(ex,'ex8.2')
  %close all
  rand('seed',0);
  N=70;
  lam=3; % Measurement noise scaling
  nnn='nnnplane'; %local version
  % changed x0=[50000,20000,-200,0,0]';
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');

  figure(1);
  dspbook('plotfix')
  h1=radarplot(y,x0);
  legend(h1,'Mtt position','Sann position')
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  plot([0 x0(1,20)],[0 x0(2,20)])
  plot([x0(1,20)-2000 x0(1,20)],[x0(2,20)-1000 x0(2,20)])
  plot([x0(1,20)-1440 x0(1,20)],[x0(2,20)-1800 x0(2,20)])
  plot([0 5000],[0 0],'--')
  hold off
  h3=text(4000,1000,'\theta');
  h4=text(12000,12000,'R');
  h1=xlabel('X');
  h2=ylabel('Y');
  %set([h1 h2 h3 h4],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/trackingex1.eps, end

  figure(2)
  t=1:length(x0);
  subplot(211)
  dspbook('plotfix')
  p1=plot(t,x0(1,:),'-',t,x0(2,:),'--');
  hold on
  plot(t,y(:,1),'-',t,y(:,2),'--');
  hold off
  h1=ylabel('Position [m]');
  subplot(212)
  dspbook('plotfix')
  p2=plot(t,x0(3,:),'-',t,x0(4,:),'--');
  hold on
  plot(t,[0;diff(y(:,1))]/5,'-',t,[0;diff(y(:,2))]/5,'--');
  axis([0 t(end) -300 300])
  hold off
  h2=ylabel('Hastighet [m/s]');
  h3=xlabel('Tid [sampel]');
  set([p1 p2],'LineWidth',2)
  legend([p2],'X','Y')  
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/trackingex2.eps, end
end


if strcmp(ex,'all') | strcmp(ex,'puckex')
  close all
  randn('seed',4);
  N=200;
  B=[0.5;1];
  A=[1 1;0 1];
  C=[1 0];
  R=1600;
  ny=size(C,1);
  sigmaF=2;
  Q=sigmaF^2;
  w=sigmaF*randn(1,N);
  e=sqrt(R)*randn(ny,N);
  x(:,1)=[2000;-20];
  y(:,1)=C*x(:,1)+e(:,1);
  % Fall 1
  xhat=zeros(2,N);
  P=[1000^2 0;0 10^2];
  L=P*C'*inv(R+C*P*C');
  xhat(:,1)=L*y(:,1);
  P=P-L*C*P;
  for t=2:N;
    x(:,t)=A*x(:,t-1)+B*w(t-1);
    y(:,t)=C*x(:,t)+e(:,t);
    xhat(:,t-1)=A*xhat(:,t-1);
    P=A*P*A'+B*Q*B';
    L=P*C'*inv(R+C*P*C');
    xhat(:,t)=xhat(:,t-1)+L*(y(:,t)-C*xhat(:,t-1));
    P=P-L*C*P;
    Pf(:,t)=P(:);
    Lf(:,t)=L;
  end
  P,L
  t=1:N;
  figure(1)
  subplot(211)
  dspbook('plotfix')
  plot(t,x(1,:),'--', t,xhat(1,:),'-')
  ylabel('Lge')
  axis([0 N -10000 2000])
  legend('Simulering','Skattning',3)
  subplot(212)
  dspbook('plotfix')
  plot(t,x(2,:),'--', t,xhat(2,:),'-')
  ylabel('Hastighet')
  legend('Simulering','Skattning',1)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/lvest.eps, end
  
  figure(2)
  subplot(221)
  dspbook('plotfix')
  plot(t,Pf(1,:),'-')
  ylabel('P_{11}(t|t)')
  subplot(222)
  dspbook('plotfix')
  plot(t,Pf(4,:),'-')
  ylabel('P_{22}(t|t)')
  subplot(223)
  dspbook('plotfix')
  plot(t,Lf(1,:),'-')
  ylabel('L_1(t)')
  subplot(224)
  dspbook('plotfix')
  plot(t,Lf(2,:),'-')
  ylabel('L_2(t)')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/QL.eps, end
    
  % Fall 2
  w=[ones(1,50) -ones(1,100) ones(1,50)];
  xhat=zeros(2,N);
  P=[1000^2 0;0 10^2];
  L=P*C'*inv(R+C*P*C');
  xhat(:,1)=L*y(:,1);
  P=P-L*C*P;
  for t=2:N;
    x(:,t)=A*x(:,t-1)+B*w(t-1);
    y(:,t)=C*x(:,t)+e(:,t);
    xhat(:,t-1)=A*xhat(:,t-1);
    P=A*P*A'+B*Q*B';
    L=P*C'*inv(R+C*P*C');
    xhat(:,t)=xhat(:,t-1)+L*(y(:,t)-C*xhat(:,t-1));
    P=P-L*C*P;
    Pf(:,t)=P(:);
    Lf(:,t)=L;
  end
  xhat2=xhat;
  L2=L;
  % Fall 3
  sigmaF=0.20;
  Q=sigmaF^2;
  w=[ones(1,50) -ones(1,100) ones(1,50)];
  xhat=zeros(2,N);
  P=[1000^2 0;0 10^2];
  L=P*C'*inv(R+C*P*C');
  xhat(:,1)=L*y(:,1);
  P=P-L*C*P;
  for t=2:N;
    x(:,t)=A*x(:,t-1)+B*w(t-1);
    y(:,t)=C*x(:,t)+e(:,t);
    xhat(:,t-1)=A*xhat(:,t-1);
    P=A*P*A'+B*Q*B';
    L=P*C'*inv(R+C*P*C');
    xhat(:,t)=xhat(:,t-1)+L*(y(:,t)-C*xhat(:,t-1));
    P=P-L*C*P;
    Pf(:,t)=P(:);
    Lf(:,t)=L;
  end
  xhat3=xhat;
  L3=L;
  % Fall 4
  sigmaF=20;
  Q=sigmaF^2;
  w=[ones(1,50) -ones(1,100) ones(1,50)];
  xhat=zeros(2,N);
  P=[1000^2 0;0 10^2];
  L=P*C'*inv(R+C*P*C');
  xhat(:,1)=L*y(:,1);
  P=P-L*C*P;
  for t=2:N;
    x(:,t)=A*x(:,t-1)+B*w(t-1);
    y(:,t)=C*x(:,t)+e(:,t);
    xhat(:,t-1)=A*xhat(:,t-1);
    P=A*P*A'+B*Q*B';
    L=P*C'*inv(R+C*P*C');
    xhat(:,t)=xhat(:,t-1)+L*(y(:,t)-C*xhat(:,t-1));
    P=P-L*C*P;
    Pf(:,t)=P(:);
    Lf(:,t)=L;
  end
  xhat4=xhat;
  L4=L;
  t=1:N;
  figure(3)
  dspbook('plotfix')
  plot(w)
  axis([0 N -1.2 1.2])
  ylabel('w(t)')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/nyF.eps, end

  figure(4)
  subplot(211)
  dspbook('plotfix')
  p=plot(t,x(1,:),'-', t,xhat2(1,:),'-',t,xhat3(1,:),'-.',t,xhat4(1,:),'--')
  set(p(1),'LineWidth',2)
  ylabel('Lge')
%  axis([0 N -10000 2000])
  legend('Simulering','Skattning \sigma_F=2','Skattning \sigma_F=0.2','Skattning \sigma_F=20',3)
  subplot(212)
  dspbook('plotfix')
  p=plot(t,x(2,:),'-', t,xhat2(2,:),'-',t,xhat3(2,:),'-.',t,xhat4(2,:),'--')
  set(p(1),'LineWidth',2)
  ylabel('Hastighet')
%  legend('Simulering','Skattning \sigma_F=2','Skattning \sigma_F=0.2','Skattning \sigma_F=20',3)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/nyX.eps, end
  
  h2=dlsim(A-L2*C*A,L2,eye(2),0,[1 zeros(1,1023)]');
  h3=dlsim(A-L3*C*A,L3,eye(2),0,[1 zeros(1,1023)]');
  h4=dlsim(A-L4*C*A,L4,eye(2),0,[1 zeros(1,1023)]');
  H2=abs(fft(h2));
  H3=abs(fft(h3));
  H4=abs(fft(h4));
  figure(5)
  subplot(211)
  dspbook('plotfix')
  loglog(2*(0:512)/1024,H2(1:513,1),'-',...
	 2*(0:512)/1024,H3(1:513,1),'-.',...
	 2*(0:512)/1024,H4(1:513,1),'--')
  axis([0 1 1e-2 1e1])
  set(gca,'XTicklabel',[])
  title('Lge')
  legend('\sigma_F=2','\sigma_F=0.2','\sigma_F=20',3)
  subplot(212)
  dspbook('plotfix')
  loglog(2*(0:512)/1024,H2(1:513,2),'-',...
	 2*(0:512)/1024,H3(1:513,2),'-.',...
	 2*(0:512)/1024,H4(1:513,2),'--')
  axis([0 1 1e-4 1e0])
  title('Hastighet')
  xlabel('Normaliserad frekvens')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kalfrek.eps, end
end

if strcmp(ex,'all') | strcmp(ex,'ex8.3')
  close all
  rand('seed',0);
  N=70;
  lam=3; % Measurement noise scaling
  nnn='nnnplane'; %local version
  % changed x0=[50000,20000,-200,0,0]';
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  
  figure(1)
  dspbook('plotfix')
  NNNmethod=1;
  P0=1e5*eye(4);
  [xhat]=adkalman(y,nnn,0.1,[],P0);
  figure(1);
  dspbook('plotfix')
  h1=radarplot(y,xhat);
  legend(h1,'Mtt position','Skattad position')
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off
  h1=title('Initial filterdesign');
  h2=xlabel('X');
  h3=ylabel('Y');
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf1.eps, end

  figure(2)
  dspbook('plotfix')
  t=1:length(x0);
  subplot(211)
  dspbook('plotfix')
  p1=plot(t,x0(1,:),'b-',t,x0(2,:),'b--',t,xhat(1,:),'b-',t,xhat(2,:),'b--');
  set(p1(1:2),'LineWidth',1)	
  set(p1(3:4),'LineWidth',2)
  
  h1=ylabel('Position [m]');
  subplot(212)
  dspbook('plotfix')
  p2=plot(t,x0(3,:),'b-',t,x0(4,:),'b--',t,xhat(3,:),'b-',t,xhat(4,:),'b--');
  set(p2(1:2),'LineWidth',1)	
  set(p2(3:4),'LineWidth',2)
  axis([0 t(end) -300 300])
  h2=ylabel('Hastighet [m/s]');
  h3=xlabel('Tid [sampel]');
  legend([p2(1:2)],'X','Y')  
%  set([p1 p2],'LineWidth',2)
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf2.eps, end

  % Improved design
  [xhat]=adkalman(y,nnn,0.5,[],1e8*eye(4));
  figure(3);
  dspbook('plotfix')
  h1=radarplot(y,xhat);
  legend(h1,'Mtt position','Skattad position')
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off
  h1=title('Initial filterdesign');
  h2=xlabel('X');
  h3=ylabel('Y');
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf3.eps, end

  figure(4)
  dspbook('plotfix')
  t=1:length(x0);
  subplot(211)
  p1=plot(t,x0(1,:),'b-',t,x0(2,:),'b--',t,xhat(1,:),'b-',t,xhat(2,:),'b--');
  set(p1(1:2),'LineWidth',1)	
  set(p1(3:4),'LineWidth',2)
  h1=ylabel('Position [m]');
  subplot(212)
  p2=plot(t,x0(3,:),'b-',t,x0(4,:),'b--',t,xhat(3,:),'b-',t,xhat(4,:),'b--');
  set(p2(1:2),'LineWidth',1)	
  set(p2(3:4),'LineWidth',2)
  axis([0 t(end) -300 300])
  h2=ylabel('Hastighet [m/s]');
  h3=xlabel('Tid [sampel]');
  legend([p2],'X','Y')  
%  set([p1 p2],'LineWidth',2)
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf4.eps, end
end


if strcmp(ex,'all') | strcmp(ex,'ex8.4')
  % design of kalmanfilter
  close all
  rand('seed',0);
  N=70;
  lam=3; % Measurement noise scaling
  nnn='nnnplane'; %local version
  % changed x0=[50000,20000,-200,0,0]';
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  
  NNNmethod=1;
  P0=1e5*eye(4);
  [xhat]=adkalman(y,nnn,0.1,[],P0);

  figure(1)
  dspbook('plotfix')
  p1=plot(sqrt(sum((x0(1:2,:)-xhat(1:2,:)).^2)));
  h1=xlabel('Tid [sampel]');
  h2=ylabel('Positionsfel [m]');
  %set([h1 h2],'FontSize',fontsize)
%  set([p1],'LineWidth',2)
  axis([0 N 0 10000])
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf5.eps, end
  
  lam=logspace(0,2,10);
  kf=[2 1];
  for k=1:length(lam);
    [xhat]=adkalman(y,nnn,lam(k),kf,P0);
    rmse(k)=sqrt(sum(sum((x0(1:2,:)-xhat(1:2,:)).^2))/length(x0));
  end
  figure(2)
  dspbook('plotfix')
  p1=semilogx(lam,rmse);
  h1=xlabel('Skalfaktor');
  h2=ylabel('RMSE [m]');
  %set([h1 h2],'FontSize',fontsize)
%  set([p1],'LineWidth',2)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf6.eps, end
  
  [dum,ind]=min(rmse);
  lamopt=lam(ind);
  [xhat]=adkalman(y,nnn,lamopt,[],P0);
  
  figure(3)
  dspbook('plotfix')
  t=1:length(x0);
  subplot(211)
  p1=plot(t,x0(1,:),'-',t,x0(2,:),'--',t,xhat(1,:),'-',t,xhat(2,:),'--');
  h1=ylabel('Position [m]');
  subplot(212)
  p2=plot(t,x0(3,:),'-',t,x0(4,:),'--',t,xhat(3,:),'-',t,xhat(4,:),'--');
  axis([0 t(end) -300 300])
  h2=ylabel('Hastighet [m/s]');
  h3=xlabel('Tid [sampel]');
  legend([p2],'X','Y')  
%  set([p1 p2],'LineWidth',2)
  %set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kf7.eps, end
end


if strcmp(ex,'all') | strcmp(ex,'ex8.5')
  % Change detection in target tracking
%  close all
  randn('seed',1);
  N=70;
  lam=3; % Measurement noise scaling
  nnn='nnnplane'; %local version
  % changed x0=[50000,20000,-200,0,0]';
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  
  NNNmethod=1;
  P0=1e5*eye(4);
%  [xhat]=adkalman(y,nnn,0.05,[],P0);
  h=3;
  nu=0.25;
  [xhat1,jhat1,xseg1,GT,ta]=detect1(y,nnn,1,[2 h nu],10);
  jhat1

  figure(2)
  dspbook('plotfix')
  h2=plot(GT,'b-');
  axis([0 N 0 h*1.5])
%  set(h2,'LineWidth',2)
  set(h2(2),'LineStyle','--')
  hold on, h1=plot([1 N],[h h]); hold off
  legend([h2;h1],'CUSUM g(t) fr hgersvng','CUSUM g(t) fr vnstersvng','Trskel')
  xlabel('Sampelnummer')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kfdetect1.eps, end

  figure(1)
  dspbook('plotfix')
  h1=radarplot(y,xhat1);
  legend(h1,'Mtt position','Skattad position')
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off

  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/kfdetect2.eps, end

end

if strcmp(ex,'all') | strcmp(ex,'ex8.6')
  close all
  load highway

  nnn='nnnplane';
  global NNNmethod
  NNNmethod=1; % four states
  kf=[2 1];
  q1=1e-3;
  [xhat1,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q1,kf,1e10*eye(4),zeros(4,1)); 

  figure(1)
  dspbook('plotfix')
  h(1)=plot(y(:,1),y(:,2));
  ax=axis;
  %hd=title('Car position from computer vision system');
  hold on
  h(2)=plot(xhat1(1,:),10+xhat1(2,:),'-');
%  set(h(2),'LineWidth',3);
  axis(ax);
  hold off
  xlabel('X')
  ylabel('Y')
  legend(h,'Mtt position','Kalmanfilterskattning',2)
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/highway1.eps, end

  figure(2)
  dspbook('plotfix')
  h(1)=plot(diff(y(:,1)),'b-');
  hold on
  h(2)=plot(diff(xhat1(1,:)),'b-');
%  set(h(2),'LineWidth',3);
  hold off
  xlabel('Tid')
  ylabel('v_X')
  set(gca,'Xlim',[0 length(y)])
  legend(h,'Hastighet frn hastighetsdifferens','Kalmanfilterskattad hastighet')
  %hd=title('Differentiated position in west-east data');
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/highway2.eps, end


  N=length(Epsi);
  figure(3)
  dspbook('plotfix')
  [Nhist,Xhist]=hist(Epsi(20:N),40);
  bar(Xhist,Nhist,'w')
  xlabel('Residual')
  if saveplots==1, print -deps /home/rt/fredrik/digsigfig/highway3.eps, end
  
end


if strcmp(ex,'all') | strcmp(ex,'ex9C.0')
  % ISI
  b=[1 -2.2 1.17];
  randn('state',0);
  N=50;
  u=sign(-0.5+rand(N,1));
  y=filter(b,1,u);

  figure(1), clf
  subplot(311)
  dspbook('plotfix')
  p1=stem(u);
  axis([0 50 -1.4 1.4])
  title('u(t)')
  set(gca,'XtickLabel',[])
%  set([p1],'LineWidth',2)

  subplot(312)
  dspbook('plotfix')
  p1=plot(y,'-o');
  title('y(t)')
  set(gca,'XtickLabel',[])
%  set([p1],'LineWidth',2)

  subplot(313)
  dspbook('plotfix')
  p1=stem(u-sign(y));
  title('u(t)-sign(y(t))')
  axis([0 50 -2.4 2.4])
%  set([p1],'LineWidth',2)

  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/isi.eps,  end
  
end

if strcmp(ex,'all') | strcmp(ex,'ex10.1wf')

disp('Exempel 10.2 (FIR-WF)')
  tau=4; % frdrjning
  m=6;   % deg(h)-deg(b)
  b=[1 -2.2 1.17];
  n=length(b);
  bb=conv(b,b)
 
  Ryy=toeplitz([bb(n:-1:1) zeros(1,m)],[bb(n:n+n-1) zeros(1,m)])
  Rsy=b;
  h=Ryy\[zeros(1,tau) Rsy(1:n)  zeros(1,m-tau)]'
  hb=conv(h,b)
  stem(hb)

end


if strcmp(ex,'all') | strcmp(ex,'ex9C.1')
  % Linjr utjmnare
  b=[1 -2.2 1.17];
  randn('state',0);
  N=200;
  L=25;  % Length of training sequence
  u=sign(-0.5+rand(N,1));
  y=filter(b,1,u);
  nn=1:10;
  V=ones(length(nn),length(nn)-1);
  for n=nn;
    for delay=1:n-1;
     [uhat,fhat,z]=lineq(y,n,[-1 1],delay,u(1:L));
     e=u(L+1:end-delay)-z(L+1+delay:end);
     V(n,delay)=1/N*e'*e;
     BER(n,delay)=u2ber(u,uhat);
    end
  end
  disp('Loss function V(n,delay):')
  V
  BER
  figure(1), clf
  dspbook('plotfix')
  p=plot(1:7,V(8,1:7));
%  set(p,'LineWidth',2)
  xlabel('Tidsfrdrjning \tau')
  ylabel('Frlustfunktion V_8(\tau)')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/equalization1.eps,  end
  
  [Vn,indn]=min(V);
  [dum,delayhat]=min(Vn);
  nhat=indn(delayhat);
  disp('Optimal parameters:')
  delay=delayhat,n=nhat
  n=8;
  [dum,delay]=min(V(8,:));
  n,delay
  [uhat,fhat,z]=lineq(y,n,[-1 1],delay,u(1:L));
  e=u(L+1:end-delay)-z(L+1+delay:end);
  Vmin=1/N*e'*e;
  BER=u2ber(u,uhat);

  figure(2)
  subplot(311)
  dspbook('plotfix')
  p1=stem(0:2,b);
  axis([0 14 -3 3])
  set(gca,'XtickLabel',[])
  h1=title('Impulssvar fr kanal');
  subplot(312)
  dspbook('plotfix')
  p2=stem(0:8,fhat);
  axis([0 14 -2 1])
  h2=title('Impulssvar utjmnare');
  set(gca,'XtickLabel',[])
  subplot(313)
  dspbook('plotfix')
  p3=stem(0:10,conv(b,fhat));
  axis([0 14 -1 2])
%  set([p1 p2 p3],'LineWidth',2)
  h3=title('Kombinerat impulssvar fr kanal och utjmnare');
%  set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/equalization2.eps,  end
end
  
if strcmp(ex,'all') | strcmp(ex,'ex9C.2')
  % Linjr utjmnare, annars som 9C.3
  b=[1 -2.2 1.17];
  randn('state',0);
  N=148;
  L=25;
  MC=50;
  
  lam=[0.01 0.02 0.05 0.1 0.2 0.5 1 2 5 10];
  e0=randn(N,1);
  flops(0);
  for k=1:length(lam);
    for l=1:MC;
      u=sign(-0.5+rand(N,1));
      y0=filter(b,1,u);
      e=sqrt(lam(k))*e0;
      yn=y0+e;
      [uhat,thhat]=lineq(yn,8,[-1 1],4,u(1:L));
      ber(l,k)=u2ber(u,uhat);
      snr(l,k)=sum(y0.^2)/sum(e.^2);
    end
  end
  f=flops/k/l
  BER=mean(ber);
  SNR=mean(snr);
  figure(1), clf
  dspbook('plotfix')
  %set(gca,'FontSize',14)
  p=semilogx(SNR,BER,'-o');
%  set(p,'LineWidth',2)
  xlabel('SNR')
  ylabel('BER')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/lineq1.eps,  end
  [SNR;BER]
  
end

if strcmp(ex,'all') | strcmp(ex,'ex9C.3')
  % Viterbi utjmnare, annars som 9C.2
  b=[1 -2.2 1.17];
  randn('state',0);
  N=148;
  L=25;
  MC=2;
  
  lam=[0.01 0.02 0.05 0.1 0.2 0.5 1 2 5 10];
  e0=randn(N,1);
  flops(0)
  for k=1:length(lam);
    for l=1:MC;
      u=sign(-0.5+rand(N,1));
      y0=filter(b,1,u);
      e=sqrt(lam(k))*e0;
      yn=y0+e;
      [uhat,thhat]=viterbi(yn,3,[-1 1],lam(k),u(1:L));
      ber(l,k)=u2ber(u,uhat);
      snr(l,k)=sum(y0.^2)/sum(e.^2);
    end
  end
  f=flops/k/l
  BER=mean(ber);
  SNR=mean(snr);
  figure(1), clf
  dspbook('plotfix')
  %set(gca,'FontSize',14)
  p=semilogx(SNR,BER,'-o');
%  set(p,'LineWidth',2)
  xlabel('SNR')
  ylabel('BER')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/equalization3.eps,  end
  [SNR;BER]
  
end

if strcmp(ex,'all') | strcmp(ex,'ex9C.4')
  disp('Demonstration of BER vs. SNR plots in equalization for a SISO noisy channel')
  disp(' ')
  echo on

  N=100;
  b=[1 -2.2 1.17];
  MC=10;
  L=[3 5 7 10];
  sigma=[0.05 0.1 0.2 0.5 1 2 5];

  for l=1:length(L);
    for m=1:length(sigma);
      randn('seed',1);
      for n=1:MC;
        u=sign(randn(N,1));
        e=sigma(m)*randn(N,1);
        y0=filter(b,1,u);
        y=y0+e;
        % BER when using a short training sequence
        [uhat,thhat]=viterbi(y,3,[-1 1],sigma(m)^2,u(1:L(l)));
        ber(n,m)=u2ber(u,uhat);
        snr(n,m)=sum(y0.^2)/sum(e.^2);
      end
    end
    BER(l,:)=mean(ber);
    SNR(l,:)=mean(snr);
  end
  BER,SNR
  leg=[];
  typ=['sdph'];
  h=[];
  close all
  figure  
  dspbook('plotfix')
  for l=1:size(SNR,1);  
    h(l)=semilogx(SNR(l,:)',BER(l,:)');
    hold on
    h(l)=semilogx(SNR(l,:)',BER(l,:)',typ(l));
    leg=str2mat(leg,['L = ',num2str(L(l))]);
  end
  leg(1,:)=[];
  xlabel('SNR')  
  ylabel('BER')
  legend(h,leg);
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/equalization4.eps,  end
end

  
if strcmp(ex,'all') | strcmp(ex,'ex9C.5')
close all
disp('Demonstration blind utjmning')

N=1000;
randn('seed',1);
b=[0.3 1 0.3]
%b=[1 -2.2 1.17];

u=sign(randn(N,1));
y=filter(b,1,u); %+0.1*randn(N,1);
nbeq=5;   % equalizer order for blindeq1

b0=[0 -0.1 1 -0.1 0]';

[theq1,uhat1,epsi1]=blindeq(y,nbeq,0.01,1,b0);
[theq2,uhat2,epsi2]=blindeq(y,nbeq,0.01,2,b0);

m1=openeye(b,theq1);
m2=openeye(b,theq2);
figure(1);
dspbook('plotfix')
h1=plot(m1,'-');  hold on
h2=plot(m2,'--'); hold off
%set([h1 h2],'LineWidth',2)
title('Prestandamtt')
xlabel('Tid [sampel]')
legend([h1 h2],'Sato','Godard',3)
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/bleq1a.eps, end


figure(2);
dspbook('plotfix')
plot([epsi1' epsi2'])
title('Residualer')
xlabel('Tid [sampel]')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/bleq1b.eps, end

figure(3);
subplot(311)
dspbook('plotfix')
h1=stem(1:3,b,'-');                   hold on
title('Kanal')
axis([0 7 -2 2])
set(gca,'XTick',[])

subplot(312)
dspbook('plotfix')
h2=stem(1:5,theq1(:,N)','-'); 
title('Utjmnare')
axis([0 7 -2 2])
set(gca,'XTick',[])

subplot(313)
dspbook('plotfix')
h3=stem(1:7,conv(b,theq1(:,N)'),'-'); hold off
title('Totalt: kanal+utjmnare')
axis([0 7 -2 2])
xlabel('Tid [sampel]')

%set([h1 h2 h3],'LineWidth',1.5)
xlabel('Tid [sampel]')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/bleq1c.eps, end

echo off

ber1=u2ber(u,uhat1);
ber2=u2ber(u,uhat2);
disp(['BER Sato       = ',num2str(ber1)])
disp(['BER Godard     = ',num2str(ber2)])
end



  
if strcmp(ex,'all') | strcmp(ex,'ex9B.1')
  close all
  load fuel
  N=length(y);
  threc1=adfilter(y,-1,0.9);
  threc2=adfilter(y,-1,0.7);
  hold on
  figure(1)
  dspbook('plotfix')
  subplot(211)
  t=1:length(y);
  h1=plot(t,y,'--',t,threc1','-');
%  set(h1(2),'LineWidth',2)
  legend([h1],'Mtning','Lngsamt filter')
  subplot(212)
  h2=plot(t,y,'--',t,threc2','-');
%  set(h2(2),'LineWidth',2)
  legend([h2],'Mtning','Snabbt filter')
  xlabel('Sampelnummer')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/fuel1.eps,  end
end

if strcmp(ex,'all') | strcmp(ex,'ex9B.2')
  close all
  load fuel
  N=length(y);
  h=5;
  nu=1;
  [threc,jhat,thseg,gt,ta]=detect1(y,-1,1,[2 h nu]);

  figure(1)
  subplot(211)
  dspbook('plotfix')
  h1=segplot(y,ta);
  set(h1(1),'LineStyle','--')
  title(' ')
  xlabel(' ')
  hold on
  h2=plot(threc);
%  set(h2,'LineWidth',2)
  legend([h1(1:2);h2],'Mtning','Alarmtider','Skattning')
  hold off
  axis([0 N 0 25])

  subplot(212)
  dspbook('plotfix')
  h2=plot(max(gt')','b-');
  axis([0 N 0 h*1.5])
%  set(h2,'LineWidth',2)
  hold on, plot([1 N],[h h]), hold off
  xlabel('Sampelnummer')
%  legend([h1 h2 h3],'Measurement','Slow filter','Fast filter')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/fuel2.eps,  end
end

if strcmp(ex,'all') | strcmp(ex,'ex10.5') | strcmp(ex,'rfi')
  close all
  figure
  dspbook('plotfix')
  plot([0 0],[0 1.2],'k',[0 1],[1 1],'k','linewidth',2)
  axis([-0.2 1 0 1.5])
  text(-0.12,1,'\mu_{glid}')
  text(-0.12,1.15,'\mu_{max}')
  xlabel('Glidhastighet')
  ylabel('Normaliserad friktionskraft \mu')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/llfriktiona.eps, end

  figure
  dspbook('plotfix')
  plot([0 0.15],[0 1.2],'k',[0.15 0.2],[1.2 1],'k',[0.2 1],[1 1],'k',...
     'linewidth',2)
  axis([-0.2 1 0 1.5])
  text(0.02,1,'\mu_{glid}')
  text(0.02,1.15,'\mu_{max}')
  ylabel('Normaliserad friktionskraft \mu')
  xlabel('Hjulglidning  s')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/llfriktionb.eps, end
end




if strcmp(ex,'all') | strcmp(ex,'ex10.1') | strcmp(ex,'tpi')

  load tpi
  
%  dt=diff([0;t1(:)]);
%  f=20:0.25:100;
%  w1f=w1-mean(w1);
%  W1=(w1f.*dt')*exp(-i*t1'*2*pi*f);
%    figure(5)
%    p1=plot(f,filtfilt(ones(10,1)/10,1,abs(W1)));
%    hold on
%    p2=plot(f,abs(W1),'r');
%    hold off
%    set(p2,'LineWidth',2)
%    if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/tpi5.eps,  end

  figure(1)
  dspbook('plotfix')
  bodeplot(etfe(w2'-mean(w2),128,1024,2e-3*2*pi)) 
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/tpi1.eps,  end

  figure(2)
  dspbook('plotfix')
  omega=(0:length(w2)-1)/length(w2)*500;
  plot(omega,abs(fft(w2)))
  axis([20 100 0 1e3])
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/tpi2.eps,  end

  figure(3)
  dspbook('plotfix')
  p2=plot(omega,abs(fft(w2-mean(w2))),'b');
  hold on
  p1=plot(omega,filtfilt(ones(40,1)/40,1,abs(fft(w2-mean(w2)))));
  set(p1,'Linewidth',3,'Color','y')
  hold off
  axis([20 100 0 1e3])
  set(p1,'LineWidth',3)
  set(p1,'Color',[1 1 1])
  xlabel('Frekvens [Hertz]')
  title('Spektralanalys: periodogram och glttning')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/tpi3.eps,  end
  
  figure(4)
  dspbook('plotfix')
  [b,a]=butter(4,[25/250 60/250]);
  w2f=filtfilt(b,a,w2);
  th=ar(w2f,2);
  wres=250*angle(roots([1 th2par(th)]))/pi
  [dum,ind]=max(filtfilt(ones(40,1),1,abs(fft(w2f))))
  wres=ind*500/length(w2f)
  th=sett(th,2e-3*2*pi);
  bodeplot([etfe(w2'-mean(w2),128,[],2e-3*2*pi) trf(th)])
  h1=xlabel('Frekvens [Hertz]');
  h2=title('Icke-parametrisk spektralanalys och AR(2) modell');
  set([h1 h2],'fontsize',14);
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/tpi4.eps,  end
  
end


if strcmp(ex,'all') | strcmp(ex,'ex10.3') | strcmp(ex,'CD')
  % CD player
  %close all
  
  
  figure(1), 
  dw=0.01;
  S=[(1:-dw:0) dw:dw:1];
  N=length(S);
  w=44.1*(0:1/N:1-1/N);
  E=0.4*ones(size(S));

  subplot(311)
  dspbook('plotfix')
  p1=plot(w,S,'-');
  hold on
  p2=plot(w,E,'--');
  hold off
  axis([0 44.1 0 1])  
%  set([p1 p2],'LineWidth',2)
  %legend([p1 p2],'Signal','Kvant.brus')
  
  subplot(312)
  dspbook('plotfix')
  S2=[S S S S];
  E2=[E E E E];
  N=length(S2);
  w2=4*44.1*(0:1/N:1-1/N);
  b=fir1(96,0.25);
  B=abs(freqz(b,1,N/2,4*44.1));
  B=[B; B(end:-1:1)];

  p1=plot(w2,S2,'-');
  hold on
  p2=plot(w2,E2,'--');
  p3=plot(w2,B,'-.');
  hold off
  axis([0 4*44.1 0 1])  
%  set([p1 p2],'LineWidth',2)
  %legend([p1 p2 p3],'Uppsamplad signal','Uppsamplat kvant.brus','Digitalt LP filter')

  subplot(313)
  dspbook('plotfix')
  S3=B'.*S2;
  E3=B'.*E2;
%  S3=[S3 S3(end:-1:1)];
%  E3=[E3 E3(end:-1:1)];
  S3=[S3 S3 S3 S3];
  E3=[E3 E3 E3 E3];
%  N=length(S3);
  w3=4*44.1*(0:1/N:4-1/N);
  
  [b,a]=butter(4,0.25);
  H=abs(freqz(b,a,N,4*44.1));

  p1=plot(w3,S3,'-');
  hold on
  p2=plot(w3,E3,'--');
  p3=plot(w2,H,'-.');
  hold off
  axis([0 600 0 1])  
  xlabel('Frekvens [Hz]')
%  set([p1 p2],'LineWidth',2)
  %legend([p1 p2 p3],'Analog signal','Analogt kvant.brus','Analogt LP filter')
  
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/CD0.eps,  end

  
  N=20;
  M=8;
  q=2;

  t=(1:N);
  y=sin(t);
  t1=(1:N*M)/M;
  y1=sin(t1);
  ya=round(q*y)/q;
  figure(2)
  subplot(311)
  dspbook('plotfix')
  p=plot(t1,y1,'-',t,y,'o',t,ya,'x');
%  set([p],'LineWidth',2)
  %legend('Analog signal','Samplad signal','Kvantiserade sampel')


  yac=zeros(1,N*M);
  yac(M:M:N*M)=ya;
  subplot(312)
  dspbook('plotfix')
  p1=plot(t1,y1); %,':',t1,yac,'-')
  hold on
  p2=stem(t1,yac);
%  set([p1;p2],'LineWidth',2)
  %legend([p1 p2(1)],'Verklig signal','versamplad signal')

  [b,a]=butter(5,.5/M);
  yacf=M*filtfilt(b,a,yac);
  subplot(313)
  dspbook('plotfix')
  p=plot(t1,y1,'--',t1,yacf,'-');
%  set([p],'LineWidth',2)
  axis([0 N -2 2])
  %legend('Verklig signal','Analog rekonstruktion')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/CD1.eps,  end

  % Alternativ: good D/A and no oversampling
  figure(3)
  dspbook('plotfix')
  q=8;
  M=8;
  t12=(1:N*M)/M;
  ya2=round(q*y)/q;
  ya2c=zeros(1,N*M);
  ya2c(M:M:N*M)=ya2;
  [b,a]=butter(5,.5/M);
  ya2cf=M*filtfilt(b,a,ya2c);
  p=plot(t12,ya2cf,'--',t1,yacf,'-');
%  set([p],'LineWidth',2)
  axis([0 N -2 2])
  legend('17 kvantiseringsniver','  5 kvantiseringsniver')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/CD2.eps,  end
end


if strcmp(ex,'all') | strcmp(ex,'ex10.4') | strcmp(ex,'GPS')
  % GPS
  close all
  figure(1)
  dspbook('plotfix')
  phi=0:0.1:2*pi+0.1;
  R=1;
  xp=[0 1 2];
  yp=[1 0 1];
  x=R*cos(phi);
  y=R*sin(phi);
  r=1;
  x1=x+r*xp(1);
  y1=y+r*yp(1);
  x2=x+r*xp(2);
  y2=y+r*yp(2);
  x3=x+r*xp(3);
  y3=y+r*yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  set(p,'Linewidth',2)
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',2)
  hold off  
  for k=1:3
    h=text(xp(k)+0.1,yp(k),['S',num2str(k)]);
    %set(h,'FontSize',fontsize)
  end
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/gps1.eps,  end

  set(p,'Linewidth',10)
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/gps4.eps,  end

  figure(2)
  dspbook('plotfix')
  r=0.9;
  x1=r*x+xp(1);
  y1=r*y+yp(1);
  x2=r*x+xp(2);
  y2=r*y+yp(2);
  x3=r*x+xp(3);
  y3=r*y+yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  set(p,'Linewidth',3)
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',3)
  hold off  
  for k=1:3
    h=text(xp(k)+0.1,yp(k),['S',num2str(k)]);
    %set(h,'FontSize',fontsize)
  end
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/gps2.eps,  end

  figure(3)
  dspbook('plotfix')
  r=1.1;
  x1=r*x+xp(1);
  y1=r*y+yp(1);
  x2=r*x+xp(2);
  y2=r*y+yp(2);
  x3=r*x+xp(3);
  y3=r*y+yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',3)
  hold off  
  for k=1:3
    h=text(xp(k)+0.1,yp(k),['S',num2str(k)]);
    %set(h,'FontSize',fontsize)
  end
  set(p,'Linewidth',3)
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/digsigfig/gps3.eps,  end
end

if strcmp(ex,'all') | strcmp(ex,'OS')
  % GPS
  %close all
  figure(1)
  dspbook('plotfix')
  phi=0:0.1:2*pi+0.1;
  R=1;
  xp=[0 1 2 3 4];
  yp=[1 0 1 0 1];
  x=R*cos(phi);
  y=R*sin(phi);
  x1=x+xp(1);
  y1=y+yp(1);
  x2=x+xp(2);
  y2=y+yp(2);
  x3=x+xp(3);
  y3=y+yp(3);
  x4=x+xp(4);
  y4=y+yp(4);
  x5=x+xp(5);
  y5=y+yp(5);
  p=plot(x1,y1,x2,y2,x3,y3,x4,y4,x5,y5);
  set(p,'Linewidth',10)
  axis([-1.2 5.2 -0.2 2.2])
  axis('equal')
  axis('off')
end

if strcmp(ex,'all') | strcmp(ex,'rfi')
%  close all
  figure(1)
  clf
  dspbook('plotfix')
  s=0.001:0.001:1;
  A=[1 0.5 0.2]';
  B=1.5;
  C=10;
  D=1;
  mu=A*sin(B*atan(C*sin(D*s)));
  p=plot(s,mu,'-k');
%  set(p,'LineWidth',2)
  xlabel('Hjulglidning s')
  ylabel('Normaliserad drivkraft \mu')
  h(1)=text(0.8,0.8*A(1)+0.06,'Asfalt');
  h(2)=text(0.8,0.8*A(2)+0.06,'Grus');
  h(3)=text(0.8,0.8*A(3)+0.08,'Sn');
  %set(h,'FontSize',fontsize)
  
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/slipcurve.eps
  end

  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  

  Phi=[ones(size(mu)) mu];
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);

  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);

  figure(2)
  subplot(211)
  dspbook('plotfix')
  plot(s)
  set(gca,'Xlim',[0 N])
  ylabel('Hjulglidning s')
  subplot(212)
  dspbook('plotfix')
  plot(mu)
  set(gca,'Xlim',[0 N])
  ylabel('Normaliserad drivkraft \mu')
  xlabel('Tid [sampel]')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi1.eps
  end


  figure(3)
  dspbook('plotfix')
  h1=plot(s(1:200),mu(1:200),'x');
  hold on
  h2=plot(s(201:N),mu(201:N),'o');
  a=axis;
  mumax=a(4);
  h3=plot([threc(1,200) threc(1,200)+threc(2,200)*mumax],[0 mumax]);
  h4=plot([threc(1,N) threc(1,N)+threc(2,N)*mumax],[0 mumax],'--');
  hold off
  ylabel('Normaliserad drivkraft \mu')
  xlabel('Hjulglidning s')
  legend([h1 h2 h3 h4],...
	 'Mtningar fre frndring',...
	 'Mtningar efter frndring',...
	 'Friktionsmodell fre frndring',...
	 'Friktionsmodell efter frndring',2);
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi2.eps
  end

  figure(4)
  dspbook('plotfix')
  h1=plot(threc');
  set(h1,'LineWidth',1);
  hold on
  h2=plot(threc2');
  set(h2,'LineWidth',2);
  h3=plot(th0','--');
  axis([0 N 0 0.05])
  hold off
  xlabel('Tid [sampel]')
  legend([h3(1) h1(1) h2(1)],'Sanna parametrar',...
	 'Skattade parametrar, snabbt filter',...
	 'Skattade parametrar, lngsamt filter',2)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi3.eps
  end

  figure(5)
  dspbook('plotfix')
  [epsi1,epsif1]=simresid([s Phi],-2,threc);
  [epsi2,epsif1]=simresid([s Phi],-2,threc2);
  plot(abs([epsi1./(1e-6+epsi2)]))
  axis([0 313 0 2])
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi4.eps
  end
  
  h=3; nu=0.2;
  [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[1 h nu]);

  figure(6)
  dspbook('plotfix')
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
%  set(h1,'LineWidth',1);
%  set(h2,'LineWidth',2);
  hold off
  xlabel('Tid [sampel]')
  legend([h1(1) h2(1)],...
	 'Skattade parametrar',...
	 'Sanna parametrar',2) 

  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi5.eps
  end
  
  figure(7)
  dspbook('plotfix')
  h2=plot(gt,'b-');
  axis([0 N 0 h*1.5])
%  set(h2,'LineWidth',1)
  hold on, plot([1 N],[h h]), hold off
  xlabel('Tid [sampel]')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi6.eps
  end
 
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
  
  figure(8)
  dspbook('plotfix')
  Phi=[ones(size(mu)) mu];
  th=Phi(1:200,:)\s(1:200)
  e=s(1:200)-Phi(1:200,:)*th;
  e=sort(e);
  e=e(3:length(e)-6);
  hist(e,30);
  sigma=std(e)
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi7.eps
  end


  h=1; nu=0.02;
  if saveplots==1, MC=50; else, MC=10; end
  TA=[];
  GT=[];
  THREC=zeros(2,N);
  for k=1:MC;
    s(1:200)=1/40*mu(1:200);
    s(201:N)=1/25*mu(201:N);
    s=s+sigma*randn(N,1)+0.0053;
    [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[2 h nu],-1);
    TA=[TA ta];
    GT=[GT gt];
    THREC=THREC+threc;
  end
  TA
  figure(20), plot(GT)
  threc=THREC/MC;

  figure(9)
  subplot(211)
  dspbook('plotfix')
  hist(TA,[200:220])
  subplot(212)
  dspbook('plotfix')
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  xlabel('Tid [sampel]')
  axis([0 N 0 0.05])
%  set(h1,'LineWidth',1);
%  set(h2,'LineWidth',2);
  hold off
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi8.eps
  end

  figure(10)
  dspbook('plotfix')
  plot(GT)

  figure(11)
  dspbook('plotfix') 
  load ceram
  k=ceram7(:,2);
  plot(.2*[1:length(k)],k)
  axis([50 80 10 70])
  xlabel('Tid [s]')
  ylabel('Glidningslutning')
  if saveplots==1;
    print -deps /home/rt/fredrik/digsigfig/rfi9.eps
  end
end


if strcmp(ex,'all') | strcmp(ex,'exA.3')
% BS2 Bootstrap for computing the uncertainty when
% estimating an AR(1) model.
% See Signal processing magazine No 1 1998, Zoubir and Boashash

close all
randn('state',0)
n=1000;
N=10;
mu=0;
sigma=1;
a0=0.7;    % AR parameter
sigma0=sqrt((1-a0^2)/N);  % theoretical variance

MC=20;
MCtot=[];
e0=mu+sigma*0.5*sum(randn(4,N)); % extra Gaussian!
y0=filter(1,[1 a0],e0)';
ahat0=-y0(1:N-1)\y0(2:N);   % ML estimate
% ahat0,a0

for j=1:MC;
  e0=mu+sigma*0.5*sum(randn(4,N)); % extra Gaussian!
  y0=filter(1,[1 a0],e0)';
  ahat=-y0(1:N-1)\y0(2:N);   % ML estimate
  sigmahat=sqrt((1-ahat^2)/N);  % theoretical variance
  % MC
  for k=1:n;
    e=mu+sigma*0.5*sum(randn(4,N));
    y=filter(1,[1 a0],e)';
    ahatMC(k)=-y(1:N-1)\y(2:N);   % ML estimate
    sigmahatMC(k)=(1-ahatMC(k)^2)/N;  % theoretical variance
  end
  % Bootstrap
  for k=1:n;
    ehat=filter([1 ahat0],1,y0)';
    ind=ceil(N*rand(1,N));
    e=ehat(ind);   % resampling
    y=filter(1,[1 ahat0],e)';
    ahatBS(k)=-y(1:N-1)\y(2:N);   % ML estimate
    sigmahatBS(k)=(1-ahatBS(k)^2)/N;  % theoretical variance
  end

  MCtot=[MCtot;sigmahat std(ahatMC) std(ahatBS)];

end
figure(1);
dspbook('plotfix')
[Nhist,Xhist]=hist(ahatMC,30);
bar(Xhist,Nhist,'w')
a=axis;
title('Skattat \theta mha. MC')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/BS2a.eps, end
figure(2);
dspbook('plotfix')
[Nhist,Xhist]=hist(ahatBS,30);
bar(Xhist,Nhist,'w')
axis(a)
title('Skattat \theta mha. BS')
if saveplots==1, print -deps /home/rt/fredrik/digsigfig/BS2b.eps, end

disp('Result from one experiment')
disp(' ')
disp('Statistics           Point estimate    Monte Carlo     Bootstrap     Theoretical')
disp(['Mean of ahat ','         ',num2str(mean(ahat)),'         ',num2str(mean(ahatMC)),'         ',num2str(mean(ahatBS)),'       ',num2str(a0)])
disp(['Std  of ahat ','         ',num2str(sigmahat),'         ',num2str(std(ahatMC)),'         ',num2str(std(ahatBS)),'       ',num2str(sigma0)])
disp(['Mean of sigmahat(ahat) ','',num2str((sigmahat)),'         ',num2str(mean(sigmahatMC)),'         ',num2str(mean(sigmahatBS)),'       ',num2str(sigma0)])

disp(' ')
disp(['Result from ',num2str(MC),' experiments'])
disp(['Mean of Std   ','         ',num2str(mean(MCtot))]);
end



if strcmp(ex,'all')
  t1=etime(clock,t000);
  clc
  disp(['Elapsed time: ',num2str(t1)])
end
