function [xhat,dhat] = ems(y)
%[xhat,deltahat] = ems(y)
%
% Expectation Maximization Segmentation
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  $

% Global simulation parameters

global N nx ny F0 F1 H Q0 Q1 R x1 P1 q

RR = kron(speye(N,N),R);
HH = kron(speye(N,N),H);

% Initial segmentation
dhat = zeros(1,N-1); 

ep = 1e-4; % Tolerance for terminating the recursion
Vold = -inf;
V = -10e7;
while abs(V-Vold) > ep

  %%%%%%%%%%%%%%%
  % Expectation %
  %%%%%%%%%%%%%%%
  
  % Given dhat, compute Psqrt -- prior covariance
  
  I0 = sparse(1:N-1,1:N-1,1-dhat);
  I1 = sparse(1:N-1,1:N-1,dhat);
    
  % Covariance for kalman smoother
  FF = sparse(N*nx,N*nx);
  FF(1:nx,1:nx) = speye(nx);
  for k=1:N-1
    FF([1:nx]+k*nx,1:nx) = (1-dhat(k)).*F0^k + dhat(k).*F1^k;
  end
  for k=1:N-1
    FF(k*nx+1:N*nx,1+nx*k:nx+nx*k) = ...
	FF(1+nx*(k-1):(N-1)*nx,1+nx*(k-1):nx*k);
  end
  GG = [P1 sparse(nx,nx*(N-1)); sparse(nx*(N-1),nx)...
	kron(I0(1:N-1,1:N-1),Q0)+kron(I1(1:N-1,1:N-1),Q1)];
  % Predicted covaraiance 
  Psqrt = FF*GG;
  
  % Smooth using a sparse QR-array algorithm
  A = [RR HH*Psqrt; sparse(N*nx,N*ny) Psqrt];
  B = qr(A')';
  Reinv = B(1:N*ny,1:N*ny) \ eye(N*ny);
  Kf = B(N*ny+1:end,1:N*ny) * Reinv;
  xhat = full((eye(nx*N,nx*N)-Kf*HH)*FF*[x1; sparse(nx*(N-1),1)] + Kf*y(:));
  Psqrt = B(N*ny+1:end,N*ny+1:end);
    
% $$$   figure(2)
%  dp = diag(Psqrt*Psqrt');
%  plot([x(:) xhat(:) xhat(:)+3*dp(:) xhat(:)-3*dp(:)])
% $$$   xh = reshape(xhat,nx,N);
% $$$   plot(xh(1,:),xh(2,:))
% $$$   drawnow
% $$$     
  %%%%%%%%%%%%%%%%
  % Maximization %
  %%%%%%%%%%%%%%%%  
  QQ0 = Q0*Q0';
  QQ1 = Q1*Q1';
  Jx = zeros(N-1,2);
  for k=1:N-1
    Pk = Psqrt(nx*(k-1)+1:nx*k,nx*(k-1)+1:nx*k)*...
	Psqrt(nx*(k-1)+1:nx*k,nx*(k-1)+1:nx*k)';
    Pl = Psqrt(nx*k+1:nx*(k+1),nx*k+1:nx*(k+1))*...
	Psqrt(nx*k+1:nx*(k+1),nx*k+1:nx*(k+1))';
    Pkl = Psqrt(nx*(k-1)+1:nx*k,nx*(k-1)+1:nx*k)*...
	Psqrt(nx*k+1:nx*(k+1),nx*k+1:nx*(k+1))';
    xk = xhat(nx*(k-1)+1:nx*k);
    xl = xhat(nx*k+1:nx*(k+1));
    % delta_k = 0
    xx = Q0\(xl - F0*xk);
    Jx(k,1) = - xx'*xx - trace(inv(QQ0)*(Pl + F0*Pk*F0' - 2*F0*Pkl)) - log(det(QQ0));
    
    % delta_k = 1
    xx = Q1\(xl - F1*xk);
    Jx(k,2) = - xx'*xx - trace(inv(QQ1)*(Pl +F1*Pk*F1'- 2*F1*Pkl)) - log(det(QQ1));    
  end
    
  J = [ones(N-1,1)*(2*log(q)) ones(N-1,1)*(2*log(1-q))];
  J = J + Jx;

% $$$   Jxd = diff(Jx,1,2);
% $$$   vm = sum(Jx(:,1));
% $$$   n = 0;
% $$$   dhat = zeros(1,N-1);
% $$$   while max(Jxd) + 2*log((1-q)/q) > 0
% $$$     %2*( + log(lam^n/prod(1:n))...
% $$$     %	- log(lam^(n-1)/prod(1:(n-1))))
% $$$     [Jm,im] = max(Jxd);
% $$$     dhat(im) = 1;
% $$$     vm = vm + Jxd(im);
% $$$     Jxd(im) = NaN;
% $$$     n = n + 1;
% $$$   end

% $$$   vm = vm + 2*n*log((1-q)/q); %log(lam^n/prod(1:n));
%keyboard
  [vm,iv] = max(J,[],2);
  iiv = find(iv == 2);
  dhat = zeros(1,N-1);
  dhat(iiv) = 1;
  Vold = V;
  V = sum(vm);
  disp(['Return:',num2str(V)])

  figure(3)
  plot(J,'*');
  legend('\delta = 0','\delta = 1');
  title('Return Q(\delta^N,\delta_p^N)');
  drawnow
  
%  pause
end
%dp = diag(Psqrt*Psqrt');
