function eqstatic
% Demo of blind equalization of static non-linearity
% Assumption: input is Gaussian with zero mean
% Method: adaptation to get a kurtosis of 3
% Kurtosis measures the size of the tail of a distribution,
%      k(u) = E(u^4)/E(u^2)^2
% A Gaussian distribution has k=3, heavier tails means k>3
% True system: y=u^th
% LMS like algorithm for estimation of th that
% uses uhat=y.^(1/thhat).
% Algorithm: 
%   thhat(i+1)=thhat(i) - mu * (k(uhat(i)) - 3)
% for batch i.

close all
th0=3;
sigma=.3;
N=2000;
u=sigma*randn(N,1);
y=abs(u).^th0.*sign(u);
blindeq(u,y)
pause
% Same thing, with additive noise
y=abs(u).^th0.*sign(u)+0.01*randn(N,1);
blindeq(u,y)

function blindeq(u,y);
N=length(y);
L=100; % batch size
thhat=ones(N+1,1);
mu=0.1;
for i=1:N/L;
	yi=y(i*L-L+1:i*L);
	ui=u(i*L-L+1:i*L);
	uhat=abs(yi).^(1/thhat(i)).*sign(yi);
	k=mean(uhat.^4)/mean(uhat.^2)^2;
	thhat(i+1)=thhat(i) + mu * (k - 3);
	plot(ui,uhat,'b.',ui,yi,'r.')
	sigma=0.3;th0=3;
	axis([-3*sigma 3*sigma  -(3*sigma)^th0 (3*sigma)^th0])  
	xlabel('u')
	ylabel('uhat and y')
	title(['True power ',num2str(th0),...
			' estimated by ',num2str(thhat(i)),...
			' at batch ',num2str(i)])
	pause(1)
end