function [xp,xf,Alfa]=imm(z,nnn,wbar,q,dsp)

% Implements the IMM algorithm by Li and Bar-Shalom
%
%   [xp,xf,Alfa]=imm(z,nnn,wbar,q)
%
% z         The output-input data vector z = [y u] or z = [y Phi].
% nnn       Model structure (should be nnnplane)
% dsp (0/1) Display some information (default 0)
% xp        State prediction xhat(t+1|t)
% xf        State filter xhat(t|t)
% wbar      Possible turn rates (default wbar=[0 -3 3])
% q         Transition probabilites (default 0.5*eye(3)+0.5/3*ones(3))
% Alfa      Probability of the elements in wbar before mixing

% The mixing step implemented by Alf Isaksson and Niclas Bergman
% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

global NNNmethod
if NNNmethod~=5;
  disp('IMM only defined for NNNmethod=5, changing to this')
  NNNmethod=5;
end

mtype=modeltype(nnn);
if mtype>4;
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nu=n(2);
  nv=n(3);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  tstart=1;
  xhat=x0;
else
  error('IMM works only for state space models')
end;
[N,nz]=size(z);

if nargin<5 dsp=0; end;
if nargin<4 q =[]; end; 
if nargin<3 wbar=[]; end; 
if isempty(wbar); wbar=[0 -3 3]; end
M=length(wbar);
if isempty(q) q=0.5*eye(M)+0.5/M*ones(M); end

lambda=1; % inverse adaptation gain

if dsp==1; clc; end;
alfa=[1 zeros(1,M-1)];
mubar=alfa;
xhat=zeros(d,M);
epsi=zeros(ny,M);
xf=zeros(d,N);
xp=zeros(d,N);
Alfa=zeros(N,M);
P=[]; for i=1:M,P=[P P0];end;

for t=tstart:N; 
    for j=1:M,
      xhatj=xhat(:,j);

      Pj=P(:,d*(j-1)+1:d*(j-1)+d);

      % --------------Time update--------------------
      wj=xhatj(5);
      xhatj(5)=xhatj(5)+wbar(j);   %cf Li and Bar-Shalom (1993)
      [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhatj,0,5);  %A=A(w)
      xhatj=A*xhatj;
      Pj=A*Pj*A'+Bv*Q*Bv';
      xhatj(5)=wj;

      % --------------Measurement update--------------------
      xbar= xhat*alfa'; %R=R(xbar)
      [C,Du,Dth,R]=feval(nnn,t,xbar,0,6);
      den=inv(lambda*R+C*Pj*C');
      epsij=y(t,:)'-C*xhatj;
      if ~isempty(u) & ~isempty(Du);
        epsij=epsij-Du*u(t,:)';
      end
      K=Pj*C'*den;
      xhatj=xhatj+K*epsij;  % Filtered output
      if ~isempty(u) & ~isempty(Bu);
        xhatj=xhatj + Bu*u(t,:)';
      end
      Pj=Pj-K*C*Pj;
      Pj=.5*(Pj+Pj');
      epsi(:,j)=epsij;
      alfa(j)=mubar(j)*sqrt(det(den))*exp(-1/2*epsij'*den*epsij);
      xhat(:,j)=xhatj;
      P(:,d*(j-1)+1:d*(j-1)+d)=Pj;
    end
    alfa=1/sum(alfa)*alfa;
    Alfa(t,:)=alfa;
    %-------------Mixing-----------------------
    mubar=alfa*q;
    muij= ((alfa'*ones(1,M))./(ones(M,1)*mubar)).*q;
    x0hat= xhat*muij; % Mixed state vectors

    for j=1:M;
      P0j= P*kron(muij(:,j),eye(5,5));
      Xj=zeros(5,5);
      for i=1:M, % Spread of the mean
        Xj=Xj+ (xhat(:,i)-x0hat(:,j))*(xhat(:,i)-x0hat(:,j))'*muij(i,j);
      end
      P00(:,d*(j-1)+1:d*(j-1)+d)=P0j+Xj;
    end
    xhat=x0hat;
    P=P00;
    xp(:,t)=mean(x0hat')';
%    xf(:,t)=xhat*alfa;
end


