function kfplot(xhat,P,x,opt,xind,tind)
  
% Plot various illustrations of Kalman filter estimates
% kfplot(xhat,P,x)  
%
% opt=1  Plot time trajetory of state estimate with confidence levels in one plot
% opt=2  Plot time trajetory of state estimate with confidence levels in subplots
% opt=3  As 2, but without legend
% opt=4  Plot 2D state plots of x(tind,xind) with confidence ellipsoids
% With opt=-k, the opt=k plot appears in the current figure/axis
% otherwise a new figure window figure(k) is created.
  
if nargin<4; opt=1; end
  
if ~isempty(find(abs(opt)==1));
  if opt>0,figure(1),end
  nx=size(xhat,1);
  book('plotfix')
  hxhat=plot(xhat','-b');
  hold on
  for i=1:nx
    hP=plot(xhat(i,:)'+sqrt(P(i:nx:end,i)),'--');
    hP=plot(xhat(i,:)'-sqrt(P(i:nx:end,i)),'--');
  end
  if nargin>2 & ~isempty(x)
    hx=plot(x','-r');
    legend([hxhat(1) hP(1) hx(1)],'xhat(t)','xhat^i(t) +/- P^{i,i}(t)','x(t)',2)
  else
    legend([hxhat(1) hP(1) hx(1)],'xhat(t)','xhat^i(t) +/- P^{i,i}(t)',2)
  end
  hold off
  set(hxhat,'LineWidth',2)
end
if ~isempty(find(abs(opt)==2));
  if opt>0,figure(2),end
  nx=size(xhat,1);
  for i=1:nx
    subplot(nx,1,i)
    book('plotfix')
    hxhat=plot(xhat(i,:),'-b');
    hold on
    hP=plot(xhat(i,:)'+sqrt(P(i:nx:end,i)),'--');
    hP=plot(xhat(i,:)'-sqrt(P(i:nx:end,i)),'--');
    if nargin>2 & ~isempty(x)
      hx=plot(x(i,:),'-r');
    end
    set(hxhat,'LineWidth',2)
  if nargin>2 & ~isempty(x)
    legend([hxhat(1) hP(1) hx(1)],'xhat(t)','xhat^i(t) +/- P^{i,i}(t)','x(t)',2)
  else
    legend([hxhat(1) hP(1) hx(1)],'xhat(t)','xhat^i(t) +/- P^{i,i}(t)',2)
  end
  hold off
  end
end

if ~isempty(find(abs(opt)==3));
  nx=size(xhat,1);
  if opt>0,figure(3),end
  for i=1:nx
    subplot(nx,1,i)
    book('plotfix')
    hxhat=plot(xhat(i,:),'-b');
    hold on
    hP=plot(xhat(i,:)'+sqrt(P(i:nx:end,i)),'--');
    hP=plot(xhat(i,:)'-sqrt(P(i:nx:end,i)),'--');
    if nargin>2 & ~isempty(x)
      hx=plot(x(i,:),'-r');
    end
    set(hxhat,'LineWidth',2)
    hold off
  end
end

if ~isempty(find(abs(opt)==4));
  if opt>0,figure(4),end
  book('plotfix')
  if nargin<5
    tind=length(xhat);
    xind=1:2;
  end
  nx=size(xhat,1);
  theta=(0:0.01:2*pi);
  v=[cos(theta);sin(theta)];
  for i=1:length(tind)
    p=plot(xhat(xind(1),tind(i)),xhat(xind(2),tind(i)),'x');
    hold on
    if ~isempty(x)
      p0=plot(x(xind(1),tind(i)),x(xind(2),tind(i)),'*');    
      set(p0,'MarkerSize',14)
      text(x(xind(1),tind(i)),x(xind(2),tind(i)),['  x(',num2str(tind(i)),')']);
    end
    if size(P,1)>nx % Time-varying
      Pt=P((tind(i)-1)*nx+1:tind(i)*nx,:);
    else
      Pt=P;
    end
    R=inv(Pt(xind,xind));
    r=sqrt(diag(v'*R*v));
    plot(xhat(xind(1),tind(i))+v(1,:)./r',xhat(xind(2),tind(i))+v(2,:)./r','--')
    set(p,'MarkerSize',14)
    text(xhat(xind(1),tind(i)),xhat(xind(2),tind(i)),['  xhat(',num2str(tind(i)),')']);
  end
  hold off
  xlabel(['xhat^',num2str(xind(1)),'(t)'])
  ylabel(['xhat^',num2str(xind(2)),'(t)'])
end

  


  