function [l]=likelihood(y,th,lam,dist);

% LIKELIHOOD computes the likelihood for data
%
% [l]=likelihood(y,th,lam,dist);
%
% Unspecified parameter (e.g. th=[]) treated as nuissance,
% and removed by marginalization (second column of l) or 
% estimation (first column of l)
%
% l       Negative log likelihood (generalized and marginalized)
% th      Location parameter (mean of distribution)
% lam     Scale parameter (variance of distribution)
% dist=1  Gaussian noise
% dist=2  Exponentially distributed noise, lambda not specified

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<4, dist=1; end
if nargin<3, lam=[]; end
if nargin<2, th=[]; end

y=y(:);
N=size(y,1);
thhat=mean(y);
lamhat=(N-1)/N*std(y);
lamhat=1/N*sum((y-thhat).^2);

if dist==1;
  if isempty(th) & ~isempty(lam)
    l(1)=N*log(2*pi*lam) + N*lamhat/lam;
    l(2)=(N-1)*log(2*pi*lam) + log(N) + N*lamhat/lam;
  elseif isempty(th) & isempty(lam)
    l(1)=N*log(2*pi)+N + N* log(lamhat);
    l(2)=N*log(2*pi)+ (N-5) + log(N) - (N-3)*log(N-5) + (N-3)*log(N*lamhat);
  elseif ~isempty(th) & isempty(lam)
    l(1)=N*log(2*pi)+N + log(lamhat);
    l(2)=N*log(2*pi)+ (N-4) - (N-2)*log(N-4) + (N-2)*log(N*lamhat);
  end
end

if dist==2;
  if isempty(th) 
    l(1)=N*(1+log(max(thhat,1e-4)));   % thhat>0 requirement
    l(2)=(N+1)*(log(N)+log(max(thhat,1e-4)))-stirling(N);   % New
  else
    l=[0 0];
  end
end