function [MDL,v,thm]=mdl(z,nn,method,par,t0)

% Computes the MDL criterion as a function of the designparameters
% for model structure nn estimated by method:
% 'arx',  no design parameter
% 'lms',  par=mu
% 'nlms', par=mu
% 'rls',  par=lam
% 'segm', par=log(q)

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

N=length(z);
y=z(:,1);
if length(nn)==1;
  d=nn;
else
  d=nn(1)+nn(2);
end;

if strcmp(method,'LS')
  [th]=arx(z,nn);
  thm=ones(N,1)*th2par(th);
  epsi=resid(z,th);
  v=1/(N-t0+1)*(sum((epsi(t0:N)).^2));
  MDL=v+d*log(N)/N;
end;

if strcmp(method,'GSM')
  n=N/par;
  v=0;
  thm=[];
  for i=1:n;   % fixed segmentation
    ztmp=z((i-1)*par+1:i*par,:);
    [th]=arx(ztmp,nn);
    [apar,P,lam]=th2par(th);
    [lam,1/par*sum(epsi.^2)];
    thm=[thm;ones(N,1)*th2par(th)];
    epsi=resid(ztmp,th);
    v=v+1/par*sum(epsi.^2);
  end
  v=v/n;
  MDL=v+n*d*log(n*par)/n/par;
end;

if strcmp(method,'Detection LS')
save matlab z nn
%  [jhat,thm]=segm(z,nn,2,10^par);
  [jhat,thseg,gt,ta,thm]=detect1(z,nn,1,[2 5 par]);
  epsi=simresid(z,nn,thseg);
  v=1/(N-t0+1)*(sum((epsi(t0:N)).^2));
  MDL=v+(length(jhat))*(d+1)*log(N)/N;
  thm=thm';
%  figure(10),plot(gt),pause
end;

if strcmp(method,'RLS')
  [thm,lamhat,Epsi]=adfilter(z,nn,par,'RLS'); thm=thm';
  v=1/(N-t0+1)*(sum((Epsi(t0:N,1)).^2));
  deltath=diff([zeros(1,d);thm]);
%  thgain=(1-par)/diag(inv(P))
  thgain=1;
%  MDL=v-d*log(1-par);
  MDL=v;
  for i=1:d;
    MDL=MDL+1/(N-t0+1)*thgain*sum(deltath(t0:N,i).^2);
  end;
end;

if strcmp(method,'LMS')
  [thm,lamhat,Epsi]=adfilter(z,nn,par,'LMS');  thm=thm';
  v=1/(N-t0+1)*(sum((Epsi(t0:N,1)).^2));
  deltath=diff([zeros(1,d);thm]);
%  thgain=(par)*diag(inv(P));
  thgain=1;
%  MDL=v-d*log(par);
  MDL=v;
  for i=1:d;
    MDL=MDL+1/(N-t0+1)*thgain*sum(deltath(t0:N,i).^2);
  end;
end;

if strcmp(method,'NLMS')
  [thm,lamhat,Epsi]=adfilter(z,nn,par,'NLMS');  thm=thm';
  v=1/(N-t0+1)*(sum((Epsi(t0:N,1)).^2));
  deltath=diff([zeros(1,d);thm]);
  thgain=(par)*diag(inv(P));
  thgain=1;
%  MDL=v-d*log(1-par);
  MDL=v;
  for i=1:d;
    MDL=MDL+1/(N-t0+1)*thgain*sum(deltath(t0:N,i).^2);
  end;
end;


