function [mtype]=modeltype(nnn);

% Determines the type of model implicitely given in nnn
%
% function [mtype]=modeltype(nnn);
%
% mtype =
% 0  No model (only noise)
% 1  Unknown mean in noise
% 2  Linear regression with regressor in z=[y PHI]
% 3  AR model
% 4  ARX model
% 5  State space model
% 6  Filename nnn specifying arbitrary state space model 
%    (see also nnn2ss)
% For mtype<3, mtype=-nnn

if isempty(nnn);
  error('No model provided (nnn==[])!')
end

if isstr(nnn)
  mtype=5;
else
  [nrow,ncol]=size(nnn);

  if nnn(1)<0; % Unknown mean or regression model
    mtype=-nnn(1);
  elseif nnn(1)==0 & ncol==1;  % No model
    mtype=0;
  elseif ncol==1;
    mtype=3;   %AR
  elseif nrow==1;
    mtype=4;   %ARX
  elseif nrow>1 %State space
    mtype=5;
  end;
end;

