function [jhat,xc,xd,yhat,dhat,threc,Alfa,Xn,V]=stiction(z,nnn,lf,q,ss,dsp)

% Detects static friction (stiction) in control valves
% using MAP estimation of change points. 
%
% [JHAT,THSEG,LAMSEG,THREC,ALFA,XN,V]=STICTION(Z,NN,LF,Q,SS,DSP) or
% THREC=STICTION(Z,NNN,LF,Q,SS,DSP)
%
% Modification of detectM.m by Fredrik Gustafsson
% done by the author and  Anders Stenman <stenman@isy.liu.se>
%
% Adaptive filtering and change detection benchmark data
%
% STICTION

% The data model is
%         / u(t),   delta(t)=0
%   x(t)= |
%         \ x(t-1), delta(t)=1 (Stiction mode)
%
% A(q)y(t)=B(q)x(t)+e(t)  (Usual ARX model as nnn)



% Stiction init
if ~all(size(nnn)==[1 3])
  disp('Incorrect number of orders specified!')
  disp('For an ARX model, NN=[na nb nk].')
  error(' ')
end

na=nnn(1);
nb=nnn(2);
nk=nnn(3);
d=na+nb;
ny=1;
[N,nz]=size(z); nu=nz-ny;
x0=zeros(d,1);
P0=1e6*eye(d);
tstart=max(na+1,nb+nk);
R=1;
Q=0;

if nu~=1
  error('Z must be of the form Z=[y u] with scalar signals.')
end

if nargin<6, dsp=0; end
if nargin<5, ss=[]; end 
if nargin<4, q =[]; end 
if nargin<3, lf=[]; end 
if isempty(lf) | lf<0, lf=2; end
if isempty(q) | q<0, q =0.5; end
if isempty(ss) | ss(1)<0, ss=d+8; end
M=ss(1);
if M>1
  if length(ss)>1, ll=ss(2); else ll=[]; end
  if length(ss)>2, minseg=ss(3); else minseg=[]; end
  if isempty(ll), ll=M-4; end
  if isempty(minseg), minseg=d+1; end
  if ll<3, ll=3; disp('Warning: lifelength LL changed to 3'); end
  % Create log(gamma(.)) function.
  for i=1:50
    loggamma(i)=log(gamma(i/2));
  end
  for i=51:N+ll
    loggamma(i)=0.5*log(2*pi)-i/2+(i-1)/2*log(i/2);
  end
elseif M<=1
  jhat=ss(2:length(ss));
  if isempty(jhat), jhat=0; end
  jhat=jhat(:);
  M=1; 
  ll=1;
else
  disp('This does not make sense')
  return
end

if lf(1)==1
  if length(lf)>1
    lambda=lf(2); lf=lf(1); 
  else
    disp(['Warning: a LAMBDA should be specified for loss function' ...
	  ' 1 (LAMBDA=1 now)']) 
    lambda=1;
    lf=lf(1);
  end
else
  lf=lf(1);
  lambda=1; 
end
if lf>1
  minseg=max([minseg,d+4]);
end

if dsp, clc, end
alfa=[0 1e6*ones(1,M-1)];
alfabar=zeros(1,M);
th=x0*ones(1,M);
thf=zeros(d,M);
epsi=zeros(ny,M);
threc=zeros(d,N);
threcf=zeros(d,N);
hist=zeros(N,M);
numseg=ones(1,M);
age=[0 ll*ones(1,M-1)];
P=repmat(P0,1,M);
lamhat=zeros(N,M);
ths=zeros(d*N,M);                    
D=zeros(1,M); 
V=zeros(1,M)+eps;
Vls=zeros(1,M);
Alfa=zeros(N,M);
Xn=ones(N,M);
yhat=zeros(N,M);

yhatt=zeros(N,1);
xct=zeros(N,1);
xdm=zeros(N,M);

% Stiction parameters.
% Assumed model:
%  xc_t = u_t       if xd=0
%  xc_t = xc_{t-1}  if xd=1
xd=[zeros(M-1,1);1]; % Discrete stiction state
xc=zeros(N,M);       % Continuous valve state

ud=diff([0; z(:,2)]);
th0 = th2par(arx(z,nnn))';

for t=tstart:N
  y=z(t,1); u=z(t,2);
  for j=1:M
    % Stiction part
    if xd(j)
      xc(t,j)=xc(t-1,j);
    else
      xc(t,j)=u;
    end
    xdm(t,j)=xd(j);
  
    % regressors
    phi=[-z(t-1:-1:t-na,1); xc(t-nk:-1:t-nk-nb+1,j)];
	
    Pj=P(:,d*(j-1)+1:d*(j-1)+d);
    denj=inv(lambda*R+phi'*Pj*phi);
    epsij=y-phi'*th(:,j);
    K=Pj*phi*denj;
    th(:,j)=th(:,j)+K*epsij;
    %th(:,j) = th0;
    Pdum=Pj-K*phi'*Pj+lambda*Q;
    P(:,d*(j-1)+1:d*(j-1)+d)=.5*(Pdum+Pdum');
    D(j)=D(j)-log(det(denj));
    V(j)=V(j)+epsij'*denj*epsij;
    Vls(j)=Vls(j)+epsij'*epsij;
    epsi(j)=epsij;
    yhat(t,j)=phi'*th(:,j);
  end
  thf=th;
  if M>1
    aind=find((age(1:M)>minseg));  % changed from ll 980910 
    if length(aind)>0,
      switch lf
       case 1
	alfabar(aind)=alfa(aind)+D(aind)+V(aind);
       case 2
	dum=age(aind)-2; 
	alfabar(aind)=alfa(aind)+D(aind)+dum.*log(V(aind))-2*loggamma(dum);
       case 3
	alfabar(aind)=alfa(aind)+age(aind).*log(V(aind)./age(aind)) ...
	    + d*numseg(aind)*log(t); 
       case 4
	alfabar(aind)=alfa(aind)+age(aind).*log(V(aind)./age(aind)) ...
	    + d*numseg(aind)*2;
       otherwise
	error(['Unknown loss function: ' int2str(lf)])
      end
      [dummy,jmin]=min(alfabar(aind));
      jmin=aind(jmin);
      threc(:,t)=th(:,jmin);
      threcf(:,t)=thf(:,jmin);
      yhatt(t)=yhat(t,jmin);
      xct(t)=xc(t,jmin);
      aind2=find((age(1:M)>ll));
      [dummy,jmax]=max(alfabar(aind2));
      Xn(t+1,:)=abs(Xn(t,:));   % number of jumps in each branch
      Xn(t+1,jmin)=-abs(Xn(t+1,jmin));   % most probable branch
      jmax=aind2(jmax);
      %t,jmin,jmax,Xn(t,:)
      if age(jmin)>minseg & age(jmax)>ll
	% Jump branch
	Xn(t,jmax)=abs(Xn(t,jmin));  % deleted branch
	Xn(t+1,jmax)=abs(Xn(t+1,jmin))+1;  % deleted branch
	
	% Stiction assumption: same model and covariance
	P(:,d*(jmax-1)+1:d*(jmax-1)+d)=P(:,d*(jmin-1)+1:d*(jmin-1)+d);
	th(:,jmax)=th(:,jmin);
	xd(jmax)=~xd(jmin);    % Switch discrete state
	xc(:,jmax)=xc(:,jmin);
	yhat(:,jmax)=yhat(:,jmin);
	xdm(:,jmax)=xdm(:,jmin);
	
	%if abs(ud(t)) < 0.1
	%  q = 0.5;
	%else
	%  q = 0.1;
	%end
	
	if lf<3
	  alfa(jmax)=alfabar(jmin)-2*log(q)+2*log(1-q); 
	else
	  alfa(jmax)=alfa(jmin);
	end
	D(jmax)=0; V(jmax)=0; Vls(jmax)=Vls(jmin);
	ths(:,jmax)=ths(:,jmin);      
	age(jmax)=0;
	hist(:,jmax)=hist(:,jmin);
	hist(t,jmax)=1;
	numseg(jmax)=numseg(jmax)+1;
	ind=length(find(hist(:,jmax)>0));
	ths(ind*d-d+1:ind*d,jmax)=th(:,jmin);
	lamhat(:,jmax)=lamhat(:,jmin);
	if lf==2
	  lamhat(ind,jmax)=V(jmin)/(age(jmin)-4); 
	else
	  lamhat(ind,jmax)=V(jmin)/(age(jmin)+1); 
	end
      end
    end
    age=age+ones(1,M);
    alfabar=alfabar-min(alfabar);
    Alfa(t,:)=alfabar;
    if dsp
      clc,t,age,aind,alfabar,jmax,jmin,th,epsi,pause
    end
  elseif M==1
    threc(:,t)=th;
    threcf(:,t)=thf;
    if ~isempty(find(t==jhat))  % jump time
      P=P0;
      th=zeros(d,1);            
    end
  end
end

if M>1
  ind=length(find(hist(:,jmin)==1));
  ths(ind*d+1:ind*d+d,jmin)=th(:,jmin);
  deltahat=hist(:,jmin);
  ind=find(deltahat==1);
  dum=length(ind);
  %No jumps in the end 
  if dum>1, if ind(dum)>N-minseg, ind=ind(1:dum-1); end, end  
  if dum==1, if ind(dum)>N-minseg, ind=[]; end, end 
  jhat=ind;
  Xn=Xn(2:N+1,:);
elseif M==1
  jmin=1;
  Xn=[];
end

jhatx=[0; jhat; N];

if lf==2
  lamhat=[lamhat(1:length(jhat),jmin); V(jmin)/(age(jmin)-4)];
else
  lamhat=[lamhat(1:length(jhat),jmin); V(jmin)/(age(jmin)+1)];
end

xc=xc(:,jmin); % Estimated valve state
yhat=yhat(:,jmin); % Estimated output
dhat=xdm(:,jmin); % Estimated stiction state

V=Vls(jmin);
thseg=[];
lamseg=[];
for i=1:length(jhatx)-1
  thseg=[thseg ths(i*d-d+1:i*d,jmin)*ones(1,jhatx(i+1)-jhatx(i))];
  lamseg=[lamseg lamhat(i)*ones(1,jhatx(i+1)-jhatx(i))];
end

% threc=threcf;  % Output filtered state for consistency
% Output predictions

if nargout==1
  jhat=threc;
end
