function [out1,out2,out3,out4,out5,out6]=nnnDCm2(t,x,uinp,flag,option);

% Sampled state space model of Dc motor with x=[phi w]'
%
%               1
%      G(s) = ------
%             s(s+1)
%
% Syntax of nnn model objects (see also S-functions in Simulink)
%   [n,x0,P0]=nnn(0,0,0,0);
%   xp=nnn(t,x,f,2);
%   y =nnn(t,x,f,3);
%   [A,Bu,Bv,Bth,Q]=nnn(t,x,f,5);
%   [C,Du,Dth,R]=nnn(t,x,f,6);
%
% Version 2 of DC motor
% Faults: actuator or angular sensor offset
%
% nnnDCm0 assumes angular measurement and state/measurement noise
% nnnDCm1 and nnnDCm2 assume angular and velocity measurements,
% disturbance but no noise

T=0.4;
nx=2; nu=1; nv=0; nd=1; nth=2; ny=2;

out1=[]; out2=[]; out3=[]; out4=[]; out5=[]; out6=[];

if flag==0;    % Initialization
    P0=zeros(nx);
    x0=zeros(nx,1);
    n=[nx nu nv nd nth ny];
    out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
    [A,Bu,Bv,Bd,Bf,Q]=feval('nnnDCm2',0,x,0,5);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    xp=A*x+Bu*u+Bd*d+Bf*f;
    out1=xp;
elseif flag==3;
    [C,Du,Df,R]=feval('nnnDCm2',0,x,0,6);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    y=C*x+Du*u+Df*f;
    out1=y;
elseif flag==4;
elseif flag==5;
  a=exp(-T);
  A=[1 1-a;0 a];
  Bu=[T-1+a; 1-a];
  Bd=[0;0];
%  Bd=[0;0];
  Bf=[Bu [0;0] ];
%  Bf=eye(2);
  Q=[];
  out1=A; out2=Bu; out4=Bd; out5=Bf; out6=Q;
elseif flag==6;  % Output equation matrices
  C=eye(2);
  Du=[0;0];
%  Dd=[0;0];
  Df=[0 1;0 0];   %zeros(2);
  R=[];
  out1=C;
  out2=Du;
  out3=Df;
  out4=R;
end







