function [out1,out2,out3,out4,out5,out6]=nnnDCm4(t,x,uinp,flag,option);

% Sampled state space model of Dc motor with x=[phi w]'
%
%               1
%      G(s) = ------
%             s(s+1)
%
% Inputs are uinp=[u;d;f;lam];
% Noise inputs are simulated within the model
% Syntax of nnn model objects (see also S-functions in Simulink)
%   [n,x0,P0]=nnn(0,0,0,0);
%   xp=nnn(t,x,f,2);
%   y =nnn(t,x,f,3);
%   [A,Bu,Bv,Bth,Q]=nnn(t,x,f,5);
%   [C,Du,Dth,R]=nnn(t,x,f,6);
%
% nnnDCm0 assumes angular measurement and state/measurement noise
% nnnDCm1 and nnnDCm2 assume angular and velocity measurements,
% disturbance but no noise
% nnnDCm3 assumes angular and velocity  measurements and
% state/measurement noise
% nnnDCm4 assumes velocity  measurements and
% state/measurement noise
  


T=0.4;
nx=2; nu=1; nv=1; nd=1; nth=2; ny=1;

out1=[]; out2=[]; out3=[]; out4=[]; out5=[]; out6=[];

if flag==0;    % Initialization
    P0=zeros(nx);
    x0=zeros(nx,1);
    n=[nx nu nv nd nth ny];
    out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
    [A,Bu,Bv,Bd,Bf,Q]=feval('nnnDCm4',0,x,0,5);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    xp=A*x+Bu*u+Bd*d+Bf*f;
    out1=xp;
elseif flag==3;
    [C,Du,Df,R]=feval('nnnDCm4',0,x,0,6);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    y=C*x+Du*u+Df*f;
    out1=y;
elseif flag==4;
  t=t+T;
  out1=t;
elseif flag==5;
  a=exp(-T);
  A=[1 1-a;0 a];
  Bu=[T-1+a; 1-a];
  Bd=[0;0];
  Bf=[Bu [0;0]]; %[0 0;1 0]; %eye(2);
  Bv=[T^2/2;T];
  Q=10;
  out1=A; out2=Bu; out3=Bv; out4=Bd; out5=Bf; out6=Q;
elseif flag==6;  % Output equation matrices
  C=[0 1];
  Du=[0];
  Df=[0 1]; %zeros(2);
  R=[1]; %1*eye(2);
  out1=C;
  out2=Du;
  out3=Df;
  out4=R;
end







