function [out1,out2,out3,out4,out5,out6]=nnnaircraft(t,x,uinp,flag,option);

% Sampled state space model of vertical dynamics of an aircraft with
%
% u =  spoiler angle [0.1 deg];
%      forward acceleration [m/s^2]
%      elevator angle [deg]
%
% y = relative altitude [m]
%     forward speed [m/s]
%     pitch angle [deg]
%
% Syntax of nnn model objects (see also S-functions in Simulink)
%   [n,x0,P0]=nnn(0,0,0,0);
%   xp=nnn(t,x,f,2);
%   y =nnn(t,x,f,3);
%   [A,Bu,Bv,Bth,Q]=nnn(t,x,f,5);
%   [C,Du,Dth,R]=nnn(t,x,f,6);
%
% Model augmented with fault states

T=0.1;
nx=5; nu=3; nv=0; nd=1; nth=6; ny=3;
nnn='nnnaircrafta';
out1=[]; out2=[]; out3=[]; out4=[]; out5=[]; out6=[];

if flag==0;    % Initialization
    P0=zeros(nx+nth);
    x0=zeros(nx+nth,1);
    n=[nx+nth nu nv nd nth ny];
    out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
    [A,Bu,Bv,Bd,Bf,Q]=feval(nnn,0,x,0,5);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    xp=A*x+Bu*u+Bd*d+Bf*f;
    out1=xp;
elseif flag==3;
    [C,Du,Df,R]=feval(nnn,0,x,0,6);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    y=C*x+Du*u+Df*f;
    out1=y;
elseif flag==4;
elseif flag==5;
  Ac=[0 0 1.1332 0 -1; 0 -0.0538 -0.1712 0 0.0705; 0 0 0 1 0;...
      0 0.0485 0 -0.8556 -1.013; 0 -0.2909 0 1.0532 -0.06859];
  Bc=[0 0 0;-0.12 1 0;0 0 0; 4.419 0 -1.665; 1.575 0 -0.0732];
  Cc=zeros(ny,nx); Dc=zeros(ny,nu);
  Gssc=ss(Ac,Bc,Cc,Dc);
  Gssd=c2d(Gssc,T);
  A=Gssd.a;
  Bu=Gssd.b;
  Bf=[Bu zeros(nx,nth-nu)];
  Bd=[0;1;0;0;0]; %zeros(nx,nd);
  out1=[A Bf;zeros(nth,nx) eye(nth)]; 
  out2=[Bu;zeros(6,3)];
  out3=[Bd zeros(nx,nth); zeros(nth,1) eye(nth)];
  out4=zeros(nx+nth,1); %Bd;
  out5=zeros(nx+nth,nth); %Bf;
  out6=diag([1 1 0 0 0 0 1]);
elseif flag==6;  % Output equation matrices
  C=[eye(ny) zeros(ny,nx-ny)];
  Du=zeros(ny,nu);
  Df=[zeros(ny,nth-ny) eye(ny)];
  out1=[C Df];
  out2=Du;
  out3=zeros(size(Df));
  out4=10*eye(ny);
end










