function [out1,out2,out3,out4,out5,out6]=nnnlabmotor(t,x,uinp,flag,option);

% State space model of feedback controlled lab motor
% Sampled state space model of controlled DC motor with x=[phi w]'
%
%               1
%      G(s) = ------,  
%             s(s+1)
%
%              0.17s^2 + 0.806s + 1
%      C(s) = ----------------------,  
%             0.625s^2 + 6 s + 0
%
%               1.67
%      F(s) = ----------,  
%             s^2 + 1.67
%
% Only used for filtering
%
%   [A,Bu,Bv,Bth,Q]=nnnlabmotor(t,x,f,5);
%   [C,Du,Dth,R]=nnnlabmotor(t,x,f,6);
%

global A Bu C Du

nx=4; nu=1; nv=1; nd=0; nth=1; ny=1;

out1=[]; out2=[]; out3=[]; out4=[]; out5=[]; out6=[];

if flag==0;    % Initialization
  T=0.01; % 2*pi/4.5;
  G=tf([0 0 140],[1 3.5 0]);         % System (DC-motor)
  C=tf([0.17 0.886 1],[0.625 6 0]);  % controller (PID)
  Gc=feedback(C*G,1);                % closed-loop system
  Gcd=c2d(Gc,T);
  Gcdss=ss(Gcd);
  [A,Bu,C,Du]=ssdata(Gcdss);
  %save labmotor A Bu C Du

  P0=eye(nx); %zeros(nx);
  x0=zeros(nx,1);
  n=[nx nu nv nd nth ny];
  out1=n; out2=x0; out3=P0;
elseif flag==5;
    Q=0.00001;  % faked noise
    Bd=[];
    Bf=Bu;
    out1=A; out2=Bu; out3=Bu; out4=Bd; out5=Bf; out6=Q;
elseif flag==6;  % Output equation matrices
  R=[1];
  Df=Du;
  out1=C;
  out2=Du;
  out3=Df;
  out4=R;
end







