function [out1,out2,out3,out4,out5,out6]=nnnmodel(t,x,uinp,flag,option);

% Model template for state space models
%
% State space model of the form 
% (see page 474 in text book or equation (10.11) in the manual)
%
% xp = A*x + Bu*u + Bv*v + Bd*d + Bth*th
%  y = C*x + Du*u + Dth*th + e
%
% Inputs are uinp=[u;d;f;lam];
% Noise inputs v,e are simulated inside the file allowing
% arbitrary distribution.
% Syntax of nnn model objects (see also S-functions in Simulink)
%   [n,x0,P0]=nnn(0,0,0,0);
%   xp=nnn(t,x,f,2);
%   y =nnn(t,x,f,3);
%   [A,Bu,Bv,Bd,Bth,Q]=nnn(t,x,f,5);
%   [C,Du,Dth,R]=nnn(t,x,f,6);
% Flags 2,3 are for simulation, and 0,5,6 for filtering.


T=1;
nx=2; nu=1; nv=0; nd=1; nth=2; ny=2;

out1=[]; out2=[]; out3=[]; out4=[]; out5=[]; out6=[];

if flag==0;    % Initialization
    P0=zeros(nx);
    x0=zeros(nx,1);
    n=[nx nu nv nd nth ny];
    out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
    [A,Bu,Bv,Bd,Bf,Q]=feval('nnnmodel',0,x,0,5);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    xp=A*x+Bu*u+Bd*d+Bf*f;
    out1=xp;
elseif flag==3;
    [C,Du,Df,R]=feval('nnnmodel',0,x,0,6);
    u=uinp(1:nu);
    d=uinp(nu+1:nu+nd);
    f=uinp(nu+nd+1:nu+nd+nth);
    y=C*x+Du*u+Df*f;
    out1=y;
elseif flag==4;
  t=t+T;
  out1=t;
elseif flag==5;
  a=0.5;
  A=[1 1-a;0 a];
  Bu=[T-1+a; 1-a];
  Bv=[];
  Bd=[0;0];
  Bf=eye(2);
  Q=[];
  out1=A; out2=Bu; out3=Bv; out4=Bd; out5=Bf; out6=Q;
elseif flag==6;  % Output equation matrices
  C=[1 0; 0 0];
  Du=[0;0];
  Df=zeros(2);
  R=[];
  out1=C;
  out2=Du;
  out3=Df;
  out4=R;
end







