function [out1,out2,out3,out4,out5,out6]=nnnplane(t,x,f,flag);

% Sampled state space model with x=[x1 x2 v1 v2 w]'
%
% Syntax one of
%   [n,x0,P0]=nnnplane(0,0,0,0);
%   xp=nnnplane(t,x,f,2);
%   y =nnnplane(t,x,f,3);
%   [A,Bu,Bv,Bd,Bth,Q]=nnnplane(t,x,f,5);
%   [C,Du,Dth,R]=nnnplane(t,x,f,6);
%
% xp=x(t+1)=g(x(t))
% F=d/dx g(x)
% G=[d/dx1(g1dot); ... d/dx5(g1dot)]
% NNNmethod global variable
% 1 Four state linear model 
% 2 Four state linear model with velocity noise parallel to velocity
% 3 Six  state linear model 
% 4 Six  state linear model with velocity noise parallel to velocity
% 5 Five state coordinated turn model, cartesian velocity, lin. disc.
% 6 Five state coordinated turn model, polar velocity, lin. disc.
% 7 Five state coordinated turn model, cartesian velocity, disc. lin. 
% 8 Five state coordinated turn model, polar velocity, disc. lin. 
% Note: 6-8 not yet implemented!

% digsig-version: changed initial position

global NNNmethod
method=NNNmethod;

T=5;
out1=[]; out2=[]; out3=[]; out4=[]; out5=[];


if flag==0;    % Initialization
  if  method>4 ;  % Coordinated turn model
    nx=5;
    P0=1e8*eye(nx); % from 100
    x0=[50000,20000,-200,0,0]';
    n=[nx 0 2 0 1 2];
    if method==5| method==7
      x0=[50000,20000,-200,0,0]';
    else
      x0=[50000,20000,200,pi,0]';
    end
  elseif  method>2;  % 6 state model
    nx=6;
    P0=1e8*eye(nx); % from 100
    x0=[50000,20000,-200,0,0,0]';
    n=[nx 0 2 0 1 2];
  else              % 4 state model
    nx=4;
    P0=1e8*eye(nx); % from 100
    x0=[50000,20000,-200,0]';
    x0=[0,0,0,0]';
    n=[nx 0 2 0 1 2];
  end
  out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
  th=f(1:length(f)-1);
  x1=x(1);
  x2=x(2);
  if  method==5;  
    % Coordinated turn model, cartesian velocity, linearized discretization
    v1=x(3);
    v2=x(4);
    w=x(5);
    %v1,T,v2,th
    if w==0;
      xp=x+[v1*T; v2*T; 0; 0; th];
    else
      xp(1)=x1+v1/w*sin(w*T)-v2/w*(1-cos(w*T));
      xp(2)=x2+v1/w*(1-cos(w*T))+v2/w*sin(w*T);
      xp(3)=v1*cos(w*T)-v2*sin(w*T);
      xp(4)=v1*sin(w*T)+v2*cos(w*T);
      xp(5)=w+th;
    end;
    out1=xp(:);
  else  % linear model
    [A,Bu,Bv,Bd,Bf,Q]=feval(nnn,t,x,f,5);
    out1=A*x;
  end

elseif flag==3;
  lambda=f(length(f));
  y=[x(1);x(2)]+lambda*100*randn(2,1);
  out1=y;
elseif flag==4;
elseif flag==5;
 if method<5;
  if method==1 | method==2;
    A=[0 0 1 0;
       0 0 0 1;
       0 0 0 0;
       0 0 0 0];
  elseif method==3 | method==4;
    A=[0 0 1 0 0 0;
       0 0 0 1 0 0;
       0 0 0 0 1 0;
       0 0 0 0 0 1;
       0 0 0 0 0 0;
       0 0 0 0 0 0];
  end;

  F=eye(size(A))+T*A+T^2/2*A*A;  % F=expm(A*T);
  qw=1;
  if method==1 | method==3;
    Qd=T*qw*eye(2);
  elseif method==2 | method==4;
    theta=atan2(x(4),x(3));
    qv=0.01*qw;   % State noise parallel to v
    Qd=[qv*cos(theta)^2+qw*sin(theta)^2  (qv-qw)*sin(theta)*cos(theta); (qv-qw)*sin(theta)*cos(theta) qv*sin(theta)^2+qw*cos(theta)^2];
  end;
  Qd=[T^3/3*Qd T^2/2*Qd; T^2/2*Qd T*Qd];   % exact sampling
  Bv=eye(4);
  if method==3| method==4;          % Six states
    Bv=[zeros(2,4); eye(4)];        % exact sampling
%     Qd=[zeros(2,6); zeros(4,2) Qd];        % exact sampling
  end;
  out1=F;
  out3=Bv;
  out6=Qd;
 elseif method==5
    v1=x(3);
    v2=x(4);
    w=x(5);
    F=eye(5);
    if w==0;
      F=[1 0 T 0 -v2*T^2/2; 
         0 1 0 T  v1*T^2/2;
         0 0 1 0 -v2*T;
         0 0 0 1 v1*T;
         0 0 0 0 1];
    else;
      F(1,3)=sin(w*T)/w;
      F(1,4)=-1/w*(1-cos(w*T));
      F(1,5)=v1*(T*cos(w*T)/w-sin(w*T)/w^2)-v2*(T*sin(w*T)/w-(1-cos(w*T))/w^2);
      F(2,3)=(1-cos(w*T))/w;
      F(2,4)=sin(w*T)/w;
      F(2,5)=v1*(T*sin(w*T)/w-(1-cos(w*T))/w^2)-v2*(T*cos(w*T)/w-sin(w*T)/w^2);
      F(3,3)=cos(w*T);
      F(3,4)=-sin(w*T);
      F(3,5)=-v1*T*sin(w*T)-v2*T*cos(w*T);
      F(4,3)=sin(w*T);
      F(4,4)=cos(w*T);
      F(4,5)=v1*T*cos(w*T)-v2*T*sin(w*T);
    end
    Bv=[0;0;0;0;1];
    Qd=T;
    out1=F;
    out3=Bv;
    out6=Qd;

 elseif method==6
 end
elseif flag==6;  % Output equation matrices
  if method<3;
    C=[eye(2) zeros(2,2)];
  elseif method<5;
    C=[eye(2) zeros(2,4)];
  elseif method==5;
    C=[eye(2) zeros(2,3)];
  end
    R=100^2*eye(2);
  out1=C;
  out4=R;
end







