clc
echo on
%     OPTIMAL SEGMENTATION
%     ====================
%
%  Generate the signal as an unknown and abruptly changing constant in white
%  measurement noise:

y=[ones(25,1);2*ones(25,1)]+0.1*randn(50,1);

% This can be written as a linear regression:
%
%      y(t)=phi(t)'theta(t)+e(t)
%
% where phi(t)=1, theta is the unknown constant and the variance, lambda, of
% the noise is 0.01. Furthermore, this can be
% expressed as an ARX model,
%
%      y(t)=b1(t)u(t)+e(t),
%
% where u(t)=1 and b(t)=theta(t). The data vector and the model structure 
% are thus:

z=[y ones(50,1)];
nn=[0 1 0];
lambda=0.01;

pause
% The parameter variation is modeled as random reset,
%
%      theta(t+1)=(1-delta(t))theta(t)+delta(t)v(t)
%
% where v(t) is an unknown parameter and delta(t) are 
% independent Bernoulli variables with success probability q. The prior
% distribution of theta is assumed to be Gaussian with variance P0. The
% design variables are chosen as follows:
%
q=0.01;
pause

% To compute the MAP segmentation, 2^50 aposteriori probabilities have to 
% be compared, requiring 2^N filters.
% Thanks to two inequalities, this number can be reduced first to N filters 
% and then to an even smaller number.
%
% Using the optseg function, the optimal segmentation is  computed.
% 
t0=clock; 
[jhat,thseg,V,likelihood,modnum]=optseg(z,nn,lambda,q,0.01);
jhat
t1=etime(clock,t0);
pause
% At the end,
modnum(length(modnum))
% filters were run in parallel, and the total estimation time was
t1
hold off
plot([y thseg])

% The optimal segmentation can also be computed by segm if the number of
% filters equals the number of data.
% [jhat,thseg,V,likelihood,modnum]=detectM(z,nn,[1 lambda],q,50);
t0=clock; 
[jhat,thseg,lam]=detectM(y,-1,[1 lambda],q,50);
jhat
t2=etime(clock,t0);
pause
% At the end, 50 filters were run in parallel, 
% and the total estimation time was now
t2
% This is however a too trivial example to judge the time performance.
% In the general case the segm function is recommended.
