function [V,TH,threc]=lossfun(z,nnn,lf,q,jumps)

% Computes the loss functions in detectM for a particular jump sequence
%
% [V,TH,threc]=lossfun(z,nnn,lf,q,jumps)
%
% z         The output-input data vector z = [y u] or z = [y Phi].
% nnn       Model structure (see help nnn)
% lf        loss function, 
%           lf=[1 lambda]  MAP estimate with known noise scaling lambda
%           lf=[2]  MAP estimate with unknown noise scaling (default)
%           lf=[3]  BIC/MDL penalty term for complexity
%           lf=[4]  AIC penalty term for complexity
% q         The probability that the system jumps at each sample 
%           (default 0.5, which gives ML)
% jumps     The jump times
% 
% The ML estimate is achieved if q=1/2 (default). 


if nargin<2; nnn=[]; end
if isempty(nnn); nnn=-1; end

mtype=modeltype(nnn);
if mtype>4;
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nu=n(2);
  nv=n(3);
  nth2=n(4);
  ny=n(5);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  tstart=1;
  xhat=x0;
else
  [tstart,d]=z2phi(z,0,nnn);
  A=1; Q=0; B=0; 
  C=1; Dss=0; R=1;
  P0=1000000000*eye(d);
  x0=zeros(d,1);
end;
[N,nz]=size(z);

   
if nargin<5 jumps=0; end;  % or [] ?
if nargin<4 q =[]; end; 
if nargin<3 lf=[]; end; 
if isempty(lf) | lf<0; lf=2; end;
if isempty(q) | q<0, q =0.5; end
if lf(1)==1; 
  if length(lf)>1;
    lambda=lf(2); lf=lf(1); 
  else
    disp('Warning: a lambda should be specified for loss function 1 (lambda=1 now)')
    lambda=1; lf=lf(1);
  end
else
  lf=lf(1); lambda=1; 
end;
% Create log(gamma(.)) function.
if lf==2;
   for i=1:50;
      loggamma(i)=log(gamma(i/2));
   end;
   for i=51:N;
      loggamma(i)=0.5*log(2*pi)-i/2+(i-1)/2*log(i/2);
   end;
end


n=1;
age=0;
V=0; D=0;
threc=zeros(N,d);
TH=[];
th=zeros(d,1);
P=P0;

if mtype>4; % state space model
  for t=tstart:N; 
    [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,th,0,5);
    [C,Du,Dth,R]=feval(nnn,t,th,0,6);
    den=inv(lambda*R+C*P*C');
    epsi=y(t)-C*th;
    if ~isempty(u) & ~isempty(Du);
      epsi=epsi-Du*u(t,:)';
    end
    K=P*C'*den;
    th=th+K*epsi;
    threc(t,:)=th';
    th=A*th;
    if ~isempty(u) & ~isempty(Bu);
      th=th + Bu*u(t,:)';
    end
    Pdum=A*(P-K*C*P)*A'+Bv*Q*Bv';
    P=.5*(Pdum+Pdum');
    D(n)=D(n)-log(det(den));
    V(n)=V(n)+epsi'*den*epsi;
    age=age+1;
    if ~isempty(find(t==jumps))
      P=10*P;
      th=th;
      n=n+1;
      D(n)=0; 
      V(n)=0;
      age(n)=0;
      TH=[TH th];
    end  
  end
  TH=[TH th];

elseif mtype<5   % linear regression
  for t=tstart:N; 
    [phi,y,u]=z2phi(z,t,nnn);
    den=inv(lambda*R+phi'*P*phi);
    epsi=y-phi'*th;
    K=P*phi*den;
    th=th+K*epsi;
    Pdum=P-K*phi'*P+Q;
    P=.5*(Pdum+Pdum');
    D(n)=D(n)-log(det(den));
    V(n)=V(n)+epsi'*den*epsi;
    age=age+1;
    threc(t,:)=th';
    if ~isempty(find(t==jumps))
      P=P0;
      th=zeros(d,1);            
      n=n+1;
      D(n)=0; 
      V(n)=0;
      age(n)=0;
      TH=[TH th];
    end  
  end
  TH=[TH th];
end;

if lf==1;
  logp = sum(D+V)+n*2*log((1-q)/q);
elseif lf==2;
  logp = sum(D)+sum( (age-d-2).*log(V) - 2*loggamma(age-d-2));
elseif lf==3;
  logp = age.*log(V./age) + d*n*log(t);
elseif lf==4;
  logp = age.*log(V./age) + d*n*2;
elseif lf==5;  
  % New feature: measurements from exponential distribution
  % z(t) = exp(th(t))
  % -log f(y|th)=log(th)+y/th;
  ybar=[];
  for jind=aind;
    ybar=[ybar mean(z(t+4-age(jind):t))];
  end
  alfabar(aind)=alfa(aind)+age(aind).*(1+log(th(:,aind)));  % MGL estimator
end;
V=logp;
threc=threc';