function [jumphat,thseg,V,likelihood,modnum]=optseg(z,nnn,R2,q,epsilon,dsp)

%    Compute the optimal ML/MAP segmentation of a signal
%
%    [jumphat,thseg,V,likelihood,modnum]=optseg(z,nnn,lam,q,epsilon,dsp)
%
%    Works only for nn structures with known noise variance!
%    See also the demo.

if nargin<6;dsp=0;end;
if nargin<5 epsilon=0.01; end;
if nargin<4; q=0.01; end; if q<0, q=0.01;end
if nargin<3 R2=-1; end
if R2<0,R2=1;end;
if length(R2)>1; R2=R2(2); end;   % compatibility

[tstart,d]=z2phi(z,0,nnn);
mtype=modeltype(nnn);
if mtype<3; tstart=1; end;  % Linear regressions
[N,nz]=size(z);
R1=1000*R2*eye(d);

alfa=zeros(1,N);
alfa(1)=1;
pointer=[1 zeros(1,N-1)];
thm=zeros(N*d,d);
Phi=zeros(N,d);
th=zeros(N*d,N);
age=zeros(1,N);
hist=zeros(N,N);
hist(1,1)=1;
P=zeros(d*N,d*N);P(1:d,1:d)=R1;
f=ones(1,N);
L=1;

%=======Loop=======================================================
t=tstart-1;   
while  t<N & abs(min(alfa))<max(alfa);
	t=t+1;
	models=pointer(find(diff(pointer(1:t))~=0));
	modnum(t)=length(models);
	if dsp==1;
        clc,disp('Time'),disp(t-1)
	disp('Models'),disp(models)
	disp('Relative probability'),disp(alfa(models))
	disp('History, t-20:t'),disp(hist(max([1 t-20]):t,models))
	disp('Theta'),disp(th(1:3*d,models));
%	disp('P'), disp(P(:,models))
	pause
	end;

%-------Step two: merging---------------------------
	models=pointer(find(diff(pointer(1:t-L))~=0));
	ind1=find(abs(alfa(models))<epsilon);
        dum=diff(ind1);
	if ~isempty(dum); ind2=find(dum==1); else; ind2=[]; end
	for i=1:length(ind2);   
		mod1=pointer(models(ind1(ind2(i))));
		mod2=pointer(models(ind1(ind2(i)+1)));
%		disp('Merging models:'),disp([mod1 mod2])
		ind=find(hist(:,mod1)==1);
		dummy=ind(length(ind)); 
		tind1=[dummy find(hist(:,mod2)==2)'];
		tind2=[find(hist(:,mod1)==2)' dummy ];
		n1=length(find(hist(:,mod1)==1))-1;
		n2=length(find(hist(:,mod2)==1))-1;
		th1=th(n1*d+1:n1*d+d,mod1);
		th2=th(n2*d-d+1:n2*d,mod2);
		P1=P(n1*d+1:n1*d+d,mod1*d-d+1:mod1*d);
		P2=P(n2*d-d+1:n2*d,mod2*d-d+1:mod2*d);
		alfamod1=abs(alfa(mod1));
		alfamod2=abs(alfa(mod2));
		for j=tind1;
			phi=Phi(j,:)';
			den=inv(R2-phi'*P1*phi);
			epsi=z(j,1)-phi'*th1;
			th1=th1-P1*phi*den*epsi;
			P1=P1+P1*phi*den*phi'*P1;
			den=R2+phi'*P1*phi;
			epsi=z(j,1)-phi'*th1;
			alfamod1=alfamod1*sqrt(det(den)/det(R2))*exp(.5*epsi'*inv(den)*epsi);
		end;

		for j=tind2;
			phi=Phi(j,:)';
			den=inv(R2-phi'*P2*phi);
			epsi=z(j,1)-phi'*th2;
			th2=th2-P2*phi*den*epsi;
			P2=P2+P2*phi*den*phi'*P2;
			den=R2+phi'*P2*phi;
			epsi=z(j,1)-phi'*th2;
			alfamod2=alfamod2*sqrt(det(den)/det(R2))*exp(.5*epsi'*inv(den)*epsi);
		end;
		if max([alfamod1 alfamod2])<max(alfa);
		if alfamod1>alfamod2;
			ind=find(pointer==mod2);
			pointer(ind)=mod1*ones(1,length(ind));
			th(n1*d+1:n1*d+d,mod1)=th1;
			P(n1*d+1:n1*d+d,mod1*d-d+1:mod1*d)=P1;
			alfa(mod1)=-abs(alfamod1);
			alfa(mod2)=0;
			hist(tind1,mod1)=2*ones(length(tind1),1);
			hist(tind1(length(tind1))+1,mod1)=1;
			th(:,mod2)=0*th(:,mod2);
		else;
			ind=find(pointer==mod1);
			pointer(ind)=mod2*ones(1,length(ind));
			th(n2*d-d+1:n2*d,mod2)=th2;
			P(n2*d-d+1:n2*d,mod2*d-d+1:mod2*d)=P2;
			alfa(mod2)=-abs(alfamod2);
			alfa(mod1)=0;
			l2=length(find(hist(tind2,mod2)==1));
			if l2>0;
				th((n2-l2)*d+1:(n2-l2)*d+d,mod2)=th(n2*d+1:n2*d+d,mod2);
				P((n2-l2)*d+1:(n2-l2)*d+d,mod2*d-d+1:mod2*d)=P(n2*d+1:n2*d+d,mod2*d-d+1:mod2*d);
			end;
			hist(tind2,mod2)=2*ones(length(tind2),1);
			hist(tind1(length(tind1))+1,mod2)=1;
			th(:,mod1)=0*th(:,mod1);
		end;
		end;
	end;

%---Measurement update-------------------------------
       [phi,y,u]=z2phi(z,t,nnn);
	alfabar=alfa;
	models=pointer(find(diff(pointer)~=0)); %,pointer
	for j=models,
		numjum=length(find(hist(:,j)==1))-1;
		thj=th(numjum*d+1:numjum*d+d,j);
		Pj=P(numjum*d+1:numjum*d+d,d*(j-1)+1:d*(j-1)+d);
		denj=(R2+phi'*Pj*phi);
		epsij=y-thj'*phi;
		alfabar(j)=alfa(j)/sqrt(denj)*exp(-.5*epsij'*inv(denj)*epsij);
		th(numjum*d+1:numjum*d+d,j)=thj+Pj*phi*inv(denj)*epsij;
		P(numjum*d+1:numjum*d+d,d*(j-1)+1:d*j)=Pj-Pj*phi*inv(denj)*phi'*Pj;
		f(j)=f(j)/sqrt(denj)*exp(-.5*epsij'*inv(denj)*epsij);
	end;
	Phi(t,:)=phi';

%-------Step one: splitting-------------------------
	[dummy,jmax]=max(alfabar);
	hist(:,t+1)=hist(:,jmax);
	hist(t+1,t+1)=1;
	pointer(t+1)=t+1;
	th(:,t+1)=th(:,jmax);
	P(:,d*t+1:d*t+d)=P(:,d*jmax-d+1:d*jmax);
	numjum=length(find(hist(:,t+1)==1))-1;
	P(numjum*d+1:numjum*d+d,d*t+1:d*t+d)=R1;
	alfabar(t+1)=q*alfabar(jmax);
	alfa=1/sum(abs(alfabar))*alfabar;
end
%======End loop===================================

[dum,Nfilters]=size(hist);
thm=th(1:t,jmax);
dum=hist(1:t,jmax);
jumphat=find(dum==1);
jumphat=[ jumphat' t];
for k=2:length(jumphat);
  thseg(jumphat(k-1):jumphat(k),:)=ones(jumphat(k)-jumphat(k-1)+1,1)*thm((k-2)*d+1:k*d-d,:)';
end
jumphat=jumphat(2:length(jumphat)-1)'-1;
  
if abs(min(alfa))<max(alfa);
  e=z(:,1)-sum(thseg'.*Phi')';
  V=e'*e/N;
  likelihood=f(jmax);
  disp(['Successful merging! epsilon may be increased. '])
  disp(['Totally, ',num2str(modnum(N)),' parallel filters were used'])
else
  disp(['The merging step failed at time ',num2str(t),'.'])
  disp(['Try to decrease epsilon'])
  jumphat=[jumphat; NaN];
  V=NaN;
  likelihood=NaN;
end;
