function m=openeye(bch,beq);

% Compute the open-eye condition
%
% m=openeye(bch,beq);
%
% If the equalizer is a perfect inverse of the channel (which is
% impossible for FIR channel and equalizer), then m=1.
% Open-eye condition corresponds to m>0, when perfect reconstruction
% is possible, if there is no noise.
% The larger m, the larger noise can be tolerated.
%
% bch  True channel FIR parameter ((nb x N) matrix for timevarying channels)
% beq  Equalizer FIR parameter ((nb x N) matrix for time-varying equalizers)

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[nch,Nch]=size(bch);
[neq,Neq]=size(beq);
if nch==1; bch=bch'; nch=Nch; Nch=1;  end;
if neq==1; beq=beq'; neq=Neq; Neq=1;  end;

N=max([Nch Neq]);
if Neq>Nch; bch=bch(:)*ones(1,Neq); end
if Neq<Nch; beq=beq(:)*ones(1,Nch); end

for t=1:N;
  h=abs(conv(bch(:,t)',beq(:,t)'));
  if max(h)>0; m(t)=2-sum(h)/max(h); else, m(t)=0; end;
end
