function beta=power2(th0,th1,R,lambda,N,alpha);

% POWER2 computes the power for testing a parameter change using 'detect2'
%
% beta=power2(th0,th1,R,lambda,alpha); 
% 
% Asymptotic expression for the power function in the two-model 
% approach for a constant known noise variance. th0 is assumed
% known and th1 estimated in a sliding window. The result holds 
% for the distance measures GLR, divergence test and normalized 
% parameter norm. For unnormalized parameter norm, use srt(N) instead.
% The average run length function is approximately 1/beta(th1)
% 
%   th0    parameter vector before change 
%   th1    parameter vector after change 
%   R      N*P(N) where P is the covariance matrix
%   lambda noise variance
%   N      size of sliding window
%   alpha  confidence level in the hypothesis test 

% 940623 Fredrik Gustafsson

Calpha=erfinv(2*(alpha)-1);
x=N*(th1-th0)'*R*(th1-th0);
PFA=(x-Calpha*sqrt(2*lambda^2*length(th0)))/sqrt(2*lambda^2*length(th0)+4*lambda*x);
beta=(erf(PFA)+1)/2;
