function [y,th]=simadfilter(z,nnn,adg,adm,lambda)

% SIMADFILTER simulates a time-varying random walk parametric model for adfilter
%
% [y]=simadfilter(z,nnn,adg,adm);
%
% z         data matrix equal to [e u] or [e Phi]
% nnn       model structure
% adg,adm   see adfilter
% lambda    measurement noise variance (default 1)

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[N,nz]=size(z);
[tstart,d]=z2phi(z,0,nnn);
mtype=modeltype(nnn);

if nargin<5; 
  lambda=1;
end
if length(adg)<d % no state space model
  if nargin<4; 
    if adg<=1 & adg>0.1;
      adm='RLS'; 
    elseif adg<0.1;
      adm='NLMS'; 
    else
      adm='WLS'; 
    end;
  end
  if strcmp(adm,'LMS') 
    P=adg*eye(d);
  end
  if strcmp(adm,'WLS') 
     disp('WLS simulation makes no sense. Changed to RLS.')
     adm='RLS';
  end
  if strcmp(adm,'RLS')
    if adg>1|adg<0; adg=0.98; disp('adg changed to 0.98'); end
    P=100*eye(d);
  end
  if strcmp(adm,'NLMS')
    P=adg*eye(d);
  end
else
  P=100*eye(d);
  R=lambda;
end

y=zeros(N,1);
th=zeros(d,N);
sigma=sqrt(lambda);

for t=tstart:N;
    [phi,e,u,w]=z2phi(z,t,nnn); 
    if strcmp(adm,'RLS')
      R=adg;
      L=P*phi;
      Q=(1/adg-1)*(P-L*inv(phi'*L+R)*L');
    elseif strcmp(adm,'NLMS')
      R=1;
      Q=adg^2*phi*phi'/(1+phi'*phi);
    elseif strcmp(adm,'NLMS')
      R=1;
      Q=adg^2*phi*phi';
    elseif strcmp(adm,'KF')
      Q=adg;
      R=lambda;
    end;
    Qsqrt=chol(Q);
    th(:,t)=th(:,t-1)+Qsqrt*randn(d,1);
    y(t)=phi'*th(:,t)+sigma*randn(1,1);
    L=P*phi;
    P=P-L*inv(phi'*L+R)*L';
end;
