function [y,u,th]=simchannel(alphabet,nn,N,th);

% SIMCHANNEL simulates a digital communication channel
%
% [y,u,th]=simchannel(alphabet,nn,N);
% [y,u]=simchannel(alphabet,nn,N,th);
%
% Generate a random input signal with elements in the alphabet
% and simulate a MIMO (ny outputs, nu inputs) FIR(nb) channel.
%
% alphabet  Finite input symbol alphabet
% th        Channel parameters, given or simulated by the function.
%           th = N x sum(sum(nn)) matrix for a time-varying channel.
%           th = sum(sum(nn)) vector for a time constant channel.
%           If th=[]; then a fading channel is simulated, which
%           is complex if the alphabet is complex, otherwise real-valued.
% nn        Orders (channel lengths) of the MIMO FIR channel. 
%           ny x nu matrix with FIR orders nb
% N         Number of simulated data points.
% y         Channel output
% u         Channel input

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

% Fading simulation currently uses Lars Lindbom's script.


if nargin<4; th=[]; end;
[ny,nu]=size(nn);

[Nth,n]=size(th);
if Nth>1 & n==1;
  th=th.'; 
  [Nth,n]=size(th);
end
if Nth==1 & n>1
  th=th.'*ones(1,N);
end
if isempty(th)
  th=[];
  for k=1:nu;
    th1=[];
    for l=1:ny;
      th2=[];
      for i=1:nn(ny,nu);
        b=b_sim(70,.9994,pi/2*70,25000,N);
        th2=[th2; b.']; 
      end
      th1=[th1;th2];
    end
    th=[th;th1];
  end
  if isreal(alphabet)
    th=real(th);
  end
end

r=ceil(length(alphabet)*rand(N,nu));
alphabet=alphabet(:);
u=alphabet(r);

y=zeros(N,ny);
for t=max(max(nn)):N;
  phi=[];
  for m=1:ny
    phiu=u(t:-1:t-nn(ny,1)+1,1);
    for k=2:nu
      phiu=[phiu; u(t:-1:t-nn(ny,nu)+1,nu)];
    end
    phi=[phi zeros(size(phi,1),1); zeros(size(phiu,1),size(phi,2)) phiu];
  end
%  phi,th(:,t)
  y(t,:)=(phi.'*th(:,t)).';
end;
