function [epsi,epsif]=simresid(z,nnn,th)

% [epsi,epsif]=simresid(z,nnn,th)
%
% Computes the residuals from the parameter vectors in th
%
% epsi is the prediction error
% For regression models, epsi(t)=y(t)-phi(t)'*th(t-1)
% and epsif(t)=y(t)-phi(t)'*th(t)
% For state space models, epsi(t)=y(t)-H(t)*x(t), where x=th
% epsif is not provided
%
% Note: to simulate the signal, set yhat=y-epsif !

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $


mtype=modeltype(nnn);
N=length(z);

if mtype<5;  % regression model

  [tstart,d]=z2phi(z,0,nnn);
  epsi=zeros(N,1);
  epsif=zeros(N,1);
  for t=tstart:N;
    [phi,y,u]=z2phi(z,t,nnn);
    epsi(t)=y-phi'*th(:,t-1);
    epsif(t)=y-phi'*th(:,t);
  end;
else   % state space model
  extmod=isstr(nnn);
  if extmod==0; 
    % Compute all state space matrices
    [A,B,C,D,R1,R2,P0,x0]=nnn2ss(nnn);
  elseif extmod==1; % Just initialize P0 and x0
    n=feval(nnn,0,0,0,0)
  end
  d=n(1);
  nu=n(2);
  ny=n(6); 
  epsi=zeros(N,ny);
  epsif=zeros(N,ny);
  y=z(:,1:ny);
  if ny<size(z,2);  % Input provided
    u=z(:,ny+1:size(z,2));
  else
    u=zeros(size(z,1),nu);
  end
  x=th;
  for t=2:N;
    if extmod==1;
      [C,D]=feval(t,x(:,t),0,6);
    end
    epsi(t,:)=(y(t,:).'-C*x(:,t-1)-D*u(t,:).').';
  end
  epsif=[];
end



