function [out1,out2,out3]=ssDC(t,x,ubar,flag,option);

% Sampled state space model of Dc motor with x=[phi w]'
%
%               1
%      G(s) = ------
%             s(s+1)
%
% Syntax of nnn model objects (see also S-functions in Simulink)
%   [n,x0,P0]=nnn(0,0,0,0);
%   xp=nnn(t,x,ubar,2);
%   y =nnn(t,x,ubar,3);
%   [A,B]=nnn(t,x,ubar,5); % Extension from Simulink
%   [C,D]=nnn(t,x,ubar,6); % Extension from Simulink
%   nnn by itself presents the model using LTI object
%   ubar=[u v d f e] 
%   B=[Bu Bv Bd Bf Be] 
%   D=[Du Dv Dd Df De]
% with dimensions n=[nx nu nv nd nf ny]
%
% DC-motor examples:
% file    ny nv nd nf purpose
% ssDC     1  1  0  0 Kalman filtering
% ssDC2df  2  1  1  2 Fault detection


% ------------------------------------
% Edit here
nnn='ssDC';
T=0.1;
nx=2; nu=1; nv=1; nd=0; nf=0; ny=1;
% ------------------------------------

if nargin==0
  feval(nnn,0,0,0,7), return
end
out1=[]; out2=[]; out3=[];

if flag==0;    % Initialization
    P0=0.01*eye(nx);
    x0=zeros(nx,1);
    n=[nx nu nv nd nf ny];
    out1=n; out2=x0; out3=P0;
elseif flag==1;
elseif flag==2;
    [A,B]=feval(nnn,0,x,0,5);
    xp=A*x+B*ubar;
    out1=xp;
elseif flag==3;
    [C,D]=feval(nnn,0,x,0,6);
    y=C*x+D*ubar;
    out1=y;
elseif flag==4;
  t=t+T;
  out1=t;
elseif flag==5;
% ------------------------------------
% Edit here
  a=exp(-T);
  A=[1 1-a;0 a];
  Bu=[T-1+a; 1-a];
  Bd=[];
  Bf=[]; 
  Bv=[T^2/2;T];
% ------------------------------------
% Don't edit here
  Be=zeros(nx,ny);
  B=[Bu Bv Bd Bf Be];
  out1=A;
  out2=B;
elseif flag==6;  % Output equation matrices
% ------------------------------------
% Edit here
  C=[1 0];
  Du=[0];
  Df=[];
% ------------------------------------
  % Don't edit here
  Dv=zeros(ny,nv);
  Dd=zeros(ny,nd);
  De=eye(ny);
  D=[Du Dv Dd Df De];
  out1=C;
  out2=D;
elseif flag==7; 
    % Create LTI object, linearized at time t, state x and input u
    [n,x0,P0]=feval(nnn,0,0,0,0);
    [A,B]=feval(nnn,t,x,ubar,5);
    [C,D]=feval(nnn,t,x,ubar,6);
    opt.n=n; opt.x0=x0; opt.P0=P0;
    opt.help='Convention: n=[nx nu nv nd nf ne], ubar=[u v d f e]';
    sys=ss(A,B,C,D,T);
    set(sys,'UserData',opt)
    uname=struct([]);
    for i=1:nu; uname{i}=['u',num2str(i)];  end
    for i=1:nv; uname{nu+i}=['v',num2str(i)]; end
    for i=1:nd; uname{nu+nv+i}=['d',num2str(i)]; end
    for i=1:nf; uname{nu+nv+nd+i}=['f',num2str(i)]; end
    for i=1:ny; uname{nu+nv+nd+nf+i}=['e',num2str(i)]; end
    set(sys,'InputName',uname)
    out1=sys;
end







