function [F,G,H,D,Q,R,P0,x0]=tnsgps()

%model
% d/dt x = Ax + Bu + Ne
%      y = Cx + Du + w
% x1=latitud error (rad)
% x2=North latitude rate error (rad/s)
% x3=East angular error (rad)
% x4=acceleration bias (m/s^2)
% x5=east gyro bias (rad/s)
% x6=gps gauss-markov


% set constant parameters in aderr.m and simalt.m simulation

%time=600;             	%total time [s]
dt=1; 			%sample time
%No=time/dt;		%number of time steps [-]

g=9.81;   		%gravity
R0=6378e3; 		%earths equatorial radius
alt=500; 		%altitude 
E_omega=7.2921e-5; 	%Earths angular velocity

Ftime=1500;             %Error state feedback after Ftime [s]
Nf=Ftime/dt;		%Error state feedback after Nf no of cycles



%GPS error  
%GPS gauss markov process
gpstau=10;	%correlation time
gpsgmsig=1.5/R0;	%gauss markov noise amplitud
gpssig=1/R0;    %measurement noise;

%ins error
%ins acc error
insabias=50e-5;  %ins acc bias [m/s^2] (50 micro g)
% insasf=0; %2e-4; %ins acc scale factor
insasig= 5e-5;    %ins acc noise [m/s^3/2] (5 micro g/sqrt(Hz))
% insagm=5e-5;     %ins acc gauss markov bias
insagmt=7200;    %   -"-               time const [s]
insagmsig=6.94e-9;  %ins acc gauss markov noise [m/s^3/2] (5 micro g/sqrt(Hz))

%abiasstep=1e-2;	%ins acc bias step
ajumptime=300;	%time when bias jumps
%ajumptime=0;
ajumpno=ajumptime/dt;

%ins gyro error
insgbias = 2e-8;  %ins gyro bias (0.004 grad/h)
% insgsf   = 0; %2e-6 %ins gyro scale factor
insgsig  = 5e-9;  %ins gyro noise (0.001 grad/sqrt(h))
% insggm   = 1.5e-8;%ins gyro gauss markov bias [rad/s] (0.003 grad/h)
insggmt  = 3600;  %ins gyro gauss markov time const
insggmsig= 1e-10; %ins gyro bias gauss markov noise [rad/s^2](0.001/sqrt(hr) )

gbiasstep=0; %10^(-6.5); %1e-5;	%ins gyro bias step
gjumptime=300;	%time when gyro bias jumps
gjumpno=gjumptime/dt;


%Initial states and covariance
P0=diag([(1000/R0)^2,(4/R0)^2,(1e-3)^2,(1e-4)^2,(1e-7)^2,(10/R0)^2]); 
x0=[500/R0,0,0,insabias,insgbias,0]';

%detection parameters

%--distance measure
dm=2;	%1=normalized sample mean 
       	%2=normalize squared sample mean

%--stopping rule
%sr=[4 23 0.5]; 	
%sr=[3 6 0.5];

                        %[1 h nu] CUSUM with threshold h and drift parameter nu
			%[2 h nu] two-sided CUSUM
			%[3 h gamma] GMA with threshold h and forgetting Gamma
                	%[4 h gamma] two-sided GMA
			%[5 h q] Bayes
%--noise scaling
%lamda=1;        %noise variance scaling                

%glr
%M=20;	%size of sliding window
%h=30;	%threshold in test
%pnu=1e-5;	%Covariance of the jump


%chi2-test limits
%chi2n1=7.9742; %alpha=1-1/3600,n=1
%chi2n10=30.8397; %n=10

%chi2n1=7.8774;%alpha=1-1/1800, n=1
%chi2n10=29.8689; %n=10

chi2n1=11.8306; %from test-run
chi2n10=29.4084; %-"-


Ns=6;	%No of states

Rh=R0+alt;

A=[ 0  1   0   0      0        0;
    0  0   -g/Rh   1/Rh      0        0;
    0  1   0   0      1        0;
    0  0    0  -1/insagmt 0     0;
    0  0    0   0   -1/insggmt  0;
    0  0    0   0      0  -1/gpstau]; 

B=zeros(6,1);

N=eye(Ns);

C=[1 0 0 0 0 -1];

D=0;

L=[0 0 0 1 0 0]'; %Jump acc bias state

% Tc=[0 0 0 1 0 0;
%   0 0 0 0 1 0]'; %Assumed possible jump states
Tc=L;
%discrete
% x(k+1) = Fx(k) + Gu(k) + e(k)
%   y(k) = Hx(k) + Du(k) + w(k)

[F,G] = c2d(A,B,dt);
H=C;
[dum,J]= c2d(A,L,dt);
[dum,T]=c2d(A,Tc,dt);

Q=diag([0,(insasig/Rh)^2,insgsig^2,insagmsig^2,insggmsig^2,gpsgmsig^2]);
[f,Qk]=c2d(A,Q,dt);
R=gpssig^2;


