% Demonstration of state space models and Kalman filtering

% Alternative 1: inside toolbox
close all
randn('state',0)



% Kalman filtering
nnn='ssDC2'; ny=2;
nnn='ssDC'; ny=1;
N=80;
u=randn(N,1);
u=0.1*[ones(4,1);zeros(N/4-4,1);ones(N/4,1);zeros(N/4,1);ones(N/4,1)];
Q=0.01^2;
R=0.1^2*eye(ny);
v=randn(N,1)*sqrt(Q);
e=randn(N,ny)*sqrt(R);
[y,x0,lam0]=simchange([u v e],nnn);

sys=feval(nnn,0,0,0,7);
[xhat,yhat,P]=adkalman([y u],sys,Q,R,[2 0]);   % Alternative 1
%[xhat,yhat,P]=adkalman([y u],nnn,Q,R,[1 0]);  % Alternative 2
kfplot(xhat,P,x0,1)
kfplot(xhat,P,x0,4,1:2,[15])
[xhat,yhat,P]=adkalman([y u],sys,Q,R,[1 0]);   % Alternative 1
hold on
kfplot(xhat,P,x0,-4,1:2,[15])
return

figure
book('plotfix')
h(1)=plot(y);
hold on
h(2)=plot(yhat,'r');
h(3)=plot(x0(1,:),'-');
h(4)=plot(x0(2,:),'--');
h(5)=plot(xhat(1,:),'-r');
h(6)=plot(xhat(2,:),'--r');
hold off
set(h(3:6),'LineWidth',2)
legend(h([1 2 3 4 5 6]),'Measured y(t)','Estimated y(t)','True position','True velocity',...
       'Estimated position','Estimated velocity',2)
h=title('True state and measurements');

return

nnn='ssDC2df';
n=feval(nnn,0,0,0,0);
nx=2; nu=1; nv=1; nd=1; nf=2; ny=2;

%diagnosability(nnn,2,eye(2))
%pause

N=80;
f1=[1;0];
f2=[0;1];
TH=[f1 -f1 f2 -f2];
jumps=[3*N/8 4*N/8 5*N/8 6*N/8];
u=randn(N,1);
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
v=0*randn(N,nv);
e=0.1*randn(N,ny);
[y,x0,lam0]=simchange([u v d e],nnn,jumps,TH,1);

figure
book('plotfix')
%h(1:2)=plot(y,'-b');
h=segplot(y,jumps);
hold on
h(3)=plot(x0(1,:),'-');
h(4)=plot(x0(2,:),'--');
hold off
set(h(3:4),'LineWidth',2)
legend(h([1 3 4]),'Measurements y','True position','True velocity',2)
h=title('True state and measurements');

% Alternative 2: using LTI objects

sys=feval(nnn,0,0,0,7)
[y,x0,lam0]=simchange([u v d e],sys,jumps,TH,1);
figure
book('plotfix')
%h(1:2)=plot(y,'-b');
h=segplot(y,jumps);
hold on
h(3)=plot(x0(1,:),'-');
h(4)=plot(x0(2,:),'--');
hold off
set(h(3:4),'LineWidth',2)
legend(h([1 3 4]),'Measurements y','True position','True velocity',2)
h=title('True state and measurements');





