function [ber,uhatout,delay,m]=u2ber(u,uhat);

% Compute bit error rate for equalization output
%
%  [ber,uhatout,delay,m]=u2ber(u,uhat);
%
% The algorithm is intelligent and looks for an appropriate
% time delay and modulus of the combined channel and equalizer.
%
% u       true input (from finite alphabet)
% uhat    estimated input. Beginning zeros are discarded
% uhatout The corrected input estimate with respect to time delay
%         and modulus of the combined channel and equalizer.
% ber     bit error rate
% delay   estimated time delay
% m       estimated modulus

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $


N=length(u);
if length(uhat)~=N; error('u and uhat must be of the same length!'), return, end

% Discard initial zeros
ind=find(uhat==0);
N0=length(ind);
u=u(N0+1:N);
uhat=uhat(N0+1:N);
u=u(:); uhat=uhat(:);
N=N-N0;

M=min([N 50]);
nfft = 2.^ceil(log(2*N)/log(2));
U=fft(([u(:)'  zeros(1,N)]),nfft);
Uhat=fft(([uhat(:)'  zeros(1,N)]),nfft);
UUhat=U.*conj(Uhat);
RUUhat=fft(UUhat,nfft);
ruuhat=real(RUUhat(1:M))/N/nfft;

[dum,i]=max(abs(ruuhat));
m=sign(ruuhat(i)); % Estimated modulus
i=i-1;  % Estimated time delay
uhatout=m*[uhat(i+1:N);zeros(i,1)];
ind=find(u~=uhatout);
ber= (length(ind)-(i))/(N-i);
delay=i;
