function [phi,y,u,w]=z2phi(z,t,nnn);

% Extract the regression vector and y,u from z
%
% function [phi,y,u,w]=z2phi(z,t,nnn);
%
% If t=0, the first possible starting time 
% is delivered together with the number of parameters
% [tstart,d,nz]=z2phi(z,0,nnn);

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $


[nrow,ncol]=size(z);
mtype=modeltype(nnn);
w=0;
u=0;

if t>0;
  if mtype==0;      % No model
      phi=0;
      y=z(t,1);
  elseif mtype==1;  % Unknown mean
      phi=1;
      y=z(t,1);
      u=1;
  elseif mtype==2;       % Linear regression
      d=ncol-1;
      phi=z(t,2:1+d).';
      y=z(t,1);
  elseif mtype==3;      % AR
      na=nnn(1);
      phi=[-z(t-1:-1:t-na,1)]; 
      y=z(t,1);
  elseif mtype==4;       % ARX
      na=nnn(1); nb=nnn(2); nk=nnn(3);
      phi=[-z(t-1:-1:t-na,1); z(t-nk:-1:t-nk-nb+1,2)]; 
      y=z(t,1);
      u=z(t,2);
   elseif  mtype==5;         % State space
         nx=nnn(1,1); nu=nnn(1,2); ny=nnn(1,3);
         phi=nnn(nx+2:nx+ny+1,1:nx)';
         y=z(t,1:ny).';
         u=z(t,ny+1:ny+nu).';
         if ncol > ny+nu;      % Used in simchange
            w=z(t,ny+nu+1:ny+nu+nx)';
         end;
      
  end;
else    % if ~(t>0)
  tstart=2;   % default
  d=1;        % default
  nz=1;       % default
  if mtype==2;            % Linear regression
      d=ncol-1;
      tstart=d+1;
      nz=d+1;
  elseif mtype==3;        % AR
      tstart=nnn(1)+1;    % na+1
      d=nnn(1);
      nz=1;
  elseif mtype==4;        % ARX
      tstart=max([nnn(1)+1,nnn(2)+nnn(3)]);
      d=nnn(1)+nnn(2);
      nz=1+1;
   elseif mtype==5;        % State space
      if isstr(nnn)
         eval(['[F,G,H]=',nnn,'(0);'])
         d=size(F,1);
         nz=size(H,1); %+size(G,2);  % ny+nu
      else
        d=nnn(1,1);   % nx
        nz=0;
     end
  end;
  phi=tstart;
  y=d;
  u=nz;
end;
