function [zout,nnnout]=zdecimate(z,nnn,decfactor)

% Decimation of AR, ARX and linear regression models
%
% zdecimate(z,nnn,decfactor)
%
% z         output input data matrix, see help nnn
% nnn       model structure, see help nnn
% decfactor decimation factor > 1
% zout      Decimated data matrix
% nnnout    NOTE: AR and ARX models are transformed to linear regressions 

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

mtype=modeltype(nnn);
if mtype>4
  error('Decimation not possible for state space models')
end

N=length(z);

if mtype<3 
  zout=z(1:decfactor:N,:);
  nnnout=nnn;
end

if mtype==3 | mtype==4 % AR or ARX
  [tstart,d]=z2phi(z,0,nnn);
  zout=[];
  nnnout=-2; % Linear regression
  for t=tstart:decfactor:N; 
    [phi,y,u]=z2phi(z,t,nnn);
    zout=[zout;y' phi'];
  end
  
end
