% Beregne sentroiden til bevegelsesbilder (eng: motion image). 
%
% Her kalkulees ogs SVD for sentroidene som angir hvilken form plottet
% har i tillegg til skjevheten.
%
% mImages: 3-D matrise med bevegelsesbilder.
% X: horisontal pikselposisjon.
% Y: vertikal pikselposisjon.
% M: matrise sammensatt av X og Y.
% skew: skjevheten til sentroidene.
% S: vektor med singulre verdier.
%
% Laget av Stian F. Olsen (2010).
%
%


tic;
for j = 1:sizeFrames(3)
    [X(j),Y(j)] = centroid(mImages(:,:,j));
end
toc;

% Justerer verdien til X og Y slik at de ligger mellom 0 og 1.
X = X/sizeFrames(2);
Y = Y/sizeFrames(1);

% Fjerner NaN (bevegelsesbilder uten hvite piksler)
X(isnan(X))=[];
Y(isnan(Y))=[];

% SVD
M = [X; Y];
S = svd(M);
skew = s(1,1)/s(2,2);

figure(33)
scatter(X,Y,'.k');
xlabel('Normalisert pikselposisjon X'),ylabel('Normalisert pikselposisjon Y');
title('Sentroidene til bevegelsesbilder fra friskt barn (P001-T03)');