function book(arg,saveplots)

% Examples in 'Adaptive filtering and Change Detection'
% by Fredrik Gustafsson, Wiley 2001.
% arg  Switch to choose example. It is of the form 'ex1.2'
% for Examples, and 'sec1.2' for illustrations that are not
% of the form of an example.
% Saveplots used by the author to create all figures
  
% Copyright Fredrik Gustafsson, Sigmoid AB
% $  $


%set(0,'DefaultLineLineWidth',1)

if nargin<2; saveplots=0; end
fontsize=14;
if strcmp(arg,'all') 
  pause off
  echo off
  more off
  t000=clock;
end

% ch1 1111111
%--------------------------------------

if strcmp(arg,'plotfix')
  set(gca,'FontSize',15)
  set(gcf,'DefaultLineLineWidth',1.5);
  set(gcf,'DefaultAxesLineWidth',1);
  set(gcf,'DefaultAxesGridLineStyle','--')
end

  
  
if strcmp(arg,'all') | strcmp(arg,'sec1.1')  
  close all
cites=[2413 417  74;...  % FD
        479  74  291;... % Eq
       4661 952  871;... % Adaptive filt/est/sp
       1921 1642 317;... % Kalman
        890 124  402;... % Target tracking
       4563 1000 666;... % adaptive control
       8894 1000 317];   % sysid
  
  key={'Fault diagnosis',...
	  'Adaptive equalizer',...
	  'Adaptive filter/SP/estimation',...
	  '(Adaptive) Kalman filter',...
	  'Target tracking',...
	  'Adaptive control',...
	  '(System) Identification'};
  
 Ntot=[588683, 856692, 2582588 ];
 
  key={'FDI',...
	  'Eq',...
	  'Adaptive',...
	  'Kalman',...
	  'Tracking',...
	  'AC',...
	  'SI'};
  
  
  
  db={'IEL','ScienceDirect','IBM patents'};  %Espacenet
  

  ind=[ 4 2 5 1 6 7 3];
  figure(1), 
  subplot(131)
  book('plotfix')
  pie(cites(ind,1),key(ind))
  title(db{1})
  colormap('gray')
  set(gca,'Clim',[0 1])

  subplot(132)
  book('plotfix')
  pie(cites(ind,2),key(ind))
  title(db{2})
  colormap('gray')
  set(gca,'Clim',[0 1])

  subplot(133)
  book('plotfix')
  pie(cites(ind,3),key(ind))
  title(db{3})
  colormap('gray')
  set(gca,'Clim',[0 1])

  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/preface1.eps, end 

  
  figure(2)
  ind=[3 4 2 5 1 6 7];
  book('plotfix')
  pie(cites(ind,3)./cites(ind,1),key(ind))
  title(db{2})
  colormap('gray')
  set(gca,'Clim',[0 1])
  title('Patents per publication')
  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/preface2.eps, end 
  
end

  
  
  
if strcmp(arg,'all') | strcmp(arg,'ex1.1')
  close all
  load fuel
  N=length(y);
  t=0.2:0.2:0.2*N;
  threc1=adfilter(y,-1,0.9);
  threc2=adfilter(y,-1,0.7);
  hold on
  figure(1)
  subplot(211)
  book('plotfix')
  h1=plot(t,y,'-',t,threc1','-');
  set(h1(2),'LineWidth',2)
  legend([h1],'Measurement','Slow filter')
  ylabel('Fuel [l/100 km]')
  axis([0 0.2*N 0 25])

  subplot(212)
  book('plotfix')
  h2=plot(t,y,'-',t,threc2','-');
  set(h2(2),'LineWidth',2)
  legend([h2],'Measurement','Fast filter')
  xlabel('Time [s]')
  ylabel('Fuel [l/100 km]')
  axis([0 0.2*N 0 25])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_1.eps, end 
    
  threc=adfilter(y,-1,0.7);
  threc=0.3*round(threc/0.3);
  threc=kron(threc(5:5:N),ones(1,5));

  figure(2)
  book('plotfix')
size(t)
  h1=plot(t,y,'-',t,threc','-');
  axis([0 0.2*N 0 25])
  set(h1(2),'LineWidth',2)
  xlabel('Time [s]')
  ylabel('Fuel [l/100 km]')
  legend([h1],'Measurement','Volvo''s filter')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_1b.eps, end
    
end


if strcmp(arg,'all') | strcmp(arg,'ex1.2')
  close all
  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
%  s(201:N)=s(201:N)+(TH0(2,2)-TH0(2,1))*mu(201:N);  % Friction change

  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);

  figure(1)
  subplot(211)
  book('plotfix')
  plot(s)
  ylabel('Slip')
%  set(gca,'LineWidth',1)
  subplot(212)
  book('plotfix')
  plot(mu)
%  set(gca,'LineWidth',1)
  ylabel('Normalized traction force')
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_2a.eps, end 


  figure(2)
  book('plotfix')
  h1=plot(s(1:200),mu(1:200),'x');
  hold on
  h2=plot(s(201:N),mu(201:N),'o');
%  set(gca,'LineWidth',1)
  a=axis;
  mumax=a(4);
  h3=plot([threc(1,200) threc(1,200)+threc(2,200)*mumax],[0 mumax]);
  h4=plot([threc(1,N) threc(1,N)+threc(2,N)*mumax],[0 mumax],'--');
  hold off
  ylabel('Normalized traction force')
  xlabel('Slip')
  legend([h1 h2 h3 h4],'Measurements before change','Measurements after change','Friction model before change','Friction model after change',2)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_2b.eps, end 

  figure(3)
  book('plotfix')
  h1=plot(threc');
  set(h1,'LineWidth',1);
  hold on
  h2=plot(threc2');
  set(h2,'LineWidth',2);
  h3=plot(th0','--');
  axis([0 N 0 0.05])
  hold off
  xlabel('Time [samples]')
  legend([h3(1) h1(1) h2(1)],'True parameters','Estimated parameters, fast filter','Estimated parameters, slow filter',2)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_2c.eps, end 

end

if strcmp(arg,'all') | strcmp(arg,'ex1.3')
  close all
  randn('seed',3);
  N=70;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;

  figure(1);
  book('plotfix')
  p1=radarplot(y,x0);
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  plot([0 x0(1,20)],[0 x0(2,20)])
  plot([x0(1,20)-2000 x0(1,20)],[x0(2,20)-1000 x0(2,20)])
  plot([x0(1,20)-1440 x0(1,20)],[x0(2,20)-1800 x0(2,20)])
  plot([0 5000],[0 0],'--')
  hold off
  h1=title('');
  h2=xlabel('X');
  h3=ylabel('Y');
  h4=text(4000,1000,'\theta');
  h5=text(12000,12000,'R');
  set([h4 h5],'FontSize',fontsize)
  legend(p1,'Measured position','True position')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_3a.eps, end

  NNNmethod=1;
  [xhat]=adfilter(y,nnn,1);
  figure(2);
  book('plotfix')
  radarplot(y,xhat);
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off
  set(gca,'Xlim',[-0.5 5]*1e4)
  xlabel('X')
  ylabel('Y')
  title('Initial filter design')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_3b.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex1.4')
  close all
  load fuel
  N=length(y);
  h=5;
  nu=0.5;
  t=0.2:0.2:0.2*N;
  [threc,jhat,thseg,gt,ta]=detect1(y,-1,2,[2 h nu]);

  figure(1)
  subplot(211)
  book('plotfix')
  h2=segplot(y,jhat);
  hold on
  h1=plot(threc);
  set(h1,'LineWidth',2)
  set(h2(1),'LineStyle','--')
  hold off
  axis([0 N 0 30])
  ylabel('Fuel [l/100 km]')
  xlabel(' ')
  legend([h1;h2],'Detection filter','Measurements','Change times')
  title(' ')

  subplot(212)
  book('plotfix')
  h2=plot(max(gt')','b-');
  axis([0 N 0 h*1.5])
  set(h2,'LineWidth',1)
  hold on, p1=plot([1 N],[h h]);, hold off
  set(p1,'LineWidth',4)
  xlabel('Time [samples]')
  ylabel('Test statistic g_t')
%  legend([h1 h2 h3],'Measurement','Slow filter','Fast filter')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_4.eps, end 
end

if strcmp(arg,'all') | strcmp(arg,'ex1.5')
  close all
  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  
%  s(201:N)=s(201:N)+(1/25-1/40)*mu(201:N);  % Friction change
  
  Phi=[ones(size(mu)) mu];
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
  h=3; nu=0.2;
  [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[1 h nu]);

  figure(1)
  subplot(211)
  book('plotfix')
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
  set(h1,'LineWidth',1);
  set(h2,'LineWidth',2);
  hold off
%  xlabel('Time [samples]')
  legend([h1(1) h2(1)],'Estimated parameters','True parameters',2)

  subplot(212)
  book('plotfix')
  h2=plot(gt,'b-');
  axis([0 N 0 h*1.5])
  set(h2,'LineWidth',1)
  hold on, p1=plot([1 N],[h h]);, hold off
  set(p1,'LineWidth',4)
  xlabel('Time [samples]')
  ylabel('Test statistic g_t')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_5.eps, end 

end

if strcmp(arg,'all') | strcmp(arg,'ex1.6')
  close all
  randn('seed',3);
  N=70;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;

  NNNmethod=1;
  h=5;
  nu=0.25;
  [xhat1,jhat1,xseg1,gt,ta]=detect1(y,nnn,1,[2 h nu],1);
  jhat1

  figure(1);
  book('plotfix')
  radarplot(y,xhat1);
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off
  set(gca,'Xlim',[-0.5 5]*1e4)
  xlabel('X')
  ylabel('Y')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_6a.eps, end
  figure(2);
  book('plotfix')
  h2=plot(max(gt')','b-');
  axis([0 N 0 h*1.5])
  set(h2,'LineWidth',1)
  hold on, p1=plot([1 N],[h h]); hold off
  set(p1,'LineWidth',4)
  xlabel('Time [samples]')
  ylabel('Test statistic g_t')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_6b.eps, end


end

if strcmp(arg,'all') | strcmp(arg,'ex1.7')
  close all
  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
%  s(201:N)=s(201:N)+(TH0(2,2)-TH0(2,1))*mu(201:N);  % Friction change

  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);

  [epsi1,epsif1]=simresid([s Phi],-2,threc);
  [epsi2,epsif1]=simresid([s Phi],-2,threc2);

  figure(4)
  subplot(211)
  book('plotfix')
  f=epsi1./(1e-5+epsi2);
  p1=plot(f);
  ylabel('|\epsilon_t^{(2)} / \epsilon_t^{(1)}|')
  xlabel('Time [samples]')
  axis([0 313 0 3])
  subplot(212)
  book('plotfix')
  ff=abs(filtfilt(ones(1,20)/20,1,[f;zeros(40,1)]));
  p2=plot(ff);
  axis([0 313 0 3])
  set([p1 p2],'LineWidth',1.5)
  ylabel('LP-filtered |\epsilon_t^{(2)} / \epsilon_t^{(1)}|')
  xlabel('Time [samples]')
  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_7.eps, end 
  
end


if strcmp(arg,'all') | strcmp(arg,'ex1.8')
  close all
  load fuel
  nu=0:0.1:1;
  h=1:1:10;
  for i=1:length(h)
    for j=1:length(nu)
      [threc,jhat,thseg,gt,ta]=detect1(y,-1,2,[2 h(i) nu(j)]);
      n(i,j)=length(jhat);
      V(i,j)=sum((y-threc').^2)/length(y);
    end
  end

  figure(1)
  book('plotfix')
  waterfall(h,nu,n')
  colormap([0 0 0])
  %mesh(nu,h,n)
  ylabel('Drift \nu')
  xlabel('Threshold h')
  zlabel('Number of alarms n')
  view([210 30])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_8a.eps, end 

  figure(2)
  book('plotfix')
  waterfall(h,nu,V')
  colormap([0 0 0])
  ylabel('Drift \nu')
  xlabel('Threshold h')
  zlabel('Sum of squared residuals')
  view([30 30])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_8b.eps, end 

end


if strcmp(arg,'all') | strcmp(arg,'ex1.9')
  close all
  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
  
  figure(1)
  subplot(211)
  book('plotfix')
  Phi=[ones(size(mu)) mu];
  th=Phi(1:200,:)\s(1:200)
  e=s(1:200)-Phi(1:200,:)*th;
  e=sort(e);
  e=e(3:length(e)-6);
  [Nhist,Xhist]=hist(e,30);
  bar(Xhist,Nhist,'w')
  sigma=std(e)
  xlabel('\epsilon_t')
  ylabel('Distribution')

  h=5; nu=0.4;
  
  if saveplots==1, MC=100; else, MC=10; end
  TA=[];
  GT=[];
  THREC=zeros(2,N);
  for i=1:MC;
    s(1:200)=1/40*mu(1:200);
    s(201:N)=1/25*mu(201:N);
    s=s+sigma*randn(N,1)+0.0053;
    [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[1 h nu]);
    TA=[TA ta];
    GT=[GT gt];
    THREC=THREC+threc;
  end
  threc=THREC/MC;

  subplot(212)
  book('plotfix')
  [Nhist,Xhist]=hist(TA,[201:209]);
  bar(Xhist,Nhist,'w')
  ylabel('Distribution')
  xlabel('Detection time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_9a.eps, end 

  figure(2)
  book('plotfix')
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
  set(h1,'LineWidth',1);
  set(h2,'LineWidth',2);
  xlabel('Time [samples]')
  legend([h1(1) h2(1)],'Estimated parameters','True parameters',2)
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_9b.eps, end 

  figure(3)
  book('plotfix')
  plot(GT)
end




if strcmp(arg,'all') | strcmp(arg,'ex1.10')
  close all
  randn('seed',3);
  N=70;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;

  NNNmethod=1;
  q=logspace(3,6,30);
  for i=1:length(q);
    [xhat]=adfilter(y,nnn,q(i));
    rms(i)=sqrt(1/(2*length(x0))*sum(sum((x0(1:2,:)-xhat(1:2,:)).^2)));
  end

  figure(1);
  book('plotfix')
  semilogx(q,rms)
  xlabel('Adaptation gain')
  ylabel('RMSE')
  [dum,ind]=min(rms);
  qopt=q(ind);
  title(['Optimal adaptation gain for KF is ',num2str(round(qopt))])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_10a.eps, end
  [xhat]=adfilter(y,nnn,qopt);

  figure(2);
  book('plotfix')
  radarplot(y,xhat);
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  hold off
  set(gca,'Xlim',[-0.5 5]*1e4)
  xlabel('X')
  ylabel('Y')
  title('Optimized filter design')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex1_10b.eps, end

end




% ch2 22222
%--------------------------------------

if strcmp(arg,'all') | strcmp(arg,'sec2.1.1') | strcmp(arg,'airbag')
  close all
  airbag
  load airbag
  [threc1,jhat1,thseg1,gt1,ta1]=detect1(z1,-1,1,[2 10 1.2],40^2);
  [threc2,jhat2,thseg2,gt2,ta1]=detect1(z2,-1,1,[2 10 1.2],40^2);
  
  Ts=1/110;
  N1=length(z1);
  t1=0:Ts:N1*Ts-Ts;
  N2=length(z2);
  t2=0:Ts:N2*Ts-Ts;
  
  figure(1)
  subplot(2,1,1)
  book('plotfix')
  p=plot(t1,[z1 threc1' thseg1']);
  set([p],'LineWidth',1);
  
  ylim=get(gca,'Ylim');
  hold on,
  for i = 1:length(jhat1);
    p=plot(Ts*jhat1(i)*[1 1],ylim);
    set([p],'LineWidth',0.5);
  end
  hold off
  ylabel('Data set 1')

  subplot(2,1,2)
  book('plotfix')
  p=plot(t2,[z2 threc2' thseg2']);
  set([p],'LineWidth',1);
  
  ylim=get(gca,'Ylim');
  hold on,
  for i = 1:length(jhat2);
    p=plot(Ts*jhat2(i)*[1 1],ylim);
    set([p],'LineWidth',0.5);
  end
  hold off
  xlabel('Time [s]')
  ylabel('Data set 2')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/airbag1.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'sec2.1.2') | strcmp(arg,'defibrator')
  load defibrator
  y=y/1000;

  figure(1)
  book('plotfix')
  plot(y)
  xlabel('Time [samples]')
  ylabel('Power')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/defibrator1.eps, end 

  %[threc,lamhat,Epsi]=adfilter(y,-1,0.99);
  figure(2)
  book('plotfix')
  plot([yhatsund])
  xlabel('Time [samples]')
  ylabel('Power')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/defibrator2.eps, end 
end


if strcmp(arg,'all')  | strcmp(arg,'sec2.1.3') | strcmp(arg,'photons')
  close all
  load photons
  y=y1(101:100:end)-y1(1:100:end-100); % Downsampling that gives gaussian distribution

  figure(1)
  subplot(211)
  book('plotfix')
  N=length(y1);
  p=plot(y1*1e-6,1:N);   % "Inverse sampling"
  set([p],'LineWidth',2);
  set(gca,'Xlim',[0 y1(N)*1e-6])
  xlabel('Time [s]')
  ylabel('Number of photons')

  subplot(212)
  book('plotfix')
  N=length(y);
  p=plot(y);
  set([p],'LineWidth',2);
  set(gca,'Xlim',[0 N])
  xlabel('Time [sampels]')
  ylabel('Time for 100 arrivals')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/photons1.eps, end
  
  y=y/500;  % Scaling to unit variance
%  y'*y/length(y)
%  y=diff(y1)/30;  
  [thrls,lamhat]=adfilter(y,-1,0.85);


  
  figure(2)
  book('plotfix')
  p=plot(lamhat);
  N=length(lamhat);
  set([p],'LineWidth',2);
  set(gca,'Xlim',[0 N])
  xlabel('Time [sampels]')
  ylabel('Variance estimate')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/photons2.eps, end

  [threc,jhat,thseg,lamseg]=detectM(y,-1,[2 1]);
  jhat

  figure(3)
  book('plotfix')
  h1=plot(thrls,'-');
  hold on
  h2=plot(threc,'-.');
  h3=plot(thseg,'--');
  hold off
  set([h1],'LineWidth',1);
  set([h2],'LineWidth',1);
  set([h3],'LineWidth',1);
  set(gca,'Xlim',[0 N])
  xlabel('Time [sampels]')
  ylabel('Arrival rate [500 \mu s/100 photons]')
  legend([h1 h2 h3],'RLS with \lambda=0.85','Recursive segmentation','Smoothed segmentation',4);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/photons3.eps, end

  figure(4)
  book('plotfix')
  segplot(y,jhat);
  set(gca,'Xlim',[0 N])
  xlabel('Time [sampels]')
  ylabel('Arrival rate [500 \mu s/100 photons]')
%  title('Number of photon arrivals in 100 intervals and segmentation')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/photons4.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'sec2.1.4') | strcmp(arg,'salesment')
  % Peter Duncan data
  close all
  load salesment
  figure(1)
  book('plotfix')
  h=plot(y);
  xlabel('Time [samples]')
  ylabel('Sales')
  set(h,'LineWidth',2);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/salesment1.eps, end 
end

if strcmp(arg,'all') | strcmp(arg,'sec2.2.1')| strcmp(arg,'altsensor')
close all
randn('seed',2)
%        Variance increases in an altitude sensor
%       ===========================================
%
% The measurement error of one sensor for height of an aircraft
% based on barometric air pressure increases drastically for velocities
% around one Mach. It's might be convenient to detect variance
% increases so the influence of these data can be decreased.

% First we load real data from a file. The data consist of the 
% residuals from the sensor signal when an accurate estimate of
% the height has been removed.

load altdata
y=hdiff(1:4:15608);

h1=figure(1);
subplot(211)
book('plotfix')
p1=plot(y);
set([p1],'LineWidth',1);
title('Barometric height residuals')

% Make an estimate of the time-varying noise variance using no
% parametric model (nnn=0)

nnn=0;
[thhat,lamhat]=adfilter(y,nnn,0.97);
subplot(212)
book('plotfix')
p2=plot(lamhat);
set([p2],'LineWidth',2);
xlabel('Time [samples]')
title('Low-pass filtered squared residuals')
print -deps /home/rt/fredrik/books/detect/figures/altsensor1.eps

% We will use the function multihyp, again without a model. 
% Use a prior probability of 0.01 for a jump at each time instant

q=0.01;
[jhat,jtype,thseg,lamseg]=multihyp(y,nnn,100,[0 1 0],q);

%[jh,ths,lams]=detectM(y,0);
% jhat,jh


h2=figure(2);
book('plotfix')
p1=semilogy([lamhat,lamseg']);
set([p1(1)],'LineWidth',1);
set([p1(2)],'LineWidth',2);
title('Low-pass filtered and segmented squared residuals')
print -deps /home/rt/fredrik/books/detect/figures/altsensor2.eps

% There are totally
length(jhat)
% segments.

% We can use the result to compute a table of noise variances as a function
% of velocity, which might be useful in filtering.
end

if strcmp(arg,'all') | strcmp(arg,'sec2.2.2') | strcmp(arg,'eeg_rat')
  close all
  load eeg_rat
  N=length(y);
  [jtime,jtype,thseg,lamseg]=multihyp(y,0,400,[0 1 0]);
  jtime
  
  figure(1)
  subplot(211)
  book('plotfix')
  p1=plot(y);
  set(gca,'Xlim',[0 N])
  h1=title('Rat EEG');

  subplot(212)
  book('plotfix')
  p1=plot(lamseg');
  set(gca,'Xlim',[0 N])
  xlabel('Time [samples]')
  set([p1],'LineWidth',2);
  h1=title('Segmentation of signal''s variance');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eeg_rat1.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'sec2.3.1')| strcmp(arg,'ash')
  close all
  load ash
  
  figure(1)
  subplot(311),  book('plotfix')
  p=plot(t1,[y1 u1]);
  title('Data y,u')
  set(gca,'Ylim',[-10 110])
  subplot(312),  book('plotfix')
  p=plot(t2,[y2 u2]);
  set(gca,'Ylim',[-10 110])
  subplot(313),  book('plotfix')
  p=plot(t3,[y3 u3]);
  set(gca,'Ylim',[-10 110])
  xlabel('Time [s]')
  
  N1=length(y1);
  N2=length(y2);
  N3=length(y3);
  phi1=[ones(size(u1)) cumsum(u1)];
  phi2=[ones(size(u2)) cumsum(u2)];
  phi3=[ones(size(u3)) cumsum(u3)];
  th1=phi1(1:round(N1/2),:)\y1(1:round(N1/2))
  th2=phi2(1:round(N2/2),:)\y2(1:round(N2/2))
  th3=phi3(1:round(N3/2),:)\y3(1:round(N3/2))
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ash1.eps, end  

  figure(2)
  subplot(311),  book('plotfix')
  plot(t1,[y1 phi1*th1])   
  title('Measurement and model simulation')
  subplot(312),  book('plotfix')
  plot(t2,[y2 phi2*th2])   
  subplot(313),  book('plotfix')
  plot(t3,[y3 phi3*th3])   
  xlabel('Time [s]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ash2.eps, end  

  [threcd1,jhat1,thseg1,gt1,ta1]=detect1([y1 phi1],-2,2,[1 1000 1.3]);

jhat1

  figure(3)
  subplot(211), 
  book('plotfix')
  plot(t1,[y1 u1])
  set(gca,'Ylim',[-10 110])
  title('Data y,u')
  set(gca,'XTick',[])
  subplot(212), 
  book('plotfix')
  plot(t1,gt1)
  title('CUSUM test statistic g_t')
  xlabel('Time [s]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ash3.eps, end  
  
  figure(4)
  threc1=adfilter([y1 phi1],-2,0.999);                
  [epsi,epsif]=simresid([y1 phi1],-2,threc1);
  y1hat=y1+epsif;
  
  subplot(211), 
  book('plotfix')
  plot(t1,[y1 y1hat])
  set(gca,'XTick',[])
  title('Measurement and model prediction')
  subplot(212), 
  book('plotfix')
  ff=0.99
  gt=filter(1-ff,[1 -ff],epsi);
  plot(gt);
  title('GMA of prediction error')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ash4.eps, end  
end

if strcmp(arg,'all') | strcmp(arg,'sec2.4.1') | strcmp(arg,'eeg_rat')
  close all
  load eeg_rat
  N=length(y);

  [jtime,jtype,thseg,lamseg]=multihyp(y,2,100);
  jtime

  figure(2)
  subplot(311)
  book('plotfix')
  p1=plot(y);
  set([p1],'LineWidth',1);
  h1=title('Rat EEG and segmentation of AR(2) model and noise variance');
  set(gca,'Xlim',[0 N])
  set([p1],'LineWidth',1);

  subplot(312)
  book('plotfix')
  p1=plot(thseg');
  set(gca,'Xlim',[0 N])
  set([p1],'LineWidth',2);

  subplot(313)
  book('plotfix')
  p1=plot(lamseg');
  xlabel('Time [samples]')
  set(gca,'Xlim',[0 N])
  set([p1],'LineWidth',2);
  book('plotfix')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eeg_rat2.eps, end 
end

if strcmp(arg,'all') | strcmp(arg,'sec2.4.2') | strcmp(arg,'eeg_human')
  close all
  load eeg_human
  y=Y(:,1);
  N=length(y);

  figure(1)
  book('plotfix')
  p1=plot(y);
  set(gca,'Xlim',[0 N])
  set([p1],'LineWidth',1);
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eeg_human1.eps, end 

  threc=adfilter(y,2,0.99);
  figure(2)
  book('plotfix')
  p1=segplot(threc',387);
  set([p1],'LineWidth',2);
  set(gca,'Xlim',[0 N])
  legend(p1,'a_1','a_2')
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eeg_human2.eps, end 
end

if strcmp(arg,'all') | strcmp(arg,'sec2.4.3') | strcmp(arg,'quake')
  close all
  load equake
  figure(1)
  book('plotfix')
  subplot(311)
  p1=plot(y(:,1));
  set([p1],'LineWidth',1);
  hd=title('Three of 16 measurement series from an earthquake');

  subplot(312)
  book('plotfix')
  p1=plot(y(:,2));
  set([p1],'LineWidth',1);
  subplot(313)
  book('plotfix')
  p1=plot(y(:,3));
  set([p1],'LineWidth',1);
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/quake1.eps, end 

  load sfquake
  figure(2)
  h=subplot(311);
  book('plotfix')
  plot(t,n);
  set(h,'XTick',[])
  hd=title('Movement in north-south direction');
  h=subplot(312);
  book('plotfix')
  plot(t,e);
  set(h,'XTick',[])
  hd=title('Movement in east-west direction');
  subplot(313)
  book('plotfix')
  plot(t,v)
  hd=title('Movement in vertical direction');
  xlabel('Time [s]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/sfquake1.eps, end 
end

if strcmp(arg,'all') | strcmp(arg,'sec2.4.4') | strcmp(arg,'speech')
  load speech
  [threc,jhat,thseg]=detectM(z,4);

  figure(1)
  subplot(211)
  book('plotfix')
  p1=segplot(z,jhat);
  set([p1],'LineWidth',2);
  title('Speech signal and segmentation using an AR(2) model');
  xlabel('')
  jhat
  subplot(212)
  book('plotfix')
  p1=plot(thseg');
  set([p1],'LineWidth',2);
  xlabel('Time [samples]')
  title('Segmentation of AR(2) parameters');

  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/speech1.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'sec2.5.1') | strcmp(arg,'labmotor')

  close all
  load  labmotor
  t=(0:0.01:25)';

  G=tf([0 0 140],[1 3.5 0]);         % System (DC-motor)
  C=tf([0.17 0.886 1],[0.625 6 0]);  % controller (PID)
  Gc=feedback(C*G,1);                % closed-loop system
  ym1=lsim(Gc,r1,t);
  ym2=lsim(Gc,r2,t);
  ym3=lsim(Gc,r3,t);
  ym4=lsim(Gc,r4,t);


  figure(1)
  book('plotfix')
  plot(t,y1-ym1+0.6,t,y2-ym2+0.2,t,y3-ym3-0.6,t,y4-ym4-0.2)
  h1=xlabel('Time [s]');
  h2=ylabel('Residuals');
  axis([0 25 -1 1])

  h3=text(9,-0.5,'Case 4: Dist. and system change');
  h4=text(9,0.7,'Case 1: Steady state');
  h5=text(9,0.3,'Case 2: Disturbances');
  h6=text(9,-0.0,'Case 3: System changes');
  set([h1 h2 h3 h4 h5 h6],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor.eps, end  
end

if strcmp(arg,'all') | strcmp(arg,'sec2.5.2') | strcmp(arg,'sheep')
  close all
  sheep=[];
  load  sheep
  z=[sheep(:,2) sheep(:,1)];
  N=length(z);

  figure(1);
  book('plotfix')
  p1=plot(1000*(1+sheep-2*[ones(N,1) zeros(N,1)]));
  set([p1],'LineWidth',2);
  xlabel('Time [samples]')
  ax=axis;ax(2)=N;axis(ax);
  h1=title('Pressure and air flow');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/sheep0.eps, end  

  [th,lamhat,e]=adfilter(z,[5 5 0],1);
  disp('VN = ')
  1000*lamhat(length(lamhat))

  nnn=0;
  [jhat1,jtype,thseg1,lamseg1]=multihyp(e,nnn,100,[0 1 0]);

  N=length(e);
  g=zeros(N,1);
  lambda=0.97;
  g=filter([1-lambda 0],[1 -lambda],e.^2);

  figure(2);
  subplot(211);
  book('plotfix')
  p1=plot(1000*(1+sheep-2*[ones(N,1) zeros(N,1)]));
  set([p1],'LineWidth',2);
  ax=axis;ax(2)=N;axis(ax);
  h1=title('Pressure and air flow');
  subplot(212);
  book('plotfix')
  thplot(g,lamseg1);
  set([p1],'LineWidth',2);
  ax=axis;  ax(2)=N; ax(4)=4e-3; axis(ax);
  title('Filtered model residual');
  xlabel('Time [samples]')
  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/sheep.eps, end  
end

if strcmp(arg,'all') | strcmp(arg,'sec2.6.1') | strcmp(arg,'path')
close all

%               Segmentation of driven path
%              =============================

% These data were collected from test drives with a Volvo 850 GLT
% using sensor signals from the ABS system. From these data, the position
% of the car can be calculated very roughly.
%
% Load and look at data

load carpath
N=length(x1);

figure(1);
book('plotfix')
plot(v)
ylabel('Velocity [m/s]')
xlabel('Time [samples]')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/path1.eps, end  

figure(2);
book('plotfix')
h=plot(x1,x2);
set([h],'LineWidth',2);
axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x1(1),x2(1)-50,'t=0 s')
text(x1(N)+50,x2(N)+50,'t=225 s')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/path2.eps, end  

% Next we compute the heading angle of the car
i=sqrt(-1);
phi=phase(i*diff(x2)'+diff(x1)')';

% The heading angle within each segment is modeled as a first order 
% polynomial in t, h(t) = c1 + c2 * t, and the segm function is used 
% with 10 parallel filters and required accuracy of the prediction errors
% corresponding to a variance of 0.05.
N=length(phi);
M=10;
nnn=-2;
t=(1:N)';
t0=clock;
[threc,jhat,thseg]=detectM([phi ones(size(t)) t],nnn,[1 0.05],.05,M);
consumedtime=etime(clock,t0)
estpath=thseg'.*[ones(size(t)) t]*[1;1];

figure(1)
book('plotfix')
h=plot(x1,x2,'-',x1(jhat),x2(jhat),'o');
set([h],'LineWidth',2);

axis('equal')
h1=xlabel('Meter');
h2=ylabel('Meter');
h3=title('Position');
h4=text(x1(1),x2(1)-50,'t=0 s');
h5=text(x1(N)+50,x2(N)+50,'t=225 s');
set([h1 h2 h3 h4 h5],'FontSize',fontsize);
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/path3.eps, end  

end


if strcmp(arg,'all') | strcmp(arg,'sec2.6.2') | strcmp(arg,'ekg')
  close all
  load ekg
  N=length(s);
  t=(1:N)';

  disp('Piecewise linear model')
  Phi=[ones(N,1) t];      
  [threc,jhat,thseg]=detectM([s Phi],-2,[1 0.01],.5,10); 
  [epsi,epsif]=simresid([s Phi],-2,thseg);

  jhat' 
  npar=size(Phi,2)*length(jhat)
  V=sum(epsif.^2)/N
  Vnorm=V*N/sum(y.^2)

  figure(1)
  subplot(211)
  book('plotfix')
  p1=plot([s]);
  set([p1],'LineWidth',2);
  axis([0 N -5 10])
  title('EKG data')
  xlabel(' ')
  
  subplot(212)
  book('plotfix')
  p1=segplot([s-epsif],jhat);
  set([p1],'LineWidth',2);
  axis([0 N -5 10])
  title('Piecewise linear model')
  xlabel('Time [samples] ')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ekg0.eps, end

end


if strcmp(arg,'all') | strcmp(arg,'sec2.8.1') | strcmp(arg,'stiction')
  % Test file for detection of static friction (stiction) in control
  % valves. Uses a filter bank segmentation approach as described by
  % Gustafsson (1999).
  %
  % Data are delivered by Krister Forsman (ABB Automation Systems)
  %
  % Author: Anders Stenman <stenman@isy.liu.se>
  % Time-stamp: <00/02/18 12:52:50 stenman>

  close all
  load stiction
  %zr = idresamp([y u],3);
  %z = detrend(zr,'constant');
  z=[y u];

   nn = [1 1 10];
  [jhat,xt,vstate,yhat,dhat,threc,dum,X] = stictionid(z,nn,2,0.1);

  figure(1)
  t=1:length(z);
  subplot(2,1,1)
  book('plotfix')
  p=plot(t,z(:,1),t,yhat,'r--');
  set([p],'LineWidth',2);
  hold on
  for j=jhat(:)'
    p=plot([j j],get(gca,'YLim'),'k--');
    set([p],'LineWidth',1);
  end
  hold off
  title('Measured and predicted output y_t')

  subplot(2,1,2)
  book('plotfix')
  [ts,ds]=stairs(t,dhat);
  p=plot(t,z(:,2),t,xt,'r--',ts,ds,'k--');
  set([p],'LineWidth',2);
  hold on
%  for j=jhat(:)'
%    p=plot([j j],get(gca,'YLim'),'k--');
%    set([p],'LineWidth',1);
%  end
  hold off
  title('Valve setpoint u_t, estimated valve state x_t and \delta_t')
  set(gcf,'NextPlot','replace')
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/stiction1.eps, end 

end

if strcmp(arg,'all') | strcmp(arg,'sec2.9.1') | strcmp(arg,'ifm')

%function epsi=modelifm(th,y);
%
%t=(1:19)';
%epsi=(th(1)./(1+exp(th(2)*(th(3)-t))).*(1-1./(1+exp(th(4)*(th(5)-t))))-y);

load nose.mat

N=90;             % Number of samples
n=15;             % Number of signals per sample
signalindex=[9 11 15];    % Used signals
%signalindex=[2:3, 5 9 11 13:15];    % Used signals
%signalindex=1:n;
sampelindex=[1:16,18, 21:N];
sampelindex=[3];


TH=[];
Vall=[];
Vnall=[];
for i=sampelindex;
  disp(['Sample nr: ',num2str(i)])
  thtemp=[];
  Vtemp=[];
  Vntemp=[];
  for j=signalindex;

    y=yall((i-1)*15+j,1:19)';
    th0=[400 2 13 0.5 18];
    th0(1)=max(y)*1.5;
    opt=[0 0 0 0 0];
    % opt(1)=display info
    % opt(5)=LevenbergMarquart (0), GN (1)
    th=leastsq('modelnose',th0,opt,[],y);
    thtemp=[thtemp th];
    t=(1:19)';
    yhat=(th(1)./(1+exp(th(2)*(th(3)-t))).*(1-1./(1+exp(th(4)*(th(5)-t)))));
    V=sum((y-yhat).^2)/19;
    %[V,yhat]=Vexpmodel(th,y);
    Vtemp=[Vtemp V];
    Vntemp=[Vntemp length(y)*V/(y'*y)];
    subplot(length(signalindex),1,find(j==signalindex))
    book('plotfix')
    h=plot(1:19,y,'--',1:19,yhat,'-');
    tmp=get(gca,'XTick');
    set(gca,'XTick',[])
    set(h,'LineWidth',2)
    legend(h,'Sensor response','Model response',2)
    title(['Sample # ',num2str(i),' and sensor # ',num2str(j)])
    drawnow
  end
  set(gca,'XTick',tmp)
  xlabel('Time [samples]')
  Vntemp
  TH=[TH thtemp'];
  Vnall=[Vnall Vntemp'];
end
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ifmnose.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'sec2.9.2') | strcmp(arg,'nmt')
  close all
  load nmt450

  figure(1)
  subplot(211)
  book('plotfix')
  h=plot(y);
  set(h,'LineWidth',2);
  h1=title('Sales figures for NMT450');
  

    %th0=[30000 1 1];
    %opt=[0 0 0 0 0];
    % opt(1)=display info
    % opt(5)=LevenbargMarquart (0), GN (1)
    %th=leastsq('modelnmt',th0,opt,[],y);


  load nmt900
  subplot(212)
  book('plotfix')
  h=plot(y);
  set(h,'LineWidth',2);
  h1=title('Sales figures for NMT900');
  xlabel('Time [samples]')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/nmt1.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'sec2.?') | strcmp(arg,'highway?')
  close all
  load highway

  figure(1)
  book('plotfix')
  h(1)=plot(y(:,1),y(:,2));
  set([h],'LineWidth',2);
  ax=axis;
  hd=title('Car position from computer vision system');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highwaya.eps, end

  figure(2)
  book('plotfix')
  h=plot(diff(y(:,1)),'b-');
  set(h,'LineWidth',2);
  hd=title('Differentiated position in east-west data');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highwayb.eps, end
end



% ch3 3333333333333
%--------------------------------------

if strcmp(arg,'all') | strcmp(arg,'ex3.1')
   close all
   randn('state',0)
   th0=[zeros(50,1);(0.02:0.02:1)';ones(50,1)];
   y=th0+randn(150,1);
   thrls=adfilter(y,-1,0.9,'RLS');
   thlms=adfilter(y,-1,0.1,'LMS');
   thkf=adfilter(y,-1,0.02,'KF');
   figure(1)
   book('plotfix')
   h1=plot([th0]);
   hold on
   h2=plot([thrls']);
   h3=plot([thlms'],'.-');
   h4=plot([thkf'],'--');
   h=plot(y,'o');
   hold off
%   set(h,'Linewidth',2)
   set([h1 h2 h3 h4],'LineWidth',1.5);
   set([h],'LineWidth',1);
   legend([h h1 h2 h3 h4],'Measurements','Signal','RLS','LMS','KF',2)

   axis([0 150 -0.5 1.5])
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex3_1.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex3.2')
   close all
   randn('state',0)
   th0=[zeros(50,1);(0.02:0.02:1)';ones(50,1)];
   y=th0+0.5*randn(150,1);
   g=0;
   h=5;
   nu=0.5;
   for t=1:length(y)
      g=g+y(t)-nu;
      if g<0, g=0; end
      if g>h, g=0; end
      gt(t)=g;
   end
   
   figure(1)
   subplot(211)
   book('plotfix')
   h1=plot([th0],'-'); hold on
   h2=plot(y,'--'); hold off
   set([h1 h2],'Linewidth',2)
   legend([h1 h2],'Signal','Measurement',4)
   subplot(212)
   book('plotfix')
   h1=plot(gt);
   set([h1],'Linewidth',2)
   legend([h1],'Test statistic g_t',2);
 
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex32.eps, end

end

if strcmp(arg,'all') | strcmp(arg,'ex3.3')
   close all
   randn('state',0)
   th0=[zeros(50,1);(0.02:0.02:1)';ones(50,1)];
   y=th0+0.5*randn(150,1);
   h=5; nu=0.5;
   [threc,jhat,thseg,gt,ta]=detect1(y,-1,1,[2 h nu]);
   jhat
   subplot(211)
   book('plotfix')
   h1=plot([th0],'--'); hold on
   h2=plot(y,':');
   h3=plot([threc],'-'); hold off
   set([h1 h2 h3],'Linewidth',2)
   legend([h1 h2 h3],'Signal','Measurement','Signal estimate',4)
   subplot(212)
   book('plotfix')
   h1=plot(max(gt'));
   set([h1],'Linewidth',2)
   legend([h1],'Test statistic g_t',2);
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex33.eps, end

end






if strcmp(arg,'all') | strcmp(arg,'ex3.5') 
  % changed lamhat in likelihood 000111
close all
N=500;
m=1; %Gaussian distribution (1) or exponential (2)
y=[zeros(N/2,1);ones(N/2,1)]+0.316*randn(N,1);
l0=likelihood(y(1:N));
for i=7:N-7; l1(i,:)=likelihood(y(1:i),[],[],m); end
for i=7:N-7; l2(i,:)=likelihood(y(i+1:N),[],[],m); end

figure(1);
subplot(211)
book('plotfix')
p1=plot(7:N-7,l1(7:N-7,1)+l2(7:N-7,1),'-',7:N-7,ones(N-13,1)*l0(1),'--');
title('MGL')
subplot(212)
book('plotfix')
p2=plot(7:N-7,l1(7:N-7,2)+l2(7:N-7,2),'-',7:N-7,ones(N-13,1)*l0(2),'--');
title('MML')
set([p1 p2],'Linewidth',2)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mlk1.eps, end

l0=likelihood(y(1:N),[],1);
for i=7:N-7; l1(i,:)=likelihood(y(1:i),[],1,m); end
for i=7:N-7; l2(i,:)=likelihood(y(i+1:N),[],1,m); end

figure(2);
subplot(211)
book('plotfix')
p1=plot(7:N-7,l1(7:N-7,1)+l2(7:N-7,1),'-',7:N-7,ones(N-13,1)*l0(1),'--');
title('MGL')
subplot(212)
book('plotfix')
p2=plot(7:N-7,l1(7:N-7,2)+l2(7:N-7,2),'-',7:N-7,ones(N-13,1)*l0(2),'--');
title('MML')
set([p1 p2],'Linewidth',2)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mlk2.eps, end

l0=likelihood(y(1:N),0);
for i=7:N-7; l1(i,:)=likelihood(y(1:i),0,[],m); end
for i=7:N-7; l2(i,:)=likelihood(y(i+1:N),1,[],m); end

figure(3);
subplot(211)
book('plotfix')
p1=plot(7:N-7,l1(7:N-7,1)+l2(7:N-7,1),'-',7:N-7,ones(N-13,1)*l0(1),'--');
title('MGL')
subplot(212)
book('plotfix')
p2=plot(7:N-7,l1(7:N-7,2)+l2(7:N-7,2),'-',7:N-7,ones(N-13,1)*l0(2),'--');
title('MML')
set([p1 p2],'Linewidth',2)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mlk3.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex3.6') 
end


if strcmp(arg,'all') | strcmp(arg,'sec3.6.1')
  close all
  load fuel
  N=length(y);
%  [threc,jhat,thseg,lamseg]=detectM(y,-1,[1 1],[5]);
  h=5;
  nu=0.5;
  [threc,jhat,thseg,gt,ta]=detect1(y,-1,2,[2 h nu]);

  figure(1)
  book('plotfix')
  h2=segplot(y,jhat);
  hold on
  h1=plot(threc);
  set(h1,'LineWidth',2)
  set(h2(1),'LineStyle','--')
  hold off
  axis([0 N 0 25])
  ylabel('Fuel [l/100 km]')
  xlabel(' ')
  legend([h1;h2],'Detection filter','Measurements','Change times')
  title(' ')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/fueldetectM.eps, end
    
end

if strcmp(arg,'all')  | strcmp(arg,'ex3.6.2') | strcmp(arg,'defibrator')

  close all
  load defibrator
  y=y/1000;
  N=length(y);
  [threc,lamhat,Epsi]=adfilter(y,-1,0.99);

  figure(1)
  subplot(211)
  book('plotfix')
  h1=plot(threc);
  set(gca,'Xlim',[0 N])
  subplot(212)
  book('plotfix')
  h2=plot(lamhat);
  set(gca,'Xlim',[0 N])
  set([h1 h2],'LineWidth',2)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/defibrator3.eps, end 

%  [threc,jhat,thseg,lamseg]=detectM(y,-1,[1 0.05]);
  h=10;
  nu=1;
  [threc,jhat,thseg,gt,ta]=detect1(y,-1,1,[2 h nu]);
  figure(2)
  subplot(311)
  book('plotfix')
  h1=plot([yhatsund]);
  set(gca,'Xlim',[0 N])
  subplot(312)
  book('plotfix')
  h2=plot(threc);
  set(gca,'Xlim',[0 N])
  subplot(313)
  book('plotfix')
  h3=plot(thseg);
  set(gca,'Xlim',[0 N])
  set([h1 h2 h3],'LineWidth',2)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/defibrator4.eps, end 

  figure(3)
  book('plotfix')
  plot(gt)
  set(gca,'Xlim',[0 N])
  set(gca,'Ylim',[0 1.2*h])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/defibrator5.eps, end 
end


% ch4 444444444444

if strcmp(arg,'all') | strcmp(arg,'ex4.3.1') | strcmp(arg,'localsearch')
close all
randn('seed',1);
jumps=[10 20 30];
TH=[1 3 6 10];
[y,th0]=simchange(randn(40,1),-1,jumps,TH);     
M=5;
[threc,jumphat,thseg,lamseg,Alfa,history]=detectM(y,-1,[1 1],0.5,M,0);

figure(1);
subplot(211)
book('plotfix')
hypplot(history,jumphat) 
title(['Local search with M=',num2str(M)])
M=8;
[threc,jumphat,thseg,lamseg,Alfa,history]=detectM(y,-1,[1 1],0.5,M,0);
subplot(212)
book('plotfix')
hypplot(history,jumphat) 
set(gca,'Ylim',[0 6])
title(['Local search with M=',num2str(M)])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/localsearch2.eps, end

figure(2);
book('plotfix')
thplot(y,th0);
title('Measurements and real parameters')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/localsearch1.eps, end

figure(3);
book('plotfix')
M=40;
[threc,jumphat,thseg,lamseg,Alfa,history]=detectM(y,-1,[1 1],0.5,M,0);
hypplot(history,jumphat) 
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/localsearch3.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'ex4.1')  | strcmp(arg,'ex4.2')  | strcmp(arg,'ex4.4') | strcmp(arg,'globalsearch')
close all
N1=100;
N2=10;
N3=0;
N=N1+N2+N3;
k=1:N1;
V1=N2*(N1+N3-k)./(N-k)+log(k)+log(N-k);
k1=k;
k=N1:N1+N2;
V2=N1*(k-N1)./k+N3*(N1+N2-k)./(N-k)+log(k)+log(N-k);
k2=k;
k=N1+N2:N1+N2+N3-1;
V3=N2*(k-N2)./(k)+log(k)+log(N-k);
k3=k;

figure(1)
subplot(211)
book('plotfix')
p=plot([k1 k2 k3],[zeros(size(k1)) ones(size(k2)) zeros(size(k3))]);
set(p,'LineWidth',2)
axis([0 110 -1 2])
title('Signal')

subplot(212)
book('plotfix')
p=plot(k1,V1, '-k',k2,V2,'-k',k3,V3,'-k');
set(p,'LineWidth',2)
title('Likelihood for k=1:N')
xlabel('Jump time')
axis([0 110 5 20])
h=text(60,17,'-log(p(k^n=1:N))');
set(h,'FontSize',fontsize)

if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/globalsearch1.eps, end

disp('If k is initialized around zero, a search algorithm')
disp('will fail to converge')
disp(' ')
disp('Another example.....')

N1=100;
N2=10;
N3=100;
N=N1+N2+N3;
k0=1:N-1; 
V0=N2*(N1+N3)/(N)*ones(size(k0))+log(N);
k=1:N1;
V1=N2*(N1+N3-k)./(N-k)+log(k)+log(N-k);
k1=k;
k=N1:N1+N2;
V2=N1*(k-N1)./k+N3*(N1+N2-k)./(N-k)+log(k)+log(N-k);
k2=k;
k=N1+N2:N1+N2+N3-1;
V3=N2*(k-N2)./(k)+log(k)+log(N-k);
k3=k;

figure(2)
subplot(211)
book('plotfix')
p=plot([k1 k2 k3],[zeros(size(k1)) ones(size(k2)) zeros(size(k3))]);
set(p,'LineWidth',2)
title('Signal')
axis([0 210 -1 2])

subplot(212)
book('plotfix')
p=plot(k1,V1,k2,V2,k3,V3,k0,V0);
set(p,'LineWidth',2)
Vopt=(log(N1)+log(N2)+log(N3))*ones(size(k0));
p=plot(k1,V1,'r',k2,V2,'r',k3,V3,'r',k0,V0,k0,Vopt);
set(p,'LineWidth',2)
title('Marginalized likelihood for k=[], k=0:N and k^n=(100,110)')
xlabel('Jump time')
axis([0 210 10 20])
h1=text(130,13,'-log(p(k^n=(100,110)))');
h2=text(130,16,'-log(p(k^n=[ ]))');
h3=text(130,19.2,'-log(p(k^n=1:N))');
set([h1 h2 h3],'FontSize',fontsize)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/globalsearch2.eps, end
 
disp(' ')
disp('No search scheme based on a local derivative could find')
disp('the global minima from an arbitrary initialization')

figure(3)
clf, book('plotfix')
disp(' ')
disp('Another example: ')
disp('Assume exponential noise distribution but noisefree measurements')
N=20;
y=0+[zeros(N/2-1,1);1;-1;zeros(N/2-1,1)];
p=stem(y);
set(p,'LineWidth',2)
axis([0 N -2 2])
%l1=likelihood(y,[],[],2);
%for k=1:N-1;
%  l2(k,:)=likelihood(y(1:k),[],[],2)+likelihood(y(k+1:N),[],[],2);
%end
%subplot(212)
%plot([1 N],[l1(:,1);l1(:,1)],1:N-1,l2(:,1))
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/globalsearch3.eps, end

end



if strcmp(arg,'all') | strcmp(arg,'optimalsearch')
  [threc,jumphat,thseg,lamseg,Alfa,history]=detectM(zeros(10,1),-1,[1 1],0.05,[10 9 0],0);
  figure(1)
  book('plotfix')
  hypplot(history,jumphat)  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/optimalsearch.eps, end
end


if strcmp(arg,'all')  | strcmp(arg,'ex4.3') | strcmp(arg,'totalsearch')
   close all
   rand('state',1)
   randn('state',3)
   N=30;
   TH=[1 4 8];
   jump=[N/3 2*N/3];
   y=simchange(randn(N,1),-1,jump,TH);
   figure(1)
   book('plotfix')
   thplot(y,TH,jump)
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/totalsearch1.eps, end
   for i=1:N;
      for j=1:N;
         V(i,j)=lossfun(y,-1,[1 1],0.5,[i j]);
      end
   end
   l=exp(-2*V);
   l=l./max(max(l));

   figure(2)
   book('plotfix')
   mesh(l)
   xlabel('k_1')
   ylabel('k_2')
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/totalsearch2.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex4.5')  | strcmp(arg,'mcmcsearch')
   close all
   rand('state',1)
   randn('state',3)
   N=30;
   TH=[1 4 8];
   jump=[N/3 2*N/3];
   y=simchange(randn(N,1),-1,jump,TH);
   figure(1)
   book('plotfix')
   p=thplot(y,TH,jump);
   set(p,'LineWidth',2)
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mcmcsearch1.eps, end

   [threc,kn]=gibbs(y,-1,[1 1],0.5,[2 500 0],[0 1 1],[5 10]);
   figure(2)
   book('plotfix')
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mcmcsearch2.eps, end
   figure(3)
   book('plotfix')
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mcmcsearch3.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'ex4.6') | strcmp(arg,'cpe')
  close all
  randn('state',0)
  sigma=1; %0.316;
  N=100;
  % H0 signal:
  y0=[1*ones(N/2,1);1*ones(N/2,1)]+sigma*randn(N,1);
  % H1 signal:
  y1=[1*ones(N/2,1);2*ones(N/2,1)]+sigma*randn(N,1);
  % Use an unknown mean model (required in cpe)
  nnn=-1;
  % Bayesian approach
  [jh,ths,GT01,U01]=cpe(y0,nnn,1);
  [jh,ths,GT11,U11]=cpe(y1,nnn,1);
  % Maximum likelihood approach
  [jh,ths,GT02,U12]=cpe(y0,nnn,2);
  [jh,ths,GT12,U12]=cpe(y1,nnn,2);
  % Non-parametric approach I
  [jh,ths,GT03,U13]=cpe(y0,nnn,3);
  [jh,ths,GT13,U13]=cpe(y1,nnn,3);
  % Non-parametric approach II
  [jh,ths,GT04,U14]=cpe(y0,nnn,4);
  [jh,ths,GT14,U14]=cpe(y1,nnn,4);
  subplot(221),   book('plotfix')
  plot([GT01,GT11]), h1=title('Bayes');
  subplot(222),  book('plotfix')
  plot([GT02,GT12]), h2=title('Maximum Likelihood');
  subplot(223),   book('plotfix')
  plot([GT03,GT13]), h3=title('Non-parametric I');
  subplot(224),   book('plotfix')
  plot([GT04,GT14]), h4=title('Non-parametric II');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex4cpe.eps, end
end

% ch4 applications

if strcmp(arg,'all') | strcmp(arg,'ex4eegrat') 
  close all
  load eeg_rat
  [th,lam,epsi]=adfilter(y,0,.97);
  [jtime,jtype,ths,lams]=multihyp(y,0,300,[0 1 0]);
  [jtime;jtype]

  figure(1);
  book('plotfix')
  segplot(y,jtime)
  h1=title('EEG signal on a rat');
  xlabel('Time [samples]')
  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eegrat1.eps, end

  figure(2);
  book('plotfix')
  h=plot([lam lams']);
  set([h],'LineWidth',2);
  xlabel('Time [samples]')
  h1=legend(h,'Recursively estimated noise variance','Segmented noise variance');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eegrat2.eps, end
end


% ch5 555555555555


if strcmp(arg,'all') | strcmp(arg,'ex5.5') 
  close all
  load eeg_rat
  N=length(y);
  [thhat,lamhat,epsi]=adfilter(y,2,1);
  thhat(:,N)
  roots([1 thhat(:,N)'])
  epsi=simresid(y,2,thhat);
  V=1/N*epsi'*epsi
  V0=1/N*y'*y
  figure(1)
  subplot(211)
  book('plotfix')
  plot(y)
  subplot(212)
  book('plotfix')
  h=plot(thhat');
  xlabel('Time [samples]')
  drawnow
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex5_0.eps, end

  [thhat,lamhat,epsi]=adfilter(y,4,1);
  thhat(:,N)
  roots([1 thhat(:,N)'])
  epsi=simresid(y,4,thhat);
  V=1/N*epsi'*epsi
end

if strcmp(arg,'all') | strcmp(arg,'ex5.5x') 
  %xxx
  close all
  load eeg_rat
  N=length(y);
  [thhat,lamhat,epsi]=adfilter(y,2,1);

end

  
if strcmp(arg,'all') | strcmp(arg,'ex5.6')
  % Steepest descent
  close all
  randn('state',1)
  disp('Minimization of x1^2+x1*x2+x2^2')
  phi=(0:0.01:2*pi);
  x=[sin(phi); cos(phi)];
  x=[1 -0.5;-0.5 1]*x;  % 
  s=1:4;
  
  figure(1)
  book('plotfix')
  plot(x(1,:)'*s,x(2,:)'*s,'b--')
  xlabel('x_1')
  ylabel('x_2')

  title('Steepest descent minimization of f(x)=x_1^2+x_1x_2+x_2^2')
  hold on

  figure(2)
  book('plotfix')
  plot(x(1,:)'*s,x(2,:)'*s,'b--')
  xlabel('x_1')
  ylabel('x_2')
  title('Stochastic steepest descent minimization of f(x)=x_1^2+x_1x_2+x_2^2')
  hold on
  N=100;

  figure(1)
  book('plotfix')
  xhat=[4;4];
  mu=0.03;
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*fprime;
  end
  p=plot(xhat(1,:)',xhat(2,:),'-b');
  plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
  set(p,'LineWidth',2)

  xhat(:,1)=[4;1];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*fprime;
  end
  p=plot(xhat(1,:)',xhat(2,:),'-b');
  plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
  set(p,'LineWidth',2)

  xhat(:,1)=[4;-1];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*fprime;
  end
  p=plot(xhat(1,:)',xhat(2,:),'-b');
  plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
  set(p,'LineWidth',2)
  
  xhat(:,1)=[4;-4];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*fprime;
  end
  p=plot(xhat(1,:)',xhat(2,:),'-b');
  plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
  set(p,'LineWidth',2)
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/steep1.eps, end

  figure(2)
  book('plotfix')
  e=3*randn(2,N);
  xhat(:,1)=[4;4];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
  end
  p=plot(xhat(1,:)',xhat(2,:),'b');
  set(p,'LineWidth',2)

  xhat(:,1)=[4;1];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
  end
  p=plot(xhat(1,:)',xhat(2,:),'b');
  set(p,'LineWidth',2)

  xhat(:,1)=[4;-1];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
  end
  p=plot(xhat(1,:)',xhat(2,:),'b');
  set(p,'LineWidth',2)

  xhat(:,1)=[4;-4];
  for i=2:N;
    x1=xhat(1,i-1);
    x2=xhat(2,i-1);
    fprime=[2*x1+x2; x1+2*x2];
    xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
  end
  p=plot(xhat(1,:)',xhat(2,:),'b');
  set(p,'LineWidth',2)
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/steep2.eps, end
end




if strcmp(arg,'all') | strcmp(arg,'ex5.7')
% Gauss-Newton
%close all
close all
randn('state',1)
disp('Minimize x1^2+x1*x2+x2^2')
phi=(0:0.01:2*pi);
x=[sin(phi); cos(phi)];
x=[1 -0.5;-0.5 1]*x;  % 
s=1:4;
figure(1)
book('plotfix')
plot(x(1,:)'*s,x(2,:)'*s,'b--')
xlabel('x_1')
ylabel('x_2')
title('Newton-Raphson minimization of f(x)=x_1^2+x_1x_2+x_2^2')
hold on
figure(2)
book('plotfix')
plot(x(1,:)'*s,x(2,:)'*s,'b--')
xlabel('x_1')
ylabel('x_2')
title('Stochastic Newton-Raphson minimization of f(x)=x_1^2+x_1x_2+x_2^2')
hold on
N=100;

figure(1)
book('plotfix')
xhat(:,1)=[4;4];
mu=0.03*inv([2 1; 1 2]);  % mu=-inv(fbis)
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*fprime;
end
p=plot(xhat(1,:)',xhat(2,:),'-b');
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
set(p,'LineWidth',2)

xhat(:,1)=[4;1];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*fprime;
end
p=plot(xhat(1,:)',xhat(2,:),'-b');
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
set(p,'LineWidth',2)

xhat(:,1)=[4;-1];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*fprime;
end
p=plot(xhat(1,:)',xhat(2,:),'-b');
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
set(p,'LineWidth',2)

xhat(:,1)=[4;-4];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*fprime;
end
p=plot(xhat(1,:)',xhat(2,:),'-b');
plot(xhat(1,1:10:N)',xhat(2,1:10:N),'o')
set(p,'LineWidth',2)
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/gauss1.eps, end

figure(2)
book('plotfix')
e=3*randn(2,N);
xhat(:,1)=[4;4];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
end
p=plot(xhat(1,:)',xhat(2,:),'b');
set(p,'LineWidth',2)

xhat(:,1)=[4;1];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
end
p=plot(xhat(1,:)',xhat(2,:),'b');
set(p,'LineWidth',2)

xhat(:,1)=[4;-1];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
end
p=plot(xhat(1,:)',xhat(2,:),'b');
set(p,'LineWidth',2)

xhat(:,1)=[4;-4];
for i=2:N;
  x1=xhat(1,i-1);
  x2=xhat(2,i-1);
  fprime=[2*x1+x2; x1+2*x2];
  xhat(:,i)=xhat(:,i-1)-mu*(fprime+e(:,i));
end
p=plot(xhat(1,:)',xhat(2,:),'b');
set(p,'LineWidth',2)
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/gauss2.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex5.8')
% Gauss-Newton with local minimum
close all
randn('state',1)

x=0:0.1:4;
f=x.^5-6*x.^4+6*x.^3+20*x.^2-38*x+20;

figure(1)
clf
book('plotfix')
plot(x,f)
xlabel('x')
ylabel('f(x)')
title('f(x)=x^5-6*x^4+6*x^3+20*x^2-38*x+20')

xstart=[0 4];
XI=[];
FI=[];
for k=1:2;
q=8;xi=zeros(1,q); 
xi(1)=xstart(k);
for i=1:q
  x0=xi(i);
  f0=x0^5-6*x0^4+6*x0^3+20*x0^2-38*x0+20;
  fprim=5*x0^4-24*x0^3+18*x0^2+40*x0-38;
  fbis=20*x0^3-72*x0^2+36*x0+40;
  xi(i+1)=xi(i)-.5*fprim/(fbis);
end
fi=xi.^5-6*xi.^4+6*xi.^3+20*xi.^2-38*xi+20;
XI(k,:)=xi;
FI(k,:)=fi;
end;

hold on
plot(XI(1,:),FI(1,:),'o')
plot(XI(2,:),FI(2,:),'*')
hold off
title('f=x^5-6*x^4+6*x^3+20*x^2-38*x+20 for x0=0 (o) and x0=4 (x)')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/gauss3.eps, end

end


if strcmp(arg,'all') | strcmp(arg,'ex5.9')
% ARMA estimation

close all
randn('state',0)
a0=-0.5;
b0=0.5;
b=[1 b0];
a=[1 a0];
y=filter(b,a,randn(200,1));
thi=poly2th([1 0],[],[1 0]);
par=[0 0];
for i=1:10
  % Warning: transpose for th2par required in Matlab R12
  dum=th2par(armax(y,thi,i));
  par(i+1,:)=dum(:)';
end
figure(4)
book('plotfix')
p1=plot(0:10,par); hold on
p2=plot(0:10,par,'o'); hold off
set([p1 p2],'Linewidth',2)
xlabel('Iteration'),
title('ARMA(1,1), th0=[-0.5; 0.5]')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arma1.eps, end


sep=0.04;
B=-1:sep:1;
A=-1:sep:1;
nA=length(A);
nB=length(B);

for i=1:nA
  for j=1:nB
    e=filter([1 A(i)],[1 B(j)],y);
    V(i,j)=sum(e.*e);
  end
end
for i=1:size(par,1);
    e=filter([1 par(i,1)],[1 par(i,2)],y);
    V2(i)=sum(e.*e);
end

figure(1)
book('plotfix')
mesh(B(1:1:nB),A(1:1:nA),log(V(1:1:nA,1:1:nB)))
xlabel('\theta_2')
ylabel('\theta_1')
title('log(V) for ARMA(1,1) model')
hold on
h=plot3(par(:,2),par(:,1),log(V2));
set(h,'Linewidth',3)
%view(150,30)

if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arma2.eps, end

figure(2)
book('plotfix')
inda=find(A>a0);
indb=find(B>b0);
inda=inda(1)-5:inda(1)+5;
indb=indb(1)-5:indb(1)+5;
mesh(B(indb),A(inda),log(V(inda,indb)))
xlabel('\theta_2')
ylabel('\theta_1')
ax=axis;
hold on
h=plot3(par(:,2),par(:,1),log(V2));
set(h,'Linewidth',3)
axis(ax)
title('log(V) for ARMA(1,1) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arma3.eps, end

figure(3)
book('plotfix')
V=log(V);
Vmin=min(min(V));
Vmax=max(max(V));
contour(B,A,V,[logspace(log10(Vmin),log10(1.2*Vmin),10) ])
hold on
p1=plot(0.5,-0.5,'*');
p2=plot(par(:,2),par(:,1));
p3=plot(par(:,2),par(:,1),'o');
set([p1 p2 p3],'Linewidth',2)
hold off
%contour(B,A,V,Vmin:(Vmax-Vmin)/50:Vmax)
xlabel('\theta_2')
ylabel('\theta_1')
title('log(V) for ARMA(1,1) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arma4.eps, end


end

if strcmp(arg,'all') | strcmp(arg,'ex5.10')
end



if strcmp(arg,'all') | strcmp(arg,'ex5.11') | strcmp(arg,'nmt')
  close all
  load nmt450
  N=length(y);
  t=1:N;
  h1=plot(t,y);
  title('Sales figures for NMT450')
  hold on
  yscale=max(y);
  y=y/yscale;
  th0=[0.0001 0.1 1];
  [t0,y0]=ode23('modelnmtode',t,0,[],th0(1),th0(2),th0(3));
  h0=plot(t0,y0*yscale,'-');

  opt=[1 0 0 0 0]; % do help foptions
  % opt(1)=display info
  % opt(5)=LevenbargMarquart (0), GN (1)
  opt(14)=10;
  th=th0;
  %fcount=[10 20 30 40 50];
  for i=1:5; %length(fcount);
    th=leastsq('modelnmt',th,opt,[],y)
    [that,yhat]=ode45('modelnmtode',t,0,[],th(1),th(2),th(3));
    h2(i)=plot(that,yhat*yscale,'--');
  end
  set([h1 h2(length(h2))],'LineWidth',2);
  hold off
  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/nmt2.eps, end

end

if strcmp(arg,'all') | strcmp(arg,'ex5.12') | strcmp(arg,'ex5.13')
% AR estimation

close all
randn('state',0)
a1=0.8;
a2=0.16;
N=1000;
b=[1];
a=[1 a1 a2];
mu=0.01;
P0=0.1*eye(2);
if saveplots~=1;
  MC=2;
else
  MC=25;
end


y=filter(b,a,randn(N,1));
par=adfilter(y,2,mu,'LMS');
par2=adfilter(y,2,0.99,'RLS',[],[],P0);

for i=2:MC;
  y=filter(b,a,randn(N,1));
  par=par+adfilter(y,2,mu,'LMS');
  par2=par2+adfilter(y,2,0.99,'RLS',[],[],P0);
end
par=par/MC;
par2=par2/MC;

par=[[0;0] par]';
par2=[[0;0] par2]';

figure(1)
book('plotfix')
plot(0:N,par),hold on
plot([0 0;N N],[a1 a2;a1 a2],':')
axis([0 N -0.2 1])
hold off
xlabel('Time'),
title('LMS with \mu=0.01 on AR(2) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ar1.eps, end


figure(2)
book('plotfix')
plot(0:N,par2),hold on
plot([0 0;N N],[a1 a2;a1 a2],':')
axis([0 N -0.2 1])
hold off
xlabel('Time'),
title('RLS with \lambda=0.99 on AR(2) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ar2.eps, end


sep=0.04;
A1=0:sep:1.5;
A2=-0.5:sep:1;
V=zeros(length(A1),length(A2));
for i=1:length(A1)
  for j=1:length(A2)
    e=filter([1 A1(i) A2(j)],[1],y);
    V(i,j)=sum(e.*e);
  end
end
for i=1:size(par,1);
    e=filter([1 par(i,:)],[1],y);
    V2(i)=sum(e.*e);
end
for i=1:size(par2,1);
    e=filter([1 par2(i,:)],[1],y);
    V3(i)=sum(e.*e);
end

figure(3)
book('plotfix')
mesh(A2(1:1:length(A2)),A1(1:1:length(A1)),log(V(1:length(A2),1:length(A1)))) 
       %log(V(1:1:end,1:1:end)))
xlabel('a_2')
ylabel('a_1')
title('log(V) for AR(2) model')
hold on
h=plot3(par(:,2),par(:,1),log(V2));
set(h,'Linewidth',2)
h=plot3(par2(:,2),par2(:,1),log(V2));
set(h,'Linewidth',2)
view(150,30)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ar3.eps, end


figure(4)
book('plotfix')
V=log(V);
Vmin=min(min(V));
Vmax=max(max(V));
contour(A2,A1,V,[logspace(log10(Vmin),log10(1.2*Vmin),10) ])
hold on
plot(a2,a1,'*')
plot(par(:,2),par(:,1))
hold off
%contour(B,A,V,Vmin:(Vmax-Vmin)/50:Vmax)
xlabel('a_2')
ylabel('a_1')
title('log(V) and LMS trajectory with \mu=0.01 on AR(2) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ar4.eps, end

figure(5)
book('plotfix')
contour(A2,A1,V,[logspace(log10(Vmin),log10(1.2*Vmin),10) ])
hold on
plot(a2,a1,'*')
plot(par2(:,2),par2(:,1))
hold off
xlabel('a_2')
ylabel('a_1')
title('log(V) and RLS trajectory with \lambda=0.99 on AR(2) model')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ar5.eps, end


end


if strcmp(arg,'all') | strcmp(arg,'ex5.14')
  % Time-frequency analysis of chirp
close all
N=5000;
y=sin(2*pi*(1:N)'.^2./N);

figure(1)
subplot(211)
book('plotfix')
h1=plot(4*pi*(1:N)'./N,'--');
hold on
h2=plot(1:1*N/4,4*pi*(1:N/4)'./N,'-');
h3=plot(1*N/4:2*N/4,2*pi-4*pi*(N/4:2*N/4)'./N,'-');
h4=plot(2*N/4:3*N/4,4*pi*(0:N/4)'./N,'-');
h5=plot(3*N/4:4*N/4,2*pi-4*pi*(N/4:2*N/4)'./N,'-');
hold off
set([h1 h2 h3 h4 h5],'LineWidth',2)
ylabel('Frequency')
xlabel('Time [samples]')
legend([h1 h2],'Fourier Transform','Discrete Fourier Transform',2)
subplot(212)
book('plotfix')
plot(y)
set(gca,'Xlim',[0 300])
xlabel('Time [samples]')
ylabel('Signal amplitude')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/chirp0.eps, end

figure(2)
book('plotfix')
specgram(y)
h1=ylabel('Frequency [Hz]');
h2=xlabel('Time [samples]');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/chirp1.eps, end

figure(3)
book('plotfix')
N=500;
y=sin(2*pi*(1:N)'.^2./N);
[th,lamhat]=adfilter(y,2,20); 
th2tfd(th,lamhat,y,64,[],[1 0]); 
h1=ylabel('Frequency [rad/s]');
h2=xlabel('Time [bins of 5 samples]');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/chirp2.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'sec5.7')
  close all
  randn('seed',1);
  nn=1;
  for ex=1:3;
    if ex==1;
      % piecewise constant parameters
      th=[-.5*ones(1,100) .5*ones(1,100)];
    elseif ex==2;
      % Time-varying parameters
      th=[-.5*ones(1,50) (-.5:.01:.5) .5*ones(1,49)];
    elseif ex==3;
      % Time-invariant parameters
      th=[-.5*ones(1,200)];
    end;

    if ex<4;
      N=200;
      e=randn(200,1);
      y=zeros(N,1);
      y(1)=e(1);
      for t=2:200;
        y(t)=-y(t-1)*th(t)+e(t);
      end;
      z=y;
    end

    t0=50;
    Methods=['RLS  ';'LMS  ';'SEGM ';'LS   ';'NLMS ';'GSM  '];
    methods=[4 1 2 3];
    Mmax=10;
    Nm=length(methods);
    MDL=zeros(Nm,Mmax);
    v=zeros(Nm,Mmax);

    for k=methods;
      if k==4;
        alg='LS';
        par=1;
      elseif k==1;
        alg='RLS';
        par=0.7:0.02:1;
      elseif k==2;
        alg='LMS';
        par=logspace(-3,-0.5,20);
      elseif k==5;
        alg='NLMS';
        par=logspace(-4,-2,20);
      elseif k==3;
        alg='Detection LS';
        par=logspace(-3,-.2,4);
        par=logspace(-1,1,10);
      elseif k==6;
        alg='GSM';
        par=[80 160 320 640 800 1066];
      end;
      disp(['Algorithm: ',alg])
      M=length(par);
      N=length(z);
      thm=zeros(N,M);

      for i=1:M;
       % disp(num2str(i))
        [MDL(k,i),v(k,i),thpar]=mdl(z,nn,alg,par(i),t0);
        thm(:,i)=thpar;
      end;
      [MDLopt(k),optind]=min(MDL(k,1:M));
      paropt(k)=par(optind);
      vopt(k)=v(k,optind);
      thpar=thm(:,optind);
      figure((ex-1)*4+k)
      clf
      subplot(211)
      set(gca,'FontSize',fontsize+2)
      p1=plot(par,MDL(k,1:M),'k-',par,v(k,1:M),'k:');
      set([p1],'LineWidth',3)
      xlabel('Adaptation gain')
      legend(p1,'MDL','Loss function V',2)
%      title(['MDL and V for algorithm ',alg])
      subplot(212)
      set(gca,'FontSize',fontsize+2)
      p1=plot(1:N,thpar,'k-',1:N,th,'k:');
      axis([0 N -1 1])
      set([p1],'LineWidth',3)
      xlabel('Time [samples]')
      legend(p1,'Estimated a_1','True a_1',2)
%      title(['Tracking for algorithm ',alg,' for par=',num2str(paropt(k))])
    
      if saveplots==1;
        eval(['print -deps /home/rt/fredrik/books/detect/figures/mdlex',num2str(ex),'m',num2str(k),'.eps'])
      end;
      disp(['Optimal parameter'])
      disp(paropt(k))
    end

    disp(' ')
    disp(['Result for ',num2str(ex)])
    disp(' ')
    disp('Method     MDLopt      paropt     V(paropt)')
    for k=methods;
      disp([Methods(k,:),'      ',num2str(MDLopt(k)),'      ',num2str(paropt(k)),'    ',num2str(vopt(k))])
    end
  end
end


if strcmp(arg,'all') | strcmp(arg,'ex5eq.0')
  % ISI
  b=[1 -2.2 1.17];
  randn('state',0);
  N=50;
  u=sign(-0.5+rand(N,1));
  y=filter(b,1,u);

  figure(1), clf
  subplot(311)
  book('plotfix')
  p1=stem(u);
  set([p1],'LineWidth',1.5)

  subplot(312)
  book('plotfix')
  p1=plot(y,'-o');
  set([p1],'LineWidth',1.5)

  subplot(313)
  book('plotfix')
  p1=stem(u-sign(y));
  set([p1],'LineWidth',1.5)
  xlabel('Time [samples]')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/isi.eps,  end
  
end



if strcmp(arg,'all') | strcmp(arg,'ex5eq.1')
  % Linear equalizer
  b=[1 -2.2 1.17];
  randn('state',0);
  rand('state',0);
  N=200;
  L=25;  % Length of training sequence
  u=sign(-0.5+rand(N,1));
  y=filter(b,1,u);
  nn=1:10;
  V=ones(length(nn),length(nn)-1);
  for n=nn;
    for delay=1:n-1;
     [uhat,fhat,z]=lineq(y,n,[-1 1],delay,u(1:L));
     e=u(L+1:end-delay)-z(L+1+delay:end);
     V(n,delay)=1/N*e'*e;
     BER(n,delay)=u2ber(u,uhat);
    end
  end
  disp('Loss function V(n,delay):')
  V
  BER
  figure(1), clf
  book('plotfix')
  p1=plot(1:5,V(6,1:5),'-');
  set(p1,'LineWidth',2)
  hold on
  p2=plot(1:7,V(8,1:7),'--');
  set(p2,'LineWidth',2)
  p3=plot(1:9,V(9,1:9),':');
  set(p3,'LineWidth',2)
  hold off
  axis([0 10 0 0.2])
  xlabel('Time delay D')
  legend([p1 p2 p3],'V(C_5,D)','V(C_7,D)','V(C_{9},D)')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/equalization1.eps,  end
  
  [Vn,indn]=min(V);
  [dum,delayhat]=min(Vn);
  nhat=indn(delayhat);
  disp('Optimal parameters:')
  delay=delayhat,n=nhat
  n=8;
  [dum,delay]=min(V(8,:));
  n,delay
  [uhat,fhat,z]=lineq(y,n,[-1 1],delay,u(1:L));
  e=u(L+1:end-delay)-z(L+1+delay:end);
  Vmin=1/N*e'*e;
  BER=u2ber(u,uhat);

  figure(2)
  subplot(311)
  book('plotfix')
  p1=stem(0:2,b);
  axis([0 14 -3 3])
  set(gca,'XtickLabel',[])
  h1=title('Impulse response for channel');
  subplot(312)
  book('plotfix')
  p2=stem(0:8,fhat);
  axis([0 14 -2 1])
  h2=title('Impulse response for equalizer');
  set(gca,'XtickLabel',[])
  subplot(313)
  book('plotfix')
  p3=stem(0:10,conv(b,fhat));
  axis([0 14 -1 2])
  set([p1 p2 p3],'LineWidth',2)
  h3=title('Combined impulse response for channel and equalizer');
%  set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/equalization2.eps,  end
end
  
if strcmp(arg,'all') | strcmp(arg,'ex5eq.2')
  % Linear equalizer, otherwise as in 5eq.3
  b=[1 -2.2 1.17];
  randn('state',0);
  N=148;
  L=25;
  MC=50;
  
  lam=[0.01 0.02 0.05 0.1 0.2 0.5 1 2 5 10];
  e0=randn(N,1);
  flops(0);
  for i=1:length(lam);
    for j=1:MC;
      u=sign(-0.5+rand(N,1));
      y0=filter(b,1,u);
      e=sqrt(lam(i))*e0;
      yn=y0+e;
      [uhat,thhat]=lineq(yn,8,[-1 1],4,u(1:L));
      ber(j,i)=u2ber(u,uhat);
      snr(j,i)=sum(y0.^2)/sum(e.^2);
    end
  end
  f=flops/i/j
  BER=mean(ber);
  SNR=mean(snr);
  figure(1), 
  book('plotfix')
  p=semilogx(SNR,BER,'-o');
  set(p,'LineWidth',2)
  xlabel('SNR')
  ylabel('BER')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/lineq1.eps,  end
  [SNR;BER]
  
end

if strcmp(arg,'all') | strcmp(arg,'ex5eq.3')
  % Viterbi equalizer, otherwise as in 5eq.2
  b=[1 -2.2 1.17];
  randn('state',0);
  N=148;
  L=25;
  MC=2;
  
  lam=[0.01 0.02 0.05 0.1 0.2 0.5 1 2 5 10];
  e0=randn(N,1);
  flops(0)
  for i=1:length(lam);
    for j=1:MC;
      u=sign(-0.5+rand(N,1));
      y0=filter(b,1,u);
      e=sqrt(lam(i))*e0;
      yn=y0+e;
      [uhat,thhat]=viterbi(yn,3,[-1 1],lam(i),u(1:L));
      ber(j,i)=u2ber(u,uhat);
      snr(j,i)=sum(y0.^2)/sum(e.^2);
    end
  end
  f=flops/i/j
  BER=mean(ber);
  SNR=mean(snr);
  figure(1), 
  book('plotfix')
  p=semilogx(SNR,BER,'-o');
  set(p,'LineWidth',2)
  xlabel('SNR')
  ylabel('BER')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/equalization3.eps,  end
  [SNR;BER]
  
end

if strcmp(arg,'all') | strcmp(arg,'ex5eq.4')
  disp('Demonstration of BER vs. SNR plots in equalization for a SISO noisy channel')
  disp(' ')
  echo on

  N=100;
  b=[1 -2.2 1.17];
  MC=10;
  L=[3 5 7 10];
  sigma=[0.05 0.1 0.2 0.5 1 2 5];

  for l=1:length(L);
    for i=1:length(sigma);
      randn('seed',1);
      for j=1:MC;
        u=sign(randn(N,1));
        e=sigma(i)*randn(N,1);
        y0=filter(b,1,u);
        y=y0+e;
        % BER when using a short training sequence
        [uhat,thhat]=viterbi(y,3,[-1 1],sigma(i)^2,u(1:L(l)));
        ber(j,i)=u2ber(u,uhat);
        snr(j,i)=sum(y0.^2)/sum(e.^2);
      end
    end
    BER(l,:)=mean(ber);
    SNR(l,:)=mean(snr);
  end
  BER,SNR
  leg=[];
  typ=['sdph'];
  close all
  figure  
  book('plotfix')
  set(gca,'FontSize',14)
  for l=1:size(SNR,1);  
    h(l)=semilogx(SNR(l,:)',BER(l,:)');
    hold on
    h(l)=semilogx(SNR(l,:)',BER(l,:)',typ(l));
    leg=str2mat(leg,['L = ',num2str(L(l))]);
  end
  leg(1,:)=[];
  xlabel('SNR')  
  ylabel('BER')
  legend(h,leg);
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/equalization4.eps,  end
end


if strcmp(arg,'all') | strcmp(arg,'ex5.20')
close all
disp('Demonstration of real time-constant SISO noise-free channel')

N=1000;
randn('seed',1);
b=[0.3 1 0.3]
%b=[1 -2.2 1.17];

u=sign(randn(N,1));
y=filter(b,1,u); %+0.1*randn(N,1);
nbeq=5;   % equalizer order for blindeq1

b0=[0 -0.1 1 -0.1 0]';

[theq1,uhat1,epsi1]=blindeq(y,nbeq,0.01,1,b0);
[theq2,uhat2,epsi2]=blindeq(y,nbeq,0.01,2,b0);

m1=openeye(b,theq1);
m2=openeye(b,theq2);
figure(1);
book('plotfix')
h1=plot(m1,'-');  hold on
h2=plot(m2,'--'); hold off
set([h1 h2],'LineWidth',2)
title('Open eye condition')
xlabel('Time [samples]')
legend([h1 h2],'Sato','Godard',3)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/bleq1a.eps, end


figure(2);
book('plotfix')
plot([epsi1' epsi2'])
title('Residuals')
xlabel('Time [samples]')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/bleq1b.eps, end

figure(3);
subplot(311)
book('plotfix')
h1=stem(1:3,b,'-');                   hold on
title('Channel')
axis([0 7 -2 2])
set(gca,'XTick',[])

subplot(312)
book('plotfix')
h2=stem(1:5,theq1(:,N)','-'); 
title('Equalizer')
axis([0 7 -2 2])
set(gca,'XTick',[])

subplot(313)
book('plotfix')
h3=stem(1:7,conv(b,theq1(:,N)'),'-'); hold off
title('Combined channel+equalizer')
axis([0 7 -2 2])
xlabel('Time [samples]')

set([h1 h2 h3],'LineWidth',1.5)
xlabel('Time [samples]')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/bleq1c.eps, end

echo off

ber1=u2ber(u,uhat1);
ber2=u2ber(u,uhat2);
disp(['BER Sato       = ',num2str(ber1)])
disp(['BER Godard     = ',num2str(ber2)])
end

%ch5 Applications

if strcmp(arg,'all') | strcmp(arg,'sec5.10.1')  | strcmp(arg,'eeghuman') 
  close all
  load eeg_human
  y=Y(:,1);
  [threc,jhat,ths]=detectM(y,4);
  jhat
  figure(1);
  book('plotfix')
  segplot(y,jhat);
  h1=title('EEG signal on a human when lights turned on at time 387');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/eeghuman.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'sec5.10.2') | strcmp(arg,'labmotor')

close all

load  labmotor
F=tf([0 1.67],[1 1.67]);           % pre-filter     
t=(0:0.01:25)';
%r=lsim(F,rnom,t)-0.02*cos(4.5*t);

N=length(y1);
z1=[y1 r1];
z2=[y2 r2];
z3=[y3 r3];
z4=[y4 r4];
zd1=[decimate(y1,10) decimate(r1,10)];
zd2=[decimate(y2,10) decimate(r2,10)];
zd3=[decimate(y3,10) decimate(r3,10)];
zd4=[decimate(y4,10) decimate(r4,10)];
nnn=[4 4 1];

%threc=adfilter(zd1,nnn,0.98);
h=3;
nu=0.05;
[threc1,jhat1,thseg,gt1,ta]=detect1(zd1,nnn,2,[1 h nu]);
[threc2,jhat2,thseg,gt2,ta]=detect1(zd2,nnn,2,[1 h nu]);
[threc3,jhat3,thseg,gt3,ta]=detect1(zd3,nnn,2,[1 h nu]);
[threc4,jhat4,thseg,gt4,ta]=detect1(zd4,nnn,2,[1 h nu]);

figure(1), subplot(211)
book('plotfix')
plot(1:10:2501,threc1')
axis([0 2501 -3 3])
title('Parameter estimation, data set 1')
subplot(212)
book('plotfix')
segplot(gt1,10*jhat1);
hold on, plot([0 N],[h h]), hold off
title('Test statistics from RLS whiteness test')
axis([0 251 0 1.2*h])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor1.eps, end  

figure(2), subplot(211)
book('plotfix')
plot(1:10:2501,threc2')
axis([0 2501 -3 3])
title('Parameter estimation, data set 2')
subplot(212)
book('plotfix')
segplot(gt2,10*jhat2);
hold on, plot([0 N],[h h]), hold off
title('Test statistics from RLS whiteness test')
axis([0 251 0 1.2*h])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor2.eps, end  

figure(3), subplot(211)
book('plotfix')
plot(1:10:2501,threc3')
axis([0 2501 -3 3])
title('Parameter estimation, data set 3')
subplot(212)
book('plotfix')
segplot(gt3,10*jhat3);
hold on, plot([0 N],[h h]), hold off
title('Test statistics from RLS whiteness test')
axis([0 251 0 1.2*h])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor3.eps, end  

figure(4), subplot(211)
book('plotfix')
plot(1:10:2501,threc4')
axis([0 2501 -3 3])
title('Parameter estimation, data set 4')
subplot(212)
book('plotfix')
segplot(gt4,10*jhat4);
hold on, plot([0 N],[h h]), hold off
title('Test statistics from RLS whiteness test')
axis([0 251 0 1.2*h])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor4.eps, end  
drawnow

end




if strcmp(arg,'all') | strcmp(arg,'sec5.10.3')  | strcmp(arg,'rfi')
%  close all
  figure(1)
  clf
  book('plotfix')
  s=0.001:0.001:1;
  A=[1 0.5 0.2]';
  B=1.5;
  C=10;
  D=1;
  mu=A*sin(B*atan(C*sin(D*s)));
  p=plot(s,mu,'-k');
  set(p,'LineWidth',2)
  xlabel('Slip s')
  ylabel('Normalized traction force \mu')
  h(1)=text(0.8,0.8*A(1)+0.06,'Asphalt');
  h(2)=text(0.8,0.8*A(2)+0.06,'Snow');
  h(3)=text(0.8,0.8*A(3)+0.08,'Ice');
  set(h,'FontSize',fontsize)
  
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi1.eps
  end

  load fricest
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  

  Phi=[ones(size(mu)) mu];
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);

  Phi=[ones(size(mu)) mu];
  threc=adfilter([s Phi],-2,0.94);
  threc2=adfilter([s Phi],-2,0.98);

  figure(2)
  subplot(211)
  book('plotfix')
  plot(s)
  ylabel('Slip s')
  subplot(212)
  book('plotfix')
  plot(mu)
  ylabel('Normalized traction force \mu')
  xlabel('Time [samples]')
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi2.eps
  end


  figure(3)
  book('plotfix')
  h1=plot(s(1:200),mu(1:200),'x');
  hold on
  h2=plot(s(201:N),mu(201:N),'o');
  a=axis;
  mumax=a(4);
  h3=plot([threc(1,200) threc(1,200)+threc(2,200)*mumax],[0 mumax]);
  h4=plot([threc(1,N) threc(1,N)+threc(2,N)*mumax],[0 mumax],'--');
  hold off
  ylabel('Normalized traction force \mu')
  xlabel('Slip s')
  legend([h1 h2 h3 h4],...
	 'Measurements before change',...
	 'Measurements after change',...
	 'Friction model before change',...
	 'Friction model after change',...
	 2);
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi3.eps
  end

  figure(4)
  book('plotfix')
  h1=plot(threc');
  set(h1,'LineWidth',1);
  hold on
  h2=plot(threc2');
  set(h2,'LineWidth',2);
  h3=plot(th0','--');
  axis([0 N 0 0.05])
  hold off
  legend([h3(1) h1(1) h2(1)],'True parameters',...
	 'Estimated parameters, fast filter',...
	 'Estimated parameters, slow filter',2)
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi4.eps
  end

  figure(5)
  book('plotfix')
  [epsi1,epsif1]=simresid([s Phi],-2,threc);
  [epsi2,epsif1]=simresid([s Phi],-2,threc2);
  plot(abs([epsi1./(1e-6+epsi2)]))
  axis([0 313 0 2])
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi5.eps
  end
  
  h=3; nu=0.2;
  [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[1 h nu]);

  figure(6)
  clf, book('plotfix')
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
  set(h1,'LineWidth',1);
  set(h2,'LineWidth',2);
  hold off
  xlabel('Time [samples]')
  legend([h1(1) h2(1)],...
	 'Estimated parameters',...
	 'True parameters',2) 

  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi6.eps
  end
  
  figure(7)
  clf, book('plotfix')
  h2=plot(gt,'b-');
  axis([0 N 0 h*1.5])
  set(h2,'LineWidth',1)
  hold on, plot([1 N],[h h]), hold off
  xlabel('Time [samples]')
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi7.eps
  end
 
  N=length(z);
  s=z(:,1);
  mu=z(:,2);  
  TH0=[0.0053 1/40; 0.0053 1/25]';
  th0=segm2par(TH0,200,N);
  
  figure(8)
  clf, book('plotfix')
  Phi=[ones(size(mu)) mu];
  th=Phi(1:200,:)\s(1:200)
  e=s(1:200)-Phi(1:200,:)*th;
  e=sort(e);
  e=e(3:length(e)-6);
  hist(e,30);
  sigma=std(e)
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi8.eps
  end


  h=5; nu=0.4;
  if saveplots==1, MC=50; else, MC=10; end
  TA=[];
  GT=[];
  THREC=zeros(2,N);
  for i=1:MC;
    s(1:200)=1/40*mu(1:200);
    s(201:N)=1/25*mu(201:N);
    s=s+sigma*randn(N,1)+0.0053;
    [threc,jhat,thseg,gt,ta]=detect1([s Phi],-2,1,[1 h nu]);
    TA=[TA ta];
    GT=[GT gt];
    THREC=THREC+threc;
  end
  threc=THREC/MC;

  figure(9)
  clf, book('plotfix')
  subplot(211)
  hist(TA,[200:220])
  subplot(212)
  h1=plot(threc');
  hold on
  h2=plot(th0','--');
  axis([0 N 0 0.05])
  set(h1,'LineWidth',1);
  set(h2,'LineWidth',2);
  hold off
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi9.eps
  end

  figure(10)
  clf, book('plotfix')
  plot(GT)

  figure(11)
  clf, book('plotfix')
  load ceram
  k=ceram7(:,2);
  p=plot(.2*[1:length(k)],k);
  set(p,'LineWidth',2);
  axis([50 80 10 70])
  xlabel('Time [s]')
  ylabel('Slip slope')
  if saveplots==1;
    print -deps /home/rt/fredrik/books/detect/figures/rfi11.eps
  end
end



% ch6 6666666
if strcmp(arg,'all') | strcmp(arg,'ex6.7') | strcmp(arg,'speech')
  close all
  load speech
  figure(1)
  subplot(211)
  book('plotfix')
  N=length(fig1120);
  plot((1:N)/12400,fig1120)
  title('Speech data with car noise')
  set(gca,'XTick',[])
  axis([0 N/12400 -5000 5000])

  subplot(212)
  book('plotfix')
  N=length(fig1118);
  plot((1:N)/12400,fig1118)
  xlabel('Time [s]')
  title('Speech data with pre-filter')
  axis([0 N/12400 -5000 5000])
  
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/speech2.eps, end 
  
  dm=2;  % Divergence test
  sr=[1 40 0.2];
  L=160;
  lambda=0.1;
  y=fig1118;
  y=y/std(y);
  [threc,jhat,thseg,gt]=detect2(y,4,dm,sr,L,lambda);

  figure(2)
  subplot(311)
  book('plotfix')
  p1=segplot(y,jhat);
  set([p1],'LineWidth',2);
  title('Speech signal and segmentation using an AR(2) model');
  xlabel('')
  jhat
  subplot(312)
  book('plotfix')
  p1=plot(thseg');
  set([p1],'LineWidth',2);
  xlabel('Time [samples]')
  title('Segmentation of AR(2) parameters');
  subplot(313)
  book('plotfix')
  plot(gt)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/speech3.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'ex6.10') 
  % Under modelling advantages on the Astrom system
close all  
Nsim=1000;
nb=30;
lambda=.1;  %0.1 in paper
N=50;
alpha0=0.99;                    % confidence level 99%
Calpha=erfinv(2*(alpha0)-1);    % Assuming AsN of V(T)
Calphachi=zeros(1,nb);
for m=1:-nb;
  Calphachi(m)=erfchi(m,alpha0);       % Assuming chi2 of V(T)
end;


%B0=fir1(nb,.5);
%B1=fir1(nb,.4);
%B0=dimpulse(.2/1.5*[0 1 .5],[1 -1.5 0.7],nb);% The "Astrom system", pole in 0.8376*exp(i*0.4592)
B0=filter(.2/1.5*[0 1 .5],[1 -1.5 0.7],[1; zeros(nb-1,1)]);
% The "Astrom system", pole in 0.8376*exp(i*0.4592)
i=sqrt(-1);
r=0.8376*exp(i*0.4);                     % Move the phase angle from 0.46 to 0.4
%B1=dimpulse(.2/1.5*[0 1 .5],[1 -real(r+r') real(r*r')],nb);
B1=filter(.2/1.5*[0 1 .5],[1 -real(r+r') real(r*r')],[1;zeros(nb-1,1)]);

figure(1),
book('plotfix')
plot([B0 B1])
title('Impulse response before and after change')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/umdet1.eps, end 

B0=B0';B1=B1';
order=[1:nb];
%order=[10 11 9 12 8 13 7 14 6 15 5 16 4 17 3 18 2 19 1 20];
th0=B0(order);
th1=B1(order);
alpha=zeros(1,nb);
beta=zeros(1,nb);
V0=zeros(nb,Nsim);
V1=zeros(nb,Nsim);
for m=1:nb;
  beta0(m)=power2(B0(1:m)',B1(1:m)',eye(m),lambda,N,alpha0);
end;

delta=.000001+((B1(order)-B0(order)).^2)';
M=(1:nb)';
limit=lambda/2/N*(1+sqrt(8*Calpha^2./M))./(1+lambda*(M+Calpha*sqrt(2*M))/N./cumsum(delta));

figure(2)
book('plotfix')
plot([delta limit])
xlabel('Model order m')
title('Variance/change trade-off')

figure(3)
book('plotfix')
plot([Calphachi' (M+Calpha*sqrt(2*M))])
title('Threshold for V(T) using chi-2 and Gaussian tests')
drawnow
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/umdet2.eps, end 

for n=1:Nsim;
  u=randn(N,1);
  e=randn(N,1);
  y0=filter(B0,1,u);
  y0n=y0+sqrt(lambda)*e;
  y1=filter(B1,1,u);
  y1n=y1+sqrt(lambda)*e;
  for m=1:nb;
    Phi=zeros(N,m);
    for l=1:m;
      Phi(:,l)=[zeros(order(l)-1,1);u(1:N-order(l)+1)];
    end;
    P=inv(Phi'*Phi);
    f1=Phi'*(y1n-y0);
    f0=Phi'*(y0n-y0);
    V0(m,n)=(f0'*P*f0-m*lambda)/sqrt(2*m*lambda^2);        % Gaussian test
    V1(m,n)=(f1'*P*f1-m*lambda)/sqrt(2*m*lambda^2);        % Gaussian test
%    V0(m,n)=f0'*P*f0/lambda;                               % chi-2 test
%    V1(m,n)=f1'*P*f1/lambda;                               % chi-2 test
    if V0(m,n) < Calpha; alpha(m)=alpha(m)+1; end;
    if V1(m,n) > Calpha; beta(m)=beta(m)+1; end;
  end;
end;
alpha=alpha/Nsim;
beta=beta/Nsim;

figure(4)
book('plotfix')
plot([alpha' beta'])
hold on
plot([alpha0*ones(size(beta0')) beta0'],'--')
hold off
title('Confidence level and power of test')
xlabel('Model order')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/umdet3.eps, end 

end



%ch6 Applications



if strcmp(arg,'all') | strcmp(arg,'sec6.4.2') | strcmp(arg,'sheep')
close all
%       Belching sheep
%      ================
sheep=[];
load  sheep
z=[sheep(:,2) sheep(:,1)];
% th=rarx(z,[5 5 0]);
%[e,R]=resid(z,th);

[th,lamhat,e]=adfilter(z,[5 5 0],1);
disp('VN = ')
1000*lamhat(length(lamhat))

nnn=0;
[jhat1,jtype,thseg1,lamseg1]=multihyp(e,nnn,100,[0 1 0]);

N=length(e);
g=zeros(N,1);
lambda=0.97;
g=filter([1-lambda 0],[1 -lambda],e.^2);

h1=figure(1);
subplot(211);
book('plotfix')
plot(1000*(1+sheep-2*[ones(N,1) zeros(N,1)]));
ax=axis;ax(2)=N;axis(ax);
title('Pressure and air flow')
subplot(212);
book('plotfix')
thplot(g,lamseg1);
ax=axis;  ax(2)=N; ax(4)=4e-3; axis(ax);
title('Filtered model residual')
xlabel('Sample number')
print -deps /home/rt/fredrik/books/detect/figures/sheep.eps
end



% ch7 77777777777


if strcmp(arg,'all') | strcmp(arg,'sec7.7.1') | strcmp(arg,'ekg')
  close all
  load ekg
  N=length(s);
  t=(1:N)';

  disp('Piecewise linear model')
  Phi=[ones(N,1) t];      
  [threc,jhat,thseg]=detectM([s Phi],-2,[1 0.01],.5,10); 
  [epsi,epsif]=simresid([s Phi],-2,thseg);

  jhat' 
  npar=size(Phi,2)*length(jhat)
  V=sum(epsif.^2)/N
  Vnorm=V*N/sum(y.^2)

  figure(1)
  subplot(211)
  book('plotfix')
  segplot([s-epsif s],jhat)
  axis([0 N -5 10])
  title('Piecewise linear model')
  xlabel(' ')
  subplot(212)
  book('plotfix')
  segplot([s-epsif s],jhat)
  axis([0 35 -5 10])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ekg1.eps, end

  disp('Piecewise quadratic model')
  Phi=[ones(N,1) t t.^2];      
  [threc,jhat,thseg]=detectM([s Phi],-2,[1 0.01],.5,10); 
  [epsi,epsif]=simresid([s Phi],-2,thseg);

  jhat' 
  npar=size(Phi,2)*length(jhat)
  V=sum(epsif.^2)/N
  Vnorm=V*N/sum(y.^2)
  figure(2)
  subplot(211)
  book('plotfix')
  segplot([s-epsif s],jhat)
  axis([0 N -5 10])
  title('Piecewise quadratic model')
  xlabel(' ')
  subplot(212)
  book('plotfix')
  segplot([s-epsif s],jhat)
  axis([0 35 -5 10])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ekg2.eps, end

end

if strcmp(arg,'all') | strcmp(arg,'ex6.7') | strcmp(arg,'speech')
  close all
  load speech
 
  [threc,jhat,thseg,gt]=detectM(z,4);

  figure(2)
  subplot(211)
  book('plotfix')
  p1=segplot(y,jhat);
  set([p1],'LineWidth',2);
  title('Speech signal and segmentation using an AR(2) model');
  xlabel('')
  jhat
  subplot(212)
  book('plotfix')
  p1=plot(thseg');
  set([p1],'LineWidth',2);
  xlabel('Time [samples]')
  title('Segmentation of AR(2) parameters');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/speech4.eps, end 
end


if strcmp(arg,'all') | strcmp(arg,'sec7.7.3') | strcmp(arg,'path')
close all

%               Segmentation of driven path
%              =============================

% These data were collected from test drives with a Volvo 850 GLT
% using sensor signals from the ABS system. From these data, the position
% of the car can be calculated very roughly.
%
% Load and look at data

load carpath
N=length(x1);
i=sqrt(-1);
phi=phase(i*diff(x2)'+diff(x1)')';

% The heading angle within each segment is modeled as a first order 
% polynomial in t, h(t) = c1 + c2 * t, and the segm function is used 
% with 10 parallel filters and required accuracy of the prediction errors
% corresponding to a variance of 0.05.
N=length(phi);
M=10;
nnn=-2;
t=(1:N)';

disp('detectM with lambda=0.05')
t0=clock;
[threc,jhat,thseg]=detectM([phi ones(size(t)) t],nnn,[1 0.05],.05,M);
consumedtime=etime(clock,t0)
estpath=thseg'.*[ones(size(t)) t]*[1;1];
jhat1=jhat;

figure(1)
book('plotfix')
h=plot(x1,x2,'-',x1(jhat),x2(jhat),'o');
set([h],'LineWidth',2);

axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x1(1),x2(1)-50,'t=0 s')
text(x1(N)+50,x2(N)+50,'t=225 s')
print -deps /home/rt/fredrik/books/detect/figures/path4.eps

figure(2)
book('plotfix')
p=plot(t,phi,'-',jhat,phi(jhat),'o',t,estpath,'-');
set([p(2:3)],'LineWidth',2);
title('Actual and segmented heading angle')
xlabel('Time [s]')
ylabel('Heading angle [rad]')
print -deps /home/rt/fredrik/books/detect/figures/path5.eps

disp('detectM with marginalized lambda')
t0=clock;
[threc,jhat,thseg]=detectM([phi ones(size(t)) t],nnn,[2],.05,M);
consumedtime=etime(clock,t0)
estpath=thseg'.*[ones(size(t)) t]*[1;1];
jhat2=jhat;

figure(3)
book('plotfix')
h=plot(x1,x2,'-',x1(jhat),x2(jhat),'o');
set([h],'LineWidth',2);

axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x1(1),x2(1)-50,'t=0 s')
text(x1(N)+50,x2(N)+50,'t=225 s')
print -deps /home/rt/fredrik/books/detect/figures/path6.eps

figure(4)
book('plotfix')
p=plot(t,phi,'-',jhat,phi(jhat),'o',t,estpath,'-');
set([p(2:3)],'LineWidth',2);
title('Actual and segmented heading angle')
xlabel('Time [s]')
ylabel('Heading angle [rad]')
print -deps /home/rt/fredrik/books/detect/figures/path7.eps

disp('Optimal detectM')
% Optimal segmentation (it takes a minute)?
  t0=clock;
%  [jhatopt,thsegopt,V,l,Nfilt]=optseg([phi ones(size(t)) t],nnn,[1 0.05],0.005,1e-6);
[threc,jhat,thseg]=detectM([phi ones(size(t)) t],nnn,[2],.05,N); %M=N
estpath=thseg'.*[ones(size(t)) t]*[1;1];
consumedtime=etime(clock,t0)
jhatopt=jhat;
  % Compare the estimated jump times
  jhat1',jhat2',jhatopt'
  % The estimates are almost identical!
  %

figure(5)
book('plotfix')
h=plot(x1,x2,'-',x1(jhat),x2(jhat),'o');
set([h],'LineWidth',2);

axis('equal')
xlabel('Meter')
ylabel('Meter')
title('Position')
text(x1(1),x2(1)-50,'t=0 s')
text(x1(N)+50,x2(N)+50,'t=225 s')
print -deps /home/rt/fredrik/books/detect/figures/path8.eps

figure(6)
book('plotfix')
p=plot(t,phi,'-',jhat,phi(jhat),'o',t,estpath,'-');
set([p(2:3)],'LineWidth',2);
title('Actual and segmented heading angle')
xlabel('Time [s]')
ylabel('Heading angle [rad]')
print -deps /home/rt/fredrik/books/detect/figures/path9.eps

% optseg does not work anymore
%  figure(4)
%  book('plotfix')
%  segplot(Nfilt',jhatopt);
%  title('Number of parallel filters and optimal segmentation') 
%  print -deps /home/rt/fredrik/books/detect/figures/path8.eps

%close([h1 h2])

end


if strcmp(arg,'all') | strcmp(arg,'sec7.7.3')  | strcmp(arg,'mcmcpath')
   close all
   rand('state',1)
   randn('state',1)
   load carpath
   [threc,jhat,thseg]=detectM(z,-2,[1 0.05],0.5);
   jhat

   [threc,kn]=gibbs(z,-2,[1 0.05],0.5,[7 50 5],[0 1 1],jhat);
   figure(2)
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mcmcpath1.eps, end
   figure(3)
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/mcmcpath2.eps, end
end


%ch8 8888888

if strcmp(arg,'all') | strcmp(arg,'ex8.1')

  P0=0.1*eye(4);
  P0(1,2)=0.07;
  P0(2,1)=0.07;
  x0=[0,0,1,1]';
  
  phi=(3*pi/4:-0.05:pi/4)';
  v=[sin(phi) cos(phi)];
  xo=[2+2*sqrt(2)*cos(phi) -2+2*sqrt(2)*sin(phi)];
  Q=0.2*eye(2);
  R=0.1*eye(2);
  B=[0 0; 0 0; 1 0; 0 1];
  A=[1 0 1 0;0 1 0 1; 0 0 1 0; 0 0 0 1];
  C=[1 0 0 0;0 1 0 0];
  x1=A*x0;
  P1=A*P0*A'+B*Q*B';
  x2=A*x1;
  P2=A*P1*A'+B*Q*B';
  x3=A*x2;
  P3=A*P2*A'+B*Q*B';
  
  figure(1)
  clf, book('plotfix')
  p1=plot(xo(:,1),xo(:,2));
  text(2.5,0.5,'x(t)')
  set(p1,'LineWidth',2)
  hold on
  p=plot(xo(10:10:30,1),xo(10:10:30,2),'o');

  phi=(0:0.1:6.3)';
  v=[sin(phi) cos(phi)];
  p0=plot(x0(1),x0(2),'o');
  vt=v*(sqrtm(P0(1:2,1:2)));
  plot(x0(1)+vt(:,1),x0(2)+vt(:,2))
  text(x0(1)+0.1,x0(2),'x(0|0)=x(0)')
  
  p1=plot(x1(1),x1(2),'o');
  vt=v*(sqrtm(P1(1:2,1:2)));
  plot(x1(1)+vt(:,1),x1(2)+vt(:,2))
  text(x1(1)+0.1,x1(2),'x(1|0)')

  p2=plot(x2(1),x2(2),'o');
  vt=v*(sqrtm(P2(1:2,1:2)));
  plot(x2(1)+vt(:,1),x2(2)+vt(:,2))
  text(x2(1)+0.1,x2(2),'x(2|0)')
  
  p3=plot(x3(1),x3(2),'o');
  vt=v*(sqrtm(P3(1:2,1:2)));
  plot(x3(1)+vt(:,1),x3(2)+vt(:,2))
  text(x3(1)+0.1,x3(2),'x(3|0)')

  set([p p0 p1 p2 p3],'LineWidth',2)

  plot([0 0.75],[0 0.75])
  plot([0.75-0.2 0.75],[0.75-0.1 0.75])
  plot([0.75-0.1 0.75],[0.75-0.2 0.75])
  axis('equal')
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfintro1.eps, end

  
  figure(2)
  clf, book('plotfix')
  p1=plot(xo(:,1),xo(:,2));
  text(2.5,0.5,'x(t)')
  set(p1,'LineWidth',2)
  hold on
  p=plot(xo(10:10:30,1),xo(10:10:30,2),'o');

  phi=(0:0.1:6.3)';
  v=[sin(phi) cos(phi)];
  p0=plot(x0(1),x0(2),'o');
  vt=v*(sqrtm(P0(1:2,1:2)));
  plot(x0(1)+vt(:,1),x0(2)+vt(:,2))
  text(x0(1)+0.1,x0(2),'x(0|0)=x(0)')

  vo1=(xo(10,:)-xo(9,:))/0.1
  sqrt(sum(vo1.^2))
  xo1=[xo(10,:) vo1]';
  y=xo(10,:)';
  
  K=P1*C'*inv(R+C*P1*C');
  P1g1=P1-P1*C'*inv(R+C*P1*C')*C*P1;
  x1g1=x1+K*(y-C*x1);
  
  p1=plot(x1(1),x1(2),'o');
  vt=v*(sqrtm(P1(1:2,1:2)));
  plot(x1(1)+vt(:,1),x1(2)+vt(:,2))
  text(x1(1)+0.1,x1(2),'x(1|0)')

  p2=plot(x1g1(1),x1g1(2),'o');
  vt=v*(sqrtm(P1g1(1:2,1:2)));
  plot(x1g1(1)+vt(:,1),x1g1(2)+vt(:,2))
  text(x1g1(1)+0.1,x1g1(2),'x(1|1)')

  set([p p0 p1 p2],'LineWidth',2)
  axis('equal')
%  axis([-0.5 2 -0.5 2])
  hold off
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfintro2.eps, end
  
  
  P0=zeros(4);
  Q=1*eye(2);
  R=1*eye(2);
  xhat=x0;
  P=P0;

  % t=1
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(10,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);  %K=0: Note the time delay caused by A,B,Q!
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

  % t=2
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(20,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

  % t=3
  % Time update
  xhat=A*xhat;
  P=A*P*A'+B*Q*B';
  yhat=C*xhat
  % Measurement update
  epsi=xo(30,:)'-C*xhat;
  K=P*C'*inv(C*P*C'+R);
  xhat=xhat + K * epsi;
  P=P-K*C*P;
  yhat=C*xhat

end

if strcmp(arg,'all') | strcmp(arg,'ex8.2')
  %close all
  rand('seed',0);
  N=70;
  lam=3; % Measurement noise scaling
  nnn='nnnplane'; %local version
  % changed x0=[50000,20000,-200,0,0]';
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04;...
       0.039;...
       -0.039]';   
  jumps=[10 16 35 41 55 61];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');

  figure(1);
  h1=radarplot(y,x0);
  legend(h1,'Measured position','True position')
  hold on
  plot([x0(1,end) x0(1,end)],[x0(2,end)-2000 x0(2,end)-12000],'--')
  plot([x0(1,end)-500 x0(1,end)-500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)+500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-12000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-2000 x0(2,end)-2000],'-')
  plot([x0(1,end)-500 x0(1,end)+500],[x0(2,end)-12000 x0(2,end)-12000],'-')
  plot([0 x0(1,20)],[0 x0(2,20)])
  plot([x0(1,20)-2000 x0(1,20)],[x0(2,20)-1000 x0(2,20)])
  plot([x0(1,20)-1440 x0(1,20)],[x0(2,20)-1800 x0(2,20)])
  plot([0 5000],[0 0],'--')
  hold off
  h3=text(4000,1000,'\theta');
  h4=text(12000,12000,'R');
  h1=xlabel('X');
  h2=ylabel('Y');
  set([h1 h2 h3 h4],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingex1.eps, end

  figure(2)
  t=1:length(x0);
  subplot(211)
  p1=plot(t,x0(1,:),'-',t,x0(2,:),'--');
  hold on
  plot(t,y(:,1),'-',t,y(:,2),'--');
  hold off
  h1=ylabel('Position [m]');
  subplot(212)
  p2=plot(t,x0(3,:),'-',t,x0(4,:),'--');
  hold on
  plot(t,[0;diff(y(:,1))]/5,'-',t,[0;diff(y(:,2))]/5,'--');
  axis([0 t(end) -300 300])
  hold off
  h2=ylabel('Velocity [m/s]');
  h3=xlabel('Time [sample]');
  legend([p2],'X','Y')  
  set([p1 p2],'LineWidth',2)
  set([h1 h2 h3],'FontSize',fontsize)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingex2.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'ex8.3')
close all
randn('state',0)
N=20;
a=0.9;
b=0.1;
c=3;
sigma=0.25;

x(1)=1;
x0(1)=1;

for t=1:N;
  x0(t+1)=a*x0(t);
  x(t+1)=a*x(t)+b*randn(1,1);
  y0(t)=c*x0(t);
  y(t)=c*x(t);
  yn(t)=y(t)+sigma*randn(1,1);
end

figure(1)
subplot(211)
book('plotfix')
h(1)=plot(x0,'r:');
hold on
h(2)=plot(x,'b-');
set(h,'LineWidth',2)
legend(h,'Noise-free x^0_t','Simulated x_t');
title('State')

subplot(212)
book('plotfix')
h(1)=plot(y0,'r:');
hold on
h(2)=plot(y,'b-');
h(3)=plot(yn,'g--');
hold off
set(h,'LineWidth',2)
legend(h,'Output with e_t=v_t=0',...
'Output with e_t=0','Output y_t')
title('Output')
xlabel('Time [samples]')
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfintro3.eps, end

xhatp(1)=0;
Pp(1)=10;
for t=1:N;
%t,yn(t),xhatp(t)
  epsi=yn(t)-c*xhatp(t);
  S(t)=c^2*Pp(t)+sigma^2;
  xhat(t)=xhatp(t)+Pp(t)*c*epsi/S(t);
  P(t)=Pp(t)-Pp(t)^2*c^2/S(t);
  xhatp(t+1)=a*xhat(t);
  Pp(t+1)=a^2*P(t)+b^2;
end
figure(2)
book('plotfix')
h(1)=plot(x0,'b:');
hold on
h(2)=plot(x,'b-');
h(3)=plot(xhat,'m-');
h(4)=plot(xhatp,'g--');
set(h(3:4),'LineWidth',3)
legend(h(1:4),'Noise-free x^0_t','Simulated x_t','Filtered x_t','Predicted x_t');
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfintro4.eps, end

figure(3)
book('plotfix')
h(1)=plot(x0,'b:');
hold on
h(2)=plot(x,'b-');
h(3)=plot(xhat,'g-');
h(4)=plot(xhat+2*sqrt(P),'g--');
h(5)=plot(xhat-2*sqrt(P),'g--');
set(h(3:5),'LineWidth',3)
legend(h(1:4),'Noise-free x^0_t','Simulated x_t','Filtered x_t','Confidence interval for x_t');
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfintro5.eps, end

end  



if strcmp(arg,'all') | strcmp(arg,'ex8.4')
close all
randn('state',0)
nnn='nnnDCm0';
N=80;
f1=[0;0];
f2=[0;0];
TH=[f1 -f1 f2 -f2];
jumps=[3*N/8 4*N/8 5*N/8 6*N/8];
u=[hamming(N/2); hamming(N/2)];
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
%d=zeros(N,1);
[y,x0]=simchange([u d],nnn,jumps,TH,1);

figure(1)
book('plotfix')
h(1)=plot([y]);
hold on
h(2)=plot(x0(1,:),'-');
h(3)=plot(x0(2,:),'--');
hold off
set(h(2:3),'LineWidth',2)
legend(h,'Measurement y','True position','True velocity',2)
h=title('True state and measurements');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/DCkf1.eps, end 

[xhat,lamhat,Epsi]=adfilter([y u],nnn,1);
[xhat1,yhat,P1]=adkalman([y u],nnn,1,[1 0]);
[xhat2,yhat,P2]=adkalman([y u],nnn,1,[2 0]);
[xhat3,yhat,P3]=adkalman([y u],nnn,1,[3 0]);
[xhatp1,yhat,Pp1]=adkalman([y u],nnn,1,[1 1]);
[xhatp2,yhat,Pp2]=adkalman([y u],nnn,1,[2 1]);
[xhatp3,yhat,Pp3]=adkalman([y u],nnn,1,[3 1]);
[xhats,yhat,Ps]=adkalman([y u],nnn,1,[4]);

figure(2),
book('plotfix')
h1=plot([xhat1'],'b-');
hold on
h2=plot([xhat2'],'b--');
h3=plot([xhat3'],'b.-');
h4=plot([xhatp1'],'g-');
h5=plot([xhatp2'],'g--');
h6=plot([xhatp3'],'g.-');
h7=plot([xhats'],'m-');
h8=plot([x0'],'r-');
hold off
legend([h1(1) h2(1) h3(1) h4(1) h5(1) h6(1) h7(1) h8(1)],...
        'Stationary KF','KF','Sqrt KF','Stationary KP',...
        'KP','Sqrt KP','Smoothing','True');

xhat=xhats;

figure(3),
book('plotfix')
plot([P1(1,1)*ones(N,1) Pp1(1,1)*ones(N,1) P2(1:2:end,1) Pp2(1:2:end,1) P3(1:2:end,1) Pp3(1:2:end,1) Ps(1:2:end,1)])
xhat=xhat1;
%[xhat1' xhat2' xhat3' P1(1,1)*ones(N,1) P2(1:2:end,1) P3(1:2:end,1) ]

pause

figure(4)
book('plotfix')
h(1)=plot([y],'b');
hold on
h(2)=plot(x0(1,:),'g-');
h(3)=plot(xhat(1,:),'r--');
hold off
set(h(2:3),'LineWidth',2)
legend(h,'Measurement y','True position','KF estimate',2);
h=title('Position');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/DCkf2.eps, end

figure(5)
book('plotfix')
h(1)=plot(diff([0;y]),'b');
hold on
h(2)=plot(x0(2,:),'g-');
h(3)=plot(xhat(2,:),'r--');
hold off
set(h(2),'LineWidth',2)
set(h(3),'LineWidth',2)
legend(h,'Estimate as diff(y)','True velocity','KF estimate',2);
h=title('Velocity');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/DCkf3.eps, end

end

if strcmp(arg,'all') | strcmp(arg,'ex8.7')
% Numerical illustration of square root filtering
close all
randn('state',0)
nnn='nnnDCm0';
N=2;
u=[hamming(N/2); hamming(N/2)];
d=zeros(N,1);
[y,x0]=simchange([u d],nnn,N,[0;0],1);

figure(1)
book('plotfix')
h(1)=plot([y]);
hold on
h(2)=plot(x0(1,:),'-');
h(3)=plot(x0(2,:),'--');
hold off
set(h(2:3),'LineWidth',2)
legend(h,'Measurement y','True position','True velocity',2)
h=title('True state and measurements');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/kfsqrt1.eps, end 

% Extract from adkalman
    [n,xhat,P]=feval(nnn,0,0,0,0);
    [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,1,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,1,xhat,0,6);
    A,Bu,C
    nx=n(1);
    nu=n(2);
    nv=n(3);
    nd=n(4);
    nth2=n(5);
    ny=n(6);
    xhat,P
    Ph=sqrtm(P)
    for t=1:N,
      qrmatrix = [sqrtm(R), C*A*Ph, zeros(ny,nv);...
                  zeros(nx,ny), A*Ph, Bv*sqrtm(Q)];
      [Qfact,Rfact]=qr(qrmatrix');
      Rfact=Rfact';
      Ph=Rfact(ny+1:ny+nx,ny+1:ny+nx);
      Sh=Rfact(1:ny,1:ny);
      Kh=Rfact(ny+1:ny+nx,1:ny);
      epsi=y(t,:).'-C*xhat;
      if ~isempty(u) & ~isempty(Du);
        epsi=epsi-Du*u(t,:).';
      end

      epsin=inv(Sh)*epsi;
      xhat=A*xhat+Kh*epsin;
      if ~isempty(u) & ~isempty(Bu);
        xhat=xhat + Bu*u(t,:).';
      end
      P=Ph*Ph';
 qrmatrix,Rfact,Ph,Kh,Sh,P=Ph*Ph',K=Kh/Sh,S=Sh*Sh
    end


end

if strcmp(arg,'all') | strcmp(arg,'ex8.11')
  randn('state',0);
  close all
  N=100;
  lam=1;
  nnn='nnnplane';
  global NNNmethod
  NNNmethod=5;

  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.025;...
       +0.025;...
       0.04;...
       -0.04]';   
  jumps=[30 40 70 80];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'no');

%  y=y+50*e;

  figure(1);
  book('plotfix')
  radarplot(y,x0)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingcoord1.eps, end

  q1=10; q2=100; q3=0.1; q4=1; q5=1e-4;
  kf=[2 1];
  NNNmethod=1; % four states
  [xhat1,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q1,kf); 
  NNNmethod=2; % four states
  [xhat2,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q2,kf); 
  NNNmethod=3; % six states
  [xhat3,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q3,kf); 
  NNNmethod=4; % six states
  [xhat4,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q4,kf); 
  NNNmethod=5; % five states
  [xhat5,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q5,kf); 

  figure(2)
  book('plotfix')
  h(1)=plot([0*1e5+sum((x0(1:2,:)-xhat1(1:2,:)).^2)],'b-');
  hold on
  h(2)=plot([1*1e5+sum((x0(1:2,:)-xhat2(1:2,:)).^2)],'b--');
  h(3)=plot([2*1e5+sum((x0(1:2,:)-xhat3(1:2,:)).^2)],'b-.');
  h(4)=plot([3*1e5+sum((x0(1:2,:)-xhat4(1:2,:)).^2)],'b:');
  h(5)=plot([4*1e5+sum((x0(1:2,:)-xhat5(1:2,:)).^2)],'b.-');
  set(h,'LineWidth',1);
  axis([0 N 0 1.5e6])
  ax=axis;
  plot([jumps' jumps']',[ax(3)*ones(size(jumps')) ax(4)*ones(size(jumps'))]','r:')
  hold off
  legend(h,'Four state linear model',...
           'Four state linear model, CT Q',...
           'Six state linear model',...
           'Six state linear model, CT Q',...
           'Five state non-linear CT model');

  hd(1)=ylabel('MSE');
  hd(2)=xlabel('Sample number');
  hd(3)=title('One step ahead prediction')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingcoord2.eps, end

  figure(5)
  book('plotfix')
  h(5)=plot([sum((x0(1:2,:)-xhat5(1:2,:)).^2)],'c');

  figure(3)
  book('plotfix')
  h(1:2)=plot([xhat1(3:4,:)'],'b-');
  hold on
  h(3:4)=plot([xhat2(3:4,:)'],'b--');
  h(5:6)=plot([xhat3(3:4,:)'],'b-.');
  h(7:8)=plot([xhat4(3:4,:)'],'b:');
  h(9:10)=plot([xhat5(3:4,:)'],'b.-');
  h0=plot([x0(3:4,:)'],'b-');
  set(h,'LineWidth',1);
  set(h0,'LineWidth',2);
  axis([0 N -300 500])
  ax=axis;
  plot([jumps' jumps']',[ax(3)*ones(size(jumps')) ax(4)*ones(size(jumps'))]','r-')
  hold off
  legend(h(1:2:9),'Four state linear model',...
           'Four state linear model, CT Q',...
           'Six state linear model',...
           'Six state linear model, CT Q',...
           'Five state non-linear CT model');

  hd(1)=ylabel('Velocity');
  hd(2)=xlabel('Sample number');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingcoord3.eps, end

  figure(4)
  book('plotfix')
  h(5:6)=plot([xhat3(5:6,:)'],'b-');
  hold on
  h(7:8)=plot([xhat4(5:6,:)'],'b--');
  set(h,'LineWidth',1);
%  axis([0 N 0 1e6])
  ax=axis;
  plot([jumps' jumps']',[ax(3)*ones(size(jumps')) ax(4)*ones(size(jumps'))]','r:')
  hold off
  legend(h(5:2:8),...
           'Six state linear model',...
           'Six state linear model, CT Q'); %,...
  hd(1)=ylabel('Acceleration');
  hd(2)=xlabel('Sample number');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/trackingcoord4.eps, end


  disp([sqrt(sum(sum((x0(1:2,:)-xhat1(1:2,:)).^2))/N),...
        sqrt(sum(sum((x0(1:2,:)-xhat2(1:2,:)).^2))/N),...
        sqrt(sum(sum((x0(1:2,:)-xhat3(1:2,:)).^2))/N),...
        sqrt(sum(sum((x0(1:2,:)-xhat4(1:2,:)).^2))/N),...
        sqrt(sum(sum((x0(1:2,:)-xhat5(1:2,:)).^2))/N)])
  disp([q1 q2 q3 q4 q5])
        
  [xhat1,jhat1,xseg1,GT,ta]=detect1(y,nnn,1,[2 3 0.2],10);

  jhat1
end


if strcmp(arg,'all') | strcmp(arg,'ex8.14')
  randn('state',0);
  close all
  N=50;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;

  NNNmethod=1;
  figure(1);
  book('plotfix')
  radarplot(y,x0)
  [xhat,yhat,P,Epsi,Epsin]=adkalman(yn,nnn,1,[2 1]); % Time-varying KF (filter)
  h=5;
  nu=0.25;
  [xhat1,jhat1,xseg1,GT,ta]=detect1(yn,nnn,1,[2 h nu],10);
  jhat1
  figure(2);
  book('plotfix')
  radarplot(y,xhat);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radardetect1a.eps, end
  figure(3);
  book('plotfix')
  radarplot(y,xhat1);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radardetect1b.eps, end
  N=length(y);
  e=(yn'-xhat(1:2,1:N))';
  
  figure(4)
  subplot(311)
  book('plotfix')
  plot(Epsi')
  hold on
  plot([1;N],[0;0],'r--')
  hold off
  handle(1)=ylabel('\epsilon_t');
  axis([0 N -1000 1000])

  subplot(312)
  book('plotfix')
  plot(sum(Epsin.^2))
  hold on
  plot([1;N],[5;5],'r--')
  hold off
  axis([0 N 0 100])
%  axis([0 length(e) 0 2e6])
  handle(2)=ylabel('\epsilon_t^T S_t^{-1} \epsilon_t');

  subplot(313)
  book('plotfix')
  plot(GT)
  hold on
  plot([1;N],[h;h],'r--')
  hold off
  axis([0 N 0 10])
  handle(3)=ylabel('g_t');
  %set(handle,'FontSize',14)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radardetect1c.eps, end

end

if strcmp(arg,'all') | strcmp(arg,'sec8.10') | strcmp(arg,'QRest')

% tracking application: estimate scaling Q, and Q,R, respectively.

randn('state',0)
N=36;
sigma1o=0; % No velocity
sigma2o=1;
sigma3o=3;

% Simulation of state space model
% x(t+1) = x(t) + v(t) + sigma1*w1(t)
% v(t+1) =        v(t) + sigma2*w2(t)
% y(t)   = x(t)        + sigma3*e(t)


w1=sigma1o*randn(N,1);
w2=sigma2o*randn(N,1);
e=sigma3o*randn(N,1);

v=filter([1 0],[1 -1],w2);
x=filter([0 1],[1 -1],w1+v);
y=x+e;

figure(1)
book('plotfix')
p=plot(y);
set(p,'LineWidth',2);

%-----Identification of variances------

% Form impulse response matrices from w1, w2, e
t=(1:N)';
H1=toeplitz(t,[1 zeros(1,N-1)]);
H2=toeplitz(ones(N,1),[1 zeros(1,N-1)]);
H3=eye(N);
% Form linear regression
r=covf(y,N);
R=toeplitz(r,[r(1) zeros(1,N-1)]);
Phi1=H1*H1';
Phi2=H2*H2';
Phi3=H3*H3';
% y = N(0,Phi1*sigma1+Phi2*sigma2+Phi3*sigma3)

% First, estimate just qv in Q
sigma1=0; 
sigma2= 1:0.1:10;
sigma3=1; 

for i=1:length(sigma1);
  for j=1:length(sigma2);
    P=(sigma3(i)^2*H3*H3'+ sigma2(j)^2*H2*H2');
    V(i,j)=y'*inv(P)*y+log(det(P));
  end
end
figure(2)
book('plotfix')
p=plot(sigma2,V);
set(p,'LineWidth',2);
xlabel('\sigma_v')
ylabel('V(\sigma_v)')
title('Minimization w.r.t. velocity noise \sigma_v')
drawnow
  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/QRest1.eps, end

[Vmin,jhat]=min(V);
sigmahat=sigma2(jhat)
Vmin

% Second, estimate both qv in Q and R=r
sigma1=0;  % 1:0.1:10;
sigma2=2:0.2:10;
sigma3=0.2:0.2:2;
clear V

for i=1:length(sigma3);
  for j=1:length(sigma2);
    P=(sigma3(i)^2*H3*H3' + sigma2(j)^2*H2*H2');
    V(i,j)=y'*inv(P)*y+log(det(P));
  end
end

[Vi,ihat]=min(V);
[Vmin,jhat]=min(Vi);
Vmin
ihat=ihat(jhat);
sigmavhat=sigma2(jhat)
sigmaehat=sigma3(ihat)

scaling=sqrt( (y'*inv(sigmaehat^2*H3*H3'+ sigmavhat^2*H2*H2')*y)/N)

figure(3)
book('plotfix')
mesh(sigma2,sigma3,V);
colormap(gray)
xlabel('\sigma_v')
ylabel('\sigma_e')
hold on
p=plot3(sigma2(jhat),sigma3(ihat),1+V(ihat,jhat),'b*');
set(p,'LineWidth',4);
hold off
view([150,30])

  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/QRest2.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'sec8.8.4')
close all

figure(1)
book('plotfix')

T=logspace(-3,1,50);
v=100;
h1=loglog(T,(v^2*(T.^4+T.^6/9)+T.^4+4*T.^2)./(v^2*(T.^6/9)+T.^2+T.^4/2),'-');
hold on
v=65;
h2=loglog(T,(v^2*(T.^4+T.^6/9)+T.^4+4*T.^2)./(v^2*(T.^6/9)+T.^2+T.^4/2),'--');
v=200;
h3=loglog(T,(v^2*(T.^4+T.^6/9)+T.^4+4*T.^2)./(v^2*(T.^6/9)+T.^2+T.^4/2),'-.');
hold off
xlabel('T [s]')
set([h1 h2 h3],'Linewidth',2)
legend([h1 h2 h3],'v=100','v=65','v=200')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/comppvcv.eps, end

randn('seed',11)
sigma=0.1;

ta=0:.01:5;
va=randn(length(ta),1);
va=cumsum(va);

tb=[0 1 1 2 2 3 3 4 4 5];
vb=va(100:100:500);
vb=kron(vb,[1;1]);

tc=[0 0.1 1 1.1 2 2.1 3 3.1 4 4.1];
vc=vb(1:2:10);

figure(2)
book('plotfix')
p1=plot(ta,-va);
set([p1],'Linewidth',1)
hold on
p1=plot(tb,-5-vb);
set([p1],'Linewidth',2)
for i=1:5;
  p1=plot([i-1 i-1],[0 vc(i)]);
  p2=plot([i-1+.1 i-1+.1],[0 vc(i)]);
  p3=plot([i-1-.05 i-1+.05],[vc(i)-1*sign(vc(i)) vc(i)+1*sign(vc(i))]);
  p4=plot([i-1+.05 i-1+.15],[vc(i)+1*sign(vc(i)) vc(i)-1*sign(vc(i))]);
  set([p1 p2 p3 p4],'Linewidth',2)
end
axis([0 5 -25 25])
xlabel('Time [samples]')
hold off
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/noises.eps, end

end


if strcmp(arg,'all') | strcmp(arg,'ex8.x') | strcmp(arg,'planepath')
  close all
  load planepath
  figure(1)
  subplot(211)
  book('plotfix')
  plot(y(:,1))
  h1=title('Measured velocities from INS');
  h2=ylabel('v_E');
  subplot(212)
  book('plotfix')
  plot(y(:,2))
  h1=ylabel('v_N');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/planepath1.eps, end 

  figure(2)
  book('plotfix')
  h=plot(p(:,1),p(:,2));
  set(h,'Linewidth',2)
  xlabel('East-West')
  ylabel('North-South')
  h1=title('Position integrated from INS');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/planepath2.eps, end 

end

if strcmp(arg,'all') | strcmp(arg,'ex8.y')
  close all
  load highway
  N=length(y);
  nnn='nnnplane';
  global NNNmethod
  NNNmethod=1; % four states
  kf=[2 1];
  q1=1e-3;
  [xhat1,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q1,kf,1e10*eye(4),zeros(4,1)); 
  
  figure(1)
  book('plotfix')
  h(1)=plot(y(:,1),y(:,2));
  ax=axis;
  hd=title('Car position from computer vision system');
  hold on
  h(2)=plot(xhat1(1,:),10+xhat1(2,:),'k--');
  axis(ax);
  set(h,'LineWidth',2);
  hold off
  xlabel('East-West Position')
  ylabel('North-South Position')
  legend(h,'Original position','Kalman filtered position',2)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highway1.eps, end

  figure(2)
  book('plotfix')
  h(1)=plot(diff(y(:,1)),'b-');
  hold on
  h(2)=plot(diff(xhat1(1,:)),'k--');
  set(h,'LineWidth',2);
  hold off
  set(gca,'Xlim',[0 N])
  ylabel('Velocity')
  xlabel('Time [samples]')
  legend(h,'Original data','Kalman filtered data')
  hd=title('Differentiated position in west-east data');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highway2.eps, end

  figure(3)
  book('plotfix')
  [Nhist,Xhist]=hist(Epsi(20:N),40);
  bar(Xhist,Nhist,'w')
  xlabel('Residual')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highway3.eps, end
  
end


if strcmp(arg,'all') | strcmp(arg,'ex8.12.3') | strcmp(arg,'GPS')
  % GPS
  %close all
  figure(1)
  book('plotfix')
  phi=0:0.1:2*pi+0.1;
  R=1;
  xp=[0 1 2];
  yp=[1 0 1];
  x=R*cos(phi);
  y=R*sin(phi);
  r=1;
  x1=x+r*xp(1);
  y1=y+r*yp(1);
  x2=x+r*xp(2);
  y2=y+r*yp(2);
  x3=x+r*xp(3);
  y3=y+r*yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  set(p,'Linewidth',3)
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',3)
  hold off  
  for i=1:3
    h=text(xp(i)+0.1,yp(i),['S',num2str(i)]);
    set(h,'FontSize',fontsize)
  end
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/gps1.eps,  end

  set(p,'Linewidth',10)
  set(p,'Color',[0.6 0.6 0.6])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/gps4.eps,  end

  figure(2)
  book('plotfix')
  r=0.9;
  x1=r*x+xp(1);
  y1=r*y+yp(1);
  x2=r*x+xp(2);
  y2=r*y+yp(2);
  x3=r*x+xp(3);
  y3=r*y+yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  set(p,'Linewidth',3)
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',3)
  hold off  
  for i=1:3
    h=text(xp(i)+0.1,yp(i),['S',num2str(i)]);
    set(h,'FontSize',fontsize)
  end
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/gps2.eps,  end

  figure(3)
  book('plotfix')
  r=1.1;
  x1=r*x+xp(1);
  y1=r*y+yp(1);
  x2=r*x+xp(2);
  y2=r*y+yp(2);
  x3=r*x+xp(3);
  y3=r*y+yp(3);
  p=plot(x1,y1,'k',x2,y2,'k',x3,y3,'k');
  hold on
  p1=plot(xp,yp,'ok');
  set(p1,'Linewidth',3)
  hold off  
  for i=1:3
    h=text(xp(i)+0.1,yp(i),['S',num2str(i)]);
    set(h,'FontSize',fontsize)
  end
  set(p,'Linewidth',3)
  axis([-1.2 3.2 -0.2 2.2])
  axis('equal')
  axis('off')
  if saveplots==1;  print -deps /home/rt/fredrik/books/detect/figures/gps3.eps,  end
end

if strcmp(arg,'all') | strcmp(arg,'OS')
  % GPS
  %close all
  figure(1)
  book('plotfix')
  phi=0:0.1:2*pi+0.1;
  R=1;
  xp=[0 1 2 3 4];
  yp=[1 0 1 0 1];
  x=R*cos(phi);
  y=R*sin(phi);
  x1=x+xp(1);
  y1=y+yp(1);
  x2=x+xp(2);
  y2=y+yp(2);
  x3=x+xp(3);
  y3=y+yp(3);
  x4=x+xp(4);
  y4=y+yp(4);
  x5=x+xp(5);
  y5=y+yp(5);
  p=plot(x1,y1,x2,y2,x3,y3,x4,y4,x5,y5);
  set(p,'Linewidth',10)
  set(p(1),'Color','b')
  set(p(2),'Color','y')
  set(p(3),'Color','k')
  set(p(4),'Color','g')
  set(p(5),'Color','r')
  axis([-1.2 5.2 -0.2 2.2])
  axis('equal')
  axis('off')
end


% ch9 999999999999



if strcmp(arg,'all') | strcmp(arg,'ex9.3')
close all
randn('state',0)
nnn='nnnDCm0';
N=160;
f1=[5;0];
f2=[0;5];
TH=[f1 -f1 f2 -f2];
jumps=[3*N/8 4*N/8 5*N/8 6*N/8];
u=[hamming(N/2); hamming(N/2)];
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
d=zeros(N,1);
[y,x0]=simchange([u d],nnn,jumps,TH,1,'pulse');

figure(1)
subplot(311)
book('plotfix')
h(1)=plot([y]);
hold on
h(2)=plot(x0(1,:),'-');
h(3)=plot(x0(2,:),'--');
hold off
set(h(2:3),'LineWidth',2)

adg=1;
[xhat,lamhat,Epsi]=adfilter([y u],nnn,adg);
h=10;
L=10;
[jhat,ta,LR,xhatglr,NUhat,mu]=glr([y u],nnn,adg,L,h,1000);
jhat,jumps,ta,NUhat

hdum=plot(LR);
set(hdum,'LineWidth',2)
hold on
hdum=plot(h*ones(N,1));
hold off
set(hdum,'LineWidth',2)
h=title('Test statistic and threshold');

%figure(2)
subplot(312)
book('plotfix')
hdum(1)=plot(xhat(2,:)','b--'); 
hold on
hdum(2)=plot(xhatglr(2,:)','b-');
hdum(3)=plot(x0(2,:)','b-');
hold off
ylabel('Angular velocity');
set(hdum([1 2]),'LineWidth',2)
%h=legend(hdum(1:3),'Kalman filter estimate','GLR estimate','True state',2);

subplot(313)
book('plotfix')
hdum(4)=plot(xhat(1,:)','b--'); 
hold on
hdum(5)=plot(xhatglr(1,:)','b-');
hdum(6)=plot(x0(1,:)','b-');
hold off
ylabel('Angle');
set(hdum([4 5]),'LineWidth',2)
h=legend(hdum(4:6),'KF estimate','GLR estimate','True state',2);

if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/DCglr1.eps, end 
end


% ch10 10101010

if strcmp(arg,'all') | strcmp(arg,'ex10.9')
  randn('state',1);
  rand('state',1);
  close all
  N=50;
  lam=0;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  nnn='nnnplane';
  global NNNmethod
  NNNmethod=5;
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+300*e;
  figure(1);
  book('plotfix')
  h=radarplot(yn,x0);
  h2=legend(h,'Measured position','True position',4);
  h1=title('Measurements');
%  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4.eps, end

  NNNmethod=1;
  lam=10;

  [xhat1,lamhat,Epsi]=adfilter(yn,nnn,1/lam);
  figure(2)
  book('plotfix')
  h=radarplot(y,xhat1);
  h1=title('Kalman filter');
  h2=legend(h,'Measured position','Estimated position',4);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking1.eps, end

  [xhat2,jhat2,xseg,GT,ta]=detect1(yn,nnn,2,[1 3 1],lam);
  figure(3)
  book('plotfix')
  plot(GT)
  % save,return
  [tmp,jhat3,tmp]=detectM(yn,nnn,[1 lam],0.1,[10 7 0]);
  [xhat3,jhat3,xseg,lamseg,Alfa,Xn]=detectM(yn,nnn,[1 lam],0.1,[1;jhat3(:)]);
  figure(13)
  book('plotfix')
  h=radarplot(y,xhat3,jhat3);
  h2=legend(h,'Measured position','Estimated position',4);
  h1=title('Filter bank');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking2.eps, end
 
  MC=100;
  Jhat2mc=[];
  Jhat3mc=[];
  for i=1:MC
    emc=randn(N,2);
    ymc=y+300*emc;
    [tmp,jhat2mc,tmp]=detect1(ymc,nnn,2,[1 3 1],lam);
    Jhat2mc=[Jhat2mc jhat2mc(:)'];
    [tmp,jhat3mc,tmp]=detectM(ymc,nnn,[1 lam],0.1,[10 7 0]);
    Jhat3mc=[Jhat3mc jhat3mc(:)'];
  end

  figure(10)
  book('plotfix')
  [Nhist,Xhist]=hist(Jhat2mc,1:N);
  bar(Xhist,Nhist,'w')
  h1=title(['Histogram over KF whiteness test using ',num2str(MC),' Monte Carlo runs']);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking3.eps, end
  figure(11)
  book('plotfix')
  [Nhist,Xhist]=hist(Jhat3mc,1:N);
  bar(Xhist,Nhist,'w')
  h1=title(['Histogram over KF filter bank using ',num2str(MC),' Monte Carlo runs']);
  drawnow
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking4.eps, end

  %figure(4)
  [xhat4,jhat4mc,jhat4]=gibbs(yn,nnn,[1 lam],0.1,[2 MC 5],[0 1 0]);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking5.eps, end
  %figure(5)
  [xhat5,jhat5mc,jhat5]=gibbs(yn,nnn,[1 lam],0.1,[5 MC 5],[0 1 0]);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking6.eps, end

  figure(6)
  book('plotfix')
  radarplot(y,xhat4,jhat4)
  title('Gibbs-Metropolis, n=2')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking7.eps, end

  figure(7)
  book('plotfix')
  radarplot(y,xhat5,jhat5)
  h1=title('Gibbs-Metropolis, n=5');
  h2=legend(h,'Measured position','Estimated position',4);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex10tracking8.eps, end

  jhatems=[15;(32:40)'];
  [xhatems,jhat,xseg,lamseg,Alfa,Xn]=detectM(yn,nnn,[1 lam],0.1,[1;jhatems]);
  radarplot(y,xhatems,jhatems);
  title('EMS')
%  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4i.eps, end

e=yn-y; e=e(11:50,:);
e1=xhat1([1 3],11:50)'-x0([1 3],11:50)';
e2=xhat2([1 3],11:50)'-x0([1 3],11:50)';
e3=xhat3([1 3],11:50)'-x0([1 3],11:50)';
e4=xhat4([1 3],11:50)'-x0([1 3],11:50)';
e5=xhat5([1 3],11:50)'-x0([1 3],11:50)';
eems=xhatems([1 3],11:50)'-x0([1 3],11:50)';
disp(['Measurement RMSE:  ',num2str(sqrt(sum(diag(e'*e))/length(e)))]);
disp(['adfilter RMSE:     ',num2str(sqrt(sum(diag(e1'*e1))/length(e)))]);
disp(['detect1  RMSE:     ',num2str(sqrt(sum(diag(e2'*e2))/length(e))),' and jump time(s): ',cdnum2str(jhat2)]);
disp(['detectM  RMSE:     ',num2str(sqrt(sum(diag(e3'*e3))/length(e))),' and jump time(s): ',cdnum2str(jhat3)]);
disp(['gibbs    RMSE:     ',num2str(sqrt(sum(diag(e4'*e4))/length(e))),' and jump time(s): ',cdnum2str(jhat4)]);
disp(['gibbs    RMSE:     ',num2str(sqrt(sum(diag(e5'*e5))/length(e))),' and jump time(s): ',cdnum2str(jhat5)]);
disp(['EMS      RMSE:     ',num2str(sqrt(sum(diag(eems'*eems))/length(e))),' and jump time(s): ',cdnum2str(jhatems)]);

  figure(7);
  book('plotfix')
  plot([e e1 e2 e3 e4])
  figure(8);
  book('plotfix')
  plot([x0(2,:)' xhat1(2,:)' xhat2(2,:)' xhat3(2,:)' xhat4(2,:)'])
end



if strcmp(arg,'all') | strcmp(arg,'ex10.10') | strcmp(arg,'blindeq')
  close all
  disp('Demonstration of real time-varying SISO channel')
  disp(' ')
  echo on

  randn('seed',1);
  N=300;
  sigma=0.05;
  [y0,u,b]=simchannel([-1 1],2,N,[]);
  y=y0+sigma*randn(N,1);
  nb=2;
  t0=clock;
  [thhat,uhat]=blindeqM([y u],nb,[-1,1],.001,64,.1,10,0);
  t1=etime(clock,t0);
  
  if thhat(1,N)*b(1,N)<0; thhat=-thhat; end;

  figure(1);
  book('plotfix')
  plot(thhat')
  hold on 
  plot(b')
  hold off

  snr=sum(sum(y0'*y0))/sum(sum((y-y0)'*(y-y0)));
  snr=10*log10(snr);
   ber=(u-uhat)'*(u-uhat)/(4*N);
  disp(['BER = ',num2str(ber)])
  disp(['SNR [dB] = ',num2str(snr)])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/blindeq1.eps, end
  if saveplots~=1;
    MC=2;
    sigma=[0.05 0.2 0.4  0.7 1];
  else
    MC=10;
    sigma=[0.05 0.1 0.2 0.3 0.4 0.5 0.7 0.85 1];
  end
  disp(['Starting MC simulations. It will take ',num2str(round(MC*length(sigma)*t1/60)),' minutes'])
  tic
  for j=1:MC;
    [y0,u,b]=simchannel([-1 1],2,N,[]);
    for i=1:length(sigma);
      waitbar(((j-1)*MC+i)/(MC*length(sigma)));
      y=y0+sigma(i)*randn(N,1);
      [thhat,uhat]=blindeqM([y u],nb,[-1,1],.001,64,.1,10,0);
      if thhat(1,N)*b(1,N)<0; thhat=-thhat; end;
      snrtmp=sum(sum(y0'*y0))/sum(sum((y-y0)'*(y-y0)));
      snr(i,j)=10*log10(snrtmp);
      ber(i,j)=(u-uhat)'*(u-uhat)/(4*N);
    end
  end
  toc
  figure(2)
  book('plotfix')
  snr=mean(snr')';
  ber=mean(ber')';
  disp(snr)
  disp(ber)
  plot(snr,ber)
  xlabel('SNR [dB]')
  ylabel('BER ')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/blindeq2.eps, end
end

% ch11 11 11 11 11 11


if strcmp(arg,'all') | strcmp(arg,'sec11.5.1')
close all
nnn='nnnDCm1';
N=80;
f1=[1;0];
f2=[0;1];
TH=[f1 -f1 f2 -f2];
jumps=[3*N/8 4*N/8 5*N/8 6*N/8];
u=randn(N,1);
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
[y,x0]=simchange([u d],nnn,jumps,TH,1);
%cdfigure(1)
%plot(y), 
%pause

L=2;
z=[y u];
[r,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
[A,Bu,Bv,Bd,Bf,Q]=feval(nnn,0,0,0,5);
[C,Du,Df,R]=feval(nnn,0,0,0,6);
Ordo,Hu,Hd,Hf,Null

figure(1)
subplot(211)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
title(['Unstructured residuals for L = ',num2str(L)])

F=eye(2);
R=eye(2);
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,R,F);
w=T*Null
subplot(212)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
axis([0 N -1 2])
title(['Structured residuals for L = ',num2str(L)])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosis1.eps, end

L=3;
r1=faultdetect(z,nnn,L);
[r2,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,R,F);
w=T*Null
figure(2)
subplot(211)
book('plotfix')
p=plot(r1);
set(p(1),'Linewidth',4);
set(p(2),'Linewidth',3);
set(p(3),'Linewidth',2);
set(p(4),'Linewidth',1);
legend(p,'r_1','r_2','r_3','r_4')
axis([0 N -1 2])
title(['Unstructured residuals for L = ',num2str(L)])
subplot(212)
book('plotfix')
p(1)=plot(r2(:,1),'-');
hold on
p(2)=plot(r2(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
axis([0 N -1 2])
title(['Structured residuals for L = ',num2str(L)])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosis2.eps, end

L=4;
r1=faultdetect(z,nnn,L);
[r2,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,R,F);
w=T*Null
figure(3)
subplot(211)
book('plotfix')
p=plot(r1);
set(p,'Linewidth',2);
% $$$ set(p(2),'Linewidth',3);
% $$$ set(p(3),'Linewidth',2);
% $$$ set(p(4),'Linewidth',1);
% $$$ legend(p,'r_1','r_2','r_3','r_4')
axis([0 N -1 2])
title(['Unstructured residuals for L = ',num2str(L)])
subplot(212)
book('plotfix')
p(1)=plot(r2(:,1),'-');
hold on
p(2)=plot(r2(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
axis([0 N -1 2])
title(['Structured residuals for L = ',num2str(L)])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosis3.eps, end

disp('Fault model: actuator or angular sensor offset')
disp('No disturbance')
F=eye(2);
R=eye(2);

nnn='nnnDCm2';
[y,x0]=simchange([u d],nnn,jumps,TH,1);

L=2;
z=[y u];
[r1,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
[r2,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L,R,F);

figure(4)
subplot(211)
book('plotfix')
p(1)=plot(r1(:,1),'-');
hold on
p(2)=plot(r1(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
title(['Unstructured residuals for L = ',num2str(L)])
subplot(212)
book('plotfix')
p(1)=plot(r2(:,1),'-');
hold on
p(2)=plot(r2(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
title(['Structured residuals for L = ',num2str(L)])
Hf,Null,NullHf=Null*Hf
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosis4.eps, end

L=3;
z=[y u];
[r1,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
[r2,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L,R,F);
figure(5)
subplot(211)
book('plotfix')
p=plot(r1);
set(p(1),'Linewidth',4);
set(p(2),'Linewidth',3);
set(p(3),'Linewidth',2);
set(p(4),'Linewidth',1);
legend(p,'r_1','r_2','r_3','r_4')
title(['Unstructured residuals for L = ',num2str(L)])
subplot(212)
book('plotfix')
p(1)=plot(r2(:,1),'-');
hold on
p(2)=plot(r2(:,2),'--');
hold off
set(p,'Linewidth',2);
legend(p,'r_1','r_2')
title(['Structured residuals for L = ',num2str(L)])
Hf,Null,NullHf=Null*Hf,rank(NullHf)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosis5.eps, end

end


if strcmp(arg,'all') | strcmp(arg,'sec11.5.2')
close all

load  labmotor
F=tf([0 1.67],[1 1.67]);           % pre-filter     
t=(0:0.01:25)';
%r=lsim(F,rnom,t)-0.02*cos(4.5*t);

N=length(y1);
z1=[y1 r1];
z2=[y2 r2];
z3=[y3 r3];
z4=[y4 r4];
zd1=[decimate(y1,10) decimate(r1,10)];
zd2=[decimate(y2,10) decimate(r2,10)];
zd3=[decimate(y3,10) decimate(r3,10)];
zd4=[decimate(y4,10) decimate(r4,10)];
nnn=[4 4 1];

G=tf([0 0 140],[1 3.5 0]);         % System (DC-motor)
C=tf([0.17 0.886 1],[0.625 6 0]);  % controller (PID)
Gc=feedback(C*G,1);                % closed-loop system
ym1=lsim(Gc,r1,t);
ym2=lsim(Gc,r2,t);
ym3=lsim(Gc,r3,t);
ym4=lsim(Gc,r4,t);


figure(5)
book('plotfix')
plot(t,y1-ym1+0.6,t,y2-ym2+0.2,t,y3-ym3-0.6,t,y4-ym4-0.2)
h1=xlabel('Time [s]');
h2=ylabel('Residuals');
axis([0 25 -1 1])

h3=text(9,-0.5,'Data set 4: disturbances and system changes');
h4=text(9,0.7,'Data set 1');
h5=text(9,0.3,'Data set 3: disturbances');
h6=text(9,-0.0,'Data set 2: system change');
set([h1 h2 h3 h4 h5 h6],'FontSize',12)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor5.eps, end  


Ts=0.1; %after decimation

Gd=c2d(G,0.1)
Cd=c2d(C,0.1)
Gcss=ss(Gc);            
Gcdss=c2d(Gcss,0.01);
Gcd=c2d(Gc,0.1)
Gcdss=ss(Gcd)

%Gcdss=ss(Gcd);
%[A,B,C,D]=ssdata(Gcdss);
th=arx(zd1,[4 4 1]);
present(th)


h=10;
nu=2;
%[xhat,yhat,P,Epsi]=adkalman(z1,'labmotor',1,[2 2]);
[xhat1,jhat1,xseg,gt1,ta]=detect1(z1,'nnnlabmotor',2,[1 h nu]);
[xhat2,jhat2,xseg,gt2,ta]=detect1(z2,'nnnlabmotor',2,[1 h nu]);
[xhat3,jhat3,xseg,gt3,ta]=detect1(z3,'nnnlabmotor',2,[1 h nu]);
[xhat4,jhat4,xseg,gt4,ta]=detect1(z4,'nnnlabmotor',2,[1 h nu]);
L=100;
%L=6; % R=F=1; gives nice results with threshold hfd=0.002
hfd=1;
[res1,xhatfd]=faultdetect(z1,'nnnlabmotor',L,1,1);
[res2,xhatfd]=faultdetect(z2,'nnnlabmotor',L,1,1);
[res3,xhatfd]=faultdetect(z3,'nnnlabmotor',L,1,1);
[res4,xhatfd]=faultdetect(z4,'nnnlabmotor',L,1,1);

figure(6), 
handle=subplot(311);
book('plotfix')
p=plot(y1-ym1);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 1')
set(handle,'XTick',[])

handle=subplot(312);
book('plotfix')
p=segplot(gt1,jhat1);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')

subplot(313)
book('plotfix')
p=plot(res1);
set(p,'Linewidth',2);
hold on, plot([0 N],[hfd hfd],'--'), hold off
hold on, plot([0 N],[-hfd -hfd],'--'), hold off
axis([0 2501 -3 3])
title('Residuals from fault detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor6.eps, end  

figure(7), 
handle=subplot(311);
book('plotfix')
p=plot(y2-ym2);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 2')
set(handle,'XTick',[])

handle=subplot(312);
book('plotfix')
p=segplot(gt2,jhat2);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')

subplot(313)
book('plotfix')
p=plot(res2);
set(p,'Linewidth',2);
hold on, plot([0 N],[hfd hfd],'--'), hold off
hold on, plot([0 N],[-hfd -hfd],'--'), hold off
axis([0 2501 -3 3])
title('Residuals from fault detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor7.eps, end  

figure(8), 
handle=subplot(311);
book('plotfix')
p=plot(y3-ym3);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 3')
set(handle,'XTick',[])

handle=subplot(312);
book('plotfix')
p=segplot(gt3,jhat3);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')

subplot(313)
book('plotfix')
p=plot(res3);
set(p,'Linewidth',2);
hold on, plot([0 N],[hfd hfd],'--'), hold off
hold on, plot([0 N],[-hfd -hfd],'--'), hold off
axis([0 2501 -3 3])
title('Residuals from fault detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor8.eps, end  

figure(9), 
handle=subplot(311);
book('plotfix')
p=plot(y4-ym4);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 4')
set(handle,'XTick',[])

handle=subplot(312);
book('plotfix')
p=segplot(gt4,jhat4);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')

subplot(313)
book('plotfix')
p=plot(res4);
set(p,'Linewidth',2);
hold on, plot([0 N],[hfd hfd],'--'), hold off
hold on, plot([0 N],[-hfd -hfd],'--'), hold off
axis([0 2501 -3 3])
title('Residuals from fault detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor9.eps, end  


figure(10), 
handle=subplot(211);
book('plotfix')
p=plot(y1-ym1);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 1')
set(handle,'XTick',[])

handle=subplot(212);
book('plotfix')
p=segplot(gt1,jhat1);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor10.eps, end  

figure(11), 
handle=subplot(211);
book('plotfix')
p=plot(y2-ym2);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 2')
set(handle,'XTick',[])

handle=subplot(212);
book('plotfix')
p=segplot(gt2,jhat2);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor11.eps, end  

figure(12), 
handle=subplot(211);
book('plotfix')
p=plot(y3-ym3);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 3')
set(handle,'XTick',[])

handle=subplot(212);
book('plotfix')
p=segplot(gt3,jhat3);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor12.eps, end  

figure(13), 
handle=subplot(211);
book('plotfix')
p=plot(y4-ym4);
set(p,'Linewidth',2);
axis([0 2501 -2 2])
title('Model residuals, data set 4')
set(handle,'XTick',[])

handle=subplot(212);
book('plotfix')
p=segplot(gt4,jhat4);
set(p(1),'Linewidth',2);
xlabel(' ')
hold on, plot([0 N],[h h]), hold off
set(handle,'XTick',[])
axis([0 2501 0 1.2*h])
title('Test statistics from whiteness change detection')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/labmotor13.eps, end  
end



if strcmp(arg,'all') | strcmp(arg,'sec11.5.3')
close all
disp('Aircraft')
nnn='nnnaircraft';
N=80;
f1=[1;0;0;0;0;0];
f2=[0;0;0;0;0;1];
TH=[f1 -f1 f2 -f2];
jumps=[1*N/8 3*N/8 4*N/8 6*N/8];
u=randn(N,3);
e=randn(N,3);
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
[y0,x0]=simchange([u d],nnn,jumps,TH,1);
y=y0+0*e;
%cdfigure(1)
%plot(y), 
%pause

disp('Standard design, unstructured residuals')
L=5;
z=[y u];
[r,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
%[A,Bu,Bv,Bd,Bf,Q]=feval(nnn,0,0,0,5)
%eig(A)
%pause
%[C,Du,Df,R]=feval(nnn,0,0,0,6)
%pause
%Ordo,Hu,Hd,Hf,Null
%pause

figure
subplot(311)
book('plotfix')
p=plot(r);
set(p,'Linewidth',2);
axis([0 N -1 2])
title(['Unstructured residuals for L = ',num2str(L)])
set(gca,'XTick',[])
disp('Standard design, structured residuals')
Fstruc=[f1 f2];
Rstruc=eye(2);
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc);
r0=r;
w=T*Null
wnz=abs(w(:));
ind=find(abs(wnz)>1e-6);
wnz=wnz(ind);
disp(['max(w)/min(w) = ',num2str(max(wnz)/min(wnz))])
[Q,R]=qr(Null); R
subplot(312)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Structured residuals for L = ',num2str(L)])
%pause

disp('Minimal order filter design')
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc,1);
w=T*Null
wnz=abs(w(:));
ind=find(abs(wnz)>1e-6);
wnz=wnz(ind);
disp(['max(w)/min(w) = ',num2str(max(wnz)/min(wnz))])
[Q,R]=qr(Null); R
subplot(313)
book('plotfix')

p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);

axis([0 N -1 2])
xlabel('Time [samples]')
title(['Structured residuals for L = ',num2str(L),' and minimum order filter'])

if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosisaircraft1.eps, end



disp('Noise and L=5')
L=5;
e=0.001*randn(N,3);
y=y0+e;

z=[y u];
[r,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
figure
subplot(311)
book('plotfix')
p=plot(r);
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Unstructured residuals for L = ',num2str(L)])
Fstruc=[f1 f2];
Rstruc=eye(2);
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc);
subplot(312)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Structured residuals for L = ',num2str(L)])
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc,1);
subplot(313)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
xlabel('Time [samples]')
title(['Structured residuals for L = ',num2str(L),' and minimum order filter'])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosisaircraft2.eps, end

disp('Noise and L=10')
L=10;
e=0.001*randn(N,3);
y=y0+e;
covy=cov(y0)
cove=cov(e)
snr=sum(sum(covy))/sum(sum(cove))

z=[y u];
[r,Ordo,Hu,Hd,Hf,Null]=faultdetect(z,nnn,L);
figure
subplot(311)
book('plotfix')
p=plot(r);
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Unstructured residuals for L = ',num2str(L)])
Fstruc=[f1 f2];
Rstruc=eye(2);
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc);

subplot(312)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Structured residuals for L = ',num2str(L)])
[r,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc,1);

subplot(313)
book('plotfix')
p(1)=plot(r(:,1),'-');
hold on
p(2)=plot(r(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
xlabel('Time [samples]')
title(['Structured residuals for L = ',num2str(L),' and minimum order filter'])
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosisaircraft3.eps, end

disp('Standard design, model error A(1,3)=1.15 (1.1332)')
figure
nnn='nnnaircraftm'; % Error model
Fstruc=[f1 f2];
Rstruc=eye(2);
L=5;
[rmodelerror,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc);

subplot(311)
book('plotfix')
p(1)=plot(r0(:,1),'-');
hold on
p(2)=plot(r0(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Design based on correct model for L = ',num2str(L)])

subplot(312)
book('plotfix')
p(1)=plot(rmodelerror(:,1),'-');
hold on
p(2)=plot(rmodelerror(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
set(gca,'XTick',[])
title(['Model error A(1,3)=1.15 (1.1332) for L = ',num2str(L)])
L=10;
[rmodelerror,Ordo,Hu,Hd,Hf,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc);
subplot(313)
book('plotfix')
p(1)=plot(rmodelerror(:,1),'-');
hold on
p(2)=plot(rmodelerror(:,2),'--');
hold off
set(p,'Linewidth',2);
axis([0 N -1 2])
xlabel('Time [samples]')
title(['Model error A(1,3)=1.15 (1.1332) for L = ',num2str(L)])
%pause
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/diagnosisaircraft4.eps, end
end



if strcmp(arg,'all') | strcmp(arg,'sec11.notyet')
  % KF bank, to be designed
close all
disp('Aircraft')
nnn='nnnaircraft'; 
N=80;
f1=[1;0;0;0;0;0];
f2=[0;0;0;0;0;1];
TH=[f1 -f1 f2 -f2];
jumps=[1*N/8 3*N/8 4*N/8 6*N/8];
u=randn(N,3);
e=randn(N,3);
d=[zeros(N/8,1); randn(N/8,1); zeros(6*N/8,1)];
[y0,x0]=simchange([u d],nnn,jumps,TH,1);
y=y0+0*e;

nnn='nnnaircrafta'; % Augmented model
save tmp
[xhatkf,yhatkf]=adkalman([y u],nnn);
[xhatM,jhat,xseg,lhat]=detectM([y u],nnn,2,0.7);

figure(1)
book('plotfix')
h1=segplot([xhatM(6:11,:)'],jhat);
hold on
h2=plot(xhatkf(6:11,:)','--');
hold off
legend([h1(1) h2(2)],'Kalman filter bank','Kalman filter');
title('Fault state tracking')
end


% ch12 1212121212

if strcmp(arg,'all') | strcmp(arg,'ex12.1')
   close all
   randn('state',0)
   th0=[zeros(50,1);(0.02:0.02:1)';ones(50,1)];
   y=th0+randn(150,1);
   th1=filter(0.1,[1 -0.9],y);
   th2=zeros(150,50);
   N=150;
   for i=1:50;
      y=th0+randn(150,1);
      th2(:,i)=filter(0.1,[1 -0.9],y);
   end
   th3=zeros(150,1000);
   for i=1:1000;
      y=th0+randn(150,1);
      th3(:,i)=filter(0.1,[1 -0.9],y);
   end
   figure(1)
   subplot(311)
   book('plotfix')
   p1=plot(th0,'--'); hold on
   p2=plot(th1,'-'); hold off
   set([p1 p2],'Linewidth',2);
   axis([0 N -0.5 1.5])
   subplot(312)
   book('plotfix')
   p1=plot(th0,'--'); hold on
   p2=plot(mean(th2'),'-');
   p3=plot(mean(th2')+std(th2'),':');
   p4=plot(mean(th2')-std(th2'),':'); hold off
   set([p1 p2 p3 p4],'Linewidth',2);
   axis([0 N -0.5 1.5])
   subplot(313)
   book('plotfix')
   p1=plot(th0,'--'); hold on
   p2=plot(mean(th3'),'-');
   p3=plot(mean(th3')+std(th3'),':');
   p4=plot(mean(th3')-std(th3'),':'); hold off
   set([p1 p2 p3 p4],'Linewidth',2);
   axis([0 N -0.5 1.5])
   xlabel('Time [samples]')
   if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex12_1.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex12.2')
  close all
  rand('seed',0);
  N=50;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;
  NNNmethod=1;
  [xhat]=adfilter(y,nnn,1,[],[],[],1e7,0);

  figure(1);
  book('plotfix')
  radarplot(y,xhat);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex12_2a.eps, end

  figure(2);
  subplot(211)
  book('plotfix')
  p=plot(sqrt(sum((x0(1:2,:)-xhat(1:2,:)).^2)));
  set([p],'Linewidth',2);
  xlabel('Time [samples]')
  ylabel('Distance error [m]')
  title('RMSE for one realization')
  set(gca,'XTick',[])

  MC=10;
  disterr=zeros(1,N);
  for i=1:MC;
    NNNmethod=5;
    [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
    NNNmethod=1;
    [xhat]=adfilter(y,nnn,1,[],[],[],1e7,0);
    disterr=disterr+sqrt(sum((x0(1:2,:)-xhat(1:2,:)).^2));
  end
  subplot(212)
  book('plotfix')
  p=plot(disterr);
  set([p],'Linewidth',2);
  xlabel('Time [samples]')
  ylabel('Distance error [m]')
  title(['RMSE for ',num2str(MC),' realizations'])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ex12_2b.eps, end
end


if strcmp(arg,'all') | strcmp(arg,'ex12.3')
% BS1 Bootstrap for computing the uncertainty when
% estimating the mean of Gaussian variables
% See Signal processing magazine No 1 1998, Zoubir and Boashash

n=1000;
N=10;
mu=10;
sigma=5;

%e0=[-2.41 4.86 6.06 9.11 10.2 12.81 13.17 14.10 15.77 15.79]; % magazine ex

MC=20;
MCtot=[];
for j=1:MC;

  e0=mu+sigma*0.5*sum(randn(4,N)); % extra Gaussian!
  muhat=mean(e0);
  sigmahat=std(e0);
  % MC
  for i=1:n;
    e=mu+sigma*0.5*sum(randn(4,N));
    muhatMC(i)=mean(e);
    sigmahatMC(i)=std(e);
  end
  % Bootstrap
  for i=1:n;
    ind=ceil(N*rand(1,N));
    e=e0(ind);   % resampling
    muhatBS(i)=mean(e);
    sigmahatBS(i)=std(e);
  end

  MCtot=[MCtot;sigmahat^2/N std(muhatMC)^2 std(muhatBS)^2];

end

figure(1);
book('plotfix')
[Nhist,Xhist]=hist(muhatMC);
bar(Xhist,Nhist,'w')
title('muhat from MC')
figure(2);
book('plotfix')
[Nhist,Xhist]=hist(muhatBS);
bar(Xhist,Nhist,'w')
title('muhat from BS')

disp('Result from one experiment')
disp(' ')
disp('Statistics           Point estimate    Monte Carlo     Bootstrap     Theoretical')
disp(['Mean of muhat ','         ',num2str(mean(muhat)),'         ',num2str(mean(muhatMC)),'         ',num2str(mean(muhatBS)),'       ',num2str(mu)])
disp(['Var of  muhat ','         ',num2str(sigmahat^2/(N)),'         ',num2str(std(muhatMC)^2),'         ',num2str(std(muhatBS)^2),'       ',num2str(sigma^2/N)])
disp(['Mean of Var   ','         ',num2str(mean(sigmahat.^2)),'         ',num2str(mean(sigmahatMC.^2)),'         ',num2str(mean(sigmahatBS.^2)),'       ',num2str(sigma^2)])

disp(' ')
disp(['Result from ',num2str(MC),' experiments'])
disp(['Mean of Var   ','         ',num2str(mean(MCtot))]);
end

if strcmp(arg,'all') | strcmp(arg,'ex12.4')
% BS2 Bootstrap for computing the uncertainty when
% estimating an AR(1) model.
% See Signal processing magazine No 1 1998, Zoubir and Boashash

close all
randn('state',0)
n=1000;
N=10;
mu=0;
sigma=1;
a0=0.7;    % AR parameter
sigma0=sqrt((1-a0^2)/N);  % theoretical variance

MC=20;
MCtot=[];
e0=mu+sigma*0.5*sum(randn(4,N)); % extra Gaussian!
y0=filter(1,[1 a0],e0)';
ahat0=-y0(1:N-1)\y0(2:N);   % ML estimate
% ahat0,a0

for j=1:MC;
  e0=mu+sigma*0.5*sum(randn(4,N)); % extra Gaussian!
  y0=filter(1,[1 a0],e0)';
  ahat=-y0(1:N-1)\y0(2:N);   % ML estimate
  sigmahat=sqrt((1-ahat^2)/N);  % theoretical variance
  % MC
  for i=1:n;
    e=mu+sigma*0.5*sum(randn(4,N));
    y=filter(1,[1 a0],e)';
    ahatMC(i)=-y(1:N-1)\y(2:N);   % ML estimate
    sigmahatMC(i)=(1-ahatMC(i)^2)/N;  % theoretical variance
  end
  % Bootstrap
  for i=1:n;
    ehat=filter([1 ahat0],1,y0)';
    ind=ceil(N*rand(1,N));
    e=ehat(ind);   % resampling
    y=filter(1,[1 ahat0],e)';
    ahatBS(i)=-y(1:N-1)\y(2:N);   % ML estimate
    sigmahatBS(i)=(1-ahatBS(i)^2)/N;  % theoretical variance
  end

  MCtot=[MCtot;sigmahat std(ahatMC) std(ahatBS)];

end

figure(1);
book('plotfix')
[Nhist,Xhist]=hist(ahatMC,30);
bar(Xhist,Nhist,'w')
a=axis;
title('Estimated \theta from MC')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/BS2a.eps, end
figure(2);
book('plotfix')
[Nhist,Xhist]=hist(ahatBS,30);
bar(Xhist,Nhist,'w')
axis(a)
title('Estimated \theta from BS')
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/BS2b.eps, end

disp('Result from one experiment')
disp(' ')
disp('Statistics           Point estimate    Monte Carlo     Bootstrap     Theoretical')
disp(['Mean of ahat ','         ',num2str(mean(ahat)),'         ',num2str(mean(ahatMC)),'         ',num2str(mean(ahatBS)),'       ',num2str(a0)])
disp(['Std  of ahat ','         ',num2str(sigmahat),'         ',num2str(std(ahatMC)),'         ',num2str(std(ahatBS)),'       ',num2str(sigma0)])
disp(['Mean of sigmahat(ahat) ','',num2str((sigmahat)),'         ',num2str(mean(sigmahatMC)),'         ',num2str(mean(sigmahatBS)),'       ',num2str(sigma0)])

disp(' ')
disp(['Result from ',num2str(MC),' experiments'])
disp(['Mean of Std   ','         ',num2str(mean(MCtot))]);
end


if strcmp(arg,'all') | strcmp(arg,'ex12.5')
  close all
  MC=250;
  h=3;
  nu=0.5;
  GT=[];
  MCjumphat=[];
  for j=1:MC;
    y0=[zeros(20,1);ones(20,1);3*ones(20,1)];
    y=y0+randn(60,1);
    [threc,jumphat,thseg,gt,ta]=detect1(y,-1,1,[2,h,nu],1);
    MCjumphat=[MCjumphat; ta(:)];
    GT=[GT gt];
  end
  figure(1);
  book('plotfix')
  [Nhist,Xhist]=hist(MCjumphat,60);
  bar(Xhist,Nhist,'w')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arl.eps, end
  figure(2);
  book('plotfix')
  plot(1:60,y,'-',1:60,y0,'--')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/arl2.eps, end
  ind1=find(MCjumphat>19 & MCjumphat<31);
  DFD1=mean(MCjumphat(ind1)-20)
  MDR1=(MC-length(ind1))/MC
  ind2=find(MCjumphat>39 & MCjumphat<51);
  DFD2=mean(MCjumphat(ind2)-40)
  MDR2=(MC-length(ind2))/MC
  FAR=(length(MCjumphat)-length(ind1)-length(ind1))/(60*MC)
end


if strcmp(arg,'all') | strcmp(arg,'ex12.6')


%    Demostration of design of CUSUM detectors
%   ===========================================
%
% Assume we observe a signal y(t) which is N(0,sigma^2) before the change and
% N(theta,sigma) after the change. We apply the CUSUM algorithm with threshold h
% and drift nu. A succesful design requires of course that theta > nu/2.
% Assume in this example that theta = nu.
%
% The average run length (ARL) function for a stopping rule for 
% detecting a change theta in the mean of a signal is defined as
% 
%        E(ta|theta)
% 
% where ta is the stopping time.
% There are two design parameter in the CUSUM test, the threshold h
% and the drift parameter nu. If sigma is the standard deviation of the noise, then
%
%             /   h      theta-nu \
%     ta = f |  ----- , ------------ |
%             \ sigma     sigma     /
%
% That is, it is a function of two arguments. We can assume that sigma=1
% and, for simplicity, nu = theta (a standard choice of drift if theta is known).
% The mean time between false alarms is f(h,-theta/2) and 
% the mean time for detection is        f(h,+theta/2).
% To check these for a given h, use
%

NMC=500;
h=3;
nu=0.5;
th=1;
mu=th-nu;  % 0.4
[ta0,N0] = cusumarl(h,-mu);
[ta1,N0] = cusumarl(h,+mu);
disp([ta0, ta1])
%
% That is, the mean time between false alarms is 4 times larger.
% N0 denotes the mean time beween resets in the CUSUM algorithm.
% A reset is caused by either an alarm or a negative test statistic.
%
% The approximation suggested by Siegmund is obtained by

[ta0siegmund] = cusumarl(h,-mu,0.5,-1)
[ta1siegmund] = cusumarl(h,+mu,0.5,-1)

% and the approximation suggested by Wald is obtained by

[ta0wald] = cusumarl(h,-mu,0.5,-2)
[ta1wald] = cusumarl(h,+mu,0.5,-2)

% The mean times do not say anything about the distribution of the run length,
% which can be quite unsymmetric. Monte Carlo simulations can be used for further
% analysis of the run length function:
%
L0=cusumMC(h,-mu,NMC);
h1=figure(1);
book('plotfix')
[Nhist,Xhist]=hist(L0,30);
bar(Xhist,Nhist,'w')
xlabel('Alarm time [samples]')
hh=title('Distribution of false alarm times');
L1=cusumMC(h,mu,NMC);

h2=figure(2);
book('plotfix')
[Nhist,Xhist]=hist(L1,30);
bar(Xhist,Nhist,'w')
xlabel('Delay for detection [samples]')
hh=title('Distribution of delay for detection');
% Compare theoretical with achieved mean times;

disp('Numerical    Wald      Siegmund       MC')
disp([ta0 ta0wald ta0siegmund mean(L0)])
disp([ta1 ta1wald ta1siegmund mean(L1)])

%close([h1 h2])


% The distribution for false alarms is basically binominal where 
% p=P(reset caused by alarm | reset)  since N0 is small.
% A rough estimate of the delay for detection is L0=h/(theta-nu);
% 
% What is really wanted is to determine h from a specified 
% mean time for detection L0. This is computed by a numerical search in
%
hmax=10;
L0=10;
h=cusumdesign(L0,mu,hmax)

% Evaluate the design by Monte Carlo simulations
L1=cusumMC(h,mu,NMC);
mean(L1)


% Finally, we can generate a table:

MU=[-0.5 0 0.5 1 1.5 2.0];
h=3;
for i=1:length(MU);
  [ta0,N0] = cusumarl(h,MU(i),0.1);    % "Exact"
  [tas,N0] = cusumarl(h,MU(i),0.1,-1); % Siegmund
  [taw,N0] = cusumarl(h,MU(i),0.1,-2); % Wald
  L1=cusumMC(h,MU(i),NMC);
  tab(i,:)=[MU(i),ta0,taw,tas,mean(L1)];
end

format short e
disp(' ')
disp('       mu       Exact   Wald     Siegmund      MC')
%disp(tab(1:3,:));
%format short
disp(tab(:,:));
disp(' ')
disp('Table 5.1 in Basseville and Nikiforov.')

h3=figure(3);
book('plotfix')
handle=semilogy(MU,tab(:,2),'-',MU,tab(:,3),'--',MU,tab(:,4),':', ...
		MU,tab(:,5),'-.');
set(handle,'Linewidth',1.5)
legend(handle,'Theoretical','Wald''s approx.','Siegmund''s approx.','Monte Carlo aprox.')
hh2=title('Average run length [samples]: from simulations and theory');
hh1=xlabel('\theta - \nu');



figure(h1)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tutcusum1.eps, end
figure(h2)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tutcusum2.eps, end
figure(h3)
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tutcusum3.eps, end

MU=[-1:0.2:2];
H=[1:0.5:5];
for i=1:length(MU);
  for j=1:length(H);
    tab2(j,i) = cusumarl(H(j),MU(i),0.1,-2);
  end
end
figure(4);
book('plotfix')
%mesh(MU(length(MU):-1:1),H(length(H):-1:1),log10(tab2))
mesh(MU(length(MU):-1:1),H,log10(tab2))
colormap(gray)
hh1=zlabel('log(ARL)');
hh2=xlabel('\theta-\nu');
hh3=ylabel('h');
if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tutcusum4.eps, end


end


if strcmp(arg,'all')
  t1=etime(clock,t000);
  disp(['Elapsed time: ',num2str(t1)])
end


% Miscellaneous










if strcmp(arg,'all') | strcmp(arg,'ex261') | strcmp(arg,'highway')
  load highway

  nnn='nnnplane';
  global NNNmethod
  NNNmethod=1; % four states
  kf=[2 1];
  q1=1e-3;
  [xhat1,yhat,P,Epsi,Epsin]=adkalman(y,nnn,q1,kf,1e10*eye(4),zeros(4,1)); 

  figure(2)
  book('plotfix')
  h(1)=plot(y(:,1),y(:,2));
  ax=axis;
  hd=title('Car position from computer vision system');
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/highway0.eps, end

end


if strcmp(arg,'all') | strcmp(arg,'ML')
randn('seed',2)
x=-2:0.1:5;
y=randn(40,1)+1;
y(1)
subplot(221)
book('plotfix')
p1=1/sqrt(2*pi*1)*exp(-0.5*(x-y(1)).^2);
plot(x,p1)
a=axis;
hold on, plot([1 1],[0 a(4)],'--'), hold off
title('l1')

subplot(222)
book('plotfix')
p=ones(size(x));
for i=1:3;
  p=p.*(1/sqrt(2*pi*1)*exp(-0.5*(x-y(i)).^2));
end
plot(x,p)
a=axis;
hold on, plot([1 1],[0 a(4)],'--'), hold off
title('l3')

subplot(223)
book('plotfix')
p=ones(size(x));
for i=1:5;
  p=p.*(1/sqrt(2*pi*1)*exp(-0.5*(x-y(i)).^2));
end
plot(x,p)
a=axis;
hold on, plot([1 1],[0 a(4)],'--'), hold off
title('l5')

subplot(224)
book('plotfix')
p=ones(size(x));
for i=1:10;
  p=p.*(1/sqrt(2*pi*1)*exp(-0.5*(x-y(i)).^2));
end
plot(x,p)
a=axis;
hold on, plot([1 1],[0 a(4)],'--'), hold off
title('l10')

if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/ml.eps, end
end

if strcmp(arg,'all') | strcmp(arg,'ex3.4')
   close all
   randn('seed',2)
x=(0:0.1:4)';
lam=0.1:0.1:3;
y=randn(40,1)+2;
p=ones(length(x),length(lam));
figure;
for i=1:length(y);
%  for k=1:length(x);
%    for l=1:length(lam);
%      p1(k,l)=1./sqrt(2*pi*lam(l))*exp(-0.5*(x(k)-y(i)).^2/lam(l));
%    end
%  end
  p1=(( ones(size(x)))*(1./sqrt(2*pi*lam)).*( exp(-0.5*(x-y(i)).^2*(1./lam))));
  p=p.*p1;
  p=p/sum(sum(p));
  figure(1)
  book('plotfix')
  mesh(lam,x,p)
  ylabel('\theta')
  xlabel('R')
  drawnow
  title(['N = ',num2str(i)])
%  pause(1/i)
  [dum,ind]=max(sum(p));
  lamhat(i)=lam(ind);
  [dum,ind]=max(sum(p'));
  thhat(i)=x(ind);
  if i==1
     if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/MLa.eps, end
  elseif i==5
     if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/MLb.eps, end
  elseif i==20
     if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/MLc.eps, end
  elseif i==40
     if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/MLd.eps, end
  end
  
end

  figure(2)
  book('plotfix')
  subplot(221)
  book('plotfix')
  plot(lam,sum(p))
  title('Marginal distribution for R')
  subplot(222)
  book('plotfix')
  plot(x,sum(p'))
  title('Marginal distribution for \theta')
  subplot(223)
  book('plotfix')
  plot(thhat)
  title('ML estimate of \theta as a function of time')
  subplot(224)
  book('plotfix')
  plot(lamhat)
  title('ML estimate of R as a function of time')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/MLe.eps, end
end


if strcmp(arg,'kin')
  randn('state',2);
  rand('state',0);
  close all
  nnn='nnnkin';
  N=100;
  jumps=50;
  TH=80;
  Lam=1;
  [y,x0]=simchange(N,nnn,jumps,TH,Lam);
  [xhat4,jhat4mc,jhat4]=gibbs(y,nnn,[1 1],0.1,[1 20 0],[1 1 1]);
  [xhat3,jhat3,xseg,lamseg,xhat3,Alfa,Xn]=detectM(y,nnn,[1 1],0.1,[12 9 0]);
  [xhat2,jhat2,xseg,GT,ta]=detect1(y,nnn,1,[2 0.6 2],1);
  figure(1)
  book('plotfix')
  hypplot(Xn)
  %plot(GT)
  [xhat1]=adfilter(y,nnn,1);

  e=x0(1,:)'-y;
  e1=xhat1(1,:)'-x0(1,:)';
  e2=xhat2(1,:)'-x0(1,:)';
  e3=xhat3(1,:)'-x0(1,:)';
  e4=xhat4(1,:)'-x0(1,:)';
  disp(['Measurement RMSE:  ',num2str(sqrt(e'*e/N))]);
  disp(['adfilter RMSE:     ',num2str(sqrt(e1'*e1/N))]);
  disp(['detect1  RMSE:     ',num2str(sqrt(e2'*e2/N)),' and jump time(s): ',cdnum2str(jhat2)]);
  disp(['detectM  RMSE:     ',num2str(sqrt(e3'*e3/N)),' and jump time(s): ',cdnum2str(jhat3)]);
  disp(['gibbs    RMSE:     ',num2str(sqrt(e4'*e4/N)),' and jump time(s): ',cdnum2str(jhat4)]);

  figure(1);
  book('plotfix')
  plot([e e1 e2 e3 e4])
  figure(2);
  book('plotfix')
  plot([x0(2,:)' xhat1(2,:)' xhat2(2,:)' xhat3(2,:)' xhat4(2,:)'])


end



   
if  strcmp(arg,'QRest')

% tracking application: estimate scaling Q, and Q,R, respectively.

randn('state',0)
N=36;
sigma1o=0; % No velocity
sigma2o=1;
sigma3o=3;

% Simulation of state space model
% x(t+1) = x(t) + v(t) + sigma1*w1(t)
% v(t+1) =        v(t) + sigma2*w2(t)
% y(t)   = x(t)        + sigma3*e(t)


w1=sigma1o*randn(N,1);
w2=sigma2o*randn(N,1);
e=sigma3o*randn(N,1);

v=filter([1 0],[1 -1],w2);
x=filter([0 1],[1 -1],w1+v);
y=x+e;

figure(1)
plot(y)

%-----Identification of variances------

% Form impulse response matrices from w1, w2, e
t=(1:N)';
H1=toeplitz(t,[1 zeros(1,N-1)]);
H2=toeplitz(ones(N,1),[1 zeros(1,N-1)]);
H3=eye(N);
% Form linear regression
r=covf(y,N);
R=toeplitz(r,[r(1) zeros(1,N-1)]);
Phi1=H1*H1';
Phi2=H2*H2';
Phi3=H3*H3';
% y = N(0,Phi1*sigma1+Phi2*sigma2+Phi3*sigma3)

% First, estimate just qv in Q
sigma1=0; 
sigma2= 1:0.1:10;
sigma3=1; 

for i=1:length(sigma1);
  for j=1:length(sigma2);
    P=(sigma3(i)^2*H3*H3'+ sigma2(j)^2*H2*H2');
    V(i,j)=y'*inv(P)*y+log(det(P));
  end
end
figure(2)
plot(sigma2,V)
xlabel('\sigma_v')
ylabel('V(\sigma_v)')
title('Minimization w.r.t. velocity noise \sigma_v')
drawnow
  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/QRest1.eps, end

[Vmin,jhat]=min(V);
sigmahat=sigma2(jhat)
Vmin

% Second, estimate both qv in Q and R=r
sigma1=0;  % 1:0.1:10;
sigma2=2:0.2:10;
sigma3=0.2:0.2:2;
clear V

for i=1:length(sigma3);
  for j=1:length(sigma2);
    P=(sigma3(i)^2*H3*H3' + sigma2(j)^2*H2*H2');
    V(i,j)=y'*inv(P)*y+log(det(P));
  end
end

[Vi,ihat]=min(V);
[Vmin,jhat]=min(Vi);
Vmin
ihat=ihat(jhat);
sigmavhat=sigma2(jhat)
sigmaehat=sigma3(ihat)

scaling=sqrt( (y'*inv(sigmaehat^2*H3*H3'+ sigmavhat^2*H2*H2')*y)/N)

figure(3)
mesh(sigma2,sigma3,V);
xlabel('\sigma_v')
ylabel('\sigma_e')
hold on
plot3(sigma2(jhat),sigma3(ihat),V(ihat,jhat),'b*')
hold off
view([150,30])

  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/QRest2.eps, end
end


if  strcmp(arg,'nmtest')
  close all
  load nmt450
  N=length(y);
  t=1:N;
  h1=plot(t,y);
  title('Sales figures for NMT450')
  hold on
  yscale=max(y);
  y=y/yscale;
  th0=[0.0001 0.1 1];
  [t0,y0]=ode23('modelnmtode',t,0,[],th0(1),th0(2),th0(3));
  h0=plot(t0,y0*yscale,'-');

  opt=[1 0 0 0 0]; % do help foptions
  % opt(1)=display info
  % opt(5)=LevenbargMarquart (0), GN (1)
  opt(14)=10;
  th=th0;
  %fcount=[10 20 30 40 50];
  for i=1:5; %length(fcount);
    th=leastsq('modelnmt',th,opt,[],y)
    [that,yhat]=ode45('modelnmtode',t,0,[],th(1),th(2),th(3));
    h2(i)=plot(that,yhat*yscale,'--');
  end
  set([h1 h2(length(h2))],'LineWidth',2);
  hold off
  if saveplots==1, print -depsc /home/rt/fredrik/books/detect/figures/nmt2.eps, end

end




if  strcmp(arg,'tracking1')
  close all
  rand('seed',0);
  N=50;
  lam=1;
  nnn='nnnplane'; 
  global NNNmethod
  NNNmethod=5;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+50*e;

  figure(1);
  book('plotfix')
  h1=radarplot(y,x0);
  legend(h1,'Measured position','True position')
  NNNmethod=1;
  [xhat]=adfilter(y,nnn,1);
  figure(2);
  book('plotfix')
  radarplot(y,xhat);
  title('Initial filter design')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radar.eps, end
  q=logspace(-2,5,30);
  for i=1:length(q);
    [xhat]=adfilter(y,nnn,q(i));
    rms(i)=sqrt(1/(2*length(x0))*sum(sum((x0(1:2,:)-xhat(1:2,:)).^2)));
  end
  figure(3);
  book('plotfix')
  semilogx(q,rms)
  xlabel('Adaptation gain')
  ylabel('RMSE')
  [dum,ind]=min(rms);
  qopt=q(ind);
  title(['Optimal adaptation gain for KF is ',num2str(round(qopt))])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radaropt.eps, end
  [xhat]=adfilter(y,nnn,qopt);
  figure(4);
  book('plotfix')
  radarplot(y,xhat);
  title('Optimized filter design')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/radaroptfilt.eps, end

end





if  strcmp(arg,'tracking4')
  randn('state',1);
  rand('state',1);
  close all
  N=50;
  lam=0;
  u=zeros(N,1);
  e=randn(N,2);
  th0=[-0.05;...
       +0.05;...
       0.075;...
       -0.075]';   
  jumps=[15 18 35 38];
  nnn='nnnplane';
  global NNNmethod
  NNNmethod=5;
  [y,x0]=simchange(N,nnn,jumps,th0,lam,'pulse');
  yn=y+300*e;

  figure(1);
  book('plotfix')
  h=radarplot(yn,x0);
  title('Measurements')
  legend(h,'Measured position','True position',4)
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4.eps, end

  NNNmethod=1;
  lam=10;

  [xhat1,lamhat,Epsi]=adfilter(yn,nnn,1/lam);
  figure(2)
  book('plotfix')
  radarplot(y,xhat1);
  title('Kalman filter')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4a.eps, end


  [xhat2,jhat2,xseg,GT,ta]=detect1(yn,nnn,2,[1 3 1],lam);
  figure(3)
  book('plotfix')
  plot(GT)
  % save,return
  [tmp,jhat3,tmp]=detectM(yn,nnn,[1 lam],0.1,[10 7 0]);
  [xhat3,jhat3,xseg,lamseg,Alfa,Xn]=detectM(yn,nnn,[1 lam],0.1,[1;jhat3(:)]);
  figure(13)
  book('plotfix')
  radarplot(y,xhat3,jhat3);
  title('Filter bank')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4h.eps, end
 
  MC=100;
  Jhat2mc=[];
  Jhat3mc=[];
  for i=1:MC
    emc=randn(N,2);
    ymc=y+300*emc;
    [tmp jhat2mc tmp]=detect1(ymc,nnn,2,[1 3 1],lam);
    Jhat2mc=[Jhat2mc jhat2mc(:)'];
    [tmp jhat3mc tmp]=detectM(ymc,nnn,[1 lam],0.1,[10 7 0]);
    Jhat3mc=[Jhat3mc jhat3mc(:)'];
  end

  figure(10)
  book('plotfix')
  [Nhist,Xhist]=hist(Jhat2mc,1:N);
  bar(Xhist,Nhist,'w')
  title(['Histogram over KF whiteness test using ',num2str(MC),' Monte Carlo runs'])
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4b.eps, end
  figure(11)
  book('plotfix')
  hist(Jhat3mc,1:N)
  title(['Histogram over KF filter bank using ',num2str(MC),' Monte Carlo runs'])
  drawnow
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4c.eps, end

  %figure(4)
  [xhat4,jhat4mc,jhat4]=gibbs(yn,nnn,[1 lam],0.1,[2 MC 5],[0 1 0]);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4d.eps, end
  %figure(5)
  [xhat5,jhat5mc,jhat5]=gibbs(yn,nnn,[1 lam],0.1,[5 MC 5],[0 1 0]);
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4e.eps, end

  figure(6)
  book('plotfix')
  radarplot(y,xhat4,jhat4)
  title('Gibbs-Metropolis, n=2')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4f.eps, end

  figure(7)
  book('plotfix')
  radarplot(y,xhat5,jhat5)
  title('Gibbs-Metropolis, n=5')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4g.eps, end

  jhatems=[15;(32:40)'];
  [xhatems,jhat,xseg,lamseg,Alfa,Xn]=detectM(yn,nnn,[1 lam],0.1,[1;jhatems]);
  radarplot(y,xhatems,jhatems);
  title('EMS')
  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tracking4i.eps, end

  e=yn-y; e=e(11:50,:);
  e1=xhat1([1 3],11:50)'-x0([1 3],11:50)';
  e2=xhat2([1 3],11:50)'-x0([1 3],11:50)';
  e3=xhat3([1 3],11:50)'-x0([1 3],11:50)';
  e4=xhat4([1 3],11:50)'-x0([1 3],11:50)';
  e5=xhat5([1 3],11:50)'-x0([1 3],11:50)';
  eems=xhatems([1 3],11:50)'-x0([1 3],11:50)';
  disp(['Measurement RMSE:  ',num2str(sqrt(sum(diag(e'*e))/length(e)))]);
  disp(['adfilter RMSE:     ',num2str(sqrt(sum(diag(e1'*e1))/length(e)))]);
  disp(['detect1  RMSE:     ',num2str(sqrt(sum(diag(e2'*e2))/length(e))),' and jump time(s): ',cdnum2str(jhat2)]);
  disp(['detectM  RMSE:     ',num2str(sqrt(sum(diag(e3'*e3))/length(e))),' and jump time(s): ',cdnum2str(jhat3)]);
  disp(['gibbs    RMSE:     ',num2str(sqrt(sum(diag(e4'*e4))/length(e))),' and jump time(s): ',cdnum2str(jhat4)]);
  disp(['gibbs    RMSE:     ',num2str(sqrt(sum(diag(e5'*e5))/length(e))),' and jump time(s): ',cdnum2str(jhat5)]);
  disp(['EMS      RMSE:     ',num2str(sqrt(sum(diag(eems'*eems))/length(e))),' and jump time(s): ',cdnum2str(jhatems)]);

  figure(7);
  book('plotfix')
  plot([e e1 e2 e3 e4])
  figure(8);
  book('plotfix')
  plot([x0(2,:)' xhat1(2,:)' xhat2(2,:)' xhat3(2,:)' xhat4(2,:)'])

%  if saveplots==1, print -deps /home/rt/fredrik/books/detect/figures/tmp.eps, end

end



if strcmp(arg,'all') | strcmp(arg,'ex8.notyet')
close all
load highway
plot(diff(y(:,1)),diff(y(:,2)))

figure(1)
book('plotfix')
[Nhist,Xhist]=hist(sum(((diff(y)'))),40);
bar(Xhist,Nhist,'w')

figure(2)
book('plotfix')
plot(y(:,1),y(:,2))
hold on
plot(y(1,1),y(1,2),'o')
nnn='nnnplane4';
[xhat,lamhat,Epsi]=adfilter(y,nnn,1);
plot(xhat(1,:),xhat(2,:),'r--')
hold off

figure(3)
book('plotfix')
size(Epsi)
[Nhist,Xhist]=hist(sum(Epsi(20:length(y),:)'),40);
bar(Xhist,Nhist,'w')

q=logspace(-6,4,40);
for i=1:length(q)
  [xhat,lamhat,Epsi]=adfilter(y,nnn,q(i));
  rms(i)=sqrt(sum(sum(Epsi.^2))/2/length(Epsi));
end
[dum,ind]=min(rms);
qopt=q(ind);

figure(4)
book('plotfix')
loglog(q,rms)
xlabel('Adaptation gain')
ylabel('RMSE')
title(['Optimal adaptation gain for KF is ',num2str(qopt)])
end
